/**
 * @section DESCRIPTION
 * This class is also horriying. It was supposed to be a pool that you could retrieve
 * memory addresses from as an alterntive to system calls for heap memory.
 */
#ifndef SL_BASICPOOL_HPP
#define SL_BASICPOOL_HPP

#include <stack>
#include <iostream>

namespace sl
{

template <typename T>
class BasicPool
{
public:
	BasicPool(std::size_t poolSize = 0xFF, std::size_t pools = 1);
	~BasicPool();
	std::size_t size() const;
	T* get();
	void put(T *p);
private:
	T **pool;
	std::stack<T*> poolStack;
	std::size_t *poolUsed, pools;
	const std::size_t poolSize;
};

template <typename T>
BasicPool<T>::BasicPool(std::size_t poolSize, std::size_t pools)
	:pools(pools)
	,poolSize(poolSize)
{
	pool = new T*[pools];
	poolUsed = new std::size_t[pools];
	for (std::size_t i = 0; i < pools; ++i)
	{
		pool[i] = new T[poolSize];
		poolUsed[i] = 0;
	}
#ifdef SL_DEBUG
	std::cout << "BasicPool created of type " << typeid(T).name() << std::endl;
#endif
}

template <typename T>
BasicPool<T>::~BasicPool()
{
	for (std::size_t i = 0; i < pools; ++i)
	{
		delete[] pool[i];
	}
	delete[] pool;
	delete[] poolUsed;
}

template <typename T>
std::size_t BasicPool<T>::size() const
{
	return poolSize * pools;
}

template <typename T>
T* BasicPool<T>::get()
{
	T* block = nullptr;
	if (poolStack.empty())
	{
		for (std::size_t i = 0; i < pools; ++i)
		{
			if (poolUsed[i] + sizeof(T) <= poolSize)
			{
				int t = 0;
				for (std::size_t j = 0; j < pools; ++j)
				{
					t += poolSize - poolUsed[j];
				}
				block = &pool[i][poolUsed[i]];
				++poolUsed[i];
				break;
			}
			else if (i == pools - 1)
			{
#ifdef SL_DEBUG
				std::cout << "Out of pool memory. Adding in " << pools * 2 << " x " << poolSize * sizeof(T) << " byte pools." << std::endl;
#endif
				T **temp = pool;
				pool = (T**)(::operator new(pools * 2 * sizeof(T)));
				for (std::size_t j = 0; j < pools; ++j)
				{
					pool[j] = temp[j];
				}
				// wat?
				::operator delete((void**)temp);
				for (std::size_t j = pools; j < pools << 1; ++j)
				{
					poolUsed[j] = 0;
				}
				pools <<= 1;
			}
		}
	}
	else
	{
		block = poolStack.top();
		poolStack.pop();
		//std::cout << "Allocated memory at " << block << " from stack. stack = " << poolStack.size() << std::endl;
	}
	return block;
}

template <typename T>
void BasicPool<T>::put(T *p)
{
	poolStack.push(p);
}

}

#endif
