#ifndef SL_DOOR_HPP
#define SL_DOOR_HPP

#include "Engine/Serialization/DeserializeConstructor.hpp"
#include "Pathfinding/NodeID.hpp"
#include "Engine/Entity.hpp"

namespace sl
{

class Door : public Entity
{
public:
	DECLARE_SERIALIZABLE_METHODS(Door)

	Door(DeserializeConstructor);
	/**
	 * @param pos The coords of the door
	 * @param w The width of the Door
	 * @param h The height of the Door
	 * @param targetPos The coordinate in the target Scene this Door opens to
	 * @param targetScene The target Scene this Door opens to
	 */
	Door(Vector2f pos, int w, int h, Vector2f target, Scene *targetcene);

	Door(Vector2f pos, int w, int h, Vector2f target, Scene *targetcene, const pf::NodeID& entrance, const pf::NodeID& exit);



	const Types::Type getType() const override;

	bool isStatic() const override;

	/**
	 * Switches the World's current AbstractScene to the one this door opens to
	 */
	void enter();

	/**
	 * Transfers an Entity from the current Scene to the target Scene
	 * Note: moves the Entity to the target x/y of this Door
	 * @param entity The Entity to transfer (must exist in the same Scene as this Door)
	 */
	void transfer(Entity *entity);

	pf::NodeID getEntranceNodeID() const;

	pf::NodeID getExitNodeID() const;

private:
	void onUpdate() override;

	void onEnter() override;

	Vector2f target;
	Scene *targetScene;
	pf::NodeID entranceNode;
	pf::NodeID exitNode;
};

} // sl

#endif // SL_DOOR_HPP