#include "Engine/Events/Event.hpp"

#include "Utility/Assert.hpp"

namespace sl
{

const std::string Event::invalidString("invalid");

const Vector2f Event::NO_LOCATION(-1234567890.f, -1234567890.f);

Event::Event(Category category, const std::string& name)
	:category(category)
	,name(name)
	,location(NO_LOCATION)
{
	ASSERT(category != EventTypeNum);
}

Event::Event(Category category, const std::string& name, const Vector2f& location)
	:category(category)
	,name(name)
	,location(location)
{
	ASSERT(category != EventTypeNum);
}

Event::Event(Category category, const std::string& name, ResponseFunction responder)
	:category(category)
	,name(name)
	,responder(responder)
	,location(NO_LOCATION)
{
	ASSERT(category != EventTypeNum);
	ASSERT(responder);
}

Event::Event(Category category, const std::string& name, ResponseFunction responder, const Vector2f& location)
	:category(category)
	,name(name)
	,responder(responder)
	,location(location)
{
	ASSERT(category != EventTypeNum);
	ASSERT(responder);
}

Event::Event(Event&& other)
	:category(other.category)
	,name(std::move(other.name))
	,responder(std::move(other.responder))
	,flags(std::move(other.flags))
	,location(other.location)
{
}

Event& Event::operator=(Event&& rhs)
{
	category = rhs.category;
	name = std::move(rhs.name);
	responder = std::move(rhs.responder);
	flags = std::move(rhs.flags);
	location = rhs.location;
	return *this;
}

Event::Event(const Event& other)
	:category(other.category)
	,name(other.name)
	,responder(other.responder)
	,flags(other.flags)
	,location(other.location)
{
}

void Event::setFlag(const std::string& flag, ls::Value value)
{
	flags[flag] = std::move(value);
}

Event::Category Event::getCategory() const
{
	return category;
}

const std::string& Event::getName() const
{
	return name;
}

const Vector2f* Event::getLocation() const
{
	return location == NO_LOCATION ? nullptr : &location;
}

void Event::respond(const std::string& msg) const
{
	ASSERT(responder);
	responder(msg);
}

const ls::Value& Event::getFlag(const std::string& flag) const
{
	static ls::Value uninitialized;
	auto it = flags.find(flag);
	ASSERT(it != flags.end());
	return it != flags.end() ? it->second : uninitialized;
}

/*static*/Event Event::copy(const Event& ev)
{
	return Event(ev);
}

} // sl
