/**
 * @section DESCRIPTION
 * An iterator for EventQueue so the user can poll the EventQueue without destroying it.
 */
#ifndef SL_EVENTITERATOR_HPP
#define SL_EVENTITERATOR_HPP

#include <deque>
#include <memory>
#include "Engine/Events/Event.hpp"

namespace sl
{

class EventIterator
{
public:
	/**
	 * Constructs an EventIterator
	 * @param eventQueue The event queue to iterate over
	 */
	EventIterator(const std::deque<std::unique_ptr<const Event>>& eventQueue);


	/**
	 * Poll the queue. If the queue isn't empty, then the poll removes the
	 * first event in the queue and returns true.
	 * @param event The pointer to an Event to set to the first event in the queue
	 * @return Whether or not there were any events left in the queue
	 */
	bool poll(const Event*& event);
private:
	//!The queue to iterate along
	const std::deque<std::unique_ptr<const Event>>& eventQueue;
	//!The iterator to represent the position in the queue
	std::deque<std::unique_ptr<const Event>>::const_iterator it;
};

}

#endif
