/**
 * @section DESCRIPTION
 * A queue of Events that happened since the last update.
 */
#ifndef SL_EVENTQUEUE_HPP
#define SL_EVENTQUEUE_HPP

#include <deque>
#include <memory>
#include "Engine/Events/Event.hpp"
#include "Engine/Events/EventIterator.hpp"

namespace sl
{

class EventQueue
{
public:
	/**
	 * Construct an EventQueue
	 */
	EventQueue();
	/**
	 * Destroy an EventQueue
	 */
	~EventQueue() {}


	/**
	 * Add an event to the EventQueue
	 * @param event The event to be added
	 */
	void addEvent(std::unique_ptr<const Event> event);
	/**
	 * Get an iterator so you can poll the EventQueue
	 * @return An EventIterator with its start at the start of this queue
	 */
	EventIterator iterator(Event::Category category) const;
	/**
	 * Empty the queue.
	 */
	void reset();
private:
	//!The Event queue
	std::deque<std::unique_ptr<const Event>> eventQueues[Event::Category::EventTypeNum];
};

}

#endif
