#include "Engine/GUI/Bar.hpp"

#include "Utility/Assert.hpp"

namespace sl
{
namespace gui
{

Bar::Bar(const Rect<int>& bounds, const sf::Color& low, const sf::Color& high, const sf::Color& outline, const sf::Color& background)
	:GUIWidget(bounds)
	,lowColor(low)
	,highColor(high)
	,back(sf::Vector2f(bounds.width, bounds.height))
	,front(sf::Vector2f(bounds.width, bounds.height))
{
	front.setFillColor(background);
	front.setOutlineColor(outline);
	back.setOutlineThickness(0.f);
	front.setOutlineThickness(1.f);
	setValue(1.f);
}

void Bar::setValue(float value)
{
	ASSERT(value >= 0.f && value <= 1.f);
	// @TODO vertical bars too
	back.setSize(sf::Vector2f(value * (front.getSize().x), back.getSize().y));
	back.setFillColor(sf::Color((1 - value) * lowColor.r + value * highColor.r,
	                            (1 - value) * lowColor.g + value * highColor.g,
	                            (1 - value) * lowColor.b + value * highColor.b,
	                            (1 - value) * lowColor.a + value * highColor.a));
}

/*	  GUIWidget private methods	   */
void Bar::onDraw(sf::RenderTarget& target) const
{
	target.draw(front);
	target.draw(back);
}

void Bar::onUpdate(int x, int y)
{
	back.setPosition(x, y);
	front.setPosition(x, y);
}

} // gui
} // sl
