#ifndef SL_GUI_BASICMENUBUILDER_HPP
#define SL_GUI_BASICMENUBUILDER_HPP

#include <functional>

#include "Engine/GUI/GUIWidget.hpp"
#include "Utility/Memory.hpp"

namespace sl
{
namespace gui
{
class TextLabel;

class BasicMenuBuilder
{
public:
	BasicMenuBuilder(int menuWidth, int maxHeight, int buttonGap, int minButtonSize, int fontSize = 2);

	~BasicMenuBuilder();

	/**
	 * Adds a button to the menu
	 * @param text The starting text to display
	 * @param function The callback to call when the button is clicked
	 */
	void addButton(const std::string& text, const std::function<void()>& function);

	/**
	 * Adds a button to the menu with changable text
	 * @param text The starting text to display
	 * @param function The callback to call when the button is clicked (input to it is the button's text label so you can change it)
	 */
	void addDynamicButton(const std::string& text, const std::function<void(TextLabel*)>& function);

	/**
	 * Both adds a "Back" button that invokes this function, and also binds pressing Key::GUIBack to it. For convenience
	 * @param function The callback to invoke
	 */
	void addBackButton(std::function<void()> function);

	/**
	 * @return The widget composed of all the buttons added so far
	 */
	std::unique_ptr<GUIWidget> create();

private:
	class MenuController : public GUIWidget
	{
	public:
		MenuController(const Rect<int>& box);

		void addButton(std::unique_ptr<GUIWidget> button, std::function<void()> callback);

	private:
		void select(int index);

		int hoverIndex;
		std::vector<GUIWidget*> buttons;
		std::vector<std::function<void()>> callbacks;
	};

	int menuWidth;
	int maxHeight;
	int buttonGap;
	int heightSoFar;
	int minButtonSize;
	int fontSize;
	std::unique_ptr<MenuController> controller;
};

} // gui
} // sl

#endif // SL_GUI_BASICMENUBUILDER_HPP