#ifndef SL_GUI_BUTTON_HPP
#define SL_GUI_BUTTON_HPP

#include "Utility/Memory.hpp"
#include "Engine/GUI/GUIWidget.hpp"
#include "Engine/GUI/TextLabel.hpp"
#include "Engine/GUI/NoBackground.hpp"

namespace sl
{
namespace gui
{

class Button : public GUIWidget
{
public:
	Button(const Rect<int>& box, std::unique_ptr<TextLabel> text, sf::Color mouseOffColour, sf::Color mouseOnColour,
		   const WidgetBackground& mouseOffBackground = NoBackground(), const WidgetBackground& mouseOnBackground = NoBackground());

	Button(std::unique_ptr<TextLabel> text, sf::Color mouseOffColour, sf::Color mouseOnColour,
		   const WidgetBackground& mouseOffBackground = NoBackground(), const WidgetBackground& mouseOnBackground = NoBackground());

	bool isSelected() const;

	TextLabel& getText();

	void setOnHover(std::function<void()> onHover);


	static std::unique_ptr<Button> createGenericButton(const Rect<int>& box, const std::string& text, std::function<void()> callback, int fontSize = 1, TextLabel::HorizontalPolicy halign = TextLabel::HorizontalPolicy::Center);

private:
	void onDraw(sf::RenderTarget& target) const override;

	void onFocusGainedOrMouseEnter() override;

	void onFocusLostOrMouseLeave() override;


	TextLabel& text;
	sf::Color mouseOffColour;
	sf::Color mouseOnColour;
	std::unique_ptr<const WidgetBackground> mouseOffBackground;
	std::unique_ptr<const WidgetBackground> mouseOnBackground;
	bool selected;
	std::function<void()> onHover;
};

} // gui
} // sl

#endif // SL_GUI_BUTTON_HPP
