#include "Engine/GUI/DrawableContainer.hpp"

#include "Engine/Graphics/Drawable.hpp"

namespace sl
{
namespace gui
{

DrawableContainer::DrawableContainer(const Rect<int>& box, Drawable *nonOwningDrawable)
	:GUIWidget(box)
	,nonOwningDrawable(nonOwningDrawable)
{
}

DrawableContainer::DrawableContainer(const Rect<int>& box, std::unique_ptr<Drawable> drawable)
	:GUIWidget(box)
	,drawable(std::move(drawable))
	,nonOwningDrawable(nullptr)
{
}

void DrawableContainer::setDrawable(std::unique_ptr<Drawable> drawable)
{
	this->drawable = std::move(drawable);
	if (this->drawable)
	{
		this->drawable->update(getBox().left, getBox().top);
	}
	nonOwningDrawable = nullptr;
}

Drawable* DrawableContainer::getDrawable() const
{
	return drawable ? drawable.get() : nonOwningDrawable;
}

/*		GUIWidget private methods		*/
void DrawableContainer::onDraw(sf::RenderTarget& target) const
{
	Drawable *d = getDrawable();
	if (d)
	{
		d->draw(target);
	}
}

void DrawableContainer::onUpdate(int x, int y)
{
	Drawable *d = getDrawable();
	if (d)
	{
		d->update(x, y);
	}
}


} // gui
} // sl
