#include "Engine/GUI/FloatingWindow.hpp"

#include "Engine/AbstractScene.hpp"
#include "Engine/GUI/Button.hpp"
#include "Engine/GUI/NormalBackground.hpp"
#include "Utility/Assert.hpp"

namespace sl
{
namespace gui
{

const int topBarHeight = 32;

FloatingWindow::FloatingWindow(std::unique_ptr<GUIWidget> window)
	:GUIWidget(Rect<int>(window->getBox().left, window->getBox().top - topBarHeight, window->getBox().width, window->getBox().height + topBarHeight))
	,clickedPos()
	,isMoving(false)
	,titleLabel(nullptr)
{
	registerMouseInputEventWithMousePos(Key::GUIClick, GUIWidget::InputEventType::Pressed,/*[&](int mouseX, int mouseY){
		clickedPos.x = mouseX - box.left;
		clickedPos.y = mouseY - box.top;
		if (clickedPos.y <= topBarHeight)
		{
			isMoving = true;
		}
	}*/std::bind(&FloatingWindow::onClick, this, std::placeholders::_1, std::placeholders::_2));

	registerMouseInputEvent(Key::GUIClick, GUIWidget::InputEventType::Released, [&](){
		isMoving = false;
	});

	registerMouseInputEventWithMousePos(Key::GUIClick, GUIWidget::InputEventType::Held, [&](int mouseX, int mouseY){
		if (isMoving)
		{
			const int diffX = mouseX - box.left - clickedPos.x;
			const int diffY = mouseY - box.top - clickedPos.y;
			if (diffX || diffY)
			{
				moveBy(diffX, diffY);
			}
		}
	});

	applyBackground(NormalBackground(sf::Color::Black, sf::Color::White));
	addWidget(std::move(window));
}

void FloatingWindow::setTitle(const std::string& name)
{
	titleLabel->setString(name);
}

void FloatingWindow::onSceneEnter()
{
	ASSERT(getParent());
	auto closeButton = Button::createGenericButton(Rect<int>(box.left + box.width - topBarHeight, box.top, topBarHeight, topBarHeight), "X", [&](){
		getParent()->removeWidget(*this);
	});
	addWidget(std::move(closeButton));

	// We don't want this happening twice (no real harm though if it does, just multiple texts)
	ASSERT(titleLabel == nullptr);
	auto title = unique<TextLabel>(Rect<int>(box.left, box.top, box.width - topBarHeight, topBarHeight), "", TextLabel::Config().valign(TextLabel::Middle).pad(16, 0));
	titleLabel = title.get();
	addWidget(std::move(title));
}

void FloatingWindow::onClick(int mouseX, int mouseY)
{
	clickedPos.x = mouseX - box.left;
	clickedPos.y = mouseY - box.top;
	isMoving = (clickedPos.y <= topBarHeight);
}

} // gui
} // sl
