#include "Engine/GUI/FontManager.hpp"

#include <fstream>
#include <sstream>
#include <utility>

#include "Utility/Assert.hpp"
#include "Utility/Logger.hpp"

namespace sl
{
namespace gui
{

FontManager* FontManager::instance = nullptr;

/*static*/void FontManager::create()
{
	ASSERT(instance == nullptr);
	if (!instance)
	{
		instance = new FontManager();
	}
}

/*static*/void FontManager::destroy()
{
	ASSERT(instance);
	if (instance)
	{
		delete instance;
		instance = nullptr;
	}
}

/*static*/const sf::Font& FontManager::getFont(const std::string& name)
{
	ASSERT(instance);
	auto it = instance->fonts.find(name);
	if (it == instance->fonts.end())
	{
		it = instance->fonts.insert(std::make_pair(name, sf::Font())).first;
		if (!it->second.loadFromFile(name))
		{
			Logger::log(Logger::GUI, Logger::Serious) << "Couldn't load " << name << std::endl;
		}
		else
			Logger::log(Logger::GUI, Logger::Debug) << "Loaded " << name << std::endl;
	}
	return it->second;
}

/*static*/const SpriteFont* FontManager::getSpriteFont(const std::string& name)
{
	ASSERT(instance);
	auto it = instance->spriteFonts.find(name);
	if (it == instance->spriteFonts.end())
	{
		std::ifstream file("resources/img/fonts/" + name + ".sff");
		if (!file)
		{
			return nullptr;
		}
		std::string flag;
		std::map<char, int> yOffsets;
		int cw = 1, ch = 1, hs = 0, vs = 0, sc = 0;
		while (file)
		{
			file >> flag;
			if (flag == "cw")
			{
				file >> cw;
			}
			else if (flag == "ch")
			{
				file >> ch;
			}
			else if (flag == "hs")
			{
				file >> hs;
			}
			else if (flag == "vs")
			{
				file >> vs;
			}
			else if (flag == "yof")
			{
				int yoff;
				file >> yoff;
				std::getline(file, flag);
				std::istringstream line(flag);
				char c;
				while (line)
				{
					line >> c;
					if (c != ' ')
					{
						yOffsets[c] = yoff;
					}
				}
			}
			else if (flag == "sc")
			{
				file >> sc;
			}
			else
			{
				ERROR("Unknown .sff flag: " + flag);
			}
		}
		it = instance->spriteFonts.insert(std::make_pair(name, SpriteFont(name, cw, ch, vs, hs, sc, std::move(yOffsets)))).first;
	}
	return &it->second;
}

} // gui
} // sl
