#include "Engine/GUI/NormalBackground.hpp"

namespace sl
{
namespace gui
{

NormalBackground::NormalBackground(const sf::Color& background, const sf::Color& outline)
	:box()
{
	for (int i = 0; i < 4; ++i)
	{
		corners[i].position.x = 0;
		corners[i].position.y = 0;
		corners[i].color = background;
	}
	for (int i = 0; i < 5; ++i)
	{
		this->outline[i].position.x = 0;
		this->outline[i].position.y = 0;
		this->outline[i].color = outline;
	}
}

/*		WidgetBackground public methods			*/
void NormalBackground::draw(sf::RenderTarget& target) const
{
	target.draw(corners, 4, sf::Quads);
	target.draw(outline, 5, sf::LinesStrip);
}

std::unique_ptr<WidgetBackground> NormalBackground::clone() const
{
	return std::make_unique<NormalBackground>(*this);
}

/*		NormalBackground private methods		*/
//NormalBackground::NormalBackground(const NormalBackground& other)
//	:box()
//{
//	for (int i = 0; i < 4; ++i)
//	{
//		corners[i].position.x = 0;
//		corners[i].position.y = 0;
//		corners[i].color = other.corners[i].color;
//	}
//	for (int i = 0; i < 5; ++i)
//	{
//		this->outline[i].position.x = 0;
//		this->outline[i].position.y = 0;
//		this->outline[i].color = other.outline[i].color;
//	}
//}

/*		WidgetBackground private methods		*/
void NormalBackground::move(int x, int y)
{
	int xdif = x - box.left;
	int ydif = y - box.top;
	if (xdif || ydif)
	{
		for (int i = 0; i < 4; ++i)
		{
			corners[i].position.x += xdif;
			corners[i].position.y += ydif;
		}
		for (int i = 0; i < 5; ++i)
		{
			outline[i].position.x += xdif;
			outline[i].position.y += ydif;
		}
		box.left = x;
		box.top = y;
	}
}

void NormalBackground::setSize(int width, int height)
{
	box.width = width;
	box.height = height;
	corners[1].position.x = corners[0].position.x + width;
	corners[2].position.x = corners[0].position.x + width;
	corners[2].position.y = corners[0].position.y + height;
	corners[3].position.y = corners[0].position.y + height;
	//	move the outline to the the corner positions
	for (int i = 0; i < 5; ++i)
	{
		outline[i].position = corners[i % 4].position;
	}
}

} // gui
} // sl
