/**
 * @section DESCRIPTION A widget that contains another (potentially bigger) widget
 * of which you can scroll along while viewing a subrect of
 */
#ifndef SL_GUI_SCROLLBAR_HPP
#define SL_GUI_SCROLLBAR_HPP

#include "Engine/GUI/GUIWidget.hpp"
#include "Utility/Memory.hpp"

namespace sl
{
namespace gui
{

class Scrollbar : public GUIWidget
{
public:
	/**
	 * @param box The bounds of the widget (area for scroll area)
	 * @param scrollWidget The widget to place inside the scoll area
	 */
	Scrollbar(const Rect<int>& box, std::unique_ptr<GUIWidget> scrollWidget = nullptr);

	/**
	 * Resets the widget inside the scroll area
	 * @param widget The new widget to be contained in the scroll area
	 */
	void setScrollArea(std::unique_ptr<GUIWidget> widget);

	/**
	 * Sets how far the scrollbar is.
	 * @param scroll The position of the scrollbar. 0 = at top, 1 = at bottom, others = assert
	 */
	void setVerticalScroll(float scroll);

	float getVerticalScroll() const;

	/**
	 * @return The width of the scrollbar
	 */
	int getScrollbarWidth() const;

	Rect<int> getVisibleRegion() const;

	void setVisiblePosition(Vector2i topLeft);

private:
	/*	  GUIWidget private methods	   */
	void onDraw(sf::RenderTarget& target) const override;

	void onUpdate(int x, int y) override;

	// mouseEnter/Leave/etc are not implemented because for focus they're called on setFocus() by onFocusGained()/etc here
	// and the mouse caused ones are performed by Scrollbar handling widget input manually.

	void onSceneEnter() override;
	void onSceneExit() override;

	void onFocusGained() override;
	void onFocusLost() override;

	void onHandleInputEvents(std::set<Key>& handled) override;
	void onHandleMouseInputEvents(std::set<Key>& handled, const sf::Vector2f& mouse, const Rect<int>& cutoff) override;

	void unfuckTextureFlipping() const;

	//! The widget inside the scroll area
	std::unique_ptr<GUIWidget> scrollWidget;
	//! Width of scrollbar
	int scrollbarWidth;
	//! Height of scrollbar (just the dragable part)
	int scrollbarHeight;
	//! @todo implement this
	float widthbarScroll;
	//! How far the scrollbar is (0 = at top, 1 = at bottom)
	float heightbarScroll;
	//! @todo implement this
	bool scrollingWidth;
	//! Whether the user has clicked the scrollbar draggy thing and is moving it
	bool scrollingHeight;
	char buf1[100];//@todo debug when back at home and see if this ever gets set
	//! The y position (relative to the scrollbar rect) that the user clicked on
	float clickYPos;
	char buf2[100];
	//! The rendered scroll widget sprite
	mutable sf::Sprite renderSprite;
	//! A texture to render to when drawing the scroll widget (so things get cut off when outside0
	mutable sf::RenderTexture renderTexture;
	//! The little rectangle on the scrollbar people can drag
	sf::RectangleShape scrollbar;

	mutable bool renderTextureNeedsUnfucking;
};

} // gui
} // sl

#endif // SL_GUI_SCROLLBAR_HPP
