#include "Engine/GUI/Slider.hpp"

#include <functional>

#include "Utility/Assert.hpp"
#include <iostream>

namespace sl
{
namespace gui
{

// @todo pass these in or something?
const int horizBorder = 16;
const int circleRadius = 8;
const int barThickness = 4;

Slider::Slider(const Rect<int>& bounds, int minValue, int maxValue, int increment)
	:GUIWidget(bounds)
	,minValue(minValue)
	,maxValue(maxValue)
	,increment(increment)
	,value(minValue)
	,circle(circleRadius)
	,bar(sf::Vector2f(bounds.width - 2 * horizBorder, barThickness))
{
	ASSERT(increment > 0);
	ASSERT(maxValue >= minValue + increment);

	circle.setFillColor(sf::Color::Black);
	circle.setOutlineColor(sf::Color::White);
	circle.setOutlineThickness(1);
	circle.setOrigin(circleRadius, circleRadius);

	bar.setFillColor(sf::Color(128, 128, 128));

	registerMouseInputEventWithMousePos(Key::GUIClick, GUIWidget::InputEventType::Held, std::bind(&Slider::onMouseDown, this, std::placeholders::_1, std::placeholders::_2));
}

int Slider::getValue() const
{
	return value;
}

void Slider::setValue(int val)
{
	value = val;
	std::cout << "Value  =  " << val << std::endl;
}

/*				private				*/
void Slider::onDraw(sf::RenderTarget& target) const
{
	target.draw(bar);
	target.draw(circle);
}

void Slider::onUpdate(int x, int y)
{
	const float midpointY = getBox().top + getBox().height / 2;
	bar.setPosition(getBox().left + horizBorder, midpointY - barThickness / 2);
	const float valuePercent = ((float) value - minValue) / (maxValue - minValue);
	circle.setPosition(bar.getPosition().x + bar.getSize().x * valuePercent, midpointY);
}

void Slider::onMouseDown(int x, int y)
{
	x -= getBox().left;
	x -= horizBorder;
	if (x < 0)
	{
		setValue(minValue);
	}
	else if (x >= bar.getSize().x)
	{
		setValue(maxValue);
	}
	else
	{
		const float valuePercent = x / bar.getSize().x;
		const int exactValue = minValue + (maxValue - minValue) * valuePercent;
		const int increments = exactValue / increment;
		setValue(increments * increment);
	}
}

} // gui
} // sl
