#ifndef SL_GUI_VALUECHANGERS_HPP
#define SL_GUI_VALUECHANGERS_HPP

#include <string>
#include <vector>

#include "Engine/GUI/Button.hpp"
#include "Engine/GUI/GUIWidget.hpp"
#include "Engine/GUI/TextLabel.hpp"
#include "Utility/Memory.hpp"

namespace sl
{
namespace gui
{

template <typename T>
class ValueChanger : public GUIWidget
{
public:
	template <typename ContainerOfT>
	ValueChanger(const ContainerOfT& values, const Rect<int>& bounds);

	const T& getValue() const;

	void keepValueUpdate(T *valueToUpdate);

	void setIndex(int newIndex);

private:
	void cycleLeft();
	void cycleRight();


	T *valueToUpdate;
	std::vector<T> values;
	int index;
	TextLabel *label;
};

template <typename T, typename ContainerOfT>
ValueChanger<T>::ValueChanger(const ContainerOfT& values, const Rect<int>& bounds)
	:GUIWidget(bounds)
	,valueToUpdate(nullptr)
	,values()
	,index(0)
{
	for (const T& value : values)
	{
		values.push_back(value);
	}
	//@TODO WRITE THE REST OF THIS FUCK IT I'LL JUST WRITE A SLIDER FOR NOW

	//std::unique_ptr<TextLabel> leftText(new TextLabel(Rect<int>()));
	//std::unique_ptr<Button> leftButton(new );
}

template <typename T>
const T& ValueChanger<T>::getValue() const
{
	return values[index];
}

template <typename T>
void ValueChanger<T>::keepValueUpdate(T *valueToUpdate)
{
	this->valueToUpdate = valueToUpdate;
}

template <typename T>
void ValueChanger<T>::setIndex(int newIndex)
{
	ASSERT(newIndex >= 0 && newIndex < values.size());
	index = newIndex;
	if (valueToUpdate)
	{
		*valueToUpdate = getValue();
	}
}

template <typename T>
void ValueChanger<T>::cycleLeft()
{
	if (index == 0)
	{
		index = values.size() - 1;
	}
	else
	{
		--index;
	}
}

template <typename T>
void ValueChanger<T>::cycleRight()
{
	if (index == values.size() - 1)
	{
		index = 0;
	}
	else
	{
		++index;
	}
}

} // gui
} // sl

#endif // SL_GUI_VALUECHANGERS_HPP