#ifndef SL_ABSTRACTTEXTBOX_HPP
#define SL_ABSTRACTTEXTBOX_HPP

#include <string>

#include "Engine/Graphics/Drawable.hpp"

namespace sl
{

class AbstractTextBox : public Drawable
{
public:
	virtual ~AbstractTextBox() {}

	virtual void setString(const std::string& str) = 0;

	virtual void appendString(const std::string& str) = 0;

	virtual const std::string& getString() const = 0;

	virtual bool backspace(int numCharsToRemove = 1) = 0;

	virtual void setArea(int width, int height) = 0;

	virtual void setFont(const std::string& fontName) = 0;

	virtual void setFontSize(int size) = 0;

	virtual void setFontColor(const sf::Color& color) = 0;

	virtual int getMinimumHeight() const = 0;

	/**
	 * Prematurely breaks lines if continuing would cause a word to be broken in half otherwise.
	 * Exceptions: If the word doesn't fit on a single line
	 * @param str The string to insert newlines into if necessary
	 * @return The resulting string with newlines possibly added where nice
	 */
	virtual std::string insertNewlines(std::string str) const = 0;

	virtual void finalize() = 0;
};

} // sl

#endif // SL_ABSTRACTTEXTBOX_HPP