#include "Engine/Graphics/ColorSwapID.hpp"

#include <algorithm>

#include "Engine/Serialization/AutoDeserialize.hpp"
#include "Engine/Serialization/AutoSerialize.hpp"
#include "Engine/Serialization/Serialization.hpp"
#include "Utility/Assert.hpp"

namespace sl
{

ColorSwapID::ColorSwapID()
	:id()
{
}

bool ColorSwapID::operator<(const ColorSwapID& rhs) const
{
	if (id.size() < rhs.id.size())
	{
		return true;
	}
	else if (id.size() > rhs.id.size())
	{
		return false;
	}
	else
	{
		for (std::vector<uint32_t>::const_iterator a = id.cbegin(), b = rhs.id.cbegin(); a != id.cend(); ++a, ++b)
		{
			if (*a < *b)
			{
				return true;
			}
			else if (*a > *b)
			{
				return false;
			}
		}
	}
	return false;
}

void ColorSwapID::getColors(std::vector<std::pair<sf::Color, sf::Color>>& colors) const
{
	colors = std::vector<std::pair<sf::Color, sf::Color> >(id.size() / 2);

	std::vector<uint32_t>::const_iterator a = id.begin();
	std::vector<std::pair<sf::Color, sf::Color> >::iterator b = colors.begin();

	while (a != id.end())
	{
		b->first.r = (*a >> 24) & 0xFF;
		b->first.g = (*a >> 16) & 0xFF;
		b->first.b = (*a >> 8) & 0xFF;
		b->first.a = (*a) & 0xFF;
		++a;
		b->second.r = (*a >> 24) & 0xff;
		b->second.g = (*a >> 16) & 0xFF;
		b->second.b = (*a >> 8) & 0xFF;
		b->second.a = (*a) & 0xFF;
		++a;
		++b;
	}
}

ColorSwapID::operator bool() const
{
	return !id.empty();
}

void ColorSwapID::serialize(Serialize& out) const
{
	AutoSerialize::serialize(out, id);
}

void ColorSwapID::deserialize(Deserialize& in)
{
	AutoDeserialize::deserialize(in, id);
}


std::ostream& operator<<(std::ostream& os, const sl::ColorSwapID& swapID)
{
	std::vector<uint32_t>::const_iterator a = swapID.id.begin();

	os << "{";
	while (a != swapID.id.end())
	{
		if (a != swapID.id.begin())
		{
			os << ", ";
		}
		os << "(";
		os << ((*a >> 16) & 0xFF) << ",";
		os << ((*a >> 8) & 0xFF) << ",";
		os << ((*a) & 0xFF);
		++a;
		os << ")->(";
		os << ((*a >> 16) & 0xff) << ",";
		os << ((*a >> 8) & 0xFF) << ",";
		os << ((*a) & 0xFF);
		++a;
		os << ")";
	}
	os << "}";

	return os;
}

ColorSwapID operator|(const ColorSwapID& lhs, const ColorSwapID& rhs)
{
	ColorSwapID ret;

	ret.id.reserve(lhs.id.size() + rhs.id.size());

	ret.id.insert(ret.id.end(), lhs.id.begin(), lhs.id.end());
	ret.id.insert(ret.id.end(), rhs.id.begin(), rhs.id.end());

	return ret;
}

} // sl
