#include "Engine/Graphics/ColorSwapLoader.hpp"

#include <fstream>

#include "Utility/Assert.hpp"
#include "Utility/JsonUtil.hpp"

namespace sl
{

ColorSwapID loadSwap(const BaseColor& base, const json11::Json::object& json)
{
	std::vector<std::pair<sf::Color, sf::Color>> swaps;
	for (const auto& entry : json)
	{
		auto it = base.find(entry.first);
		if (it != base.end())
		{
			const json11::Json::array& rgb = entry.second.array_items();
			ASSERT(rgb.size() == 3);
			// @todo remove this check once data us put into the json
			if (rgb[0].is_string())
			{
				//swaps.push_back(std::make_pair(it->second, sf::Color::Magenta));
				continue;
			}
			swaps.push_back(std::make_pair(it->second, sf::Color(rgb[0].int_value(), rgb[1].int_value(), rgb[2].int_value())));
		}
	}
	return ColorSwapID(swaps);
}


std::map<std::string, ColorSwapID> loadSwaps(const json11::Json::object& json)
{
	std::map<std::string, ColorSwapID> output;
	BaseColor base;
	for (const auto& color : json.at("base").object_items())
	{
		const json11::Json::array& rgb = color.second.array_items();
		base[color.first] = sf::Color(rgb[0].int_value(), rgb[1].int_value(), rgb[2].int_value());
	}
	for (const auto& swap : json)
	{
		if (swap.first != "base")
		{
			output.insert(std::make_pair(swap.first, std::move(loadSwap(base, swap.second.object_items()))));
		}
	}
	return output;
}

std::map<std::string, ColorSwapID> loadSwapsFile(const std::string& file)
{
	std::string err;
	json11::Json data = readJsonFile(file, err);
	if (!err.empty())
	{
		ERROR(err);
	}
	const json11::Json::object& swaps = data.object_items();
	return loadSwaps(swaps);
}

} // sl
