#include "Engine/Graphics/Framerate.hpp"

#include "Engine/Graphics/TextureManager.hpp"
#include "Utility/Assert.hpp"
#include "Utility/Mapping.hpp"

namespace sl
{

Framerate::Framerate(int fixedSpeed)
	:basis(Basis::Fixed)
	,fixedSpeed(fixedSpeed)
{
}

Framerate::Framerate(std::vector<int> variedSpeeds)
	:basis(Basis::Varied)
	,fixedSpeed(-1)
	,varied(mapVecIndexed(variedSpeeds, [](int i, int len) { return Frame{ i, len }; }))
{
}

Framerate::Framerate(std::vector<Frame> mixedOrder)
	:basis(Basis::Mixed)
	,fixedSpeed(-1)
	,varied(std::move(mixedOrder))
{
}

std::vector<Framerate::Frame> Framerate::makeAnimationSpeeds(const std::string& fname, int cellWidth) const
{
	ASSERT(cellWidth > 0);
	Texture texture(TextureManager::getTexture(fname));
	const sf::Vector2u& textureSize = texture.get().getSize();
	ASSERT(textureSize.x % cellWidth == 0);
	const int frameCount = textureSize.x / cellWidth;
	if (basis == Basis::Fixed)
	{
		std::vector<Frame> ret(frameCount);
		for (int i = 0; i < frameCount; ++i)
		{
			ret[i] = Frame{ i, fixedSpeed };
		}
		return ret;
	}
	else if (basis == Basis::Varied)
	{
		ASSERT(varied.size() == frameCount);
	}
	return varied;
}

} // sl