#ifndef SL_FRAMERATE_HPP
#define SL_FRAMERATE_HPP

#include <string>
#include <vector>

namespace sl
{

class Framerate
{
public:
	struct Frame
	{
		int index;
		int duration;
	};
	/**
	 * Every frame renders in-order for fixedSpeed frames each.
	 */
	Framerate(int fixedSpeed);

	/**
	 * @param variedSpeeds. Plays in-order with the ith frame rendering for variedSpeeds[i] frames.
	 */
	Framerate(std::vector<int> variedSpeeds);

	/**
	 * Allows out-of-order arbitrary animation.
	 * @param mixedOrder A pair of <frame, length> specifying which frame to play for how many frames.
	 */
	Framerate(std::vector<Frame> mixedOrder);

	/**
	 * Agnostic to how it was initialized - will construct a set of speeds
	 */
	std::vector<Frame> makeAnimationSpeeds(const std::string& fname, int cellWidth) const;

private:
	enum class Basis
	{
		Fixed,
		Varied,
		Mixed
	};
	Basis basis;
	int fixedSpeed;
	std::vector<Frame> varied;
};

} // sl

#endif // SL_FRAMERATE_HPP