#include "Engine/Graphics/Line.hpp"

#include <algorithm>

namespace sl
{

Line::Line(int x1, int y1, int x2, int y2, sf::Color colour)
	:x(std::min(x1, x2))
	,y(std::min(y1, y2))
{
	points[0].color = colour;
	points[1].color = colour;
	this->setPositions(x1, y1, x2, y2);
}

Line::Line(int x1, int y1, int x2, int y2, sf::Color colour1, sf::Color colour2)
	:x(std::min(x1, x2))
	,y(std::min(y1, y2))
{
	points[0].color = colour1;
	points[1].color = colour2;
	this->setPositions(x1, y1, x2, y2);
}

void Line::update(int x, int y)
{
	const int xdif = x - this->x;
	const int ydif = y - this->y;

	if (xdif || ydif)
	{
		points[0].position.x += xdif;
		points[0].position.y += ydif;
		points[1].position.x += xdif;
		points[1].position.y += ydif;
		currentBounds.left = x;
		currentBounds.top = y;
	}
}

const Rect<int>& Line::getCurrentBounds() const
{
	return currentBounds;
}

void Line::draw(sf::RenderTarget& target) const
{
	target.draw(points, 2, sf::Lines);
}

void Line::setPositions(int x1, int y1, int x2, int y2)
{
	points[0].position.x = x1;
	points[0].position.y = y1;
	points[1].position.x = x2;
	points[1].position.y = y2;
	currentBounds.left = std::min(x1, x2);
	currentBounds.top = std::min(x1, x2);
	currentBounds.width = abs(x1 - x2);
	currentBounds.height = abs(y1 - y2);
}

void Line::setPositions(int x1, int y1, int x2, int y2, int x, int y)
{
	this->x = x;
	this->y = y;
	this->setPositions(x1, y1, x2, y2);
}

void Line::setColour(sf::Color colour)
{
	points[0].color = colour;
	points[1].color = colour;
}

}
