#ifndef SL_LINE_HPP
#define SL_LINE_HPP

#include <SFML/Graphics/Color.hpp>
#include <SFML/Graphics/Vertex.hpp>
#include "Engine/Graphics/Drawable.hpp"

namespace sl
{

class Line : public Drawable
{
public:
	/**
	 * Constructs a line with the origin being the top left of the AABB formed by the line
	 * @param x1 first endpoint's X coordinate
	 * @param y1 first endpoint's Y coordinate
	 * @param x2 second endpoint's X coordinate
	 * @param y2 second endpoint's Y coordinate
	 * @param colour The colour of the line
	 */
	Line(int x1, int y1, int x2, int y2, sf::Color colour);
	/**
	 * Constructs a line with the origin being the top left of the AABB formed by the line
	 * This line can have two colours.
	 * @param x1 first endpoint's X coordinate
	 * @param y1 first endpoint's Y coordinate
	 * @param x2 second endpoint's X coordinate
	 * @param y2 second endpoint's Y coordinate
	 * @param colour1 The colour of the first endpoint
	 * @param colour2 The colour of the second endpoint
	 */
	Line(int x1, int y1, int x2, int y2, sf::Color colour1, sf::Color colour2);

	/*			Drawable public methods			*/
	void update(int x, int y) override;

	const Rect<int>& getCurrentBounds() const override;

	void draw(sf::RenderTarget& target) const override;

	/**
	 * Sets the positions of the endpoints of the line.
	 * THIS DOES NOT SET THE OFFSET FROM THE ORIGIN THOUGH!
	 * @param x1 first endpoint's X coordinate
	 * @param y1 first endpoint's Y coordinate
	 * @param x2 second endpoint's X coordinate
	 * @param y2 second endpoint's Y coordinate
	 */
	void setPositions(int x1, int y1, int x2, int y2);

	/**
	 * Sets the positions of the endpoints of the line as well as the origin
	 * @param x1 first endpoint's X coordinate
	 * @param y1 first endpoint's Y coordinate
	 * @param x2 second endpoint's X coordinate
	 * @param y2 second endpoint's Y coordinate
	 * @param x The origin's X coordinate
	 * @param y The origin's Y coordinate
	 */
	void setPositions(int x1, int y1, int x2, int y2, int x, int y);

	/**
	 * Sets the colour of the line (duh)
	 * @param colour The colour to use...
	 */
	void setColour(sf::Color colour);

private:
	//!The two endpoints (contains coordinate+colour)
	sf::Vertex points[2];
	//!The origin coordinates
	int x, y;

	Rect<int> currentBounds;
};

} // sl

#endif // SL_LINE_HPP