#include "Engine/Graphics/RenderTarget.hpp"

#include <algorithm>

#include "Engine/Input/Input.hpp"
#include "Utility/Assert.hpp"

#ifdef SL_DEBUG
	#include "Engine/Settings.hpp"
	#include "Utility/Random.hpp" 
#endif // SL_DEBUG

namespace sl
{

RenderTarget::RenderTarget(sf::RenderTarget& target)
	:target(target)
	,drawQueue()
{
	drawQueue.reserve(0xFFF);
}

void RenderTarget::draw(const sf::Drawable& drawable, const sf::RenderStates& renderState)
{
	/*  INSERT CODE HERE	*/
	//target.draw(drawable);
	sfDrawQueue.push_back(std::make_pair(&drawable, renderState));
}

void RenderTarget::draw(const Drawable& drawable)
{
	drawQueue.push_back(&drawable);
}

void RenderTarget::draw(const sf::Vertex *vertices, unsigned int vertexCount, sf::PrimitiveType type)
{
	sfDrawVectorQueue.push_back(std::make_tuple(vertices, vertexCount, type));
}

/**
 * Function used just to get right render depth in RenderTarget::render()
 * @param a Drawable to compare
 * @param b Drawable to compare
 * @return True if a should be above below b
 */
static bool sortByDepth(const Drawable *a, const Drawable *b)
{
	const int aDepth = a->getDepth();
	const int bDepth = b->getDepth();
	if (aDepth == bDepth)
	{
		const int aY = a->getCurrentBounds().top + a->getCurrentBounds().height + a->getHeight() + a->getBottomPointOffset();
		const int bY = b->getCurrentBounds().top + b->getCurrentBounds().height + b->getHeight() + b->getBottomPointOffset();
		if (aY == bY)
		{
			return a->getDepthConflictPower() > b->getDepthConflictPower();
		}
		else
		{
			return aY < bY;
		}
	}
	else
	{
		return aDepth > bDepth;
	}
}

void RenderTarget::render()
{
	std::sort(drawQueue.begin(), drawQueue.end(), sortByDepth);

	for (const Drawable *drawable : drawQueue)
	{
		drawable->draw(target);
	}
#ifdef SL_DEBUG
	if (Settings::fuckYouSingletonHack()->get("draw_y_bottom"))
	{
		sf::RectangleShape bottom;
		for (const Drawable *drawable : drawQueue)
		{
			bottom.setPosition(drawable->getCurrentBounds().left, drawable->getCurrentBounds().bottom() + drawable->getBottomPointOffset());
			bottom.setSize(sf::Vector2f(drawable->getCurrentBounds().width, 2));
			bottom.setFillColor(Random::get().choose({ sf::Color::Red, sf::Color::Green, sf::Color::Blue, sf::Color::Cyan, sf::Color::Magenta, sf::Color(255, 255, 0) }));
			target.draw(bottom);
		}
	}
#endif // SL_DEBUG
	drawQueue.clear();
	for (const auto& sfDrawable : sfDrawQueue)
	{
		target.draw(*sfDrawable.first, sfDrawable.second);
	}
	sfDrawQueue.clear();
	for (auto it = sfDrawVectorQueue.begin(); it != sfDrawVectorQueue.end(); ++it)
	{
		target.draw(std::get<0>(*it), std::get<1>(*it), std::get<2>(*it));
	}
	sfDrawVectorQueue.clear();
}

sf::RenderTarget& RenderTarget::getSFMLTarget()
{
	return target;
}

} // sl
