#ifndef SL_SFDRAWABLE_HPP
#define SL_SFDRAWABLE_HPP

#include "Engine/Graphics/Drawable.hpp"

namespace sl
{

template <typename T>
class SFDrawable : public Drawable
{
public:
	SFDrawable(const T& data = T());


	void update(int x, int y) override;

	const Rect<int>& getCurrentBounds() const override;

	void draw(sf::RenderTarget& target) const override;

	const T& getData() const;
	T& getData();

private:
	T data;
	Rect<int> bounds;
};

template <typename T>
SFDrawable<T>::SFDrawable(const T& data)
	:Drawable()
	,data(data)
	,bounds(data.getGlobalBounds().left, data.getGlobalBounds().top, data.getGlobalBounds().width, data.getGlobalBounds().height)
{
}


template <typename T>
void SFDrawable<T>::update(int x, int y)
{
	data.setPosition(x, y);
	const sf::FloatRect globalBounds = data.getGlobalBounds();
	bounds.left = globalBounds.left;
	bounds.top = globalBounds.top;
	bounds.width = globalBounds.width;
	bounds.height = globalBounds.height;
}

template <typename T>
const Rect<int>& SFDrawable<T>::getCurrentBounds() const
{
	return bounds;
}

template <typename T>
void SFDrawable<T>::draw(sf::RenderTarget& target) const
{
	target.draw(data);
}

template <typename T>
const T& SFDrawable<T>::getData() const
{
	return data;
}

template <typename T>
T& SFDrawable<T>::getData()
{
	return data;
}

} // sl

#endif // SL_SFDRAWABLE_HPP