#include "Engine/Graphics/SFTextBox.hpp"

#include <algorithm>

#include "Engine/GUI/FontManager.hpp"
#include "Utility/Assert.hpp"
#include "Utility/StringUtil.hpp"

namespace sl
{

SFTextBox::SFTextBox(const Rect<int>& box)
	:bounds(0, 0, box.width, box.height)
	,maxLineWidth(0)
{
}


void SFTextBox::update(int x, int y)
{
	rendertext.setPosition(x, y);
	bounds.left = x;
	bounds.top = y;
	bounds.width = rendertext.getLocalBounds().width;//maxLineWidth;
	bounds.height = rendertext.getLocalBounds().height;
}

const Rect<int>& SFTextBox::getCurrentBounds() const
{
	return bounds;
}


void SFTextBox::setString(const std::string& str)
{
	text = str;
	words.clear();
	splitBy(words, text, ' ');
	wrapWords();
}

void SFTextBox::appendString(const std::string& str)
{
	// @todo optimize?
	setString(getString() + str);
}

const std::string& SFTextBox::getString() const
{
	ASSERT(text == rendertext.getString());
	return text;
}

bool SFTextBox::backspace(int numCharsToRemove)
{
	ERROR("unimplemented");
	return false;
}

void SFTextBox::setArea(int width, int height)
{
	bounds.width = width;
	bounds.height = height;
	wrapWords();
}

void SFTextBox::setFont(const std::string& fontName)
{
	rendertext.setFont(gui::FontManager::getFont(fontName));
}

void SFTextBox::setFontSize(int size)
{
	rendertext.setCharacterSize(size);
}

void SFTextBox::setFontColor(const sf::Color& color)
{
#ifdef OLD_SFML
	rendertext.setColor(color);
#else
	rendertext.setFillColor(color);
	rendertext.setOutlineColor(color);
#endif
}

int SFTextBox::getMinimumHeight() const
{
	return rendertext.getLocalBounds().height;
}

std::string SFTextBox::insertNewlines(std::string str) const
{
	// TODO: maybe make this insert newlines if we ever use SFML text again? right now everything is spritetext...
	return str;
}

void SFTextBox::finalize()
{
}

// private
void SFTextBox::draw(sf::RenderTarget& target) const
{
	target.draw(rendertext);
}

void SFTextBox::wrapWords()
{
	if (words.empty())
	{
		return;
	}
	//	Now we need to set rendertext up so that words wrap properly...
	const sf::Font* font = rendertext.getFont();
	const unsigned int fontSize = rendertext.getCharacterSize();
	int wordLength = 0;
	int lineWidth = 0;
	const int spaceSize = font->getGlyph(' ', fontSize, false).advance;
	text.clear();
	for (const std::string& word : words)
	{
		//	sum up the space used to write each word (including kerning)
		wordLength = font->getGlyph(word[0], fontSize, false).advance;
		for (int i = 1; i < word.length(); ++i)
		{
			wordLength += font->getGlyph(word[i], fontSize, false).advance + font->getKerning(word[i - 1], word[i], fontSize);
		}

		if (lineWidth == 0)
		{
			text += word;
			lineWidth += wordLength;
		}
		else
		{
			if (lineWidth + spaceSize + wordLength > bounds.width)
			{
				text += '\n';
				lineWidth = spaceSize + wordLength;
			}
			else
			{
				text += ' ';
				lineWidth += spaceSize;
			}
			text += word;
			lineWidth += wordLength;
		}
		maxLineWidth = std::max(lineWidth, maxLineWidth);
	}
	rendertext.setString(text);
}

} // sl
