#ifndef SL_SFTEXTBOX_HPP
#define SL_SFTEXTBOX_HPP

#include <SFML/Graphics/Text.hpp>

#include <vector>

#include "Engine/Graphics/AbstractTextBox.hpp"

namespace sl
{

class SFTextBox : public AbstractTextBox
{
public:
	SFTextBox(const Rect<int>& box);


	void update(int x, int y) override;

	const Rect<int>& getCurrentBounds() const override;

	void setString(const std::string& str) override;

	void appendString(const std::string& str) override;

	const std::string& getString() const override;

	bool backspace(int numCharsToRemove = 1) override;

	void setArea(int width, int height) override;

	void setFont(const std::string& fontName) override;

	void setFontSize(int size) override;

	void setFontColor(const sf::Color& color) override;

	int getMinimumHeight() const override;

	std::string insertNewlines(std::string str) const override;

	void finalize() override;

private:

	void draw(sf::RenderTarget& target) const override;
	
	/**
	* Takes all words in words and inserts them into text while making them
	* wrap around with a newline when they hit the bounds.
	* This method also sets the position of rendertext to the proper place.
	*/
	void wrapWords();

	

	std::vector<std::string> words;

	sf::Text rendertext;

	std::string text;

	Rect<int> bounds;

	int maxLineWidth;
};

} // sl

#endif // SL_SFTEXTBOX_HPP