#include "Engine/Graphics/Sprite.hpp"

#include "Engine/Graphics/ColorSwapID.hpp"
#include "Engine/Graphics/TextureManager.hpp"
#include "Engine/Serialization/AutoDeserialize.hpp"
#include "Engine/Serialization/AutoSerialize.hpp"
#include "Utility/Assert.hpp"

namespace sl
{

Sprite::Sprite(DeserializeConstructor)
	:texture(deserializeConstructor)
{
}

// THE REASON THIS IS CONST CHAR * IS BECAUSE FOR SOME FUCKING REASON MSVC++ CRASHES
// WITH INTERNAL COMPILER ERROR IF THIS A STD::STRING IN SOME PLACES
// I think it has to do with constructing Sprites inside of initializer_lists somewhere
Sprite::Sprite(const char *str, int originX, int originY)
	:texture(TextureManager::getTexture(str))
	,sprite(texture.get())
	,currentBounds(originX, originY, texture.get().getSize().x, texture.get().getSize().y)
{
	this->init(originX, originY);
}

Sprite::Sprite(const char *str, const ColorSwapID& swapID, int originX, int originY)
	:texture(TextureManager::getTexture(str, swapID))
	,sprite(texture.get())
	,currentBounds(originX, originY, texture.get().getSize().x, texture.get().getSize().y)
{
	this->init(originX, originY);
}

void Sprite::init(int originX, int originY)
{
	sprite.setOrigin(originX, originY);

	//depthCheckSubrect = maximumBounds;
	//depthCheckSubrectReturn = maximumBounds;
}

/*	  Drawable public methods		 */
void Sprite::update(int x, int y)
{
	sprite.setPosition(x, y);
	currentBounds.left = sprite.getGlobalBounds().left;
	currentBounds.top = sprite.getGlobalBounds().top;
	currentBounds.width = sprite.getGlobalBounds().width;
	currentBounds.height = sprite.getGlobalBounds().height;
}

const Rect<int>& Sprite::getCurrentBounds() const
{
	return currentBounds;
}


void Sprite::rotate(float newAngle)
{
	sprite.setRotation(360.f - newAngle);
}

void Sprite::setColor(const sf::Color& color)
{
	sprite.setColor(color);
}

/*	  Sprite public methods	*/
float Sprite::getRotation() const
{
	return 360.f - sprite.getRotation();
}

void Sprite::setRenderedSubrect(const Rect<int>& subrect)
{
	sprite.setTextureRect(sf::IntRect(subrect.left, subrect.top, subrect.width, subrect.height));
}

const Texture& Sprite::getTexture() const
{
	return texture;
}

void Sprite::setAlpha(float alpha)
{
	ASSERT(alpha >= 0.f && alpha <= 1.f);
	sprite.setColor(sf::Color(255, 255, 255, alpha * 255));
}

void Sprite::setOrigin(const Vector2i& offset)
{
	sprite.setOrigin(offset.x, offset.y);
}

void Sprite::mirrorHorizontally()
{
	sprite.setScale(-sprite.getScale().x, 1.f);
}

void Sprite::serialize(Serialize& out) const
{
	serializeDrawable(out);
	ASSERT(texture.filename() != "invalid");
	out.s("filename", texture.filename());
	out.f("originX", sprite.getOrigin().x);
	out.f("originY", sprite.getOrigin().y);
	AutoSerialize::serialize(out, currentBounds);
}

void Sprite::deserialize(Deserialize& in)
{
	deserializeDrawable(in);
	texture = TextureManager::getTexture(in.s("filename"));
	sprite.setTexture(texture.get());
	const float originX = in.f("originX");
	const float originY = in.f("originY");
	sprite.setOrigin(originX, originY);
	AutoDeserialize::deserialize(in, currentBounds);
}

std::string Sprite::serializationType() const
{
	return "Sprite";
}

bool Sprite::shouldSerialize() const
{
	return true;
}

/*	  Drawable private methods		*/
void Sprite::draw(sf::RenderTarget& target) const
{
	target.draw(sprite);
#ifdef SL_DEBUG

#endif
}

} // sl
