/* throw this class out please or write an AnimatedSprite or something */
#ifndef SL_WRAPPEDSPRITE_HPP
#define SL_WRAPPEDSPRITE_HPP

#include "Engine/Graphics/Drawable.hpp"
#include "Engine/Graphics/Texture.hpp"
#include "Engine/Serialization/DeserializeConstructor.hpp"

namespace sl
{

class ColorSwapID;

class Sprite : public Drawable
{
public:
	Sprite(DeserializeConstructor);
	/**
	 * Creates a RAII managed sprite from a texture file
	 * @param str The filename of the image (located in /resources/img/)
	 * @param originX The x offset this image will be displayed at (0 = is top left)
	 * @param originY The y offset this image will be display at (0 = top left)
	 * @param top The smallest pixel of this image fixed to the ground
	 */
	Sprite(const char *str, int originX = 0, int originY = 0);

	/**
	 * Creates a RAII managed sprite from a texture file, but colour-swapped
	 * @param str The filename of the image (located in /resources/img/)
	 * @param swapID The colour-swap to be performed
	 * @param originX The x offset this image will be displayed at (0 = is top left)
	 * @param originY The y offset this image will be display at (0 = top left)
	 * @param top The smallest pixel of this image fixed to the ground
	 */
	Sprite(const char *str, const ColorSwapID& swapID, int originX = 0, int originY = 0);

	/*	  Drawable public methods		 */
	void update(int x, int y) override;

	const Rect<int>& getCurrentBounds() const override;

	
	/**
	 * Rotates image
	 * @param newAngle The new angle this image will be rotated to in degrees (0=default)
	 */
	void rotate(float newAngle) override;

	void setColor(const sf::Color& color) override;

	/*	  Sprite public methods	*/
	float getRotation() const;

	void mirrorHorizontally();

	/**
	 * Sets the subrect of the sprite to draw (by default is entire texture)
	 * @param subrect The subrect to display
	 */
	void setRenderedSubrect(const Rect<int>& subrect);

	const Texture& getTexture() const;

	void setAlpha(float alpha);

	void setOrigin(const Vector2i& offset);

	void draw(sf::RenderTarget& target) const override;


	void serialize(Serialize& out) const override;

	void deserialize(Deserialize& in) override;

	std::string serializationType() const override;

	bool shouldSerialize() const override;

private:
	void init(int originX, int originY);



	//!The RAII texture contained in this class
	Texture texture;
	//!The sprite representing any transforms done to the texture
	sf::Sprite sprite;
	//!The current bounds of this sprite with the current rotation
	Rect<int> currentBounds;
};

}

#endif
