#ifndef SL_SPRITETEXT_HPP
#define SL_SPRITETEXT_HPP

#include <SFML/Graphics/RenderTexture.hpp>
#include <SFML/Graphics/Sprite.hpp>

#include <memory>
#include <string>

#include "Engine/Graphics/AbstractTextBox.hpp"

namespace sl
{

class SpriteFont;

class SpriteText : public AbstractTextBox
{
public:
	SpriteText(const std::string& fontName, const Rect<int>& box, int spaceGap, int scale, const std::string& str = std::string());
	~SpriteText();

	// Drawable
	void update(int x, int y) override;

	const Rect<int>& getCurrentBounds() const override;

	// AbstractTextBox
	void setString(const std::string& str) override;

	void appendString(const std::string& str) override;

	const std::string& getString() const override;

	bool backspace(int numCharsToRemove = 1) override;

	void setArea(int width, int height) override;

	void setFont(const std::string& fontName) override;

	/**
	 * Sets the size of the font - for SpriteFonts this is the x/yscale of the sprite, not the height in pixels
	 * @param size The new scale
	 */
	void setFontSize(int size) override;

	void setFontColor(const sf::Color& color) override;

	int getMinimumHeight() const override;

	std::string insertNewlines(std::string str) const override;

	void finalize() override;

private:

	void draw(sf::RenderTarget& target) const override;

	void refresh();

	void remakeRenderTexture();



	const SpriteFont *font;
	int scale;
	sf::Color color;
	std::string text;
	Rect<int> bounds;
	Rect<int> scaledBounds;
	int spaceGap;
	int symbolsAcross;
	std::unique_ptr<sf::RenderTexture> render;
	sf::Texture texture;
	sf::Sprite sprite;
	Vector2i cursor;
	Vector2i textureSize;
#ifdef SL_DEBUG
	// SpriteText has two modes with respect to newlines.
	// 1) Newlines pre-inserted. It assumes input had newlines already in it. This is activated as soon as you call insertNewlines()
	// 2) Newlines not present. It will automatically wrap around to the next line. This is the default behaviour.
	mutable bool inputTextHadNewlines;
#endif // SL_DEBUG

	friend class AbstractScene; // for draw()
};

} // sl

#endif // SL_SPRITETEXT_HPP
