#include "Engine/Graphics/Texture.hpp"

namespace sl
{

Texture::Texture(DeserializeConstructor)
	:owner(nullptr)
	,id("invalid")
	,swapID()
	,isAColorSwap(false)
	,textureData(nullptr)
{
}

Texture::Texture(TextureManager *owner, const std::string& id, const sf::Texture *textureData)
	:owner(owner)
	,id(id)
	,swapID()
	,isAColorSwap(false)
	,textureData(textureData)
{
	owner->addReference(id);
}

Texture::Texture(TextureManager *owner, const std::string& id, const ColorSwapID& swapID, const sf::Texture *textureData)
	:owner(owner)
	,id(id)
	,swapID(swapID)
	,isAColorSwap(true)
	,textureData(textureData)
{
	owner->addReference(id, swapID);
	owner->addReference(id);
}

Texture::Texture(const Texture& copy)
	:owner(copy.owner)
	,id(copy.id)
	,swapID(copy.swapID)
	,isAColorSwap(copy.isAColorSwap)
	,textureData(copy.textureData)
{
	if (owner)
	{
		if (isAColorSwap)
		{
			owner->addReference(id, swapID);
		}
		owner->addReference(id);
	}
}

Texture::~Texture()
{
	if (isAColorSwap)
	{
		owner->removeReference(id, swapID);
	}
	owner->removeReference(id);
}

const Texture& Texture::operator=(const Texture& copy)
{
	if (owner)
	{
		if (this->isAColorSwap)
		{
			this->owner->removeReference(id, swapID);
		}
		this->owner->removeReference(id);
	}
	this->id = copy.id;
	this->owner = copy.owner;
	if (copy.owner)
	{
		this->owner->addReference(copy.id);
		if (copy.isAColorSwap)
		{
			copy.owner->addReference(copy.id, copy.swapID);
		}
	}
	this->textureData = copy.textureData;
	this->isAColorSwap = copy.isAColorSwap;
	this->swapID = copy.swapID;
	return *this;
}

const sf::Texture& Texture::get() const
{
	return *textureData;
}

const std::string& Texture::filename() const
{
	return id;
}

} // sl
