#ifndef SL_TILEGRID_HPP
#define SL_TILEGRID_HPP

#include "Engine/Graphics/Drawable.hpp"
#include "Utility/Grid.hpp"
#include "Utility/Rect.hpp"

namespace sf
{
class Sprite;
class Texture;
class RenderTarget;
} // sf

namespace sl
{

class TileGrid : public Drawable
{
public:
	TileGrid(int xOffset, int yOffset, int width, int height, int cellSizeX, int cellSizeY);

	/*	  Drawable public methods	 */
	void update(int x, int y) override;

	const Rect<int>& getCurrentBounds() const override;


	/*	  TileGrid public methods	 */
	void setTexture(int x, int y, const sf::Texture& texture);

	void setTexture(int x, int y, const sf::Texture& texture, const sf::IntRect& textureRect);

	void resetTextures();

	void setVisibleArea(const Rect<int>& bBox);

	void set(int xOffset, int yOffset, int width, int height, int cellSizeX, int cellSizeY);

private:
	TileGrid(const TileGrid&) = delete;
	TileGrid& operator=(const TileGrid&) = delete;

	/*	  Drawble private methods	 */
	void draw(sf::RenderTarget& target) const;

	/*	  TileGrid private methods	*/
	void recalculateVisibleTiles();

	Grid<sf::Sprite> tiles;
	int left;      //  in pixels
	int top;       //  in pixels
	int cellSizeX; //  in pixels
	int cellSizeY; //  in pixels
	Rect<int> bBox;
	Rect<int> visibleTilesByIndices;
	Rect<int> visibleTilesByPixels;
};

} // sl

#endif
