#ifndef SL_CONTROLLER_HPP
#define SL_CONTROLLER_HPP

#include <memory>

#include "Engine/Input/Input.hpp"
#include "Engine/Input/Key.hpp"
#include "Engine/Input/KeyBind.hpp"

namespace sl
{

class ControllerBinds;
class InputHandle;

class Controller
{
public:
	Controller(const InputHandle& input, const ControllerBinds& binds, unsigned int gamepadId = 0);

	//void setInputHandle(const InputHandle& input);

	//void unsetInputHandle();


	bool pressed(Key key) const;
	bool released(Key key) const;
	bool held(Key key) const;

	void setJoystickID(unsigned int id);
	float axis(Input::Axis axis) const;

	/**
	* @return the custom action id pressed. If multiple were pressed in a given frame one is picked arbitrarily. NO_CUSTOM_ACTION if none pressed.
	*/
	CustomAction customActionPressed() const;

	/**
	 * @return A KeyBind to the last pressed key, but only if it isn't already bound to a REGULAR key. CustomActions are overwritten.
	 */
	std::unique_ptr<KeyBind> getJustPressedUnboundKey() const;

private:
	bool checkPressed(const KeyBind& bind) const;
	bool checkReleased(const KeyBind& bind) const;
	bool checkHeld(const KeyBind& bind) const;

	const InputHandle *input;
	const ControllerBinds *binds;
	unsigned int gamepadId;
};

} // sl

#endif // SL_CONTROLLER_HPP
