#ifndef SL_CONTROLLERBINDS_HPP
#define SL_CONTROLLERBINDS_HPP

#include <map>
#include <vector>

#include "Engine/Input/Input.hpp"
#include "Engine/Input/Key.hpp"
#include "Engine/Input/KeyBind.hpp"

namespace sl
{

class ControllerBinds
{
public:
	void addKeybind(Key key, KeyBind bind);
	void clearKeybinds(Key key);
	void clearKeybinds();
	void setKeybinds(Key key)
	{
		clearKeybinds(key);
	}
	template <typename Bind, typename... Binds>
	void setKeybinds(Key key, Bind head, Binds... tail)
	{
		setKeybinds(key, tail...);
		addKeybind(key, KeyBind(head));
	}

	void registerCustomAction(CustomAction action, KeyBind bind);

	void removeCustomActionKeybind(KeyBind bind);

	bool isBound(KeyBind keyBind) const;

	const std::vector<KeyBind>* getBinds(Key key) const;

	const std::map<KeyBind, CustomAction>& getCustomActions() const;

	std::string getBindsString(Key key, bool excludeDisconnectedGamepads = true) const;

private:
	//! All keys and their normal keybinds. And Key here cannot be used as a CustomAction.
	std::map<Key, std::vector<KeyBind>> keybinds;
	//! How many times a keybind is registered. Used to see what can be used as a CustomAction
	std::map<KeyBind, int> regularBindCount;
	std::map<KeyBind, CustomAction> customActions;
};

} // sl

#endif // SL_CONTROLLERBINDS_HPP