#include "Engine/Input/InputHandle.hpp"

#include "Engine/Input/KeyBindDetection.hpp"
#include "Engine/AbstractScene.hpp"

namespace sl
{

InputHandle::InputHandle(const Input& input, const AbstractScene& scene)
	:input(input)
	,scene(scene)
{
}

/*			keyboard			*/
bool InputHandle::isKeyPressed(Input::KeyboardKey key) const
{
	return scene.isActive() && input.isKeyPressed(key);
}

bool InputHandle::isKeyReleased(Input::KeyboardKey key) const
{
	return scene.isActive() && input.isKeyReleased(key);
}

bool InputHandle::isKeyHeld(Input::KeyboardKey key) const
{
	return scene.isActive() && input.isKeyHeld(key);
}

/*			mouse				*/
bool InputHandle::isMousePressed(Input::MouseButton button) const
{
	return scene.isActive() && input.isMousePressed(button);
}

bool InputHandle::isMouseReleased(Input::MouseButton button) const
{
	return scene.isActive() && input.isMouseReleased(button);
}

bool InputHandle::isMouseHeld(Input::MouseButton button) const
{
	return scene.isActive() && input.isMouseHeld(button);
}

/*			gamepad			*/
bool InputHandle::isGamepadButtonPressed(unsigned int gamepadId, Input::GamepadButton button) const
{
	return scene.isActive() && input.isGamepadButtonPressed(gamepadId, button);
}

bool InputHandle::isGamepadButtonReleased(unsigned int gamepadId, Input::GamepadButton button) const
{
	return scene.isActive() && input.isGamepadButtonReleased(gamepadId, button);
}

bool InputHandle::isGamepadButtonHeld(unsigned int gamepadId, Input::GamepadButton button) const
{
	return scene.isActive() && input.isGamepadButtonHeld(gamepadId, button);
}

bool InputHandle::isGamepadAxisPressed(unsigned int gamepadId, Input::AxisButton axis) const
{
	return scene.isActive() && input.isGamepadAxisPressed(gamepadId, axis);
}

bool InputHandle::isGamepadAxisReleased(unsigned int gamepadId, Input::AxisButton axis) const
{
	return scene.isActive() && input.isGamepadAxisReleased(gamepadId, axis);
}

bool InputHandle::isGamepadAxisHeld(unsigned int gamepadId, Input::AxisButton axis) const
{
	return scene.isActive() && input.isGamepadAxisHeld(gamepadId, axis);
}

float InputHandle::axis(int gamepadId, Input::Axis axis) const
{
	if (scene.isActive())
	{
		return input.axis(gamepadId, axis);
	}
	return 0;
}

std::vector<KeyBind> InputHandle::getJustPressed(int gamepad) const
{
	if (scene.isActive())
	{
		return pollJustPressedKeyBinds(input, gamepad);
	}
	return std::vector<KeyBind>();
}

} // sl