#ifndef SL_INPUTHANDLE_HPP
#define SL_INPUTHANDLE_HPP

#include "Engine/Input/Input.hpp"
#include "Engine/Input/KeyBind.hpp"

namespace sl
{

class AbstractScene;

class InputHandle
{
public:
	InputHandle(const Input& input, const AbstractScene& scene);

	/**
	 * Check if a key has just been released this update
	 * @ param key The key to check for. See the Key enum
	 * @ return whether or not it was released
	 */
	bool isKeyReleased(Input::KeyboardKey key) const;
	/**
	 * Check if a key has just been pressed this update
	 * @ param key The key to check for. See the Key enum
	 * @ return whether or not it was pressed
	 */
	bool isKeyPressed(Input::KeyboardKey key) const;
	/**
	 * Check if a key is being held this update
	 * @ param key The key to check for. See the Key enum
	 * @ return whether or not the key is held down
	 */
	bool isKeyHeld(Input::KeyboardKey key) const;
	/**
	 * Check if a mouse button has just been released this update
	 * @ param button The button to check for. See the Button enum
	 * @ return whether or not it was released
	 */
	bool isMouseReleased(Input::MouseButton button) const;
	/**
	 * Check if a mouse button has just been pressed this update
	 * @ param button The button to check for. See the Button enum
	 * @ return whether or not it was pressed
	 */
	bool isMousePressed(Input::MouseButton button) const;
	/**
	 * Check if a mouse button is being held down this update
	 * @ param button The button to check for. See the Button enum
	 * @ return whether or not it is held down
	 */
	bool isMouseHeld(Input::MouseButton button) const;

	bool isGamepadButtonPressed(unsigned int gamepadId, Input::GamepadButton button) const;

	bool isGamepadButtonReleased(unsigned int gamepadId, Input::GamepadButton button) const;

	bool isGamepadButtonHeld(unsigned int gamepadId, Input::GamepadButton button) const;

	bool isGamepadAxisPressed(unsigned int gamepadId, Input::AxisButton axis) const;

	bool isGamepadAxisReleased(unsigned int gamepadId, Input::AxisButton axis) const;

	bool isGamepadAxisHeld(unsigned int gamepadId, Input::AxisButton axis) const;

	float axis(int gamepadId, Input::Axis axis) const;

	/**
	 * @gamepad The gamepad id to query from
	 * @return KeyBinds for all the input methods (buttons, keys, etc) that were just pressed.
	 */
	std::vector<KeyBind> getJustPressed(int gamepad) const;

private:
	const Input& input;
	const AbstractScene& scene;
};

}

#endif