#include "Engine/Input/KeyBind.hpp"

namespace sl
{

KeyBind::KeyBind(Input::KeyboardKey key)
	:keyCode(static_cast<int>(key))
	,deviceType(DeviceType::Keyboard)
{
	//Logger::log(Logger::Core, Logger::Debug) << "KeyBind(Keyboard)\n";
}

KeyBind::KeyBind(Input::MouseButton button)
	:keyCode(static_cast<int>(button))
	,deviceType(DeviceType::Mouse)
{
	//Logger::log(Logger::Core, Logger::Debug) << "KeyBind(Mouse)\n";
}

KeyBind::KeyBind(Input::GamepadButton button)
	:keyCode(static_cast<int>(button))
	,deviceType(DeviceType::GamepadButton)
{
	//Logger::log(Logger::Core, Logger::Debug) << "KeyBind(GamepadButton)\n";
}

KeyBind::KeyBind(Input::AxisButton axis)
	:keyCode(static_cast<int>(axis))
	,deviceType(DeviceType::GamepadAxis)
{
}

unsigned int KeyBind::code() const
{
	return keyCode;
}

KeyBind::DeviceType KeyBind::type() const
{
	return deviceType;
}

bool KeyBind::operator<(const KeyBind& rhs) const
{
	if (deviceType == rhs.deviceType)
	{
		return keyCode < rhs.keyCode;
	}
	return deviceType < rhs.deviceType;
}

std::string KeyBind::string() const
{
	switch (deviceType)
	{
	case DeviceType::Keyboard:
		return Input::name(static_cast<Input::KeyboardKey>(keyCode));
	case DeviceType::Mouse:
		return Input::name(static_cast<Input::MouseButton>(keyCode));
	case DeviceType::GamepadAxis:
		return Input::name(static_cast<Input::AxisButton>(keyCode), 0);
	case DeviceType::GamepadButton:
		return Input::name(static_cast<Input::GamepadButton>(keyCode), 0);
	}
	return "???";
}

} // sl