#ifndef SL_KEYBIND_HPP
#define SL_KEYBIND_HPP

#include "Engine/Input/Input.hpp"

namespace sl
{

class KeyBind
{
public:
	enum class DeviceType
	{
		Keyboard,
		Mouse,
		GamepadButton,
		GamepadAxis
	};

	explicit KeyBind(Input::KeyboardKey key);

	explicit KeyBind(Input::MouseButton button);

	explicit KeyBind(Input::GamepadButton button);

	explicit KeyBind(Input::AxisButton axis);

	unsigned int code() const;

	DeviceType type() const;

	bool operator<(const KeyBind& rhs) const;

	std::string string() const;

private:
	unsigned int keyCode;
	DeviceType deviceType;
};

namespace XBOX360
{
static const KeyBind A(0);
static const KeyBind B(1);
static const KeyBind X(2);
static const KeyBind Y(3);
static const KeyBind LeftBumper(4);
static const KeyBind RightBumper(5);
static const KeyBind Back(6);
static const KeyBind Start(7);
static const KeyBind LeftStickPressed(8);
static const KeyBind RightStickPressed(9);
static const KeyBind LeftTrigger(Input::AxisButton::Z);
static const KeyBind RightTrigger(Input::AxisButton::Z_Neg);
static const KeyBind LeftStickLeft(Input::AxisButton::X_Neg);
static const KeyBind LeftStickRight(Input::AxisButton::X);
static const KeyBind LeftStickDown(Input::AxisButton::Y);
static const KeyBind LeftStickUp(Input::AxisButton::Y_Neg);
static const KeyBind RightStickRight(Input::AxisButton::U);
static const KeyBind RightStickLeft(Input::AxisButton::U_Neg);
static const KeyBind RightStickDown(Input::AxisButton::R);
static const KeyBind RightStickUp(Input::AxisButton::R_Neg);
static const KeyBind DPadUp(Input::AxisButton::PovY);
static const KeyBind DPadDown(Input::AxisButton::PovY_Neg);
static const KeyBind DPadRight(Input::AxisButton::PovX);
static const KeyBind DPadLeft(Input::AxisButton::PovX_Neg);
static const Input::Axis RightToLeftTriggers = Input::Axis::Z;
static const Input::Axis LeftStickHoriz = Input::Axis::X;
static const Input::Axis LeftStickVert = Input::Axis::Y;
static const Input::Axis RightStickHoriz = Input::Axis::U;
static const Input::Axis RightStickVert = Input::Axis::R;
static const Input::Axis DPadHoriz = Input::Axis::PovY;
static const Input::Axis DPadVert = Input::Axis::PovX;
};

} // sl

#endif // SL_KEYBIND_HPP