#include "Engine/Input/KeyBindDetection.hpp"

#include "Engine/Input/Input.hpp"

namespace sl
{

std::vector<KeyBind> pollJustPressedKeyBinds(const Input& input, int gamepad)
{
	std::vector<KeyBind> justPressed;
	for (int i = 0; i < sf::Keyboard::KeyCount; ++i)
	{
		const Input::KeyboardKey key = static_cast<Input::KeyboardKey>(i);
		if (input.isKeyPressed(key))
		{
			justPressed.push_back(KeyBind(key));
		}
	}
	for (int i = 0; i < sf::Mouse::ButtonCount; ++i)
	{
		const Input::MouseButton button = static_cast<Input::MouseButton>(i);
		if (input.isMousePressed(button))
		{
			justPressed.push_back(KeyBind(button));
		}
	}
	if (sf::Joystick::isConnected(gamepad))
	{
		for (int i = 0; i < sf::Joystick::ButtonCount; ++i)
		{
			const Input::GamepadButton button = static_cast<Input::GamepadButton>(i);
			if (input.isGamepadButtonPressed(gamepad, button))
			{
				justPressed.push_back(KeyBind(button));
			}
		}
		for (int i = 0; i < sf::Joystick::AxisCount * 2; ++i)
		{
			const Input::AxisButton axis = static_cast<Input::AxisButton>(i);
			if (input.isGamepadAxisPressed(gamepad, axis))
			{
				justPressed.push_back(KeyBind(axis));
			}
		}
	}
	return justPressed;
}

} // sl
