#include "Engine/Physics/PhysicsComponent.hpp"
#include <list>
#include "Engine/Scene.hpp"
#include "Utility/Trig.hpp"

namespace sl
{

// TODO: move this into a Utility file?
static inline float sign(float x)
{
	return x >= 0.f ? 1.f : -1.f;
}

PhysicsComponent::PhysicsComponent(Entity& owner, Scene *scene)
	:owner(owner)
	,pos(owner.getPos())
	,scene(scene)
	,speed(20, 0)
	,previousPosition(owner.getPos())
{
}

void PhysicsComponent::update()
{
	//Entity *check = nullptr;

	owner.setPos(pos + speed);


	if (scene->getViewRect().intersects(owner.getMask().getBoundingBox()))
	{
		Vector2f vecResult;
		const Scene::RaycastResult castResult = scene->raycast(owner, {"StaticGeometry"}, speed, vecResult);
		if (castResult != Scene::RaycastResult::MovedFully)
		{
			std::cout << "speed = (" << speed.x << ", " << speed.y << ")" << std::endl;

			const Vector2f remainingMotion = owner.getPos() + speed - vecResult;
			const float remainingMotionDist = std::max(remainingMotion.length(), 3.f);

			//ASSERT(remainingMotionDist <= speed.length());
			
			owner.setPos(vecResult - speed.normalized() * 2.f);

			Vector2f horizResult;
			scene->raycast(owner, {"StaticGeometry"}, Vector2f(remainingMotionDist, 0.f), horizResult);
			Vector2f vertResult;
			scene->raycast(owner, {"StaticGeometry"}, Vector2f(0.f, remainingMotionDist), vertResult);
				
			std::cout << "remainingMotion = (" << remainingMotion.x << ", " << remainingMotion.y << ")" << std::endl;

			const float horizDist = (horizResult - owner.getPos()).length();
			const float vertDist = (vertResult - owner.getPos()).length();

			const float requiredThreshold = remainingMotionDist * 0.45f;
			const float slideThreshhold = remainingMotionDist * 0.75f;
			
			std::cout << "requiredThreshold = " << requiredThreshold << "\tvertDist = " << vertDist << "\thorizDist = " << horizDist << std::endl;
			if (horizDist >= requiredThreshold && vertDist < requiredThreshold)
			{
				if (horizDist >= slideThreshhold)
				{
					speed.x = sign(speed.x) * horizDist;
					speed.y = 0.f;

				}
				else
				{
					// move horizontally and bounce vertically a bit
					speed.x = sign(speed.x) * (horizDist * 0.8f + vertDist * 0.2f);
					speed.y = speed.y * -0.1f;

					std::cout << "bounce vert" << std::endl;

					
				}
				onVelocChange(speed, BounceType::Vertical);
			}
			else if (horizDist < requiredThreshold && vertDist >= requiredThreshold)
			{
				if (vertDist >= slideThreshhold)
				{
					speed.x = 0.f;
					speed.y = sign(speed.y) * vertDist;
				}
				else
				{
					// move vertically and bounce horizontally a bit
					speed.x = speed.x * -0.1f;
					speed.y = sign(speed.y) * (vertDist * 0.8f + horizDist * 0.2f);

					std::cout << "bounce horiz" << std::endl;

					
				}
				onVelocChange(speed, BounceType::Horizontal);
			}
			else // we hit a corner (from inside or outside)
			{
				// bound back
				std::cout << "bounce back" << std::endl;

				owner.setPos(pos-speed);
				speed *= -0.25f;
				onVelocChange(speed, BounceType::Back);
			}

			//owner.setPos(previousPosition + speed);//vecResult + speed);
		}
		else
		{
			owner.setPos(vecResult);
		}
	}

	previousPosition = pos;
}

void PhysicsComponent::addMotion(float dir, float amount)
{
	speed += lengthdir(amount, dir);
}

float PhysicsComponent::getSpeed() const
{
	return pointDistance(0, 0, speed.x, speed.y);
}

const Vector2<float>& PhysicsComponent::getSpeedVector() const
{
	return speed;
}

void PhysicsComponent::setSpeed(float speed)
{
	//	Check to see if we even had a speed before...
	if (this->speed.x != 0 || this->speed.y != 0)
	{
		float ratio = speed / this->getSpeed();

		this->speed.x *= ratio;
		this->speed.y *= ratio;
	}
	else
	{
		//	0 degree angle fuck you
		this->speed.x = 0;
	}
}

void PhysicsComponent::setSpeed(const Vector2<float>& speed)
{
	this->speed = speed;
}

void PhysicsComponent::setSpeed(float xspeed, float yspeed)
{
	speed.x = xspeed;
	speed.y = yspeed;
}

void PhysicsComponent::setOnVelocChange(const std::function<void(const Vector2f&, BounceType)>& onChange)
{
	onVelocChange = onChange;
}

} // sl
