#ifndef SL_PHYSICSCOMPONENT_HPP
#define SL_PHYSICSCOMPONENT_HPP

#include <functional>

#include "Engine/Entity.hpp"
#include "Utility/Vector.hpp"

namespace sl
{

class Scene;

class PhysicsComponent
{
public:
	PhysicsComponent(Entity& owner, Scene *scene);

	void update();

	/**
	 * Adds a force to the component
	 * @param dir The direction of the force
	 * @param amount The velocity of the force
	 */
	void addMotion(float dir, float amount);

	/**
	 * Returns the velocity of the component
	 * @return The velocity as a scalar
	 */
	float getSpeed() const;

	/**
	 * Returns the velocity of the component
	 * @return The velocity as a vector of the x/y velocities
	 */
	const Vector2<float>& getSpeedVector() const;

	/**
	 * Sets the velocity of the object keeping its previous direction.
	 * Note:	if the object had no previous velocity, then the direction
	 *			is assumed to be 0 (ie facing right)
	 * @param speed The new velocity for the component
	 */
	void setSpeed(float speed);

	
	void setSpeed(const Vector2<float>& speed);

	void setSpeed(float xspeed, float yspeed);
	
	enum class BounceType
	{
		Back,
		Horizontal,
		Vertical
	};

	void setOnVelocChange(const std::function<void(const Vector2f&, BounceType)>& onChange);

private:
	Entity& owner;
	const Vector2<float>& pos;	//	to avoid lots of method calls
	Scene *scene;
	Vector2<float> speed;
	Vector2<float> previousPosition;
	float speedScalar;
	std::function<void(const Vector2f&, BounceType)> onVelocChange;

	friend class Scene;
};

} // sl

#endif