/**
 * @section DESCRIPTION
 * An allocator for use in STL containers that will allocate from a pool,
 * rather than from the heap every time. This was created mostly because
 * QuadTree can do a lot of stuff with lists that involve a lot of heap allocs
 * from within std::list as nodes transfer from one to another. This class
 * kind of sucks balls though and didn't even end up working properly since
 * std::list nodes are 12 (or 24) bytes and Entity pointers are 4 (or 8)
 * bytes, and my BasicPool class is shitty and hopes for uniform sizes
 * I could fix this, but then it doesn't seem like it would be any faster
 * than not using a pool in the first place. I'll have to try it out later.
 */
#ifndef SL_POOLALLOCATOR_HPP
#define SL_POOLALLOCATOR_HPP
#include <typeinfo>
#include <limits>
#include <iostream>
#include "Engine/BasicPool.hpp"

namespace sl
{

template <typename T>
class PoolAllocator
{
public:
	// These typedefs are REQUIRED for Allocators:
	typedef T			   value_type;
	typedef T*			  pointer;
	typedef const T*		const_pointer;
	typedef T&			  reference;
	typedef const T&		const_reference;
	typedef std::size_t	 size_type;
	typedef std::ptrdiff_t  difference_type;
	// As is this:
	template <class U>
	struct rebind
	{
		typedef PoolAllocator<U> other;
	};

	PoolAllocator() throw();
	PoolAllocator(const PoolAllocator&) throw();
	template <typename U>
	PoolAllocator(const PoolAllocator<U>&) throw();
	PoolAllocator(BasicPool<T> *pool) throw();
	~PoolAllocator() throw();

	pointer address(reference value) const;
	const_pointer address(const_reference value) const;
	std::size_t max_size() const throw();
	pointer allocate(std::size_t num, const void * = 0);
	void construct(pointer p, const_reference value);
	void destroy(pointer p);
	void deallocate(pointer p, std::size_t num);

	void setPool(BasicPool<T> *newPool);
private:
	/*T **pool;
	std::stack<pointer> poolStack;
	const std::size_t poolSize;
	std::size_t *poolUsed, pools;*/
	BasicPool<T> *pool;
};

template <typename T>
PoolAllocator<T>::PoolAllocator() throw()
	:pool(nullptr)
{
#ifdef POOL_DEBUG
	std::cout << "def ctor of type we are " << this << std::endl;
	std::cout << "We are a \n" << typeid(*this).name() << std::endl << std::endl;
	std::cout << "We take up " << sizeof(T) << " bytes." << std::endl << std::endl;
#endif
}

template <typename T>
PoolAllocator<T>::PoolAllocator(BasicPool<T> *pool) throw()
	:pool(pool)
{
#ifdef POOL_DEBUG
	std::cout << "def ctor of type we are " << this << std::endl;
	std::cout << "We are a \n" << typeid(*this).name() << std::endl << std::endl;
	std::cout << "We take up " << sizeof(T) << " bytes." << std::endl << std::endl;
#endif
}

template <typename T>
PoolAllocator<T>::PoolAllocator(const PoolAllocator& derp) throw()
	:pool(nullptr)
{
#ifdef POOL_DEBUG
	std::cout << "copy ctor from" << &derp << "\tand we are " << this << std::endl;
	std::cout << "We are a \n" << typeid(*this).name() << "\n\n and input is a \n" << typeid(derp).name() << std::endl << std::endl;
	std::cout << "We take up " << sizeof(T) << " bytes." << std::endl << std::endl;
#endif
}

template <typename T>
template <typename U>
PoolAllocator<T>::PoolAllocator(const PoolAllocator<U>& derp) throw()
	:pool(nullptr)
{
#ifdef POOL_DEBUG
	std::cout << "copy ctor alt from " << &derp << "\tand we are " << this << std::endl;
	std::cout << "We are a \n" << typeid(*this).name() << "\n\n and input is a \n" << typeid(derp).name() << std::endl << std::endl;
	std::cout << "We take up " << sizeof(T) << " bytes." << std::endl << std::endl;
#endif
}

template <typename T>
PoolAllocator<T>::~PoolAllocator() throw()
{
}

template <typename T>
T* PoolAllocator<T>::address(reference value) const
{
	return &value;
}

template <typename T>
const T* PoolAllocator<T>::address(const_reference value) const
{
	return &value;
}

template <typename T>
std::size_t PoolAllocator<T>::max_size() const throw()
{
	return std::numeric_limits<std::size_t>::max() / sizeof(T);
}

template <typename T>
T* PoolAllocator<T>::allocate(std::size_t num, const void *)
{
	//std::cout << "S";
	//std::cout.flush();
	pointer block;
	/*if (num == 1)
	{
		block = pool->get();
	}
	else	// I hate you.
	*/{
		//std::cerr << "You are a faggot for using this class on arrays. Enjoy your standard allocation, faggot." << std::endl;
		block = (pointer)(::operator new(num * sizeof(T)));
	}
	//std::cout << "E";
	//std::cout.flush();
	return block;
}

template <typename T>
void PoolAllocator<T>::construct(pointer p, const_reference value)
{
	new((void*)p)T(value);
}

template <typename T>
void PoolAllocator<T>::destroy(pointer p)
{
	p->~T();
}

template <typename T>
void PoolAllocator<T>::deallocate(pointer p, std::size_t num)
{
	/*if (num == 1)
	{
		//std::cout << "Deallocated memory at " << p << ". Put into stack, now = " << poolStack.size() << std::endl;
		pool->put(p);
	}
	else	// I STILL hate you
	*/{
		//std::cerr << "You are STILL a faggot for using this class on arrays. Enjoy your standard deallocation, faggot." << std::endl;
		::operator delete((void*)p);
	}
}

template <typename T>
void PoolAllocator<T>::setPool(BasicPool<T> *newPool)
{
	pool = newPool;
}

template <class T1, class T2>
bool operator== (const PoolAllocator<T1>&, const PoolAllocator<T2>&) throw()
{
   return true;
}

template <class T1, class T2>
bool operator!= (const PoolAllocator<T1>&, const PoolAllocator<T2>&) throw()
{
   return false;
}

}

#endif
