/**
 * @DESCRIPTION Meant to be used via composition to anything that can be used in a Ref<Type>
 * This is here to reduce code-duplication largely, but also to minimize potential fuckery with the alive counter.
 * For example, to have a Ref<Entity> then Entity should contian a Refable as a member.
 */
#ifndef SL_REFABLE_HPP
#define SL_REFABLE_HPP

#include <memory>

namespace sl
{

namespace ref_impl
{
template <bool Owning>
class RefBase;
}

class Refable
{
public:
	Refable();

	~Refable();


	bool canDestroy() const;

private:
	// it's probably not a good idea to copy/move any T containing a Refable,
	// as it will break the references in any Ref<T> that may or may not be live
	Refable(const Refable& other) = delete;
	Refable& operator=(const Refable& rhs) = delete;

	//! Alive reference. This should always be 1 or more. 0 means we deleted this object already. higher than 1 means someone has a lock on this object and we shouldn't delete it
	std::shared_ptr<int> alive;

	friend class ref_impl::RefBase<false>;
	friend class ref_impl::RefBase<true>;
};

} // sl

#endif // SL_REFABLE_HPP