#ifndef SL_SCENEPOS_HPP
#define SL_SCENEPOS_HPP

#include "Engine/Serialization/AutoDeserialize.hpp"
#include "Utility/Vector.hpp"

namespace sl
{

class Entity;
class Deserialize;
class Serialize;
class Scene;

class ScenePos
{
public:
	ScenePos()
		:scene(nullptr)
		,pos()
	{} // should not be used, only for later initialization/etc

	ScenePos(const Entity& entity);

	ScenePos(Scene& scene, const Vector2f& pos)
		:scene(&scene)
		,pos(pos)
	{}

	Scene* getScene() const { return scene; }

	const Vector2f& getPos() const { return pos; }

	static ScenePos createImpl(Deserialize& in);

private:
	Scene *scene;
	Vector2f pos;
};

namespace AutoDeserialize
{
template <>
inline ScenePos create(Deserialize& in)
{
	// Since we need to have the definition of Scene here, but can't define the specializaiton itself in the .cpp
	return ScenePos::createImpl(in);
}
} // AutoDeserialize

namespace AutoSerialize
{
void serialize(Serialize& out, const ScenePos& sp);
} // AutoSerialize

} //  sl

#endif // SL_SCENEPOS_HPP