#ifndef SL_BINARYDESERIALIZE_HPP
#define SL_BINARYDESERIALIZE_HPP

#include "Engine/Serialization/Serialization.hpp"

#include <fstream>
#include <functional>
#include <memory>
#include <stack>
#include <vector>

namespace sl
{

class BinaryDeserialize : public Deserialize
{
public:
	BinaryDeserialize(const std::string& fname, std::map<std::string, std::function<std::unique_ptr<Serializable>()>> ptrAllocators);

	uint8_t u8(const std::string& name) override;

	uint16_t u16(const std::string& name) override;

	uint32_t u32(const std::string& name) override;

	uint64_t u64(const std::string& name) override;

	bool b(const std::string& name) override;

	int i(const std::string& name) override;

	float f(const std::string& name) override;

	std::string s(const std::string& name) override;

	uint32_t raw(const std::string& name, void *val, uint32_t len) override;

	Serializable* ptr(const std::string& name) override;

	std::unique_ptr<Serializable> loadSerializable(const std::string& name) override;

	int startArray(const std::string& name) override;

	void endArray(const std::string& name) override;

	void startObject(const std::string& name) override;

	void endObject(const std::string& name) override;
	
	void finish() override;

	World* getWorld() const override;

	void setWorld(World *world) override;

private:

	void debugField(const std::string& name, const std::string& type);

	std::ifstream in;
	bool debug;
	enum class StackType
	{
		Array,
		Object
	};
	std::stack<std::pair<StackType, std::string>> debugStack;
	std::map<uint64_t, Serializable*> ptrs;
	std::map<uint64_t, std::unique_ptr<Serializable>> undeserializedPtrs;
	std::map<std::string, std::function<std::unique_ptr<Serializable>()>> ptrAllocators;
	World *world;
#ifdef SL_DEBUG
	std::ofstream debugFile;
#endif // SL_DEBUG
};

} // sl

#endif // SL_BINARYDESERIALIZE_HPP