#ifndef SL_BINARYSERIALIZE_HPP
#define SL_BINARYSERIALIZE_HPP

#include <fstream>
#include <map>
#ifdef SL_DEBUG
	#include <set>
#endif // SL_DEBUG

#include "Engine/Serialization/Serialization.hpp"

#include <stack>

namespace sl
{

class BinarySerialize : public Serialize
{
public:
	BinarySerialize(const std::string& fname);

	~BinarySerialize();

	void u8(const std::string& name, uint8_t val) override;

	void u16(const std::string& name, uint16_t val) override;

	void u32(const std::string& name, uint32_t val) override;

	void u64(const std::string& name, uint64_t val) override;

	void b(const std::string& name, bool val) override;

	void i(const std::string& name, int val) override;

	void f(const std::string& name, float val) override;

	void s(const std::string& name, const std::string& val) override;

	void raw(const std::string& name, const void *val, uint32_t len) override;

	void ptr(const std::string& name, const Serializable *val) override;

	void saveSerializable(const std::string& type, const Serializable *val) override;

	void startArray(const std::string& name, int length) override;

	void endArray(const std::string& name) override;

	void startObject(const std::string& name) override;

	void endObject(const std::string& name) override;

	void finish() override;


private:

	void debugField(const std::string& name, const std::string& type);

	std::ofstream out;
	bool debug;
	enum class StackType
	{
		Array,
		Object
	};
	std::stack<std::pair<StackType, std::string>> debugStack;
#ifdef SL_DEBUG
	// checks which poiners someone called ptr() on, so that we can verify that
	// ptrCounts contains it by the end of the serialiation, or else we can't
	// possibly deserialize the data
	std::set<const Serializable*> savedPtrs;
	std::set<const Serializable*> registeredPtrs;
	std::ofstream debugFile;
#endif // SL_DEBUG
};

} // sl

#endif // SL_BINARYSERIALIZE_HPP
