#include "Engine/Settings.hpp"

#include <fstream>
#include <iostream>

#include "Utility/Assert.hpp"
#include "Utility/JsonUtil.hpp"


namespace sl
{

Settings* Settings::shittyHackSingleton = nullptr;

const char *Settings::filename = "options.json";

Settings::Settings()
	:systemSettingName("you should not see this")
{
	shittyHackSingleton = this;
}

bool Settings::exists(const std::string& id) const
{
	return settings.count(id);
}

void Settings::createSetting(const std::string& id, const std::string& name, int defaultValue)
{
	settings.insert(std::make_pair(id, Setting(name)));
	Setting& setting = settings.at(id);
	setting.value = defaultValue;
}

void Settings::createSystemSetting(const std::string& id, int value)
{
	auto it = settings.insert(std::make_pair(id, Setting(systemSettingName)));
	it.first->second.value = value;
}

void Settings::set(const std::string& id, int value)
{
	Setting& setting = settings.at(id);
	set(setting, value);
}

void Settings::set(Setting& setting, int value)
{
	setting.value = value;
	if (setting.onChange)
	{
		setting.onChange(value);
	}
}

int Settings::get(const std::string& id) const
{
	auto it = settings.find(id);
	if (it == settings.end())
	{
		ERROR("accessing non-existent setting " + id);
		return -1;
	}
	return it->second.value;
}

void Settings::setValuesByRange(const std::string& id, int min, int max, int increment)
{
	Setting& setting = settings.at(id);
	setting.begin = [min]
	{
		return min;
	};
	setting.end = [max]
	{
		return max;
	};
	setting.next = [increment](int x)
	{
		return x + increment;
	};
}

void Settings::setValuesCusom(const std::string& id, const std::function<int()>& begin, const std::function<int()>& end, const std::function<int(int)> next)
{
	Setting& setting = settings.at(id);
	setting.begin = begin;
	setting.end = end;
	setting.next = next;
}

void Settings::setNameFunc(const std::string& id, const std::function<std::string(int)>& nameFunc)
{
	settings.at(id).name = nameFunc;
}

const Settings::Setting& Settings::getSetting(const std::string& setting) const
{
	return settings.at(setting);
}



const std::map<std::string, Settings::Setting>& Settings::hackyIterator() const
{
	return settings;
}

void Settings::setOnChangeCallback(const std::string& setting, const std::function<void(int)>& callback)
{
	settings.at(setting).onChange = callback;
}

bool Settings::save() const
{
	std::ofstream file(filename);
	json11::Json::array settingList;

	if (!file.is_open())
	{
		return false;
	}

	for (const auto &setting : settings)
	{
		if (setting.second.properName != systemSettingName)
		{
			json11::Json::object obj;
			obj["name"] = setting.first;
			obj["value"] = setting.second.value;
			settingList.push_back(std::move(obj));
		}
	}

	file << json11::Json(settingList).dump();
	return true;
}

bool Settings::load()
{
	std::string error;
	json11::Json data = readJsonFile(filename, error);

	if (!data.is_null())
	{
		const json11::Json::array& settingList = data.array_items();
		for (auto &object : settingList)
		{
			const std::string& name = object["name"].string_value();
			if (name.empty())
				continue;
			Setting& setting = settings.at(name);
			setting.value = object["value"].int_value();
		}
	}

	for (auto &setting : settings)
	{
		if (setting.second.properName != systemSettingName)
		{
			Setting& updateSetting = setting.second;
			set(updateSetting, updateSetting.value);
		}
	}
	return true;
}

//Singleton stuff
/*static*/Settings* Settings::fuckYouSingletonHack()
{
	return shittyHackSingleton;
}

/*static*/int Settings::increment(int setting)
{
	return setting + 1;
}

} // sl
