/**
 * @section DESCRIPTION
 * The part of a cell door the user can interact with and use to open/close
 * The physical part that stops people from walking through is, however, still
 * controlled through WallGrid to retain consistency with the other walls
 */
#ifndef SL_CELLDOOR_HPP
#define SL_CELLDOOR_HPP

#include "Engine/Entity.hpp"
#include "Engine/Serialization/DeserializeConstructor.hpp"

namespace sl
{

class WallGrid;

class CellDoor : public Entity
{
public:
	//DECLARE_SERIALIZABLE_METHODS(CellDoor)
	//CellDoor(DeserializeConstructor);
	/**
	 * @param tx The x location in TILES
	 * @param ty The y location in TILES
	 * @param grid The WallGrid this door exists in
	 */
	CellDoor(int tx, int ty, WallGrid& grid);

	bool isStatic() const override;

	const Type getType() const override;

	void open();

	void close();


private:

	void onUpdate() override;


	//! The WallGrid this door exists within
	WallGrid *grid;
	//! Whether the door is open or not
	bool opened;
	//! x position of the tile in grid this exists at
	int tx;
	//! y position of the tile in grid this exists at
	int ty;
};

} // sl

#endif // SL_CELLDOOR_HPP