#ifndef SL_DUNGEONCONSTRUCTIONGUI_HPP
#define SL_DUNGEONCONSTRUCTIONGUI_HPP

#include <SFML/Graphics/RectangleShape.hpp>

#include "Engine/Graphics/Texture.hpp"
#include "Engine/Graphics/TileGrid.hpp"
#include "Engine/GUI/WidgetHandle.hpp"
#include "Engine/GUI/WidgetList.hpp"
#include "Home/DungeonObjectDatabase.hpp"
#include "Home/WallGrid.hpp"
#include "Town/Generation/Interiors/HomeObjectDatabase.hpp"
#include "Town/Generation/Interiors/HomeTileDatabase.hpp"
#include "Utility/Grid.hpp"

#include <set>

namespace sl
{

class Dungeon;

namespace gui
{
class DrawableContainer;
class TextLabel;
} // gui

namespace towngen
{
class EditingUndo;
} // towngen

class DungeonConstructionGUI : public gui::GUIWidget
{
public:
	DungeonConstructionGUI(Dungeon& dungeon);


	void setViewPos(const Vector2f& pos);

private:
	enum class State
	{
		EditingLayout,
		EditingWallStyle,
		EditingFloorStyle,
		PlacingStuff,
		DesignatingRooms,
		TransferingInhabitants
	};

	void onUpdate(int x, int y) override;

	void onMouseHover(int x, int y) override;

	void beforeDraw(sf::RenderTarget& target) const override;

	void onSceneEnter() override;


	void initChangeStateBar();
	
	void onLeftClick(int x, int y);

	void onRightClick(int x, int y);

	void onLeftHold(int x, int y);

	void onRightHold(int x, int y);

	void registerStateChangeGUI(const char *str, State newState, int col);

	void changeState(State newState);

	void updateGridOverlay();

	void registerSubstateGUI(std::unique_ptr<GUIWidget> widget);

	void registerSubtateChangeGUI(const char *str, State newSubstate, int col, int row);

	void updatePlacementPreview();

	void showMessage(const char *str);
	enum class TransferSide
	{
		Left,
		Right
	};
	void transferNPC(TransferSide side, int npcIndex);

	void scrollRooms(TransferSide side, int offset);

	void createTransferGUI(TransferSide side);

	void addObject(int x, int y, const std::string& type, const json11::Json::object *instanceAttributes = nullptr);

	void addTile(int tx, int ty, const std::string& id);

	void addWall(int tx, int ty, const std::string& id);

	void removeWallIfRedundant(int tx, int ty);

	int snapToGrid(int x) const;

	int snapToHalfGrid(int x) const;

	Vector2i snapToHalfGrid(int x, int y);

	Vector2i screenToLevel(int x, int y) const;

	bool canPlaceObject(int mx, int my) const;



	State state;
	Dungeon &dungeon;

	mutable Grid<sf::RectangleShape> gridOverlay;

	std::vector<gui::GUIWidget*> substateWidgets;

	gui::TextLabel *popup;

	unsigned int rotation;

	int tileSize;

	int hackTimer;

	int currentlySelectedItemId;

	// transfer inhabitants info
	int leftRoomIndex;

	int rightRoomIndex;

	Rect<int> viewBox;


	/// ----------------------- Data used for exporting -----------------------
	//! A collection of Undo objects corresponding to all placed objects for the purpose of undoing them
	std::set<towngen::EditingUndo*> objects;
	//! Attributes that correspond to things in the objects member. Stuff is automatically removed from here by it too via undo()
	std::map<const towngen::EditingUndo*, json11::Json::object> instanceAttributes;
	//! All tiles placed indexed by their position (in tiles)
	std::map<Vector2i, std::string> tiles;
	//! All walls placed indexed by their position (in tiles)
	std::map<Vector2i, std::string> walls;
	//! The tile in the home that corresponds to the top left corner of the building exterior
	Vector2i homeOrigin;


	/// -------------- Data used for the GUI portion of the editor ---------------
	//! The currently selected object's type id
	std::string currentObject;
	//! The currently selected tile's type id
	std::string currentTile;
	//! The currently selected wall's type id
	std::string currentWall;
	//! The cursor which shows a transparent preview of what you're about to place
	gui::WidgetHandle<gui::DrawableContainer> objectPreview;
	//! Rectangular cursor to show where we're placing things
	gui::WidgetHandle<gui::DrawableContainer> cursor;
	//! cursor's graphical rectangle
	sf::RectangleShape *cursorRectShape;
};

} // sl

#endif // SL_DUNGEONCONSTRUCTIONGUI_HPP