#include "Home/DungeonObjectDatabase.hpp"

#include "Utility/Assert.hpp"
#include "Utility/JsonUtil.hpp"

namespace sl
{

DungeonObjectDatabase::Data::const_iterator DungeonObjectDatabase::begin() const
{
	return data.cbegin();
}

DungeonObjectDatabase::Data::const_iterator DungeonObjectDatabase::end() const
{
	return data.cend();
}

int DungeonObjectDatabase::size() const
{
	return data.size();
}

/*static*/DungeonObjectDatabase& DungeonObjectDatabase::get()
{
	static DungeonObjectDatabase instance;
	return instance;
}

// private
DungeonObjectDatabase::DungeonObjectDatabase()
{
	load();
}

void DungeonObjectDatabase::load()
{
	std::string err;
	json11::Json file = readJsonFile("data/gendata/dungeonobjects.json", err);
	if (!err.empty())
	{
		ERROR(err);
	}
	else
	{
		for (const json11::Json& entryJson : file.array_items())
		{
			const json11::Json::object& fields = entryJson.object_items();
			const std::string& name = fields.at("name").string_value();
			ASSERT(data.count(name) == 0);
			Entry& entry = data[name];
			entry.desc = fields.at("desc").string_value();
			entry.style[Entry::GIRLY] = fields.at("style_girly").number_value();
			entry.style[Entry::FANCY] = fields.at("style_fancy").number_value();
			entry.style[Entry::COMFY] = fields.at("style_comfy").number_value();
			entry.color = colorPrefFromString(fields.at("color").string_value());
			entry.cost = fields.at("cost").number_value();
			for (const json11::Json& rep : fields.at("objects").array_items())
			{
				entry.representations.push_back(rep.string_value());
			}
			ASSERT(!entry.representations.empty());
		}
	}

}

} // sl
