#include "Home/Internet/Hyperlink.hpp"

#include "Engine/GUI/TextLabel.hpp"
#include "Home/Internet/Internet.hpp"

namespace sl
{

Hyperlink::Hyperlink(Internet& internet, sf::Color onColour, sf::Color offColour)
	:GUIWidget(Rect<int>(0, 0, 0, 0))
	,linkComponents()
	,selected(false)
	,internet(internet)
	,onColour(onColour)
	,offColour(offColour)
	,textSize(12)
{
}

void Hyperlink::addLinkComponent(int x, int y, const std::string& text, const std::string& target)
{
	auto textLabel = unique<gui::TextLabel>(Rect<int>(x, y, 384, 0), text);
	textLabel->setColour(offColour);
	textLabel->extendVerticallyToFitText();
	// TODO: reenable
	//textLabel->registerMouseInputEvent(Key::GUIClick, GUIWidget::InputEventType::Pressed, std::bind(&Internet::visitPage, &internet, target));

	// if it's the first one, then this object takes on its dimensions
	if (linkComponents.empty())
	{
		move(x, y);
		setSize(textLabel->getBox().width, textLabel->getBox().height);
	}
	else
	{
		const Rect<int>& labelBox = textLabel->getBox();

	}

	linkComponents.push_back(textLabel.get());

	addWidget(std::move(textLabel));
}

void Hyperlink::onDraw(sf::RenderTarget& target) const
{
}

void Hyperlink::onUpdate(int x, int y)
{
}

void Hyperlink::onMouseHover(int x, int y)
{
	bool isMouseOn = false;
	for (gui::TextLabel *link : linkComponents)
	{
		if (link->getBox().containsPoint(x, y))
		{
			isMouseOn = true;
			break;
		}
	}
	if (isMouseOn && !selected)
	{
		for (gui::TextLabel *link : linkComponents)
		{
			link->setColour(onColour);
		}
	}
	else if (!isMouseOn && selected)
	{
		for (gui::TextLabel *link : linkComponents)
		{
			link->setColour(offColour);
		}
	}
	selected = isMouseOn;
}

void Hyperlink::onMouseLeave(int x, int y)
{
	for (gui::TextLabel *link : linkComponents)
	{
		link->setColour(onColour);
	}
	selected = false;
}

} // sl
