#ifndef SL_HYPERLINK_HPP
#define SL_HYPERLINK_HPP

#include <string>
#include <vector>

#include "Engine/GUI/GUIWidget.hpp"

namespace sl
{

class Internet;

namespace gui
{
class TextLabel;
} // gui

/**
 * This class is to represent the GUI button in webpages for hyperlinks
 * it is necessary over just a TextLabel + Button since links can spill
 * over multiple lines and must be composed over multiple of them to give
 * proper mouse-over behaviour
 */
class Hyperlink : public gui::GUIWidget
{
public:
	Hyperlink(Internet& internet, sf::Color onColour, sf::Color offColour);

	void addLinkComponent(int x, int y, const std::string& text, const std::string& target);

private:
	void onDraw(sf::RenderTarget& target) const override;

	void onUpdate(int x, int y) override;

	void onMouseHover(int x, int y) override;

	void onMouseLeave(int x, int y) override;



	//non-owning pointer. all children are actual GUIWidget children of this
	std::vector<gui::TextLabel*> linkComponents;
	bool selected;
	Internet& internet;
	sf::Color onColour;
	sf::Color offColour;
	int textSize;
};

} // sl

#endif // SL_HYPERLINK_HPP