#include "Home/PlayerBed.hpp"

#include "Engine/GUI/BasicMenuBuilder.hpp"
#include "Engine/GUI/NormalBackground.hpp"
#include "Engine/Game.hpp"
#include "Engine/Scene.hpp"
#include "Menus/PauseScene.hpp"
#include "Player/Player.hpp"
#include "Town/World.hpp"

namespace sl
{

PlayerBed::PlayerBed(int x, int y)
	:Entity(x - 4, y, 32 + 8, 24, "PlayerBed")
{
}

bool PlayerBed::isStatic() const
{
	return true;
}

const Entity::Type PlayerBed::getType() const
{
	return Entity::makeType("PlayerBed");
}

void PlayerBed::onUpdate()
{
	Player *player = scene->checkCollision<Player>(*this);
	if (player)
	{
		player->addActionCallout(Player::CalloutType::Use, "Advance Time", [this] {
			// to capture to get current scene
			auto base = unique<gui::GUIWidget>(Rect<int>(scene->getViewWidth(), scene->getViewHeight()));
			constexpr int width = 320;
			gui::BasicMenuBuilder builder(width, scene->getViewHeight(), 24, 32, 2);
			World *world = scene->getWorld();
			auto advanceAndQuit = [world, b = base.get()](int days) {
				return [world, days, b] {
					world->advanceDate(days, true);
					b->getScene()->destroy();
				};
			};
			builder.addButton("Advance 1 day", advanceAndQuit(1));
			builder.addButton("Advance 1 week", advanceAndQuit(7));
			builder.addButton("Advance 1 month", advanceAndQuit(31));
			builder.addButton("Advance 1 year", advanceAndQuit(365));
			builder.addBackButton([b = base.get()] { b->getScene()->destroy(); });
			
			auto built = builder.create();
			// center it
			const int horizBorder = (scene->getViewWidth() - built->getBox().width) / 2;
			const int vertBorder = (scene->getViewHeight() - built->getBox().height) / 2;
			built->move(horizBorder, vertBorder);
			built->update();
			
			base->addWidget(std::move(built));

			scene->getGame()->addScene(unique<PauseScene>(scene->getWorld(), std::move(base)));
		});
	}
}

// private
void PlayerBed::deserialize(Deserialize& in)
{
	deserializeEntity(in);
}

void PlayerBed::serialize(Serialize& out) const
{
	serializeEntity(out);
}

} // sl
