#ifndef SL_ROOM_HPP
#define SL_ROOM_HPP

#include <SFML/System/Vector2.hpp>

#include <vector>

#include "Home/RoomDoor.hpp"

namespace sl
{

class NPC;
class Scene;
class Deserialize;
class Serialize;

class Room
{
public:
	Room(RoomDoor& door);

	void addInhabitant(NPC& npc);

	void removeInhabitant(NPC& npc);

	const std::vector<NPC*>& getInhabitants() const;

	int getID() const;

	std::vector<Vector2i>& getPositions();

	const std::vector<Vector2i>& getPositions() const;

	// mutable access explicitly disallowed since we want to access open() / close() via wallgrid to update its tiles right.
	inline const RoomDoor& getDoor() const;

	void findInhabitants(Scene *scene, bool removeOldOnes = true);


	void deserialize(Deserialize& in);

	void serialize(Serialize& out) const;


	static int computeID(int x, int y);

private:

	std::vector<Vector2i> positions;
	std::vector<NPC*> inhabitants;
	// should this even exist to be stored or always computed?
	//int ID;
	RoomDoor& door;
};

const RoomDoor& Room::getDoor() const
{
	return door;
}

} // sl

#endif