#include "Home/RoomDoor.hpp"

#include "Engine/Scene.hpp"
#include "Home/CellDoor.hpp"
#include "Home/WallGrid.hpp" 

namespace sl
{

RoomDoor::RoomDoor(RoomDoor&& other)
	:position(other.position)
	,opened(other.opened)
	,orientation(other.orientation)
	,cellDoor(other.cellDoor)
{
	other.cellDoor = nullptr;
}

RoomDoor::RoomDoor(const Vector2i& position, Orientation orientation, WallGrid *grid)
	:position(position)
	,opened(false)
	,orientation(orientation)
	,cellDoor(nullptr)
{
	if (grid)
	{
		cellDoor = new CellDoor(x(), y(), *grid);
		grid->getScene().addEntityToList(cellDoor);
	}
}

RoomDoor& RoomDoor::operator=(RoomDoor&& rhs)
{
	position = rhs.position;
	opened = rhs.opened;
	orientation = rhs.orientation;
	std::swap(cellDoor, rhs.cellDoor);
	return *this;
}

RoomDoor::~RoomDoor()
{
	if (cellDoor)
	{
		cellDoor->destroy();
	}
}

unsigned int RoomDoor::x() const
{
	return position.x;
}

unsigned int RoomDoor::y() const
{
	return position.y;
}

bool RoomDoor::isOpen() const
{
	return opened;
}

RoomDoor::Orientation RoomDoor::getOrientation() const
{
	return orientation;
}

void RoomDoor::open()
{
	if (!opened)
	{
		opened = true;
		cellDoor->open();
	}
}

void RoomDoor::close()
{
	if (opened)
	{
		opened = false;
		cellDoor->close();
	}
}

} // sl