#ifndef SL_WALLGRID_HPP
#define SL_WALLGRID_HPP

#include <functional>
#include <string>
#include <vector>

#include "Engine/Graphics/Texture.hpp"
#include "Home/WallType.hpp"
#include "Home/FloorType.hpp"
#include "Pathfinding/PathGrid.hpp"
#include "Utility/Grid.hpp"


namespace sl
{

class StaticGeometry;
class Scene;
class Entity;
class TileGrid;
class RoomDoor;

class WallGrid : public pf::PathGrid
{
public:
	WallGrid(int x, int y, int width, int height, int tileWidth, int tileHeight, Scene& scene, std::function<const std::string&(int, int)> wallType);


	//---------------------  PathGrid  ---------------------
	bool isSolid(int x, int y) const override;

	bool canGoRight(int x, int y) const override;

	bool canGoLeft(int x, int y) const override;

	bool canGoUp(int x, int y) const override;

	bool canGoDown(int x, int y) const override;

	int getWidth() const override;

	int getHeight() const override;

	int getTileSize() const override;


	//---------------------  WallGrid  ---------------------
	void fill(int x, int y);

	void remove(int x, int y);

	void print() const;

	void openAllDoors();

	void closeAllDoors();

	bool openDoorAt(int x, int y);

	bool closeDoorAt(int x, int y);

	const RoomDoor* getDoorAt(int x, int y) const;

	void addDoor(RoomDoor& door);

	void removeDoor(RoomDoor& door);

	void updateAllTiles();

	void reset(int x, int y, int width, int height, int tileWidth, int tileHeight);

	Scene& getScene();

private:


	void updateTilesSurrounding(int x, int y);

	void updateTile(int x, int y);



	Grid<std::vector<Entity*>> wallEntities;
	Grid<bool> walls;
	std::function<const std::string&(int, int)> wallType;
	Grid<RoomDoor*> doorGrid;
	Scene& scene;
	int x;
	int y;
	int width;  // in tiles
	int height; // in tiles
	int tileWidth;
	int tileHeight;
};

} // sk

#endif // SL_WALLGRID_HPP
