#include "Items/Hotbar.hpp"

#include "Utility/Math.hpp"
#include "Utility/Assert.hpp"

namespace sl
{

Hotbar::Hotbar(Inventory& inventory)
	:inventory(inventory)
	,selectedIndex(0)
{
}

void Hotbar::addItem(Inventory::ItemRef item)
{
	ASSERT(equipped.size() < MAX_SIZE);
	equipped.push_back(item);
}

void Hotbar::removeItem(Inventory::ItemRef item)
{
	auto it = equipped.begin();
	for (int i = 0; i < equipped.size(); ++i, ++it)
	{
		if (*it == item)
		{
			// index might need fixing up
			if (i < selectedIndex)
			{
				// mod is fine since i < s  => s > 0
				--selectedIndex;
			}
			equipped.erase(it);
			return;
		}
	}
	ERROR("Item not found");
}

void Hotbar::cycleSelected(int offset)
{
	selectedIndex = mod(selectedIndex + offset, equipped.size());
}

const Inventory::ItemRef* Hotbar::getSelected(int offset) const
{
	if (std::abs(offset) >= (int)equipped.size() || equipped.size() == 0)
	{
		return nullptr;
	}
	return get(mod(selectedIndex + offset, equipped.size()));
}

const Inventory::ItemRef* Hotbar::get(int index) const
{
	ASSERT(index >= 0 && index < equipped.size());
	return &*std::next(equipped.cbegin(), index);
}

int Hotbar::getSize() const
{
	return equipped.size();
}

} // sl
