#include "Items/Inventory.hpp"

#include "Engine/Serialization/AutoDeserialize.hpp"
#include "Engine/Serialization/AutoSerialize.hpp"
#include "Engine/Serialization/Serialization.hpp"

namespace sl
{

Inventory::ItemRef Inventory::addItem(Item item)
{
	auto& list = items[item.getType()];
	list.push_back(std::move(item));
	return std::prev(list.end());
}

Item Inventory::removeItem(ItemRef item)
{
	Item ret(std::move(*item));
	auto it = items.find(item->getType());
	ASSERT(it != items.end());
	it->second.erase(item);
	return ret;
}

void Inventory::transferItem(ItemRef item, Inventory& destination)
{
	auto it = items.find(item->getType());
	ASSERT(it != items.end());
	destination.addItem(std::move(*item));
	it->second.erase(item);
}

void Inventory::queryItems(std::vector<ItemRef>& output, Item::Categories categories)
{
	for (auto& typeItems : items)
	{
		const Item::Categories itemCategories = Item::categories(typeItems.first);
		if ((itemCategories & categories) != 0)
		{
			for (auto item = typeItems.second.begin(), end = typeItems.second.end(); item != end; ++item)
			{
				output.push_back(item);
			}
		}
	}
}

void Inventory::queryItems(std::vector<ItemRef>& output, const Item::Type& type)
{
	const auto it = items.find(type);
	if (it != items.end())
	{
		for (auto item = it->second.begin(), end = it->second.end(); item != end; ++item)
		{
			output.push_back(item);
		}
	}
}

void Inventory::serialize(Serialize& out) const
{
	out.startObject("Inventory");
	AutoSerialize::serialize(out, items);
	out.endObject("Inventory");
}

void Inventory::deserialize(Deserialize& in)
{
	in.startObject("Inventory");
	AutoDeserialize::deserialize(in, items);
	in.endObject("Inventory");
}

std::vector<Item::Type> Inventory::getTypesContained() const
{
	std::vector<Item::Type> types;
	types.reserve(items.size());
	for (const auto& item : items)
	{
		types.push_back(item.first);
	}
	return types;
}

} // namespace sl
