#ifndef SL_INVENTORY_HPP
#define SL_INVENTORY_HPP

#include <initializer_list>
#include <list>
#include <map>
#include <memory>
#include <string>
#include <type_traits>
#include <vector>

#include "Items/Item.hpp"
#include "Utility/Assert.hpp"
#include "Utility/TypeToString.hpp"

namespace sl
{

class Deserialize;
class Serialize;

class Inventory
{
public:
	typedef std::list<Item>::iterator ItemRef;
	class ItemComp
	{
	public:
		bool operator()(const ItemRef& lhs, const ItemRef& rhs) const
		{
			return *lhs < *rhs;
		}
	};
	class ItemCompByType
	{
	public:
		bool operator()(const ItemRef& lhs, const ItemRef& rhs) const
		{
			return lhs->getType() < rhs->getType();
		}
	};

	ItemRef addItem(Item item);

	Item removeItem(ItemRef item);

	void transferItem(ItemRef item, Inventory& destination);

	void queryItems(std::vector<ItemRef>& output, Item::Categories categories = Item::AllCategories);

	// Remove this? Should this even be the responsibility of Inventory?
	//template <typename Container>
	//void queryItems(std::vector<ItemRef>& output, const Container& types)
	//{
	//	for (const Item::Type& type : types)
	//	{
	//		queryItems(output, type);
	//	}
	//}

	void queryItems(std::vector<ItemRef>& output, const Item::Type& type);



	void serialize(Serialize& out) const;

	void deserialize(Deserialize& in);

	std::vector<Item::Type> getTypesContained() const;

private:
	std::map<Item::Type, std::list<Item>> items;
};

} // namespace sl

#endif // SL_INVENTORY_HPP
