#include "Items/ItemPickup.hpp"

#include "Engine/Graphics/Sprite.hpp"
#include "Engine/Serialization/AutoDeserialize.hpp"
#include "Engine/Serialization/AutoSerialize.hpp"
#include "Engine/Serialization/Serialization.hpp"
#include "Items/ItemToDrawables.hpp"
#include "Utility/TypeToString.hpp"

namespace sl
{

DECLARE_TYPE_TO_STRING(ItemPickup)

ItemPickup::ItemPickup(DeserializeConstructor)
	:Entity(0, 0, 16, 16, "ItemPickup")
	,item(Item(0))
	,hover(0.f)
{
}

ItemPickup::ItemPickup(Vector2f pos, Item item)
	:Entity(pos.x, pos.y, 16, 16, "ItemPickup")
	,item(std::move(item))
	,hover(0.f)
{
	auto drawable = itemToDrawable(this->item.getType(), true);
	addDrawable(std::move(drawable));
}

const Item& ItemPickup::getItem() const
{
	return item;
}

const Entity::Type ItemPickup::getType() const
{
	return Entity::makeType("ItemPickup");
}

void ItemPickup::onUpdate()
{
	setHeight(2.f * (1.f + sinf(hover)));
	hover += 0.05f;
}

void ItemPickup::deserialize(Deserialize& in)
{
	AutoDeserialize::fields(in, item, hover);
}

void ItemPickup::serialize(Serialize& out) const
{
	AutoSerialize::fields(out, item, hover);
}

} // sl