#include "Items/ItemToDrawables.hpp"

#include "Engine/Graphics/Sprite.hpp"
#include "Engine/GUI/DrawableContainer.hpp"
#include "Engine/GUI/NormalBackground.hpp"

namespace sl
{

std::unique_ptr<Drawable> itemToDrawable(Item::Type type, bool center)
{
	auto sprite = unique<Sprite>(("items/" + Item::typeToStr(type) + ".png").c_str());
	if (center)
	{
		sprite->setOrigin(sprite->getCurrentBounds().size() / 2);
	}
	return sprite;
}

std::unique_ptr<gui::GUIWidget> itemToGUI(Item::Type type)
{
	auto drawable = itemToDrawable(type, false);
	const Rect<int> bounds = drawable->getCurrentBounds();
	return unique<gui::DrawableContainer>(bounds, std::move(drawable));
}

ItemToBoxedGUI::ItemToBoxedGUI(const Rect<int>& bounds)
	:bounds(bounds)
{
}

std::unique_ptr<gui::GUIWidget> ItemToBoxedGUI::operator()(Item::Type type) const
{
	auto drawable = itemToDrawable(type, false);
	drawable->update(bounds.left + (bounds.width - drawable->getCurrentBounds().width) / 2,
	                 bounds.top + (bounds.height - drawable->getCurrentBounds().height) / 2);
	auto container = unique<gui::DrawableContainer>(bounds, std::move(drawable));
	container->applyBackground(gui::NormalBackground(sf::Color::Black, sf::Color::White));
	return container;
}

} // sl