#ifndef SL_ITEMUSEACTION_HPP
#define SL_ITEMUSEACTION_HPP

#include <memory>
#include <vector>

namespace sl
{

class ItemUseTarget;
class NPC;
class Target;

class ItemUseAction
{
public:
	virtual ~ItemUseAction() {}

	bool getTargets(std::vector<NPC*> output, const ItemUseTarget& useTarget, const Target& target) const;

	/**
	 * Checks if an NPC is a viable target for this action. Returns true by default.
	 * @param npc The NPC to check validity for
	 * @return Whether {\ref npc} is a valid target for this action
	 */
	virtual bool canUse(const NPC& npc) const;

	enum class UseResult
	{
		Success,
		Fail,
		InProgress
	};
	virtual UseResult use(NPC& targetNPC) = 0;

protected:
	ItemUseAction() {}

private:
	ItemUseAction(ItemUseAction&) = delete;
	ItemUseAction& operator=(ItemUseAction&) = delete;
};

} // sl

#endif // SL_ITEMUSEACTION_HPP