#include "Menus/GameOver.hpp"

#include <functional>
#include <sstream>

#include "Engine/Game.hpp"
#include "Town/World.hpp"

#include "Engine/Graphics/Animation.hpp"
#include "Engine/Graphics/SFDrawable.hpp"
#include "Engine/Graphics/Sprite.hpp"
#include "Engine/GUI/BasicMenuBuilder.hpp"
#include "Engine/GUI/Button.hpp"
#include "Engine/GUI/DrawableContainer.hpp"
#include "Engine/GUI/Scrollbar.hpp"
#include "Engine/GUI/NormalBackground.hpp"
#include "Engine/GUI/WidgetGrid.hpp"
#include "Menus/GameOver.hpp"
#include "Menus/Main/MainMenu.hpp"
#include "NPC/LayeredSpriteLoader.hpp"
#include "NPC/NPC.hpp"
#include "NPC/SoulConjugator.hpp"
#include "Police/CrimeDatabase.hpp"
#include "Player/PlayerStats.hpp"
#include "Utility/Assert.hpp"
#include "Utility/Memory.hpp"

namespace sl
{

GameOver::GameOver(Game *game, World *world, const PlayerStats& playerStats)
	:AbstractScene(game, 640, 480, -2)
	,world(world)
	,playerStats(playerStats)
	,subMenu(SubMenu::Main)
{
	widget.setScene(this);

	changeSubMenu(subMenu);
}


/*			private			*/
const sf::Vector2f GameOver::getView() const
{
	return sf::Vector2f(getViewWidth() / 2, getViewHeight() / 2);
}

void GameOver::onUpdate(Timestamp timestamp)
{
	widget->update();

	deleteNextStep.reset(nullptr);
}

void GameOver::onDraw(RenderTarget& target)
{
}

void GameOver::changeSubMenu(SubMenu subMenu)
{
	this->subMenu = subMenu;
	widget.disable();
	deleteNextStep = widget.take();
	switch (subMenu)
	{
	case SubMenu::Main:
		{
			gui::BasicMenuBuilder builder(400, getViewHeight(), 16, 24);

			//builder.addButton("Crimes", std::bind(&GameOver::changeSubMenu, this, SubMenu::Crimes));
			builder.addButton("Main Menu", std::bind(&GameOver::exit, this));

			widget.setHandle(builder.create());
				
			const int horizBorder = (getViewWidth() - widget->getBox().width) / 2;
			const int vertBorder = (getViewHeight() - widget->getBox().height) / 2;
				
			widget->move(horizBorder, vertBorder);
			widget->update();
		}
		break;
	case SubMenu::Crimes:
		{
			const int menuWidth = 400;
			const int titleHeight = 32;
			const int infoHeight = 200;
			const int scrollHeight = 160;
			const int footerHeight = 32;
			const int menuHeight = titleHeight + infoHeight + scrollHeight + footerHeight;
			const int crimeListY = titleHeight + infoHeight;
			
			const gui::NormalBackground background(sf::Color::Black, sf::Color::White);

			auto crimeListBox = unique<gui::GUIWidget>(Rect<int>(menuWidth, menuHeight));
			crimeListBox->applyBackground(background);

			// TODO HACK DEMO renable game over menu
			//const CriminalRecords& criminalRecords = world->getPolice().getCriminalRecords();

			//std::map<Soul::ID, std::vector<Crime>> crimesByVictim = criminalRecords.getCrimesForMenu();

			//auto crimeList = unique<gui::WidgetGrid>(0, 0, 1, crimesByVictim.size());

			// TODO: use widgetlist here instead.


			// core preview

			//// scroll area
			//auto scrollbar = unique<gui::Scrollbar>(Rect<int>(0, crimeListY, menuWidth, scrollHeight));

			//// create victim boxes for the scroll area
			//int victimIndex = 0;
			//int sentenceInMonths = 0;
			//for (const auto& crimes : crimesByVictim)
			//{
			//	const int nameListHeight = 32;
			//	
			//	const Soul& soul = *world->getSoul(crimes.first);

			//	const int yOffset = victimIndex * nameListHeight;//crimeList->getBox().top;
			//	// name
			//	auto nameSlot = unique<gui::GUIWidget>(Rect<int>(0, yOffset, menuWidth - scrollbar->getScrollbarWidth(), nameListHeight));

			//	// preview image (part of name widget)
			//	auto npcPreview = unique<gui::DrawableContainer>(Rect<int>(0, yOffset, 32, 32));
			//	npcPreview->setDrawable(LayeredSpriteLoader::create(LayeredSpriteLoader::Config(soul.appearance, "walk", 32, 666, Vector2i(-4, -4)), "south"));
			//	nameSlot->addWidget(std::move(npcPreview));
			//	nameSlot->applyBackground(background);
			//	
			//	// count crimes to create list
			//	std::map<Crime::Type, int> crimeCount;
			//	for (const Crime& crime : crimes.second)
			//	{
			//		++crimeCount[crime.type];
			//	}
			//	std::stringstream crimeListString;
			//	bool first = true;
			//	for (const auto& crime : crimeCount)
			//	{
			//		const CrimeDatabase::Entry& crimeEntry = CrimeDatabase::get().getEntry(crime.first);
			//		if (!first)
			//		{
			//			crimeListString << ", ";
			//		}
			//		first = false;
			//		crimeListString << crimeEntry.name;
			//		if (crime.second > 1)
			//		{
			//			crimeListString << " x" << crime.second;
			//		}
			//		sentenceInMonths += crime.second * crimeEntry.sentenceLengthInMonths;
			//	}
			//	const Rect<int> nameBox(32, yOffset, menuWidth - scrollbar->getScrollbarWidth() - 32, nameListHeight);
			//	std::unique_ptr<gui::Button> nameButton = gui::Button::createGenericButton(nameBox, SoulConjugator(soul).fullName() + "\t - \t" + crimeListString.str(), [&](){
			//		//...
			//	}, 12);
			//	nameSlot->addWidget(std::move(nameButton));

			//	crimeList->setWidget(0, victimIndex++, std::move(nameSlot));
			//}

			//// header
			//std::stringstream gameOverTitle;
			//gameOverTitle << "Game Over";
			//const int years = sentenceInMonths / 12;
			//const int months = sentenceInMonths % 12;
			//if (years || months)
			//{
			//	gameOverTitle << " - Sentence:";
			//}
			//if (years)
			//{
			//	gameOverTitle << " " << years << " years";
			//}
			//if (months)
			//{
			//	gameOverTitle << " " << months << " months";
			//}
			//crimeListBox->addWidget(unique<gui::TextLabel>(Rect<int>(menuWidth, titleHeight), gameOverTitle.str(), gui::TextLabel::Config().centered()));
			//

			//// scroll area
			//scrollbar->setScrollArea(std::move(crimeList));
			//crimeListBox->addWidget(std::move(scrollbar));

			// footer
			const Rect<int> nameBox(0, menuHeight - footerHeight, menuWidth, footerHeight);
			std::unique_ptr<gui::Button> nameButton = gui::Button::createGenericButton(nameBox, "Exit to Main Menu", [&](){
				exit();
			}, 12);
			crimeListBox->addWidget(std::move(nameButton));


			widget.setHandle(std::move(crimeListBox));
				
			const int horizBorder = (getViewWidth() - widget->getBox().width) / 2;
			const int vertBorder = (getViewHeight() - widget->getBox().height) / 2;
				
			widget->move(horizBorder, vertBorder);
			widget->update();
		}
		break;
	default:
		ERROR("undefined state transition");
		break;
	}
	widget.enable();
}

void GameOver::exit()
{
	Game *game = getGame();

	game->addScene(unique<MainMenu>(game));

	world->destroy();
	destroy();
}

} // sl
