#include "Menus/Main/GalleryTest.hpp"

#include "Engine/GUI/DrawableContainer.hpp"
#include "Engine/GUI/NormalBackground.hpp"
#include "Engine/GUI/WidgetList.hpp"
#include "NPC/Stats/SoulGenerationSettings.hpp"
#include "NPC/Stats/Soul.hpp"
#include "NPC/Stats/NPCColorSwaps.hpp"
#include "NPC/AnimationDatabase.hpp"
#include "NPC/ClothesDatabase.hpp"
#include "NPC/LayeredSpriteLoader.hpp"
#include "Utility/Assert.hpp"
#include "Utility/Mapping.hpp"
#include "Utility/MapUtil.hpp"
#include "Utility/Random.hpp"
#include "Utility/Ranges.hpp"

#include <experimental/filesystem>

namespace sl
{
namespace gui
{

static constexpr int width = 512;
static constexpr int height = 384;
static constexpr int wborder = 8;
static constexpr int hborder = 8;
static constexpr int imageWidth = 256;
static constexpr int imageHeight = 192;
static constexpr int listWidth = (width - 2 * wborder - imageWidth) / 2;
static constexpr int listHeight = height - 2 * hborder;

static const std::string options[] = {
	"hair style",
	"hair color",
	"eyes",
	"skin",
	"age",
	"outfit",
	"outfit color",
	"accessory",
	//"accessory color",
	"panty",
	"panty color",
	"socks",
	"socks color",
	"bra",
	"bra color",
	"pose",
	"action",
	"face",
	"cum",
	"animation"
};

static const std::string ages[] = {
	"toddler",
	"kindergartner",
	"elementary schooler",
	"middle schooler"
};

static const std::string noFilesFoundInCurrentPose = "n/a";

static const std::vector<std::string> listDirectoriesAt(const std::string& path)
{
	namespace fs = std::experimental::filesystem;

	std::vector<std::string> directories;
	for (const fs::directory_entry& file : fs::directory_iterator(path))
	{
		if (fs::is_directory(file.path()))
		{
			directories.push_back(file.path().stem().string());
		}
	}
	return directories;
}

static const std::vector<std::string> listImagesAt(const std::string& path)
{
	namespace fs = std::experimental::filesystem;

	std::vector<std::string> images;
	for (const fs::directory_entry& file : fs::directory_iterator(path))
	{
		if (file.path().extension() == ".png")
		{
			images.push_back(file.path().stem().string());
		}
	}
	return images;
}


std::string identity(const std::string& s)
{
	return s;
}

// TODO: move somewhere common? not as much of a need I think?
static const std::vector<DNA::EyeColor> eyeColours = enumRangeInclusive(DNA::EyeColor::Brown, DNA::EyeColor::Black);
static const std::vector<DNA::HairColor> hairColours = enumRangeInclusive(DNA::HairColor::PlatinumBlonde, DNA::HairColor::LightRed);
static const std::vector<DNA::SkinColor> skinColours = enumRangeInclusive(DNA::SkinColor::Caucasian, DNA::SkinColor::DarkAsian);

GalleryTest::GalleryTest()
	:GUIWidget(Rect<int>(wborder, hborder, width, height))
	,npc()
	,sublist(nullptr)
	,picture(nullptr)
	,pose("talking")
	,action("default")
	,face("default")
	,animation(Framerate(5))
	,eyeColor(npc.expressedDNA.getEyeColor())
	,skinColor(npc.expressedDNA.getSkinColor())
	,hairColor(npc.expressedDNA.getHairColor())
{
	const Soul soul(Random::get(), SoulGenerationSettings());
	npc.init(soul);
	ClothesDatabase::get().dressLoliAppropriately(Random::get(), npc);
	// to avoid crash
	npc.setClothing(ClothingType::Bra, "training");

	reloadPoseList();
	reloadActionList();
	reloadFaceList();

	auto optionList = unique<WidgetList<std::string>>(Rect<int>(wborder, hborder, listWidth, listHeight), identity, std::bind(&GalleryTest::onOptionChange, this, std::placeholders::_1));
	optionList->updateItems(options);
	addWidget(std::move(optionList));

	ageMap[ages[0]] = 2 * 365;
	ageMap[ages[1]] = 6 * 365;
	ageMap[ages[2]] = 9 * 365;
	ageMap[ages[3]] = 13 * 365;

	clothing[ClothingType::Accessory].push_back("none");
	clothing[ClothingType::Outfit].push_back("none");
	clothing[ClothingType::Panties].push_back("none");
	clothing[ClothingType::Socks].push_back("none");
	clothing[ClothingType::Bra].push_back("none");
	for (const std::pair<std::string, ClothesDatabase::Info>& clothingDef : ClothesDatabase::get())
	{
		clothing[clothingDef.second.type].push_back(clothingDef.first);
	}

	onOptionChange("age");

	refreshPicture();
}

void GalleryTest::onOptionChange(const std::string& option)
{
	const Rect<int> listBox(wborder + listWidth, hborder, listWidth, listHeight);
	std::unique_ptr<GUIWidget> listWidget;

	auto clothingOptions = [&listWidget,&listBox,&option,this](ClothingType type, const std::string string)
	{
		if (option == string)
		{
			auto list = unique<WidgetList<std::string>>(listBox, identity, [type,this](const std::string& clothes) {
				if (clothes == "none")
				{
					npc.clothingLayers.fromEnum(type) = false;
				}
				else
				{
					npc.clothingLayers.fromEnum(type) = true;
					npc.setClothing(type, clothes);
				}
				refreshPicture();
			});
			list->updateItems(clothing.at(type));
			listWidget = std::move(list);
			return true;
		}
		else if (option == string + " color")
		{
			auto list = unique<WidgetList<std::string>>(listBox, identity, [type,this](const std::string& color) {
				npc.setClothingColour(type, color);
				refreshPicture();
			});
			const ClothesDatabase::Info& info = ClothesDatabase::get().getInfo(npc.getClothing(type));
			list->updateItems(getKeys(info.swaps));
			listWidget = std::move(list);
			return true;
		}
		return false;
	};

	if (option == "hair style")
	{
		reloadHairList();
		auto list = unique<WidgetList<std::string>>(listBox, identity, [&](const std::string& hairstyle) {
			npc.setHair(hairstyle);
			refreshPicture();
		});
		list->updateItems(hairstyles);
		listWidget = std::move(list);
	}
	else if (option == "hair color")
	{
		auto list = unique<WidgetList<DNA::HairColor>>(listBox, &DNA::getHairColorString, [&](DNA::HairColor hair) {
			hairColor = hair;
			npc.bodySwapID = toColorSwapID(skinColor) | toColorSwapID(eyeColor) | toColorSwapID(hairColor);
			npc.cachedHairSwapID = toColorSwapID(skinColor) | toColorSwapID(eyeColor) | toColorSwapID(hairColor);
			refreshPicture();
		});
		list->updateItems(hairColours);
		listWidget = std::move(list);
	}
	else if (option == "eyes")
	{
		auto list = unique<WidgetList<DNA::EyeColor>>(listBox, &DNA::getEyeColorString, [&](DNA::EyeColor eyes) {
			eyeColor = eyes;
			npc.bodySwapID = toColorSwapID(skinColor) | toColorSwapID(eyeColor) | toColorSwapID(hairColor);
			npc.cachedHairSwapID = toColorSwapID(skinColor) | toColorSwapID(eyeColor) | toColorSwapID(hairColor);
			refreshPicture();
		});
		list->updateItems(eyeColours);
		listWidget = std::move(list);
	}
	else if (option == "skin")
	{
		auto list = unique<WidgetList<DNA::SkinColor>>(listBox, &DNA::getSkinColorString, [&](DNA::SkinColor skin) {
			skinColor = skin;
			npc.bodySwapID = toColorSwapID(skinColor) | toColorSwapID(eyeColor) | toColorSwapID(hairColor);
			npc.cachedHairSwapID = toColorSwapID(skinColor) | toColorSwapID(eyeColor) | toColorSwapID(hairColor);
			refreshPicture();
		});
		list->updateItems(skinColours);
		listWidget = std::move(list);
	}
	else if (option == "age")
	{
		auto list = unique<WidgetList<std::string>>(listBox, identity, [&](const std::string& age) {
			npc.ageDays = ageMap.at(age);
			refreshPicture();
		});
		list->updateItems(ages);
		listWidget = std::move(list);
	}
	else if (clothingOptions(ClothingType::Outfit, "outfit"))
	{
	}
	else if (clothingOptions(ClothingType::Panties, "panty"))
	{
	}
	else if (clothingOptions(ClothingType::Socks, "socks"))
	{
	}
	else if (clothingOptions(ClothingType::Accessory, "accessory"))
	{
	}
	else if (clothingOptions(ClothingType::Bra, "bra"))
	{
	}
	else if (option == "pose")
	{
		reloadPoseList();
		auto list = unique<WidgetList<std::string>>(listBox, identity, [&](const std::string& pose) {
			this->pose = pose;
			refreshPicture();
		});
		list->updateItems(poses);
		listWidget = std::move(list);
	}
	else if (option == "action")
	{
		reloadActionList();
		auto list = unique<WidgetList<std::string>>(listBox, identity, [&](const std::string& action) {
			this->action = action;
			refreshPicture();
		});
		list->updateItems(actions);
		listWidget = std::move(list);
	}
	else if (option == "face")
	{
		reloadFaceList();
		auto list = unique<WidgetList<std::string>>(listBox, identity, [&](const std::string& face) {
			if (face != noFilesFoundInCurrentPose)
			{
				this->face = face;
				refreshPicture();
			}
		});
		list->updateItems(faces);
		listWidget = std::move(list);
	}
	else if (option == "cum")
	{
		reloadCumList();
		auto list = unique<WidgetList<std::string>>(listBox, identity, [&](const std::string& cum) {
			if (cum != noFilesFoundInCurrentPose)
			{
				if (npc.getCumLayers().count(cum) == 0)
				{
					npc.addCumLayer(cum);
				}
				else
				{
					npc.removeCumLayer(cum);
				}
			}
			refreshPicture();
		});
		list->updateItems(cumTypes);
		listWidget = std::move(list);
	}
	else if (option == "animation")
	{
		auto list = unique<WidgetList<std::string>>(listBox, identity, [&](const std::string& animation) {
			this->animation = animation == "default" ? FramerateSpec(5) : FramerateSpec(animation);
			refreshPicture();
		});
		auto animations = mapVec(AnimationDatabase::get().getEntries(), [](const std::pair<std::string, AnimationDatabase::Entry>& entry) {
			return entry.first;
		});
		animations.push_back("default");
		list->updateItems(animations);
		listWidget = std::move(list);
	}
	else
	{
		ERROR("invalid option:" + option);
	}
	if (sublist)
	{
		removeWidget(*sublist);
	}
	sublist = listWidget.get();
	addWidget(std::move(listWidget));
}

void GalleryTest::refreshPicture()
{
	const Rect<int> imageBounds(wborder + 2 * listWidth, hborder, imageWidth, imageHeight);
	;
	auto pictureWidget = unique<DrawableContainer>(
		imageBounds,
		LayeredSpriteLoader::createPortrait(
			LayeredSpriteLoader::Config(npc, pose, 256).animate(animation, true),
			action.c_str(),
			face.c_str()));
	pictureWidget->applyBackground(NormalBackground(sf::Color(154, 137, 211), sf::Color::Transparent));
	if (picture)
	{
		removeWidget(*picture);
	}
	picture = pictureWidget.get();
	addWidget(std::move(pictureWidget));
}

void GalleryTest::reloadPoseList()
{
	poses = listDirectoriesAt(makePath());
}

void GalleryTest::reloadActionList()
{
	// TODO: handle when the body doesn't have a different pose, but the others do?
	actions = listImagesAt(makePath() + pose + "\\body\\");
}

template <typename T>
static std::vector<T> merge(std::initializer_list<std::vector<T>> vectors)
{
	std::set<T> added;
	std::vector<T> result;
	for (const std::vector<T>& vec : vectors)
	{
		for (const T& elem : vec)
		{
			if (added.insert(elem).second)
			{
				result.push_back(elem);
			}
		}
	}
	return result;
}

void GalleryTest::reloadFaceList()
{
	auto newFaces = merge({
		listImagesAt(makePath() + pose + "\\face\\" + action + "\\"),
		listImagesAt(makePath() + pose + "\\face\\default\\")
	});
	faces = newFaces.empty() ? std::vector<std::string>({ noFilesFoundInCurrentPose }) : std::move(newFaces);
}

void GalleryTest::reloadHairList()
{
	hairstyles = listDirectoriesAt(makePath() + pose + "\\hair\\");
}

void GalleryTest::reloadCumList()
{
	auto newCumTypes = listDirectoriesAt(makePath() + pose + "\\cum\\");
	cumTypes = newCumTypes.empty() ? std::vector<std::string>({ noFilesFoundInCurrentPose }) : std::move(newCumTypes);
}

std::string GalleryTest::makePath() const
{
	// TODO: make this read those offset JSON files then use the proper path
	if (npc.getAgeYears() < 3.5f)
	{
		return "resources\\img\\" + npc.getResourcePath() + "\\popups\\";
	}
	// but for now just use elem since I think all the defaults are there
	return "resources\\img\\loli\\elem\\popups\\";
}

} // gui
} // sl