#include "Menus/Main/MainMenu.hpp"

#include <functional>

#include "Engine/Serialization/BinaryDeserialize.hpp"
#include "Engine/Game.hpp"
#include "Engine/Graphics/SFDrawable.hpp"
#include "Engine/GUI/BasicMenuBuilder.hpp"
#include "Engine/GUI/Button.hpp"
#include "Engine/GUI/DrawableContainer.hpp"
#include "Engine/GUI/NormalBackground.hpp"
#include "Engine/GUI/TextLabel.hpp"
#include "Engine/GUI/WidgetList.hpp"
#include "Menus/InputMenu.hpp"
#include "Menus/SettingChangeButtonCreator.hpp"
#ifdef SL_DEBUG
	#include "Menus/HomeEditor/HomeEditor.hpp"
#endif // SL_DEBUG
#include "Player/Fetishes.hpp"
#include "Player/PlayerGen.hpp"
#include "Town/Generation/TownGenerator.hpp"
#include "Town/Map/Map.hpp"
#include "Town/Map/MapData.hpp"
#include "Town/Town.hpp"
#include "Town/DeserializationAllocators.hpp"
#include "Utility/Assert.hpp"
#include "Utility/Memory.hpp"
#include "Utility/Ranges.hpp"

//	remove later:
#include "Home/BuildingLevel.hpp"
#include "Town/World.hpp"
#include "Engine/Door.hpp"
#include "Town/Cars/Car.hpp"

#include "Engine/GUI/Slider.hpp"


#include "Menus/Main/GalleryTest.hpp"


namespace sl
{

const int loadbarwidth = 320;
const int loadbarheight = 24;

const int mapWidth = 320;
const int mapHeight = 192;

const int tileSize = 16;

MainMenu::MainMenu(Game *game)
	:AbstractScene(game, game->getWindowWidth(), game->getWindowHeight(), -1)
	,subMenu(SubMenu::Main)
	,loadingText(nullptr)
	,loadingBar(nullptr)
	,minimap(nullptr)
	,generatedWorld()
{
	widget.setScene(this);

	changeSubMenu(SubMenu::Main);
}

/*				private					*/
const sf::Vector2f MainMenu::getView() const
{
	return sf::Vector2f(getViewWidth() / 2, getViewHeight() / 2);
}

void MainMenu::onUpdate(Timestamp timestamp)
{
	//GUIWidget& widget = *submenus[subMenu];
	widget->update();

	deleteNextStep.reset(nullptr);
}

void MainMenu::onDraw(RenderTarget& target)
{
}


void MainMenu::changeSubMenu(SubMenu subMenu)
{
	this->subMenu = subMenu;
	widget.disable();
	deleteNextStep = widget.take();
	widget.setHandle(unique<gui::GUIWidget>(getViewRect()));
	//auto it = submenus.find(subMenu);
	//if (it == submenus.end())
	{
		//WidgetHandle<GUIWidget>& newWidget = submenus[subMenu];
		switch (subMenu)
		{
		case SubMenu::Main:
			{
				const int titleHeight = 128;
				auto title = unique<gui::TextLabel>(Rect<int>(getViewWidth(), titleHeight), "SIM LOLI", gui::TextLabel::Config().centered().fontSize(10));
				widget->addWidget(std::move(title));

				gui::BasicMenuBuilder builder(320, getViewHeight() - titleHeight, 16, 32);

				builder.addButton("New Game", std::bind(&MainMenu::changeSubMenu, this, SubMenu::NewGame));
				builder.addButton("Load Game"/*"Skip To Dungeon"*/, [&]()
				{
					BinaryDeserialize in("save_game_test", deserializationAllocators(getGame()));
					generatedWorld = loadSerializable<World>(in, "root");
					start();
					// repurposed to a skip-to-debug-dungeon
					//destroy();
					//const int width = 640;
					//const int height = 480;
					//BuildingLevel *top = new BuildingLevel(getGame(), width, height, width, height, 0, 0);
					//BuildingLevel *basement = new BuildingLevel(getGame(), width, height, width, height, 0, 0);
					//BuildingLevel *dungeon = new BuildingLevel(getGame(), width * 4, height * 4, width, height, 0, 0);

					//World *world = new World(getGame(), 640, 480, GameTime(1996, 24, TimeOfDay(6, 0)));
					//world->setTimeRate(16.f);
					//world->addScene(top, true);
					//world->addScene(basement, true);
					//world->addScene(dungeon, true);
					//world->setCurrentScene(top);

					//top->loadUpperFloor(Random::get(), basement);
					//basement->loadBasement(Random::get(), top, dungeon);
					//dungeon->loadDungeon(Random::get(), basement);

					//getGame()->addScene(world);
				});
				builder.addButton("Debug Art Gallery", std::bind(&MainMenu::changeSubMenu, this, SubMenu::GalleryTest));
				builder.addButton("Options", std::bind(&MainMenu::changeSubMenu, this, SubMenu::Options));
				builder.addButton("Exit", std::bind(&MainMenu::exit, this));

				std::unique_ptr<gui::GUIWidget> menuButtons = builder.create();

				const int horizBorder = (getViewWidth() - menuButtons->getBox().width) / 2;
				const int vertBorder = (getViewHeight() - titleHeight - menuButtons->getBox().height) / 2;

				menuButtons->move(horizBorder, vertBorder + titleHeight);
				menuButtons->update();

				widget->addWidget(std::move(menuButtons));
				//newWidget.setHandle(std::move(widget));
			}
			break;
		case SubMenu::NewGame:
			{
				const int gap = 8;
				const int menuWidth = 320;
				gui::BasicMenuBuilder builder(menuWidth, getViewHeight(), gap, 24);

				builder.addButton("Generate", std::bind(&MainMenu::generate, this));
				builder.addButton("Start", std::bind(&MainMenu::start, this));
				builder.addButton("Player Settings", std::bind(&MainMenu::changeSubMenu, this, SubMenu::GenSettings));
				builder.addBackButton(std::bind(&MainMenu::changeSubMenu, this, SubMenu::Main));

				std::unique_ptr<gui::GUIWidget> menuButtons = builder.create();

				const int horizBorder = (getViewWidth() - menuButtons->getBox().width) / 2;
				const int mapHorizBorder = (menuWidth - mapWidth) / 2;

				int ySoFar = 8;

				auto loadingTextLabel = unique<gui::TextLabel>(Rect<int>(mapWidth, loadbarheight), gui::TextLabel::Config().centered());
				loadingText = loadingTextLabel.get();


				std::unique_ptr<gui::DrawableContainer> loadbarWidget = unique<gui::DrawableContainer>(Rect<int>(loadingTextLabel->getBox().width, loadingTextLabel->getBox().height));
				SFDrawable<sf::RectangleShape> *loadBarDrawable = new SFDrawable<sf::RectangleShape>(sf::RectangleShape());
				loadbarWidget->setDrawable(std::unique_ptr<Drawable>(loadBarDrawable));
				loadingBar = &loadBarDrawable->getData();
				loadingBar->setFillColor(sf::Color(0, 160, 0, 164));
				loadingBar->setOrigin(-1, 0);
				loadbarWidget->move(horizBorder + mapHorizBorder, ySoFar);
				loadbarWidget->applyBackground(gui::NormalBackground(sf::Color::Black, sf::Color::White));


				ySoFar += loadingTextLabel->getBox().height + gap;

				std::unique_ptr<gui::DrawableContainer> minimapWidget = unique<gui::DrawableContainer>(Rect<int>(horizBorder + mapHorizBorder - 1, ySoFar - 1, mapWidth + 2, mapHeight + 2));
				Grid<TileType> allDirt(mapWidth, mapHeight, TileType::Dirt);
				minimap = new map::Map(/*map::MapData(mapWidth, mapHeight, &allDirt, nullptr, nullptr), */tileSize, Vector2i(mapWidth, mapHeight), Vector2i(-1, -1));
				minimapWidget->setDrawable(std::unique_ptr<Drawable>(minimap));
				minimapWidget->applyBackground(gui::NormalBackground(sf::Color::Black, sf::Color::White));

				ySoFar += mapHeight + gap;

				menuButtons->move(horizBorder, ySoFar);
				menuButtons->update();

				loadbarWidget->addWidget(std::move(loadingTextLabel));
				widget->addWidget(std::move(menuButtons));
				widget->addWidget(std::move(loadbarWidget));
				widget->addWidget(std::move(minimapWidget));

				//widget->addWidget(unique<Slider>(Rect<int>(horizBorder, 100, 400, 24), 90, 100, 3));



				//newWidget.setHandle(std::move(widget));
			}
			break;
		case SubMenu::Options:
			{
				gui::BasicMenuBuilder builder(320, getViewHeight(), 16, 24);

				builder.addButton("Controls", std::bind(&MainMenu::changeSubMenu, this, SubMenu::Input));
				builder.addButton("Gamepad Config", std::bind(&MainMenu::changeSubMenu, this, SubMenu::GamepadConfig));
				builder.addButton("I am a sick fuck", std::bind(&MainMenu::changeSubMenu, this, SubMenu::Fetishes));
				Settings& settings = getGame()->getSettings();
				addSettingChangeButton(builder, settings, "metric");
				addSettingChangeButton(builder, settings, "res_scale");
				addSettingChangeButton(builder, settings, "blood_visible");
				addSettingChangeButton(builder, settings, "portraits_visible");
				addSettingChangeButton(builder, settings, "bind_first_use");
#ifdef SL_DEBUG
				//builder.addButton("Level Editor", [&]{
				//	destroy();
				//	getGame()->addScene(new hed::HomeEditor(getGame()));
				//});
#endif
				builder.addBackButton([&]{
					getGame()->getSettings().save();
					changeSubMenu(SubMenu::Main);
				});

				std::unique_ptr<gui::GUIWidget> menuButtons = builder.create();

				const int horizBorder = (getViewWidth() - menuButtons->getBox().width) / 2;
				const int vertBorder = (getViewHeight() - menuButtons->getBox().height) / 2;

				menuButtons->move(horizBorder, vertBorder);
				menuButtons->update();

				widget->addWidget(std::move(menuButtons));

				//newWidget.setHandle(std::move(widget));
			}
			break;
		case SubMenu::GalleryTest:
			{
				widget->addWidget(unique<gui::GalleryTest>());

				widget->addWidget(gui::Button::createGenericButton(Rect<int>(16+240, getViewHeight() - 32, getViewWidth() - 8-16-240, 24), "Back", std::bind(&MainMenu::changeSubMenu, this, SubMenu::Main)));
			}
			break;
		case SubMenu::Input:
			{
				widget->addWidget(gui::makeKeybindMenu(getGame()->getMutablePlayerControls(), std::bind(&MainMenu::changeSubMenu, this, SubMenu::Options)));
			}
			break;
		case SubMenu::GamepadConfig:
			{
				widget->addWidget(gui::makeGamepadConfigurationMenu(std::bind(&MainMenu::changeSubMenu, this, SubMenu::Options)));
			}
			break;
		case SubMenu::Fetishes:
			{
			gui::BasicMenuBuilder builder(320, getViewHeight(), 16, 24);

			addFetishChangeButtons(builder);
			builder.addBackButton([&] {
				changeSubMenu(SubMenu::Options);
			});

			std::unique_ptr<gui::GUIWidget> menuButtons = builder.create();

			const int horizBorder = (getViewWidth() - menuButtons->getBox().width) / 2;
			const int vertBorder = (getViewHeight() - menuButtons->getBox().height) / 2;

			menuButtons->move(horizBorder, vertBorder);
			menuButtons->update();

			widget->addWidget(std::move(menuButtons));
			}
			break;
		case SubMenu::GenSettings:
			{
				constexpr int listWidth = 320;
				constexpr int listHeight = 400;
				const int horizBorder = (getViewWidth() - listWidth) / 2;
				const int vertBorder = (getViewHeight() - listHeight) / 2;
				static const std::vector<DNADatabase::GenePool> pools = enumRangeInclusive(DNADatabase::GenePool::Caucasian, DNADatabase::GenePool::Arabic);
				auto poolList = unique<gui::WidgetList<DNADatabase::GenePool>>(Rect<int>(horizBorder, vertBorder, listWidth, listHeight), [](DNADatabase::GenePool pool) {
					switch (pool)
					{
					case DNADatabase::GenePool::Caucasian:
						return "Caucasian";
					case DNADatabase::GenePool::NordicCaucasian:
						return "Nordic";
					case DNADatabase::GenePool::MediterraneanCaucasian:
						return "Mediterranean";
					case DNADatabase::GenePool::Asian:
						return "East-Asian";
					case DNADatabase::GenePool::Hispanic:
						return "Hispanic";
					case DNADatabase::GenePool::Tropical:
						return "Pacific";
					case DNADatabase::GenePool::Nigger:
						return "African";
					case DNADatabase::GenePool::Indian:
						return "South-Asian";
					case DNADatabase::GenePool::SouthEastAsian:
						return "South-East-Asian";
					case DNADatabase::GenePool::Arabic:
						return "West-Asian";
					}
					ERROR("unhandled");
					return "???";
				}, [this](DNADatabase::GenePool pool) {
					playerGenPool = static_cast<int>(pool);
					changeSubMenu(SubMenu::NewGame);
				});
				poolList->updateItems(pools).trim();
				widget->addWidget(std::move(poolList));
			}
			break;
		default:
			ERROR("undefined state transition");
			break;
		}
		//newWidget.setScene(this);
		//newWidget.enable();
	}
	widget.enable();
}

void MainMenu::exit()
{
	getGame()->end();
}

void MainMenu::start()
{
	//ASSERT(generatedWorld);
	if (generatedWorld)
	{
		destroy();

		getGame()->addScene(std::move(generatedWorld));
	}
}

void MainMenu::generate()
{
	const int blocksWide = 8;
	const int blocksHigh = 7;
	const int tilesWidePerBlock = 48;
	const int tilesHighPerBlock = 40;
	const int tilesWide = tilesWidePerBlock * blocksWide;
	const int tilesHigh = tilesHighPerBlock * blocksHigh;

	// @SAVEMAP
	generatedWorld = unique<World>(getGame(), getViewWidth(), getViewHeight()/*tilesWide * 32, tilesHigh * 32*/, GameTime(2018, Month::December, 25, TimeOfDay(7, 00)));

	unsigned int seed = 0;//219471593;
	do
	{
		seed = Random::get().random(0xFFFFFFF);
	}
	while (seed == 0);
	Logger::log(Logger::TownGen, Logger::Debug) << "generating a town with seed = " << seed << std::endl;
	//seed = 210846990;
	Random rng(seed);
	getGame()->towngenSeed = seed;
	towngen::TownGenerator generator(blocksWide, blocksHigh, tilesWidePerBlock, tilesHighPerBlock, tileSize, rng);

	auto listener = std::bind(&MainMenu::generationListener, this, std::placeholders::_1, std::placeholders::_2, std::placeholders::_3);

	generator.generate(listener);

	generator.exportMap(*generatedWorld, listener);
}

void MainMenu::generationListener(int progress, const char *category, const map::MapData& minimapData)
{
	//std::cout << "[" << progress << "]  -  " << category << std::endl;

	loadingText->setString(category);
	loadingBar->setSize(sf::Vector2f((loadbarwidth - 1) * progress / 100, loadbarheight - 1));
	minimap->setData(minimapData);
	minimap->setPositionInMap(0.5f, 0.5f);

	getGame()->redraw();
}

} // sl
