#include "Menus/Pause/PauseMenu.hpp"

#include <functional>

#include "Engine/Game.hpp"
#include "Town/World.hpp"
#include "Engine/GUI/BasicMenuBuilder.hpp"
#include "Menus/Main/MainMenu.hpp"
#include "Menus/SettingChangeButtonCreator.hpp"
#include "Utility/Assert.hpp"
#include "Utility/Memory.hpp"

#include "NPC/NPC.hpp"

namespace sl
{

PauseMenu::PauseMenu(Game *game, World *world)
	:AbstractScene(game, game->getWindowWidth(), game->getWindowHeight(), -2)
	,world(world)
	,subMenu(SubMenu::Main)
{
	widget.setScene(this);

	changeSubMenu(subMenu);
}


/*			private			*/
const sf::Vector2f PauseMenu::getView() const
{
	return sf::Vector2f(getViewWidth() / 2, getViewHeight() / 2);
}

void PauseMenu::onUpdate(Timestamp timestamp)
{
	if (input.isKeyPressed(Input::KeyboardKey::Escape))
	{
		close();
	}

	widget->update();

	deleteNextStep.reset(nullptr);
}

void PauseMenu::onDraw(RenderTarget& target)
{
}

void PauseMenu::changeSubMenu(SubMenu subMenu)
{
	this->subMenu = subMenu;
	widget.disable();
	deleteNextStep = widget.take();
	switch (subMenu)
	{
	case SubMenu::Main:
		{
			gui::BasicMenuBuilder builder(400, getViewHeight(), 16, 24);

			builder.addButton("Options", std::bind(&PauseMenu::changeSubMenu, this, SubMenu::Options));
			builder.addButton("Save", [&]() {
				getGame()->scheduleSerialization("save_game_test", world);
			});
			builder.addButton("Main Menu", std::bind(&PauseMenu::exit, this));
			builder.addBackButton(std::bind(&PauseMenu::close, this));

			widget.setHandle(builder.create());
				
			const int horizBorder = (getViewWidth() - widget->getBox().width) / 2;
			const int vertBorder = (getViewHeight() - widget->getBox().height) / 2;
				
			widget->move(horizBorder, vertBorder);
			widget->update();
		}
		break;
	case SubMenu::Options:
		{
			gui::BasicMenuBuilder builder(400, getViewHeight(), 16, 24);

			Settings& settings = getGame()->getSettings();
			addSettingChangeButton(builder, settings, "metric");
			addSettingChangeButton(builder, settings, "res_scale");
			addSettingChangeButton(builder, settings, "blood_visible");
			addSettingChangeButton(builder, settings, "portraits_visible");
			addSettingChangeButton(builder, settings, "bind_first_use");
			builder.addBackButton(std::bind(&PauseMenu::changeSubMenu, this, SubMenu::Main));

			widget.setHandle(builder.create());
				
			const int horizBorder = (getViewWidth() - widget->getBox().width) / 2;
			const int vertBorder = (getViewHeight() - widget->getBox().height) / 2;
				
			widget->move(horizBorder, vertBorder);
			widget->update();
		}
		break;
	default:
		ERROR("undefined state transition");
		break;
	}
	widget.enable();
}

void PauseMenu::close()
{
	world->start();
	getGame()->removeScene(this);
}

void PauseMenu::exit()
{
	Game *game = getGame();

	game->addScene(unique<MainMenu>(game));

	world->destroy();
	destroy();
}

} // sl
