#include "Menus/PhotoViewer.hpp"

#include "Engine/GUI/DrawableContainer.hpp"
#include "Engine/GUI/NormalBackground.hpp"
#include "Player/Player.hpp"
#include "Town/World.hpp"
#include "Utility/DataToString.hpp"

namespace sl
{
namespace gui
{

static constexpr int border = 32; // the white border around a photo
static constexpr int width = 256 + 2*border;
static constexpr int height = 192 + 2*border;
static constexpr int x = (512 - width) / 2;
static constexpr int y = (384 - height) / 2;

PhotoViewer::PhotoViewer(Player& player)
	:GUIWidget(Rect<int>(x, y, width, height))
	,player(player)
	,index(0)
	,photoDrawable(nullptr)
{
	auto textLabel = std::make_unique<TextLabel>(Rect<int>(x, y + height - border, width, border), TextLabel::Config().centered());
	text = textLabel.get();
	addWidget(std::move(textLabel));
	text->setColour(sf::Color(64, 64, 64));

	applyBackground(gui::NormalBackground(sf::Color::White, sf::Color(64, 64, 64)));

	cyclePhotos(0); // to update everything
	registerInputEvent(Key::GUILeft, InputEventType::Pressed, [this] {
		cyclePhotos(-1);
	});
	registerInputEvent(Key::GUIRight, InputEventType::Pressed, [this] {
		cyclePhotos(1);
	});
	registerInputEvent(Key::GUIBack, InputEventType::Released, [this] {
		close();
	});
}

void PhotoViewer::onUpdate(int x, int y)
{

}

void PhotoViewer::close()
{
	this->getScene()->destroy();
}

void PhotoViewer::cyclePhotos(int cycle)
{
	auto& photos = player.getStats().stalkingData.getPhotos();
	if (!photos.empty())
	{
		index = (index + cycle) % photos.size();
		
		if (photoDrawable)
		{
			removeWidget(*photoDrawable);
		}
		auto photo = std::make_unique<DrawableContainer>(Rect<int>(x + border, y + border, 256, 192), photos[index].photo.makeDrawable());
		photoDrawable = photo.get();
		addWidget(std::move(photo));
		photoDrawable->applyBackground(gui::NormalBackground(photos[index].photo.background, sf::Color(64, 64, 64)));
		const Soul& subject = *player.getWorld()->getSoul(photos[index].id);
		std::stringstream ss;
		ss << subject.getFirstName() << " " << subject.getLastName();
		ss << " - " << formatTime(photos[index].time.time);
		ss << " - " << formatDateShort(photos[index].time.date);
		text->setString(ss.str());
	}
	else
	{
		text->setString("No photos available");
	}
}

} // gui
} // sl