#include "Menus/SettingChangeButtonCreator.hpp"

#include <sstream>

#include "Engine/Settings.hpp"
#include "Engine/GUI/BasicMenuBuilder.hpp"
#include "Engine/GUI/TextLabel.hpp"
#include "Player/Fetishes.hpp"

namespace sl
{

void addSettingChangeButton(gui::BasicMenuBuilder& builder, Settings& settings, const std::string& settingId)
{
	const Settings::Setting& setting = settings.getSetting(settingId);
	auto formatName = [&]() -> std::string
	{
		std::stringstream ss;
		ss << setting.properName << ": " << setting.name(setting.value);
		return ss.str();
	};
	builder.addDynamicButton(formatName(), [formatName, &setting, &settings, settingId](gui::TextLabel* textLabel)
	{
		const int newSetting = setting.value == setting.end() ? setting.begin() : setting.next(setting.value);
		settings.set(settingId, newSetting);
		textLabel->setString(formatName());
	});
}

void addFetishChangeButtons(gui::BasicMenuBuilder& builder)
{
	for (const std::pair<std::string, bool>& fetish : Fetishes::get().list())
	{
		static const auto formatName = [](const std::string& id) -> std::string
		{
			std::stringstream ss;
			ss << id << ": " << (Fetishes::get().isEnabled(id) ? "On" : "Off");
			return ss.str();
		};
		builder.addDynamicButton(formatName(fetish.first), [id = fetish.first](gui::TextLabel* textLabel)
		{
			Fetishes::get().set(id, !Fetishes::get().isEnabled(id));
			textLabel->setString(formatName(id));
		});
	}
}

} // sl
