#ifndef SL_ANIMATIONDATABASE_HPP
#define SL_ANIMATIONDATABASE_HPP

#include "Engine/Graphics/Framerate.hpp"
#include "NPC/NPCSpriteLayers.hpp"

#include <map>
#include <string>

namespace sl
{

class AnimationDatabase
{
public:
	class Entry
	{
	public:
		const Framerate& get() const
		{
			return base;
		}

		const Framerate& get(NPCSpriteLayer layer) const
		{
			auto it = overrides.find(layer);
			return it != overrides.end() ? it->second : base;
		}
	private:
		Entry()
			:base(0)
		{
		}

		Framerate base;
		std::map<NPCSpriteLayer, Framerate> overrides;

		friend AnimationDatabase;
	};
	const Entry* get(const std::string& id) const;

	const std::map<std::string, Entry>& getEntries() const;
	
	
	static const AnimationDatabase& get();

private:
	
	AnimationDatabase();
	
	std::map<std::string, Entry> entries;
};

} // sl

#endif // SL_ANIMATIONDATABASE_HPP