#include "NPC/Bed.hpp"

#include "Engine/Graphics/Sprite.hpp"
#include "Engine/Serialization/AutoDeserialize.hpp"
#include "Engine/Serialization/AutoSerialize.hpp"
#include "Utility/Random.hpp"
#include "Utility/TypeToString.hpp"

namespace sl
{

DECLARE_TYPE_TO_STRING(Bed)

Bed::Bed(const Rect<int>& bounds)
	:Entity(bounds.left, bounds.top, bounds.width, bounds.height, "Bed")
	,occupant(nullptr)
{
	loadEmptyDrawables();
}

const Types::Type Bed::getType() const
{
	return Entity::makeType("Bed");
}

void Bed::setOccupant(NPC *occupant)
{
	if (occupant)
	{
		loadInUseDrawables();
	}
	else
	{
		loadEmptyDrawables();
	}
	//ASSERT(this->occupant == nullptr);
	this->occupant = occupant;
}

NPC* Bed::getOccupant() const
{
	return occupant;
}

Vector2f Bed::getSleepPos() const
{
	return Vector2f(
		pos.x + getMask().getBoundingBox().width / 2,
		pos.y + getMask().getBoundingBox().height / 2);
}

void Bed::addExitPos(const Vector2f& exitPos)
{
	exitPositions.push_back(exitPos);
}

Vector2f Bed::getExitPos(Random& rng) const
{
	if (exitPositions.empty())
	{
		return pos + Vector2f(12, 12);
	}
	return rng.choose(exitPositions);
}

const std::vector<Vector2f>& Bed::getExitPositions() const
{
	return exitPositions;
}

// Serializable
void Bed::serialize(Serialize& out) const
{
	serializeEntity(out);
	SAVEREF(occupant);
	AutoSerialize::serialize(out, exitPositions);
}

void Bed::deserialize(Deserialize& in)
{
	deserializeEntity(in);
	LOADREF(occupant);
	AutoDeserialize::deserialize(in, exitPositions);
	setOccupant(occupant);
}

/*	  Entity private methods	  */
void Bed::onUpdate()
{
}

void Bed::loadInUseDrawables()
{
	//clearNonSetDrawables();
	//Sprite *sheets = new Sprite("furniture/bed_girly_inuse_sheets.png", 5, 9);
	//Sprite *bottom = new Sprite("furniture/bed_girly_inuse_bottom.png", 5, 9);
	//sheets->setDepth(-10);
	//bottom->setDepth(10);
	//addDrawable(std::unique_ptr<Drawable>(sheets));
	//addDrawable(std::unique_ptr<Drawable>(bottom));
}

void Bed::loadEmptyDrawables()
{
	//clearNonSetDrawables();
	//addDrawable(std::unique_ptr<Drawable>(new Sprite("furniture/bed_girly.png", 5, 9)));
}

} // sl
