/**
 * @section DESCRIPTION
 * Passivley responds to a request (via event system) for a conversation from the player.
 * This should be used in combination with an active node (ie ForgetfulSelector)
 * Returns SUCCESS when the conversation ends
 * Returns RUNNING when the conversation is active
 * Returns FAILURE if there is no request
 */
#ifndef SL_AI_ACCEPTTALKWITHPLAYER_HPP
#define SL_AI_ACCEPTTALKWITHPLAYER_HPP

#include <string>

#include "NPC/Behaviors/BehaviorTreeNode.hpp"
#include "Utility/Memory.hpp"

namespace sl
{
namespace ai
{

class AcceptTalkWithPlayer : public BehaviorTreeNode
{
public:
	enum class Type
	{
		Absolute, // Takes the startNode as-is and uses it as the node
		ContextLookup // looks up startNode as a string in context to get the actual node
	};
	AcceptTalkWithPlayer(std::string startNode = "begin", Type type = Type::Absolute);

	Result execute(BehaviorContext& context) override;

	void abort(BehaviorContext& context) override;

private:
	//! Whether the node is currently talking with the player.
	bool talking;
	//! Dialogue node id to start the dialogue at, or key to find it in context.
	std::string startNode;
	//! Action to execute after dialogue, if the dialogue commands us to.
	std::unique_ptr<BehaviorTreeNode> action;
	//! How to treat startNode to get the actual start node
	Type type;
};

} // ai
} // sl

#endif // SL_AI_ACCEPTTALKWITHPLAYER_HPP