#include "NPC/Behaviors/AlwaysSucceed.hpp"

#include "NPC/Behaviors/Not.hpp"
#include "Utility/Assert.hpp"

namespace sl
{
namespace ai
{

AlwaysSucceed::AlwaysSucceed(std::unique_ptr<BehaviorTreeNode> node, const char *debugText)
	:BehaviorTreeNode(debugText)
	,node(std::move(node))
{
#ifdef SL_DEBUG
	this->node->setParent(this);
#endif // SL_DEBUG
}

BehaviorTreeNode::Result AlwaysSucceed::execute(BehaviorContext& context)
{
#ifdef SL_DEBUG
	Result result = node->debugExecute(context);
#else // SL_DEBUG
	Result result = node->execute(context);
#endif // SL_DEBUG
	if (result != Result::Running)
	{
		return Result::Success;
	}
	return result;
}

void AlwaysSucceed::abort(BehaviorContext& context)
{
	node->abort(context);
}

std::unique_ptr<BehaviorTreeNode> neverSucceed(std::unique_ptr<BehaviorTreeNode> node)
{
	return std::make_unique<Not>(std::move(node));
}

} // ai
} // sl