#include "NPC/Behaviors/BehaviorTreeNode.hpp"

#ifdef SL_DEBUG
	#include "Engine/AbstractScene.hpp"
#endif // SL_DEBUG

namespace sl
{
namespace ai
{

BehaviorTreeNode::BehaviorTreeNode(const char *debugTypeName)
{
#ifdef SL_DEBUG
	type = debugTypeName;
	parent = nullptr;
	isResultValid = false;
#endif // SL_DEBUG
}

#ifdef SL_DEBUG
BehaviorTreeNode::Result BehaviorTreeNode::debugExecute(BehaviorContext& context)
{
	lastResult = execute(context);
	isResultValid = true;
	return lastResult;
}

void BehaviorTreeNode::drawAsTree(AbstractScene& scene, const Vector2f& pos, int width, const Vector2f& parPos) const
{
	if (width < 128) width = 128;
	for (int i = 0; i < children.size(); ++i)
	{
		const Vector2f childPos(pos.x - width / 2 + (i + 0.5f) * (width / children.size()), pos.y + 32);
		children[i]->drawAsTree(scene, childPos, width / children.size(), pos);
	}
	sf::Color col = sf::Color::Black; // invalid result - node hasn't been executed
	if (isResultValid)
	{
		switch (lastResult)
		{
		case Result::Success:
			col = sf::Color::Green;
			break;
		case Result::Failure:
			col = sf::Color::Red;
			break;
		case Result::Running:
			col = sf::Color::Yellow;
			break;
		}
	}
	if (parent)
	{
		scene.debugDrawLine("draw_btree", parPos, pos, col);
	}
	scene.debugDrawCircle("draw_btree", pos, 16.f, col);
	scene.debugDrawText("draw_btree", pos, type);
	isResultValid = false;
}

void BehaviorTreeNode::createDebugInfo(json11::Json::object& output) const
{
	json11::Json::object textObj;
	textObj["name"] = type;
	output["text"] = std::move(textObj);

	json11::Json::array childArray;
	for (const BehaviorTreeNode *child : children)
	{
		json11::Json::object childObj;
		child->createDebugInfo(childObj);
		childArray.push_back(std::move(childObj));
	}
	output["children"] = std::move(childArray);

	onCreateDebugInfo(output);
}

void BehaviorTreeNode::setParent(BehaviorTreeNode *p)
{
	p->children.push_back(this);
	parent = p;
}
#endif // SL_DEBUG

} // ai
} // sl
