#ifndef SL_AI_BEHAVIORTREENODE_HPP
#define SL_AI_BEHAVIORTREENODE_HPP

#ifdef SL_DEBUG
	#include <vector>
	#include "Utility/Vector.hpp"
	#include <json11/json11.hpp>
#endif // SL_DEBUG

namespace sl
{

#ifdef SL_DEBUG
class AbstractScene;
#endif // SL_DEBUG

namespace ai
{

class BehaviorContext;

class BehaviorTreeNode
{
public:
	virtual ~BehaviorTreeNode() {}


	enum class Result
	{
		Success,
		Failure,
		Running
	};

	virtual Result execute(BehaviorContext& context) = 0;

	/**
	 * Code to be called on a failure to do any clean-up/resetting
	 * This is to avoid weird behaviour if a parent node prematurely
	 * aborts an operation for whatever reason.
	 * If your node has any children, you almost definitely want to overload this
	 * and call it on some/all of those children.
	 */
	virtual void abort(BehaviorContext& context) {}

#ifdef SL_DEBUG
	Result debugExecute(BehaviorContext& context);

	void drawAsTree(AbstractScene& scene, const Vector2f& pos, int width, const Vector2f& parPos = Vector2f()) const;

	void createDebugInfo(json11::Json::object& output) const;

	virtual void onCreateDebugInfo(json11::Json::object& output) const {}

	void setParent(BehaviorTreeNode *p);
#endif // SL_DEBUG
protected:
	BehaviorTreeNode(const char *debugTypeName);

private:

#ifdef SL_DEBUG
	std::vector<const BehaviorTreeNode*> children;
	BehaviorTreeNode *parent;
	Result lastResult;
	const char *type;
	mutable bool isResultValid;
#endif // SL_DEBUG
};

} // ai
} // sl

#endif // SL_BEHAVIORTREENODE_HPP