#include "NPC/Behaviors/CanHear.hpp"

#include "NPC/Behaviors/BehaviorContext.hpp"
#include "NPC/Stealth/Noise.hpp"
#include "NPC/NPC.hpp"
#include "Engine/Scene.hpp"

#include <algorithm>

namespace sl
{
namespace ai
{

CanHear::CanHear(float concernLevel)
	:BehaviorTreeNode("CanHear")
	,concernLevel(concernLevel)
{
}

CanHear::CanHear(float concernLevel, const std::string& resultSaveLocation)
	:BehaviorTreeNode("CanHear")
	,concernLevel(concernLevel)
	,resultSaveLocation(resultSaveLocation)
{
}

BehaviorTreeNode::Result CanHear::execute(BehaviorContext& context)
{
	std::list<Noise*> noises;
	// @TODO implement circular collison masks so this will be proper
	context.getNPC().getScene()->cull(noises, context.getNPC().getMask().getBoundingBox());
	if (!noises.empty())
	{
		const float maxNoiseLevel = (*std::max_element(noises.cbegin(), noises.cend(),
			[](const Noise* a, const Noise *b) -> bool
		{
			return a->getLoudness() < b->getLoudness();
		}
		))->getLoudness();
		const float drownedOutThreshold = 0.6f * maxNoiseLevel; // can hear all sounds at least 60% as loud as the loudest one
		Noise *mostConcerning = nullptr;
		for (Noise *noise : noises)
		{
			if (noise->getLoudness() >= drownedOutThreshold &&
				noise->getConcernLevel() >= concernLevel &&
				(!mostConcerning || noise->getConcernLevel() > mostConcerning->getConcernLevel()))
			{
				mostConcerning = noise;
			}
		}
		if (mostConcerning)
		{
			if (!resultSaveLocation.empty())
			{
				context.setData<Target>(resultSaveLocation, Target(ScenePos(*mostConcerning)));
			}
			return BehaviorTreeNode::Result::Success;
		}
	}
	return BehaviorTreeNode::Result::Failure;
}


} // ai
} // sl