#include "NPC/Behaviors/CanSee.hpp"

#include "NPC/Behaviors/BehaviorContext.hpp"
#include "NPC/NPC.hpp"
#include "Engine/Scene.hpp"

#include <algorithm>

namespace sl
{
namespace ai
{

CanSee::CanSee(std::unique_ptr<MultiTargetLocator> locator, std::string resultSaveLocation, bool blocking)
	:BehaviorTreeNode("CanSee")
	,locator(std::move(locator))
	,resultSaveLocation(std::move(resultSaveLocation))
	,running(false)
	,blocking(blocking)
{
}

BehaviorTreeNode::Result CanSee::execute(BehaviorContext& context)
{
	if (!running)
	{
		targets = locator->findTargets(context);
	}
	Result result = Result::Failure;
	// any MultiTargetLocator should return targets in order of importance (ie closeness), so we can simply greedily run through and look in order
	for (const Target& target : targets)
	{
		if (target.isCalculating())
		{
			// why would you use a pathfinding target on a CanSee? Oh well
			result = Result::Running;
			if (blocking)
			{
				break;
			}
		}
		if (target && context.getNPC().canSee(target.getPos()))
		{
			if (!resultSaveLocation.empty())
			{
				context.setData(resultSaveLocation, target);
			}
			return Result::Success;
		}
	}
	if (result != Result::Running)
	{
		abort(context);
	}
	return result;
}

void CanSee::abort(BehaviorContext&)
{
	running = false;
	// TODO: cancel pathfing calculations if our targets were from that?
}


} // ai
} // sl