#include "NPC/Behaviors/Chance.hpp"

#include "NPC/Behaviors/BehaviorContext.hpp"
#include "Utility/Assert.hpp"

namespace sl
{
namespace ai
{

Chance::Chance(std::unique_ptr<BehaviorTreeNode> node, float probability)
	:BehaviorTreeNode("Chance")
	,node(std::move(node))
	,probabilityFunc([probability](BehaviorContext&) -> float { return probability; })
	,isComputing(false)
{
#ifdef SL_DEBUG
	this->node->setParent(this);
#endif // SL_DEBUG
}

Chance::Chance(std::unique_ptr<BehaviorTreeNode> node, std::function<float(BehaviorContext&)> probabilityFunc)
	:BehaviorTreeNode("Chance")
	,node(std::move(node))
	,probabilityFunc(std::move(probabilityFunc))
	,isComputing(false)
{
#ifdef SL_DEBUG
	this->node->setParent(this);
#endif // SL_DEBUG
}

BehaviorTreeNode::Result Chance::execute(BehaviorContext& context)
{
	if (!isComputing)
	{
		if (!context.rng().chance(probabilityFunc(context)))
		{
			abort(context);
			return Result::Failure;
		}
	}
#ifdef SL_DEBUG
	Result result = node->debugExecute(context);
#else // SL_DEBUG
	Result result = node->execute(context);
#endif // SL_DEBUG
	isComputing = (result == Result::Running);
	return result;
}

void Chance::abort(BehaviorContext& context)
{
	if (isComputing)
	{
		isComputing = false;
		node->abort(context);
	}
}


} // ai
} // sl