#include "NPC/Behaviors/Counter.hpp"

#include "NPC/Behaviors/BehaviorContext.hpp"

namespace sl
{
namespace ai
{

Counter::Counter(std::unique_ptr<BehaviorTreeNode> node, std::string counterName)
	:BehaviorTreeNode("Counter")
	,node(std::move(node))
	,counterName(std::move(counterName))
	,initialized(false)
{
#ifdef SL_DEBUG
	this->node->setParent(this);
#endif // SL_DEBUG
}

BehaviorTreeNode::Result Counter::execute(BehaviorContext& context)
{
	if (!initialized)
	{
		context.pushVariable(counterName, ls::Value(0));
		initialized = true;
	}
	const Result result =
#ifdef SL_DEBUG
		node->debugExecute(context);
#else // SL_DEBUG
		node->execute(context);
#endif // SL_DEBUG
	context.setVariable(counterName, context.getVariable(counterName) + ls::Value(1));
	if (result != Result::Running)
	{
		abort(context);
	}
	return result;
}

void Counter::abort(BehaviorContext& context)
{
	if (initialized)
	{
		initialized = false;
		context.popVariable(counterName);
		node->abort(context);
	}
}

} // ai
} // sl