#include "NPC/Behaviors/ForgetfulSelector.hpp"

#include "Utility/Assert.hpp"

namespace sl
{
namespace ai
{

ForgetfulSelector::ForgetfulSelector(const char *customName)
	:BehaviorTreeNode(customName)
	,children()
{
}

BehaviorTreeNode::Result ForgetfulSelector::execute(BehaviorContext& context)
{
	ASSERT(!children.empty());
	Result result = Result::Success;
	for (auto& child : children)
	{
#ifdef SL_DEBUG
		result = child->debugExecute(context);
#else // SL_DEBUG
		result = child->execute(context);
#endif // SL_DEBUG
		if (result != Result::Failure)
		{
			return result;
		}
	}
	return result;
}

void ForgetfulSelector::addChild(std::unique_ptr<BehaviorTreeNode> child)
{
#ifdef SL_DEBUG
	child->setParent(this);
#endif // SL_DEBUG
	children.push_back(std::move(child));
}

void ForgetfulSelector::abort(BehaviorContext& context)
{
	// is this the correct behaviour?
	for (auto& child : children)
	{
		child->abort(context);
	}
}

} // ai
} // sl