/**
 * @section DESCRIPTION
 * Actively goes up to the player and tries to initiate a conversation.
 * If the player does not respond in time, it will force the conversation.
 * Returns SUCCESS on conversation initiation
 * Returns RUNNING when on-route to (or waiting for a response from) the player
 * Returns FAILURE when player is unreachable
 */
#ifndef SL_AI_INITIATETALKWITHPLAYER_HPP
#define SL_AI_INITIATETALKWITHPLAYER_HPP

#include <string>

#include "NPC/Behaviors/BehaviorTreeNode.hpp"
#include "Utility/Memory.hpp"

namespace sl
{
namespace ai
{

class InitiateTalkWithPlayer : public BehaviorTreeNode
{
public:
	InitiateTalkWithPlayer(std::string startNode);

	Result execute(BehaviorContext& context) override;

	void abort(BehaviorContext& context) override;

private:
	std::string startNode;
	Result walkResult;
	int waitTimer;
	int timesToWait;
	std::unique_ptr<BehaviorTreeNode> walkToPlayer;
	bool talking;
	std::unique_ptr<BehaviorTreeNode> action;
};

} // ai
} // sl

#endif // SL_AI_INITIATETALKWITHPLAYER_HPP