#include "NPC/Behaviors/LoliBehaviors.hpp"

#include "Engine/Game.hpp"
#include "Items/ItemPickup.hpp"
#include "NPC/Behaviors/AcceptTalkWithPlayer.hpp"
#include "NPC/Behaviors/CanHear.hpp"
#include "NPC/Behaviors/CanSee.hpp"
#include "NPC/Behaviors/Chance.hpp"
#include "NPC/Behaviors/CheckTime.hpp"
#include "NPC/Behaviors/Chloroformed.hpp"
#include "NPC/Behaviors/CustomCondition.hpp"
#include "NPC/Behaviors/FixedResult.hpp"
#include "NPC/Behaviors/FollowSchedule.hpp"
#include "NPC/Behaviors/ForgetfulSelector.hpp"
#include "NPC/Behaviors/ForgetfulSequence.hpp"
#include "NPC/Behaviors/InitiateTalkWithPlayer.hpp"
#include "NPC/Behaviors/JoinActivity.hpp"
#include "NPC/Behaviors/MemoryNode.hpp"
#include "NPC/Behaviors/MovementBehaviors.hpp"
#include "NPC/Behaviors/Not.hpp"
#include "NPC/Behaviors/Parallel.hpp"
#include "NPC/Behaviors/PeeNode.hpp"
#include "NPC/Behaviors/RandomSelector.hpp"
#include "NPC/Behaviors/Selector.hpp"
#include "NPC/Behaviors/Sequence.hpp"
#include "NPC/Behaviors/SleepInBed.hpp"
#include "NPC/Behaviors/Util.hpp"
#include "NPC/Behaviors/UtilityNode.hpp"
#include "NPC/Behaviors/WalkTo.hpp"
#include "NPC/Behaviors/Wait.hpp"
#include "NPC/Behaviors/WalkTo.hpp"
#include "NPC/Behaviors/WeightMap.hpp"
#include "NPC/Behaviors/While.hpp"
#include "NPC/Locators/BehaviorContextStoredTargetLocator.hpp"
#include "NPC/Locators/FixedTargetLocator.hpp"
#include "NPC/Locators/PathfindingTargetLocator.hpp"
#include "NPC/Locators/DynamicTargetLocator.hpp"
#include "NPC/Locators/EntityMultiLocator.hpp"
#include "NPC/States/LoliRoutineState.hpp"
#include "NPC/States/SleepingState.hpp"
#include "NPC/Stealth/Suspicion.hpp"
#include "NPC/GroupAI/GameOfTag.hpp"
#include "NPC/NPC.hpp"
#include "NPC/Bed.hpp"
#include "NPC/Locators/EntityLocator.hpp"
#include "NPC/Stealth/Noise.hpp"
#include "Player/Player.hpp"
#include "Police/CrimeEntity.hpp"
#include "Town/Town.hpp"
#include "Town/World.hpp"
#include "Utility/Logger.hpp"
#include "Utility/Random.hpp"
#include "Utility/Trig.hpp"
#include "Utility/Math.hpp"
#include "Utility/Memory.hpp"

namespace sl
{
namespace ai
{

static constexpr int hours = 60 * 60;

static std::unique_ptr<BehaviorTreeNode> fleeScaryPlayerNode();

static std::unique_ptr<BehaviorTreeNode> hasUnreportedCrimesNode(bool recordId);

static std::unique_ptr<BehaviorTreeNode> lookDir(FacingDirection dir);

static std::unique_ptr<BehaviorTreeNode> tellAdultsCrimesNode();

static std::unique_ptr<BehaviorTreeNode> parkActivitiesNode();

static std::unique_ptr<BehaviorTreeNode> playTagNode();

static std::unique_ptr<BehaviorTreeNode> chaseCandy();

static std::unique_ptr<BehaviorTreeNode> noticeThings();

static float useToiletPriority(BehaviorContext& context);

static std::unique_ptr<BehaviorTreeNode> useToiletNode();

static std::unique_ptr<BehaviorTreeNode> wettingNode();

// ----------------------------------------------------------------------------
// ----------------------------- ROUTINE NODE ---------------------------------
std::unique_ptr<BehaviorTreeNode> routineNode()
{
	auto root = unique<ForgetfulSelector>();

	root->addChild(unique<Chloroformed>());

	root->addChild(noticeThings());

	// DEMO HACK
	//root->addChild(wettingNode());

	auto utilityRoot = unique<UtilityNode>();

	// TODO: base this off of her fear for the player, etc rather than just crimes
	auto witnessedCrimes = unique<Sequence>();
	{
		witnessedCrimes->addChild(hasUnreportedCrimesNode(true));
		witnessedCrimes->addChild(fleeScaryPlayerNode());
		witnessedCrimes->addChild(tellAdultsCrimesNode());
	}
	utilityRoot->addChild(1.f, unique<MemoryNode>(std::move(witnessedCrimes), MemoryNode::var("crimeId")));

	static const char *strNoiseToInvestigate = "noise_to_investigate";
	auto stareAt = unique<Sequence>("Investigate");
	{
		stareAt->addChild(unique<CanHear>(Noise::Level::concerning, strNoiseToInvestigate));
		//auto stare = [](BehaviorContext& context) -> BehaviorTreeNode::Result
		//{
		//	// @HACK
		//	context.getNPC().face(context.getNPC().getScene()->getPlayer()->getPos());
		//	return BehaviorTreeNode::Result::Success;
		//};
		//stareAt->addChild(unique<CustomCondition>(stare));
		//stareAt->addChild(unique<Wait>(30));
		stareAt->addChild(unique<WalkTo>(unique<BehaviorContextStoredTargetLocator>(strNoiseToInvestigate)));
	}
	// TODO: base off of the noise's concern level
	utilityRoot->addChild(0.3f, std::make_unique<MemoryNode>(std::move(stareAt), MemoryNode::var(strNoiseToInvestigate)));

	utilityRoot->addChild(0.35f, chaseCandy());

	utilityRoot->addChild(10.45f, unique<AcceptTalkWithPlayer>("begin"));

	// DEMO HACK
	//utilityRoot->addChild(useToiletPriority, useToiletNode());

	// TODO: comment this out and enable the below code.
	//auto dailyRoutine = unique<Selector>();
	//{
	//	auto schoolRoutine = unique<Sequence>();
	//	{
	//		schoolRoutine->addChild(unique<CheckTime>(RecurringTimeInterval(TimeOfDay(7, 30), hours * 7, RecurringTimeInterval::Weekdays)));

	//		schoolRoutine->addChild(goToTownNode());

	//		schoolRoutine->addChild(unique<WalkTo>(unique<DynamicTargetLocator>("school_desk")));

	//		schoolRoutine->addChild(unique<While>(unique<CheckTime>(RecurringTimeInterval(TimeOfDay(7, 30), hours * 7, RecurringTimeInterval::Weekdays))));
	//	}
	//	dailyRoutine->addChild(std::move(schoolRoutine));

	//	auto parkRoutine = unique<Sequence>();
	//	{
	//		const RecurringTimeInterval parkTime(TimeOfDay(14, 30), hours * 5, RecurringTimeInterval::Weekdays);

	//		parkRoutine->addChild(unique<CheckTime>(parkTime));

	//		parkRoutine->addChild(goToTownNode());

	//		parkRoutine->addChild(unique<WalkTo>(unique<EntityLocator>("ParkMarker")));
	//		// @todo make them walk to the park with the min distance between home/school

	//		parkRoutine->addChild(While::make(unique<CheckTime>(parkTime), parkActivitiesNode()));
	//	}
	//	dailyRoutine->addChild(std::move(parkRoutine));

	//	auto homeRoutine = unique<Sequence>();
	//	{
	//		homeRoutine->addChild(unique<Not>(unique<CheckTime>(RecurringTimeInterval(TimeOfDay(7, 30), hours * 12, RecurringTimeInterval::Weekdays))));

	//		homeRoutine->addChild(unique<WalkTo>(unique<DynamicTargetLocator>("bedroom")));

	//		homeRoutine->addChild(unique<CustomCondition>([](BehaviorContext& context) -> BehaviorTreeNode::Result
	//		{
	//			NPC& npc = context.getNPC();
	//			GameTime wakeupTime = context.getGameTime();
	//			TimeOfDay sevenThirtyAm(7, 30);
	//			if (wakeupTime.time.getTotalSeconds() > sevenThirtyAm.getTotalSeconds())
	//			{
	//				wakeupTime.date.advance(1);
	//			}
	//			wakeupTime.time = sevenThirtyAm;
	//			npc.changeState(unique<SleepingState>(npc, wakeupTime, static_cast<Bed*>(npc.getSoul()->getKnownEntity("bed"))));
	//			return BehaviorTreeNode::Result::Success;
	//		}, "GetInBed"));
	//		// this tree will end here since moving to the SleepingState will switch states and thus stop using this tree
	//	}
	//	dailyRoutine->addChild(std::move(homeRoutine));
	//}
	//utilityRoot->addChild(0.15f, std::move(dailyRoutine));
	// TODO: reenable this and comment out the above.
	auto schedule = std::make_unique<FollowSchedule>();
	{
		// TODO: bathroom breaks, etc
		schedule->addChild("school", makebt<Sequence>(
			unique<WalkTo>(unique<DynamicTargetLocator>("school_desk")),
			unique<CustomCondition>([](BehaviorContext& context) {
				NPC& npc = context.getNPC();
				const Entity *desk = context.getSoul().getKnownEntity("school_desk");
				const Vector2f sittingPos = desk->getPos() + Vector2f(8, 12);
				const float sittingDist = (sittingPos - npc.getPos()).length();
				bool sitting = sittingDist < 2;
				if (!sitting && sittingDist < 16)
				{
					npc.setPos(sittingPos);
					sitting = true;
				}
				if (sitting)
				{
					npc.face(North);
					if (context.rng().chance(1.f / (30 * 30)))
					{
						npc.makeTextBubble(
							context.rng().choose(
								"When are we ever gonna use this?",
								"This is boring...",
								"Wow, cool, teacher!",
								"I can't wait until recess!",
								"So that's how babies are made..?",
								"How long until lunch break?",
								"I'm gonna be a scientist when I grow up!",
								"How do you solve for x?",
								"What's an integral?",
								"Why does sin^2(x)+cos^2(x) = 1?",
								"Why do we always have to find x?",
								"I wonder how big Teacher's dick is...",
								"Oh, so then X(G) = theta(G)?"));
					}
					return BehaviorTreeNode::Result::Running;
				}
				return BehaviorTreeNode::Result::Failure;
			})));

		schedule->addChild("play_in_park", makebt<Sequence>(
			unique<WalkTo>(unique<EntityLocator>("ParkMarker")),
			parkActivitiesNode()));

		schedule->addChild("sleep", unique<SleepInBed>());
	}
	utilityRoot->addChild(FollowSchedule::getPriority, std::move(schedule));

	root->addChild(std::move(utilityRoot));

	return std::move(root);
}


// ----------------------------------------------------------------------------
// ----------------------------- CAPTIVE NODE ---------------------------------
std::unique_ptr<BehaviorTreeNode> captiveNode()
{
	auto root = unique<Selector>();

	root->addChild(unique<Chloroformed>());

	auto utilityRoot = unique<UtilityNode>();
	// DEMO HACK real probability?
	utilityRoot->addChild(13376662342.f, unique<AcceptTalkWithPlayer>("captivity/begin"));
	
	// Sleep
	auto sleep = unique<Sequence>();
	{
		//auto findBed = unique<EntityLocator>("Bed");
		//findBed->filter([](Entity* e) {
		//	return static_cast<const Bed*>(e)->getOccupant() == nullptr;
		//});
		//sleep->addChild(unique<WalkTo>(std::move(findBed), "found_bed"));
		sleep->addChild(unique<WalkTo>(unique<PathfindingTargetLocator>(
		[](const pf::NodeID& node, ai::BehaviorContext *context) -> bool {
			LOCK_NPC_OR_ABORT;
			if (node.getScene() == npc->getScene())
			{
				const int t = node.getScene()->getTileSize();
				std::list<Bed*> beds;
				node.getScene()->cull(beds, Rect<int>((node.getX() + 0.25f) * t, (node.getY() + 0.25f) * t, t * 0.5f, t * 0.5f));
				for (Bed *bed : beds)
				{
					if (bed->getOccupant() == nullptr)
					{
						context->setVariable("found_bed", ls::Value(bed));
						return true;
					}
				}
			}
			return false;
		})));
		sleep->addChild(unique<SleepInBed>("found_bed"));
	}
	utilityRoot->addChild(
		[](BehaviorContext& context) {
			return (context.getGameTime().time < TimeOfDay(7, 30) || TimeOfDay(23, 0) < context.getGameTime().time)
				? 10.f
				: 0.f;
		},
		unique<MemoryNode>(std::move(sleep), MemoryNode::var("found_bed", ls::Value(nullptr)))
	);

	// Escape Sequence
	auto escapeSequence = unique<Sequence>();
	{
		// run home and report to mommy/daddy
		escapeSequence->addChild(unique<WalkTo>(unique<DynamicTargetLocator>("bedroom")));
		escapeSequence->addChild(unique<CustomCondition>([](BehaviorContext& context) -> BehaviorTreeNode::Result
		{
			NPC& npc = context.getNPC();
			for (const int crimeId : npc.getSoul()->getUnreportedCrimes())
			{
				PoliceForce& police = npc.getScene()->getWorld()->getPolice();
				const Crime *crime = police.getCriminalRecords().getCrime(crimeId);
				police.reportCrime(crimeId);
			}
			// go back to normal routine as if nothing happened...
			npc.changeState(unique<LoliRoutineState>(npc));
			return BehaviorTreeNode::Result::Success;
		}, "Report"));
	}
	utilityRoot->addChild(
		[](BehaviorContext& context) -> float {
			// TODO: re-enable + implement some timer on escapes
			//if (timeSinceLastEscape < someThreshold)
			{
				return 0.f;
			}
			Soul& npc = *context.getNPC().getSoul();
			// low npc fear -> don't fear enough to leave
			// high npc fear -> too scared to leave
			const float fearFactor = 2.f * (0.5f - std::abs(npc.getFear() - 0.5f));
			const float loveFactor = (1.f - npc.getLove()) / 2.f;
			const float hopeFactor = 1.f - npc.getHope();
			const float desireToLeave = fearFactor * loveFactor * hopeFactor;
			const float leaveChance = 2.f * std::max(0.f, desireToLeave - 0.5f);
			return leaveChance;
		},
		std::move(escapeSequence)
	);

	auto tryEat = unique<Sequence>();
	{
		// if master is around, look for him and ask for food
		tryEat->addChild(unique<InitiateTalkWithPlayer>("captivity/ask_for_food"));
		// else see if there is food in the cell, if so eat it
	}
	utilityRoot->addChild(
		[](BehaviorContext& context) -> float {
			Soul& npc = *context.getNPC().getSoul();
			// at what point a loli should seek food
			const float hungerThreshold = 0.5f;
			// then linearly scale range from hungerThreshold -> 0.0, to 1.0 -> 1.0
			return std::max(0.f, (npc.getStatus().hunger - hungerThreshold) / (1.f - hungerThreshold));
		},
		std::move(tryEat)
	);

	// wander cell
	auto wanderCell = std::make_unique<Sequence>();
	{
		wanderCell->addChild(unique<WalkTo>(unique<PathfindingTargetLocator>(
			[&](const pf::NodeID& node, ai::BehaviorContext*) -> bool
		{
			return Random::get().chance(0.1f);
		})));
		wanderCell->addChild(unique<Wait>(30, 90));
	}
	utilityRoot->addChild(0.2f, std::move(wanderCell));

	// run away
	utilityRoot->addChild(
		[](BehaviorContext& context) -> float {
			return 1.f - context.getSoul().getWillingness();
		},
		fleeScaryPlayerNode());

	root->addChild(std::move(utilityRoot));

	return std::move(root);
}


// ----------------------------------------------------------------------------
// ------------------------------- TEST NODE ----------------------------------
std::unique_ptr<BehaviorTreeNode> testNode()
{
	auto root = unique<ForgetfulSelector>();

	root->addChild(unique<Chloroformed>());

	root->addChild(unique<AcceptTalkWithPlayer>("captivity/begin"));


	//root->addChild(unique<WalkTo>(unique<EntityLocator>("Player")));


	auto topLevelPassive = unique<Selector>("Passive");
	{
		//auto trickOrTreat = unique<Sequence>();
		//{
		//	trickOrTreat->addChild(goToTownNode());
		//	trickOrTreat->addChild(unique<WalkTo>(unique<PathfindingTargetLocator>(
		//		[](const pf::NodeID& node, BehaviorContext *context) -> bool
		//	{
		//		LOCK_NPC_OR_ABORT;
		//		if (node.getScene()->getType() != "Town")
		//		{
		//			return false;
		//		}
		//		const int ts = node.getScene()->getTileSize();
		//		Entity *door = node.getScene()->checkCollision(Rect<int>(node.getX()*ts, node.getY()*ts, ts, ts), "Door");
		//		if (!door)
		//		{
		//			return false;
		//		}
		//		Soul& soul = *npc->getSoul();
		//		const std::string ptrAsStr = std::to_string((int)door);
		//		if (soul.playerKnows(ptrAsStr))
		//		{
		//			return false;
		//		}
		//		soul.setPlayerKnowledge(ptrAsStr); // @HACK
		//		return true;
		//	}, pf::defaultWeight)));
		//	trickOrTreat->addChild(unique<Wait>(30));
		//	trickOrTreat->addChild(unique<CustomCondition>([](BehaviorContext& context) {
		//		context.getNPC().makeTextBubble("Trick or treat!");
		//		return BehaviorTreeNode::Result::Success;
		//	}));
		//	trickOrTreat->addChild(unique<Wait>(90));
		//	trickOrTreat->addChild(unique<CustomCondition>([](BehaviorContext& context) {
		//		context.getNPC().makeTextBubble(context.rng().choose({ "Yummy!", "Thanks!" }));
		//		return BehaviorTreeNode::Result::Success;
		//	}));
		//}
		//topLevelPassive->addChild(std::move(trickOrTreat));

		//auto playInPark = unique<Sequence>("PlayInPark");
		//{
		//	playInPark->addChild(goToTownNode());
		//	playInPark->addChild(unique<WalkTo>(unique<EntityLocator>("ParkMarker")));
		//	playInPark->addChild(parkActivitiesNode());
		//}
		//topLevelPassive->addChild(std::move(playInPark));
	}
	//root->addChild(std::move(topLevelPassive));

	return std::move(root);
}



// ----------------------------------------------------------------------------
// ------------------------------ MISC NODES ----------------------------------

std::unique_ptr<BehaviorTreeNode> fleeScaryPlayerNode()
{
	auto root = unique<Parallel>();

	auto runAwayTarget = unique<PathfindingTargetLocator>([](const pf::NodeID& node, ai::BehaviorContext *context) -> bool {
		LOCK_NPC_OR_ABORT;
		return !context->getPlayer() || worldDistance(*context->getPlayer(), ScenePos(*node.getScene(), node.scenePos())) > 9; // 9 tiles, since world pos
	}, pf::noHeuristic, "flee_player");
	root->addChild(unique<WalkTo>(std::move(runAwayTarget)));

	
	root->addIgnoredChild(unique<CustomCondition>([](BehaviorContext& context) {
		if (context.rng().chance(1.f / (60.f + context.getScene()->instanceCount("NPCTextBox"))))
		{
			context.getNPC().makeTextBubble(context.rng().choose({ "Get away!", "Aaaah!", "Aaaa!", "Help!", "Noo!" }));
		}
		return true;
	}), true);

	return root;
}

std::unique_ptr<BehaviorTreeNode> hasUnreportedCrimesNode(bool recordId)
{
	return unique<CustomCondition>([recordId](BehaviorContext& context) {
		for (int id : context.getNPC().getSoul()->getUnreportedCrimes())
		{
			if (!context.getWorld()->getPolice().getCriminalRecords().getCrime(id)->consensual)
			{
				if (recordId)
				{
					context.setVariable("crimeId", ls::Value(id));
				}
				return true;
			}
		}
		return false;
	});
}

// TODO: move?
std::unique_ptr<BehaviorTreeNode> lookDir(FacingDirection dir)
{
	return unique<CustomCondition>([dir](BehaviorContext& context) {
		context.getNPC().face(dir);
		return BehaviorTreeNode::Result::Success;
	});
}

std::unique_ptr<BehaviorTreeNode> tellAdultsCrimesNode()
{
	auto root = unique<Sequence>();

	root->addChild(hasUnreportedCrimesNode(false));

	auto nearestAdult = unique<EntityLocator>("NPC");
	nearestAdult->filter([](Entity *entity) {
		NPC *npc = static_cast<NPC*>(entity);
		return npc->getSoul()->getType() != "loli";
	});
	root->addChild(unique<WalkTo>(std::move(nearestAdult)));

	auto tellCrimes = unique<Sequence>();
	{
		tellCrimes->addChild(unique<CustomCondition>([](BehaviorContext &context) {
			NPC& npc = context.getNPC();
			const Crime *crime = context.getWorld()->getPolice().getCriminalRecords().getCrime(context.getVariable("crimeId").asInt());
			if (context.getWorld()->getPolice().reportCrime(crime->id) != nullptr)
			{
				const std::string targetString = (crime->victim == npc.getSoul()->getID()) ? "me" : "her";
				const std::string end = context.rng().choose({ "...", "!" });
				switch (crime->type)
				{
				case Crime::Type::AttemptedKidnapping:
				case Crime::Type::Kidnapping:
					npc.makeTextBubble("He kidnapped " + targetString + end);
					break;
				case Crime::Type::Rape:
				case Crime::Type::SexualAssault:
					npc.makeTextBubble("He touched " + targetString + " where he shouldn't have" + end);
					break;
				case Crime::Type::Murder:
					npc.makeTextBubble("H-He killed her!");
					break;
				case Crime::Type::Assault:
					npc.makeTextBubble("He hit " + targetString + end);
					break;
				}
			}
			npc.getSoul()->setCrimeReported(crime->id, true);
			return BehaviorTreeNode::Result::Success;
		}));
		tellCrimes->addChild(unique<Wait>(90));
	}
	root->addChild(
		unique<MemoryNode>(
			While::make(hasUnreportedCrimesNode(true), std::move(tellCrimes)),
			MemoryNode::var("crimeId")));

	return unique<WeightMap>(std::move(root), "flee_player");
}

std::unique_ptr<BehaviorTreeNode> parkActivitiesNode()
{	
	// TODO: check if in park? walk to park if not?
	auto parkActivities = unique<Selector>("ParkActivities");
	{
		// @todo play with other lolis
		{
		}

		parkActivities->addChild(playTagNode());

		auto wanderAroundPark = unique<Sequence>("WanderAroundPark");
		{
			std::unique_ptr<TargetLocator> findClosebyPath = unique<PathfindingTargetLocator>(
				[](const pf::NodeID& node, ai::BehaviorContext *context) -> bool
			{
				LOCK_NPC_OR_ABORT;
				if (node.getScene()->getType() != "Town")
				{
					return false;
				}
				const Town* town = static_cast<Town*>(node.getScene());
				const int ts = node.getScene()->getTileSize();
				const int dist = pointDistance(node.getX() * ts, node.getY() * ts, npc->getPos().x, npc->getPos().y);
				return isParkPath(town->getTileAtInTileCoords(node.getX(), node.getY()))
					&& Random::get().chance(0.1f)
					&& dist < 640
					&& dist > 128;

			});

			wanderAroundPark->addChild(unique<WalkTo>(std::move(findClosebyPath)));
		}
		parkActivities->addChild(std::move(wanderAroundPark));
	}
	
	return std::move(parkActivities);
}

static NPC* getIt(BehaviorContext& context)
{
	GameOfTag *ret = context.getVariable("activity").asPtr<GameOfTag>();
	return ret->getIt();
}

static TileType tileAt(const pf::NodeID& node)
{
	return dynamic_cast<Town*>(node.getScene())->getTileAtInTileCoords(node.getX(), node.getY());
}

std::unique_ptr<BehaviorTreeNode> playTagNode()
{
	auto root = unique<ForgetfulSelector>("tag");

	auto respondToJoinRequests = unique<CustomCondition>([](BehaviorContext& context) -> BehaviorTreeNode::Result
	{
		Event *ev;
		while ((ev = context.pollEvents()) != nullptr)
		{
			if (ev->getCategory() == Event::Category::NPCInteraction && ev->getName() == "GameOfTag_join")
			{
				// @TODO make it depend on how much they like the other NPC
				ev->respond("yes");
			}
		}
		return BehaviorTreeNode::Result::Failure;
	}, "joinResp");
	root->addChild(std::move(respondToJoinRequests));

	auto chase = unique<ForgetfulSequence>("chase");
	{
		chase->addChild(unique<CustomCondition>([](BehaviorContext& context) -> BehaviorTreeNode::Result
		{
			GameOfTag *tag = context.getVariable("activity").asPtr<GameOfTag>();
			if (tag->getIt() != &context.getNPC())
			{
				return BehaviorTreeNode::Result::Failure;
			}
			if (tag->isTaggingAllowedNow())
			{
				return BehaviorTreeNode::Result::Success;
			}
			// so they don't move or do anything until the cooldown is done
			return BehaviorTreeNode::Result::Running;
		}, "it?"));
		chase->addChild(unique<WalkTo>(unique<PathfindingTargetLocator>(
			[](const pf::NodeID& node, ai::BehaviorContext *context) -> bool
		{
			LOCK_NPC_OR_ABORT;
			GameOfTag *tag = context->getVariable("activity").asPtr<GameOfTag>();
			NPC *closest = nullptr;
			float closestDist;
			for (NPC *participant : tag->getParticipants())
			{
				const float dist = pointDistance(participant->getPos(),  node.scenePos());
				if (participant != npc && (!closest || dist < closestDist))
				{
					closest = participant;
					closestDist = dist;
				}
			}
			return !closest || closestDist <= 16.f;
		})));
		
	}
	root->addChild(std::move(chase));

	auto run = unique<ForgetfulSequence>("run");
	{
		run->addChild(unique<CustomCondition>([](BehaviorContext& context) -> BehaviorTreeNode::Result
		{
			// check if you've been tagged!
			GameOfTag *tag = context.getVariable("activity").asPtr<GameOfTag>();
			NPC *it = tag->getIt();
			NPC& npc = context.getNPC();
			if (pointDistance(it->getPos(), npc.getPos()) <= 19.f && tag->isTaggingAllowedNow())
			{
				it->makeTextBubble("Tag! You're it!");
				tag->setIt(npc);
				return BehaviorTreeNode::Result::Failure;
			}
			return it != &npc ? BehaviorTreeNode::Result::Success : BehaviorTreeNode::Result::Failure;
		}, "!it?"));
		run->addChild(unique<WalkTo>(unique<PathfindingTargetLocator>(
			[](const pf::NodeID& node, ai::BehaviorContext *context) -> bool
		{
			LOCK_NPC_OR_ABORT;
			const float dist = pointDistance(npc->getPos(), node.scenePos());
			return dist >= 96.f;
		}, 
			[](const pf::NodeID& node, ai::BehaviorContext *context) -> float
		{
			TileType tile = tileAt(node);
			if (isRoad(tile))
			{
				return 10000.f;
			}
			if (isSidewalk(tile))
			{
				return 100.f;
			}
			if (tile == CrosswalkH || tile == CrosswalkV)
			{
				return 1000.f;
			}
			const float minDist = 128.f;
			const float ret = 19.f * std::max(0.f, minDist - pointDistance(getIt(*context)->getPos(), context->getNPC().getPos()));
			return ret;
		})));
	}
	root->addChild(std::move(run));

	auto continueCondition = unique<CustomCondition>([](BehaviorContext& context) -> BehaviorTreeNode::Result
	{
		GameOfTag *tag = context.getVariable("activity").asPtr<GameOfTag>();
		// @TODO don't just randomly run the game x times, maybe let GameOfTag decide?
		if (tag->getRoundsPlayed() > 5 && context.rng().chance(0.4f))
		{
			return BehaviorTreeNode::Result::Failure;
		}
		return BehaviorTreeNode::Result::Success;
	}, "doneTag?");

	return unique<JoinActivity<GameOfTag>>(While::make(std::move(continueCondition), std::move(root)));
}


std::unique_ptr<BehaviorTreeNode> chaseCandy()
{
	static const char *strFoundCandy = "found_candy_target";
	static auto makeSeeCandy = [] {
		auto closeByCandy = unique<EntityMultiLocator>("ItemPickup", LOCATOR_NO_MAX_TARGETS, LocatorHeuristic::ExactlyClosest, 10);
		closeByCandy->filter([](Entity *e) {
			return !e->isDead() && static_cast<ItemPickup*>(e)->getItem().hasComponent(Item::Category::Candy);
		});
		return unique<CanSee>(std::move(closeByCandy), strFoundCandy);
	};
	auto oohCandy = unique<Selector>();
	{
		// while we know about candy, grab it
		auto pickUpCandy = unique<Sequence>();
		{
			pickUpCandy->addChild(unique<WalkTo>(unique<BehaviorContextStoredTargetLocator>(strFoundCandy)));
			pickUpCandy->addChild(unique<CustomCondition>([](BehaviorContext& context) {
				const Target *candyTarget = context.getData<Target>(strFoundCandy);
				if (candyTarget && candyTarget->getEntity())
				{
					ItemPickup *item = static_cast<ItemPickup*>(candyTarget->getEntity());
					// TODO: eat the candy?
					context.getNPC().makeTextBubble(context.rng().choose({ "Yum!", "Mmmm", "...where does this candy keep coming from?", "Candy!" }));
					item->destroy();
					// Target is basically auto-consumed since we destroy the Entity
					//context.writePOD(strFoundCandy, Target()); // consume target (should this be a built-in functionality somewhere?)
				}
				return BehaviorTreeNode::Result::Success;
			}));
			// after we get once piece, try spinning around to look for more
			auto lookAroundForCandy = unique<Selector>();
			{
				auto lookForCandyInDir = [](FacingDirection dir) {
					auto seq = unique<Sequence>();
					seq->addChild(lookDir(dir));
					seq->addChild(makeSeeCandy());
					seq->addChild(unique<Wait>(30));
					return seq; // TODO: get around this error by actually making some ref-counting Entity refernece thing instead of this (to avoid onCandy's makeSeeCandy node reference this non-existent Entity)
				};
				lookAroundForCandy->addChild(lookForCandyInDir(FacingDirection::North));
				lookAroundForCandy->addChild(lookForCandyInDir(FacingDirection::East));
				lookAroundForCandy->addChild(lookForCandyInDir(FacingDirection::South));
				lookAroundForCandy->addChild(lookForCandyInDir(FacingDirection::West));
			}
			pickUpCandy->addChild(std::move(lookAroundForCandy));
		}
		oohCandy->addChild(While::make(unique<CustomCondition>([](BehaviorContext& context) -> bool {
			// TODO: make this its own function (targetExists(const char*)? This seems pretty general-purpose...
			return static_cast<bool>(*context.getData<Target>(strFoundCandy));
		}), std::move(pickUpCandy)));
		// otherwise look in front of you
		oohCandy->addChild(makeSeeCandy());
	}
	return unique<MemoryNode>(std::move(oohCandy), MemoryNode::var(strFoundCandy, ls::Value(DynamicStore::make<Target>())));
}

std::unique_ptr<BehaviorTreeNode> noticeThings()
{
	auto notice = unique<Parallel>();

	// make the default node one that runs forever
	notice->addChild(unique<FixedResult>(BehaviorTreeNode::Result::Running));

	notice->addIgnoredChild(unique<CustomCondition>([](BehaviorContext& context) {
		std::list<CrimeEntity*> crimes;
		context.getScene()->cull(crimes, rectAround(context.getNPC().getPos(), 256));
		for (CrimeEntity *crime : crimes)
		{
			if (context.getSoul().isCrimeKnown(crime->getCrimeId()) && context.getNPC().canSee(crime->getPos()))
			{
				context.getSoul().setCrimeReported(crime->getCrimeId(), false);
			}
		}
		// continue on even if there was nothing, since we have the next check anyway
		return BehaviorTreeNode::Result::Success;
	}), true);

	// then pass up the tree that we actually failed (so we preserve the internal running of noticing, if needed)
	// (But we should probably make these all success/failure only nodes)
	return unique<FixedResult>(BehaviorTreeNode::Result::Failure, std::move(notice));
}

float useToiletPriority(BehaviorContext& context)
{
	return 0.f;// context.getNPC().getSoul()->getUrinaryUrges();
}

std::unique_ptr<BehaviorTreeNode> useToiletNode()
{
	auto useToilet = unique<Sequence>();

	useToilet->addChild(unique<WalkTo>(unique<EntityLocator>("Toilet")));
	useToilet->addChild(lookDir(South));
	useToilet->addChild(unique<PeeNode>(false));

	return useToilet;
}

std::unique_ptr<BehaviorTreeNode> wettingNode()
{
	auto seq = unique<Sequence>();

	seq->addChild(unique<CustomCondition>([](BehaviorContext& context) { return context.getSoul().getUrinaryUrges() >= 1.f; }));
	seq->addChild(unique<CustomCondition>([](BehaviorContext& context) {
		context.getNPC().makeTextBubble("N-No! I can't hold it!");
		return true;
	}));
	auto wettingParallel = unique<Parallel>();
	{
		wettingParallel->addChild(unique<PeeNode>(true));
		auto complain = unique<Sequence>();
		{
			complain->addChild(unique<Wait>(150, 280));
			complain->addChild(unique<CustomCondition>([](BehaviorContext& context) {
				const auto& clothes = context.getSoul().appearance.clothingLayers;
				if (clothes.panties || clothes.outfit)
				{
					context.getNPC().makeTextBubble(context.rng().choose({
						"I can't believe it...",
						"...",
						"They're soaked now..."
					}));
				}
				return true;
			}));
		}
		wettingParallel->addIgnoredChild(std::move(complain), true);
	}
	seq->addChild(std::move(wettingParallel));

	return seq;
}

} // ai
} // sl
