#include "NPC/Behaviors/MovementBehaviors.hpp"

#include "NPC/Behaviors/BehaviorContext.hpp"
#include "NPC/Behaviors/CustomCondition.hpp"
#include "NPC/Behaviors/Selector.hpp"
#include "NPC/Behaviors/WalkTo.hpp"
#include "NPC/Locators/PathfindingTargetLocator.hpp"
#include "Town/Town.hpp"
#include "Utility/Memory.hpp"

namespace sl
{
namespace ai
{

std::unique_ptr<BehaviorTreeNode> goToTownNode()
{
	auto orNode = unique<Selector>();
	orNode->addChild(unique<CustomCondition>([](BehaviorContext& context) -> bool
	{
		return context.getNPC().getScene()->getType() == "Town";
	}, "inTown?"));
	auto townLocator = unique<PathfindingTargetLocator>(
		[](const pf::NodeID& node, ai::BehaviorContext*) -> bool
	{
		return node.getScene()->getType() == "Town";
	});
	orNode->addChild(unique<WalkTo>(std::move(townLocator)));
	return std::move(orNode);
}

} // ai
} // sl