#ifndef SL_AI_PARALLEL_HPP
#define SL_AI_PARALLEL_HPP

#include <memory>
#include <vector>

#include "NPC/Behaviors/BehaviorTreeNode.hpp"

namespace sl
{
namespace ai
{

class Parallel : public BehaviorTreeNode
{
public:
	enum class ReturnStrategy
	{
		//! Returns SUCCESS when any child succceeds, and failure when all childrren fail.
		AnySucceed,
		//! Returns SUCCESS when all children succeed, and failure when any fail.
		AllMustSucceed
	};
	Parallel(ReturnStrategy strat = ReturnStrategy::AnySucceed, const char *customName = "Parallel");

	Result execute(BehaviorContext& context) override;

	/**
	 * Adds a child whose return value is respected
	 */
	void addChild(std::unique_ptr<BehaviorTreeNode> child);

	/**
	 * Adds a child whose return value is IGNORED
	 * @param node Node to run
	 * @param repeating Whether to repeatedly run the node once it finishes. If false, only runs to SUCCESS/FAILURE once.
	 */
	void addIgnoredChild(std::unique_ptr<BehaviorTreeNode> child, bool repeating);

	void abort(BehaviorContext& context) override;

private:
	struct Child
	{
		std::unique_ptr<BehaviorTreeNode> node;
		bool ignored;
		bool repeating;
		//! The last terminal result from running. Set initially to RUNNING
		Result terminalResult = Result::Running;
	};
	std::vector<Child> children;
	ReturnStrategy strat;
	bool running;
};

} // ai
} // sl

#endif // SL_AI_PARALLEL_HPP