#include "NPC/Behaviors/PlayAnimation.hpp"

#include "Engine/Graphics/Animation.hpp"
#include "NPC/Behaviors/BehaviorContext.hpp"
#include "NPC/NPC.hpp"

namespace sl
{
namespace ai
{

PlayAnimation::PlayAnimation(const std::string& key)
	:PlayAnimation([key](BehaviorContext&) { return key; })
{
}

PlayAnimation::PlayAnimation(std::function<std::string(BehaviorContext&)> key)
	:BehaviorTreeNode("Anim")
	,key(std::move(key))
	,running(false)
{
}


BehaviorTreeNode::Result PlayAnimation::execute(BehaviorContext& context)
{
	if (!running)
	{
		context.getNPC().setDrawableSet(key(context));
		running = true;
	}

	Animation& animation = const_cast<Animation&>(static_cast<const Animation&>(*context.getNPC().getCurrentDrawable()));
	if (animation.isDone())
	{
		running = false;
		animation.reset(); // Do we want this in general?
		return Result::Success;
	}
	return Result::Running;
}

void PlayAnimation::abort(BehaviorContext& context)
{
	running = false;
}


} // ai
} // sl