#include "NPC/Behaviors/RandomSelector.hpp"

#include "NPC/Behaviors/BehaviorContext.hpp"
#include "Utility/Assert.hpp"
#include "Utility/Random.hpp"

namespace sl
{
namespace ai
{

RandomSelector::RandomSelector(const char *customDebugName)
	:BehaviorTreeNode(customDebugName)
	,children()
	,childRunningIndex(RESHUFFLE_INDICES)
{
}

BehaviorTreeNode::Result RandomSelector::execute(BehaviorContext& context)
{
	if (childRunningIndex == RESHUFFLE_INDICES)
	{
		context.rng().shuffle(indices.begin(), indices.end());
		childRunningIndex = 0;
	}
	ASSERT(!children.empty());
	Result result;
#ifdef SL_DEBUG
	while ((result = children[indices[childRunningIndex]]->debugExecute(context)) == Result::Failure)
#else // SL_DEBUG
	while ((result = children[indices[childRunningIndex]]->execute(context)) == Result::Failure)
#endif // SL_DEBUG
	{
		if (++childRunningIndex == children.size())
		{
			childRunningIndex = RESHUFFLE_INDICES;
			return result;
		}
	}
	if (result == Result::Success)
	{
		childRunningIndex = RESHUFFLE_INDICES;
	}
	else if (result == Result::Failure)
	{
		childRunningIndex = RESHUFFLE_INDICES;
	}
	return result;
}

void RandomSelector::abort(BehaviorContext& context)
{
	if (childRunningIndex != RESHUFFLE_INDICES)
	{
		childRunningIndex = RESHUFFLE_INDICES;
		for (const std::unique_ptr<BehaviorTreeNode>& child : children)
		{
			child->abort(context);
		}
	}
}

void RandomSelector::addChild(std::unique_ptr<BehaviorTreeNode> child)
{
#ifdef SL_DEBUG
	child->setParent(this);
#endif // SL_DEBUG
	children.push_back(std::move(child));
	indices.push_back(indices.size());
	childRunningIndex = RESHUFFLE_INDICES;
}

} // ai
} // sl