#include "NPC/Behaviors/RunAtMost.hpp"

#include "Utility/Assert.hpp"

namespace sl
{
namespace ai
{

RunAtMost::RunAtMost(std::unique_ptr<BehaviorTreeNode> node, int times)
	:BehaviorTreeNode("RunAtMost")
	,node(std::move(node))
	,currentTimes(0)
	,times(times)
{
	ASSERT(times > 0);
#ifdef SL_DEBUG
	this->node->setParent(this);
#endif // SL_DEBUG
}

BehaviorTreeNode::Result RunAtMost::execute(BehaviorContext& context)
{
	if (currentTimes++ >= times)
	{
		abort(context);
		return Result::Failure;
	}
#ifdef SL_DEBUG
	Result result = node->debugExecute(context);
#else // SL_DEBUG
	Result result = node->execute(context);
#endif // SL_DEBUG
	return result;
}

void RunAtMost::abort(BehaviorContext& context)
{
	if (currentTimes != 0)
	{
		currentTimes = 0;
		node->abort(context);
	}
}


} // ai
} // sl