#include "NPC/Behaviors/Selector.hpp"

#include "Utility/Assert.hpp"

namespace sl
{
namespace ai
{

Selector::Selector(const char *customDebugName)
	:BehaviorTreeNode(customDebugName)
	,children()
	,childRunningIndex(-1)
{
}

BehaviorTreeNode::Result Selector::execute(BehaviorContext& context)
{
	if (childRunningIndex == -1)
	{
		childRunningIndex = 0;
	}
	ASSERT(!children.empty());
	Result result;
#ifdef SL_DEBUG
	while ((result = children[childRunningIndex]->debugExecute(context)) == Result::Failure)
#else // SL_DEBUG
	while ((result = children[childRunningIndex]->execute(context)) == Result::Failure)
#endif // SL_DEBUG
	{
		if (++childRunningIndex == children.size())
		{
			childRunningIndex = -1;
			return result;
		}
	}
	if (result == Result::Success)
	{
		childRunningIndex = -1;
	}
	return result;
}

void Selector::abort(BehaviorContext& context)
{
	if (childRunningIndex != -1)
	{
		children[childRunningIndex]->abort(context);
		childRunningIndex = -1;
	}
}

void Selector::addChild(std::unique_ptr<BehaviorTreeNode> child)
{
#ifdef SL_DEBUG
	child->setParent(this);
#endif // SL_DEBUG
	children.push_back(std::move(child));
}

} // ai
} // sl