#include "NPC/Behaviors/Sequence.hpp"

#include "Utility/Assert.hpp"

namespace sl
{
namespace ai
{

Sequence::Sequence(const char *customDebugName)
	:BehaviorTreeNode(customDebugName)
	,children()
	,childRunningIndex(CLEAN)
{
}

BehaviorTreeNode::Result Sequence::execute(BehaviorContext& context)
{
	if (childRunningIndex == CLEAN)
	{
		childRunningIndex = 0;
	}
	ASSERT(!children.empty());
	Result result;
#ifdef SL_DEBUG
	while ((result = children[childRunningIndex]->debugExecute(context)) == Result::Success)
#else // SL_DEBUG
	while ((result = children[childRunningIndex]->execute(context)) == Result::Success)
#endif // SL_DEBUG
	{
		if (++childRunningIndex == children.size())
		{
			childRunningIndex = CLEAN;
			return result;
		}
	}
	if (result == Result::Failure)
	{
		childRunningIndex = CLEAN;
	}
	return result;
}

void Sequence::abort(BehaviorContext& context)
{
	if (childRunningIndex != -1)
	{
		children[childRunningIndex]->abort(context);
		childRunningIndex = -1;
	}
}

void Sequence::addChild(std::unique_ptr<BehaviorTreeNode> child)
{
#ifdef SL_DEBUG
	child->setParent(this);
#endif // SL_DEBUG
	children.push_back(std::move(child));
}


std::unique_ptr<Sequence> ifThen(std::unique_ptr<BehaviorTreeNode> condition, std::unique_ptr<BehaviorTreeNode> body)
{
	auto seq = std::make_unique<Sequence>();
	seq->addChild(std::move(condition));
	seq->addChild(std::move(body));
	return seq;
}

} // ai
} // sl