#ifndef SL_AI_SEQUENCE_HPP
#define SL_AI_SEQUENCE_HPP

#include <memory>
#include <vector>

#include "NPC/Behaviors/BehaviorTreeNode.hpp"

namespace sl
{
namespace ai
{

class Sequence : public BehaviorTreeNode
{
public:
	Sequence(const char *customDebugName = "Sequence");

	Result execute(BehaviorContext& context) override;

	void abort(BehaviorContext& context) override;

	void addChild(std::unique_ptr<BehaviorTreeNode> child);

	static std::unique_ptr<Sequence> make(std::unique_ptr<BehaviorTreeNode> node);

	template <typename... Args>
	static std::unique_ptr<Sequence> make(std::unique_ptr<BehaviorTreeNode> node, Args... args);

private:
	std::vector<std::unique_ptr<BehaviorTreeNode>> children;
	int childRunningIndex;

	static const int CLEAN = -1;
};

extern std::unique_ptr<Sequence> ifThen(std::unique_ptr<BehaviorTreeNode> condition, std::unique_ptr<BehaviorTreeNode> body);

} // ai
} // sl

#endif // SL_AI_SEQUENCE_HPP