#include "NPC/Behaviors/SleepInBed.hpp"

#include "Engine/Scene.hpp"
#include "NPC/Behaviors/AcceptTalkWithPlayer.hpp"
#include "NPC/Bed.hpp"
#include "Utility/Assert.hpp"
#include "Utility/Random.hpp"

namespace sl
{
namespace ai
{

SleepInBed::SleepInBed(const char *key)
	:BehaviorTreeNode("SleepInBed")
	,key(key)
	,running(false)
	,sleepDialogue(unique<ai::AcceptTalkWithPlayer>("sleeping_begin"))
	,zzz(0)
{
}

BehaviorTreeNode::Result SleepInBed::execute(BehaviorContext& context)
{
	if (!running)
	{
		Bed *bed = key
			? context.getVariable(key).asPtr<Bed>()
			: dynamic_cast<Bed*>(context.getSoul().getKnownEntity("bed"));
		if (bed->getOccupant() != nullptr)
		{
			abort(context);
			return Result::Failure;
		}
		NPC& npc = context.getNPC();
		npc.setPos(bed->getSleepPos());
		bed->setOccupant(&npc);
		npc.face(South);
		context.getSoul().sleep();
		running = true;
	}
	if (zzz-- <= 0)
	{
		context.getNPC().makeTextBubble("Zzz...");
		zzz = 120;
	}
	return sleepDialogue->execute(context);
}

void SleepInBed::abort(BehaviorContext& context)
{
	if (running)
	{
		context.getSoul().wakeup();
		
		Bed *bed = static_cast<Bed*>(context.getSoul().getKnownEntity("bed"));
		const int bedExitPositions = bed->getExitPositions().size();
		Vector2f exitPos;
		if (bedExitPositions > 1)
		{
			// get out of bed at a random valid bed exit position
			const int bedIndexOffset = Random::get().random(bedExitPositions);
			for (int bedIndex = 0; bedIndex < bedExitPositions; ++bedIndex)
			{
				const Vector2f p = bed->getExitPositions()[(bedIndexOffset + bedIndex) % bedExitPositions];
				const int w = context.getNPC().getMask().getBoundingBox().width;
				const int h = context.getNPC().getMask().getBoundingBox().height;
				Rect<int> exitArea(p.x - 2 / w, p.y - h / 2, w, h);
				if (context.getScene()->checkCollision(exitArea, "StaticGeometry") == nullptr)
				{
					exitPos = bed->getExitPositions()[(bedIndexOffset + bedIndex) % bedExitPositions];
					break;
				}
			}
		}
		else
		{
			exitPos = bed->getExitPos(Random::get());
		}
		context.getNPC().setPos(exitPos);
		bed->setOccupant(nullptr);
		sleepDialogue->abort(context);
		running = false;
	}
}

} // ai
} // sl