#include "NPC/Behaviors/UtilityNode.hpp"

#include <algorithm>

#include "Utility/Assert.hpp"
#include "Utility/Ranges.hpp"

namespace sl
{
namespace ai
{

UtilityNode::UtilityNode(const char *customName)
	:BehaviorTreeNode(customName)
	,children()
{
}

BehaviorTreeNode::Result UtilityNode::execute(BehaviorContext& context)
{
	ASSERT(!children.empty());
	std::vector<float> priorities(children.size());
	for (int i = 0; i < children.size(); ++i)
	{
		priorities[i] = children[i].first(context);
	}
	std::vector<int> indices = range<int>(0, children.size());
	std::sort(indices.begin(), indices.end(), [&](int a, int b) {
		return priorities[a] > priorities[b];
	});
	Result result = Result::Success;
	for (int i : indices)
	{
#ifdef SL_DEBUG
		result = children[i].second->debugExecute(context);
#else // SL_DEBUG
		result = children[i].second->execute(context);
#endif // SL_DEBUG
		if (result != Result::Failure)
		{
			return result;
		}
	}
	return result;
}

void UtilityNode::addChild(PriorityFn f, std::unique_ptr<BehaviorTreeNode> child)
{
#ifdef SL_DEBUG
	child->setParent(this);
#endif // SL_DEBUG
	children.push_back(std::make_pair(std::move(f), std::move(child)));
}

void UtilityNode::addChild(float p, std::unique_ptr<BehaviorTreeNode> child)
{
	addChild([p](BehaviorContext&) { return p; }, std::move(child));
}

void UtilityNode::abort(BehaviorContext& context)
{
	// is this the correct behaviour?
	for (auto& child : children)
	{
		child.second->abort(context);
	}
}

} // ai
} // sl