#include "NPC/Behaviors/Wait.hpp"

#include "NPC/Behaviors/BehaviorContext.hpp"
#include "Utility/Assert.hpp"

namespace sl
{
namespace ai
{

static constexpr int RECALCULATE_COUNTDOWN = -1;


Wait::Wait(int ticksToWait)
	:Wait([ticksToWait](BehaviorContext&) { return ticksToWait; })
{
}

Wait::Wait(int minTicks, int maxTicks)
	:Wait([minTicks, maxTicks](BehaviorContext& context) { return context.rng().randomInclusive(minTicks, maxTicks); })
{
}

Wait::Wait(const std::function<int(BehaviorContext&)>& tickGenerator)
	:BehaviorTreeNode("Wait")
	,tickGenerator(tickGenerator)
	,countdown(RECALCULATE_COUNTDOWN)
{
}

BehaviorTreeNode::Result Wait::execute(BehaviorContext& context)
{
	if (countdown == RECALCULATE_COUNTDOWN)
	{
		countdown = tickGenerator(context);
	}
	ASSERT(countdown >= 0);
	if (countdown == 0)
	{
		abort(context);
		return Result::Success;
	}
	--countdown;
	return Result::Running;
}

void Wait::abort(BehaviorContext& context)
{
	countdown = RECALCULATE_COUNTDOWN;
}


} // ai
} // sl