#include "NPC/Behaviors/WalkSpeed.hpp"

#include "NPC/Behaviors/BehaviorContext.hpp"
#include "NPC/NPC.hpp"
#include "Utility/Assert.hpp"

namespace sl
{
namespace ai
{

WalkSpeed::WalkSpeed(std::unique_ptr<BehaviorTreeNode> node, float speed)
	:WalkSpeed(std::move(node), [speed](BehaviorContext&) -> float { return speed; })
{
}

WalkSpeed::WalkSpeed(std::unique_ptr<BehaviorTreeNode> node, std::function<float(BehaviorContext&)> speed)
	:BehaviorTreeNode("WalkSpeed")
	,node(std::move(node))
	,speed(std::move(speed))
{
#ifdef SL_DEBUG
	this->node->setParent(this);
#endif // SL_DEBUG
}

BehaviorTreeNode::Result WalkSpeed::execute(BehaviorContext& context)
{
	const float cachedSpeed = context.getNPC().getWalkSpeedMultiplier();
	context.getNPC().setWalkSpeedMultiplier(speed(context));
#ifdef SL_DEBUG
	Result result = node->debugExecute(context);
#else // SL_DEBUG
	Result result = node->execute(context);
#endif // SL_DEBUG
	context.getNPC().setWalkSpeedMultiplier(cachedSpeed);
	return result;
}

void WalkSpeed::abort(BehaviorContext& context)
{
	node->abort(context);
}

} // ai
} // sl