#include "NPC/Behaviors/WalkTo.hpp"

#include "NPC/Behaviors/BehaviorContext.hpp"
#include "NPC/NPC.hpp"

namespace sl
{
namespace ai
{

WalkTo::WalkTo(std::unique_ptr<TargetLocator> locator)
	:WalkTo(std::move(locator), std::string())
{
}

WalkTo::WalkTo(std::unique_ptr<TargetLocator> locator, std::string storeKey)
	:BehaviorTreeNode("WalkTo")
	,locator(std::move(locator))
	,isTargetValid(false)
	,target()
	,storeKey(std::move(storeKey))
{
}

BehaviorTreeNode::Result WalkTo::execute(BehaviorContext& context)
{
	if (!isTargetValid)
	{
		target = locator->findTarget(context);
		if (!storeKey.empty())
		{
			context.setData(storeKey, target);
		}
		isTargetValid = true;
	}

	if (target.isCalculating())
	{
		return Result::Running;
	}

	target.absorbPathData(); // ugh pls remove this @TODO this is gross

	if (!target)
	{
		abort(context);
		return Result::Failure;
	}

	context.setNPCTarget(target);

	const NPCMoveResult move = context.npcMoveTowardsTarget();

	if (move == NPCMoveResult::TargetReached)
	{
		abort(context);
		return Result::Success;
	}
	else if (move == NPCMoveResult::Moved || move == NPCMoveResult::CalculatingPath)
	{
		return Result::Running;
	}
	else
	{
		abort(context);
		return Result::Failure;
	}
}

void WalkTo::abort(BehaviorContext& context)
{
	isTargetValid = false;
}

} // ai
} // sl
