#include "NPC/Behaviors/WeightMap.hpp"

#include "NPC/Behaviors/BehaviorContext.hpp"
#include "NPC/NPC.hpp"
#include "Utility/Assert.hpp"

namespace sl
{
namespace ai
{

WeightMap::WeightMap(std::unique_ptr<BehaviorTreeNode> node, std::string map)
	:WeightMap(std::move(node), [map](BehaviorContext&) -> std::string { return map; })
{
}


WeightMap::WeightMap(std::unique_ptr<BehaviorTreeNode> node, std::function<std::string(BehaviorContext&)> map)
	:BehaviorTreeNode("WeightMap")
	,node(std::move(node))
	,weightMap(std::move(map))
{
#ifdef SL_DEBUG
	this->node->setParent(this);
#endif // SL_DEBUG
}

BehaviorTreeNode::Result WeightMap::execute(BehaviorContext& context)
{
	const std::string cachedMap = context.getNPC().getCurrentPathfindingWegithMap();
	context.getNPC().setPathfindingWeightMap(weightMap(context));
#ifdef SL_DEBUG
	Result result = node->debugExecute(context);
#else // SL_DEBUG
	Result result = node->execute(context);
#endif // SL_DEBUG
	context.getNPC().setPathfindingWeightMap(cachedMap);
	return result;
}

void WeightMap::abort(BehaviorContext& context)
{
	node->abort(context);
}

} // ai
} // sl