#include "NPC/Behaviors/While.hpp"

#include "NPC/Behaviors/AlwaysSucceed.hpp"
#include "NPC/Behaviors/Sequence.hpp"
#include "NPC/Behaviors/ForgetfulSequence.hpp"
#include "NPC/Behaviors/Util.hpp"
#include "Utility/Assert.hpp"
#include "Utility/Memory.hpp"

namespace sl
{
namespace ai
{

While::While(std::unique_ptr<BehaviorTreeNode> node)
	:BehaviorTreeNode("While")
	,node(std::move(node))
{
#ifdef SL_DEBUG
	this->node->setParent(this);
#endif // SL_DEBUG
}

/*static*/std::unique_ptr<BehaviorTreeNode> While::makeActive(std::unique_ptr<BehaviorTreeNode> condition, std::unique_ptr<BehaviorTreeNode> node)
{
	return std::make_unique<While>(
		makebt<ForgetfulSequence>(
			std::move(condition),
			std::make_unique<AlwaysSucceed>(std::move(node), "Body")));
}

/*static*/std::unique_ptr<BehaviorTreeNode> While::make(std::unique_ptr<BehaviorTreeNode> condition, std::unique_ptr<BehaviorTreeNode> node)
{
	return std::make_unique<While>(
		makebt<Sequence>(
			std::move(condition),
			std::make_unique<AlwaysSucceed>(std::move(node), "Body")));
}


BehaviorTreeNode::Result While::execute(BehaviorContext& context)
{
#ifdef SL_DEBUG
	Result result = node->debugExecute(context);
#else // SL_DEBUG
	Result result = node->execute(context);
#endif // SL_DEBUG
	if (result == Result::Failure)
	{
		return Result::Failure;
	}
	return Result::Running;
}

void While::abort(BehaviorContext& context)
{
	node->abort(context);
}

} // ai
} // sl