#include "NPC/ClothesDatabase.hpp"

#include <fstream>
#include <iostream>
#include <sstream>

#include "Engine/Graphics/ColorSwapLoader.hpp"
#include "NPC/NPCAppearance.hpp"
#include "Utility/Assert.hpp"
#include "Utility/JsonUtil.hpp"
#include "Utility/Random.hpp"

namespace sl
{

std::unique_ptr<ClothesDatabase> ClothesDatabase::database;


const ClothesDatabase::Info& ClothesDatabase::getInfo(const std::string& name) const
{
	ASSERT(info.count(name));
	return info.at(name);
}

void ClothesDatabase::load()
{
	// STUB DATA
	auto& tj = info["tshirtjeans"];
	tj.type = ClothingType::Outfit;
	tj.swaps["regular"];
	publiclyWornClothes[static_cast<int>(tj.type)].insert("tshirtjeans", 1);

	auto& pl = info["uniform"];
	pl.type = ClothingType::Outfit;
	pl.swaps["regular"];

	auto& gh = info["ghost"];
	gh.type = ClothingType::Accessory;
	gh.swaps["regular"];

	readClothesFile(ClothingType::Outfit, "data/clothes/outfits.json");
	readClothesFile(ClothingType::Accessory, "data/clothes/accessories.json");
	readClothesFile(ClothingType::Panties, "data/clothes/panties.json");
	readClothesFile(ClothingType::Socks, "data/clothes/socks.json");
	readClothesFile(ClothingType::Bra, "data/clothes/bras.json");
}

std::string ClothesDatabase::getRandomClothes(Random& random, ClothingType type) const
{
	return publiclyWornClothes[static_cast<int>(type)].get(random);
}

std::string ClothesDatabase::getRandomSwap(Random& random, const std::string& clothesType) const
{
	const auto it = info.find(clothesType);
	if (it != info.end())
	{
		double total = 0.0;
		for (const auto& data : it->second.swaps)
		{
			total += data.second.popularity;
		}
		double n = random.randomFloat(total);

		for (const auto& data : it->second.swaps)
		{
			n -= data.second.popularity;
			if (n <= 0.0001)
			{
				return data.first;
			}
		}
		ERROR("This is broken -- maybe incrase the negligible value?");
	}
	ERROR("DOES NOT EXIST");
	return "DOES NOT EXIST";
}

void ClothesDatabase::dressLoliAppropriately(Random& random, NPCAppearance& npc) const
{
	// @todo make up outfits made up of accesories + clothes
	//setClothing(ClothingType::Outfit, cdb.getRandomClothes(random, ClothingType::Outfit));
	//setClothing(ClothingType::Accessory, cdb.getRandomClothes(random, ClothingType::Accessory));
	if (npc.getAgeYears() <= 7.5)
	{
		if (random.chance(0.5f))
		{
			npc.setClothing(ClothingType::Outfit, "kindergarten_outfit");
			npc.setClothing(ClothingType::Accessory, "kindergarten_hat");
			if (random.chance(0.5f))
			{
				npc.setClothing(ClothingType::Socks, "normal");
			}
		}
		else
		{
			npc.setClothing(ClothingType::Outfit, "dress");
			if (random.chance(0.5f))
			{
				npc.setClothing(ClothingType::Socks, ProportionList<const char*>("normal", 0.7f, "thighhighs", 0.3f).get(random));
			}
		}
	}
	else if (npc.getAgeYears() <= 11.5)
	{
		npc.setClothing(ClothingType::Outfit, random.choose({ "school_uniform", "kindergarten_outfit", "dress" }));
		if (random.chance(0.7f))
		{
			npc.setClothing(ClothingType::Socks, ProportionList<const char*>("normal", 0.6f, "thighhighs", 0.4f).get(random));
		}
	}
	else
	{
		npc.setClothing(ClothingType::Outfit, ProportionList<const char*>("school_uniform", 1.f, "dress", 0.6f).get(random));
		npc.setClothing(ClothingType::Bra, "training");
		npc.setClothing(ClothingType::Socks, random.choose({ "normal", "thighhighs" }));
	}
	// TODO: uncomment once art for doggy anims is in elsewhere.
	npc.setHair( ProportionList<const char*>("long", 1.f, "short", 0.5f, "pigtails", 1.f, "pigtails2", 0.3f).get(random));
	//npc.setHair("long");
	npc.setClothing(ClothingType::Panties, "plain");
	if (!npc.getClothing(ClothingType::Bra).empty())
	{
		npc.setClothingColour(ClothingType::Panties, npc.getClothingColour(ClothingType::Bra));
	}
}

ClothesDatabase::ClothesMap::const_iterator ClothesDatabase::begin() const
{
	return info.cbegin();
}

ClothesDatabase::ClothesMap::const_iterator ClothesDatabase::end() const
{
	return info.cend();
}

// private
void ClothesDatabase::readClothesFile(ClothingType type, const std::string& fname)
{
	std::string err;
	const json11::Json data = readJsonFile(fname, err);
	if (!err.empty())
	{
		ERROR(err);
		return;
	}
	for (const json11::Json& clothingDef : data.array_items())
	{
		const json11::Json::object& obj = clothingDef.object_items();
		const std::string& id = obj.at("id").string_value();
		Info& def = info[id];
		def.type = type;
		def.exactName = obj.at("exactName").string_value();
		def.genericName = obj.at("genericName").string_value();
		auto it = obj.find("genericTopName");
		if (it != obj.end())
		{
			def.genericTopName = it->second.string_value();
		}
		it = obj.find("genericBottomName");
		if (it != obj.end())
		{
			def.genericBottomName = it->second.string_value();
		}
		it = obj.find("popularity");
		if (it != obj.end())
		{
			publiclyWornClothes[static_cast<int>(type)].insert(id, it->second.number_value());
		}
		auto swaps = loadSwaps(obj.at("swaps").object_items());
		for (const std::pair<std::string, ColorSwapID>& swap : swaps)
		{
			def.swaps[swap.first].swapID = swap.second;
		}
		it = obj.find("animOverrides");
		if (it != obj.end())
		{
			ASSERT(type == ClothingType::Outfit || type == ClothingType::Panties); // extend or remove as-needed.
			for (const std::pair<std::string, json11::Json>& byAction : it->second.object_items())
			{
				for (const std::pair<std::string, json11::Json>& byLayer : byAction.second.object_items())
				{
					const NPCSpriteLayer layer = strToNPCSpriteLayer(byLayer.first);
					if (layer == NPCSpriteLayer::Unsupported)
					{
						ERROR("Unsupported or unknown Sprite Layer: " + byLayer.first);
					}
					auto r = def.animOverrides.insert(
						std::make_pair(
							std::make_pair(byAction.first, layer),
							byLayer.second.string_value()));
					ASSERT(r.second);
				}
			}
		}
	}
}

// static
ClothesDatabase& ClothesDatabase::get()
{
	if (!database)
	{
		database = std::make_unique<ClothesDatabase>();
		database->load(); // todo move?
	}
	return *database;
}

} // sl