#ifndef SL_CLOTHESDATABASE_HPP
#define SL_CLOTHESDATABASE_HPP

#include <map>
#include <memory>
#include <string>
#include <vector>

#include "Engine/Graphics/ColorSwapID.hpp"
#include "NPC/ClothingType.hpp"
#include "NPC/NPCSpriteLayers.hpp"
#include "Utility/ProportionList.hpp"

namespace sl
{

class NPCAppearance;
class Random;

class ClothesDatabase
{
public:
	class Info
	{
	public:
		class Data
		{
		public:
			Data()
				:popularity(1.0)
			{
			}

			ColorSwapID swapID;
			double popularity;
		};

		std::map<std::string, Data> swaps;
		ClothingType type;
		std::string exactName;
		std::string genericName;
		// only use these 2 for outfits
		std::string genericTopName;
		std::string genericBottomName;
		std::map<std::pair<std::string, NPCSpriteLayer>, std::string> animOverrides;
	};
	
	typedef std::map<std::string, Info> ClothesMap;

	

	const Info& getInfo(const std::string& name) const;

	void load();

	std::string getRandomClothes(Random& random, ClothingType type) const;

	std::string getRandomSwap(Random& random, const std::string& clothesType) const;

	
	

	void dressLoliAppropriately(Random& random, NPCAppearance& npc) const;


	ClothesMap::const_iterator begin() const;

	ClothesMap::const_iterator end() const;

	static ClothesDatabase& get();

private:
	void readClothesInfo(const std::string& fname, const std::string& name);

	void readClothesFile(ClothingType type, const std::string& fname);



	ClothesMap info;
	ProportionList<std::string> publiclyWornClothes[CLOTHING_TYPE_COUNT];


	static std::unique_ptr<ClothesDatabase> database;
};


} // sl

#endif // SL_CLOTHESDATABASE_HPP