#include "NPC/Dialogue/AttributeLoading.hpp"

#include "NPC/Stats/Soul.hpp"

namespace sl
{

void initAttributes(const std::string& name, const Soul& soul, std::map<std::string, Attribute>& attributes)
{
	attributes.insert(std::make_pair(name + "strength", Attribute(0, 1, std::bind(&Soul::strength, &soul))));
	attributes.insert(std::make_pair(name + "intelligence", Attribute(0, 1, std::bind(&Soul::intelligence, &soul))));
	attributes.insert(std::make_pair(name + "gullibility", Attribute(0, 1, std::bind(&Soul::gullibility, &soul))));
	attributes.insert(std::make_pair(name + "charisma", Attribute(0, 1, std::bind(&Soul::charisma, &soul))));
	attributes.insert(std::make_pair(name + "lewd", Attribute(0, 1, [&soul] {
		return 0.5f; // TODO implement
	})));

	attributes.insert(std::make_pair(name + "hunger", Attribute(0, 1, [&soul] {
		return soul.getStatus().hunger;
	})));
	attributes.insert(std::make_pair(name + "thirst", Attribute(0, 1, [&soul] {
		return soul.getStatus().thirst;
	})));

	attributes.insert(std::make_pair(name + "willingness", Attribute(0, 1, [&soul] {
		return soul.getWillingness();
	})));

	// personality stuff
	attributes.insert(std::make_pair(name + "dominance", Attribute(-1, 1, std::bind(&Soul::dominance, &soul))));
	attributes.insert(std::make_pair(name + "open_to_cautious", Attribute(-1, 1, std::bind(&Soul::openToCautious, &soul))));
	attributes.insert(std::make_pair(name + "organized_to_careless", Attribute(-1, 1, std::bind(&Soul::organizedToCareless, &soul))));
	attributes.insert(std::make_pair(name + "extrovert_to_introvert", Attribute(-1, 1, std::bind(&Soul::extrovertToIntrovert, &soul))));
	attributes.insert(std::make_pair(name + "friendly_to_unkind", Attribute(-1, 1, std::bind(&Soul::friendlyToUnkind, &soul))));
	attributes.insert(std::make_pair(name + "nervous_to_confident", Attribute(-1, 1, std::bind(&Soul::nervousToConfident, &soul))));

	
	// mood related stuff
	attributes.insert(std::make_pair(name + "mood_pleasure", Attribute(Mood::min, Mood::max, [&soul] { return soul.getStatus().currentMood.getValue(Mood::Pleasure); })));
	attributes.insert(std::make_pair(name + "mood_arousal", Attribute(Mood::min, Mood::max, [&soul] { return soul.getStatus().currentMood.getValue(Mood::Arousal); })));
	attributes.insert(std::make_pair(name + "mood_dominance", Attribute(Mood::min, Mood::max, [&soul] { return soul.getStatus().currentMood.getValue(Mood::Dominance); })));
}

} // sl
