#include "NPC/Dialogue/AttributeRange.hpp"

#include "Parsing/Binding/Bindings.hpp"

namespace sl
{

AttributeRange::AttributeRange(std::unique_ptr<ls::Expression> val, float scalar)
	:min(std::move(val))
	,max()
	,scalar(scalar)
{
}

AttributeRange::AttributeRange(std::unique_ptr<ls::Expression> min, std::unique_ptr<ls::Expression> max, float scalar)
	:min(std::move(min))
	,max(std::move(max))
	,scalar(scalar)
{
}


float AttributeRange::calculateDifference(const ls::Bindings& context, float value) const
{
	const float minVal = min->evaluate(context).asFloat();
	const float maxVal = max ? max->evaluate(context).asFloat() : minVal;
	if (value < minVal)
	{
		return (minVal - value) * scalar;
	}
	else if (value > maxVal)
	{
		return (value - maxVal) * scalar;
	}
	return 0.f;
}

float AttributeRange::getScalar() const
{
	return scalar;
}

} // sl
