#include "NPC/Dialogue/ItemQuery.hpp"

#include "NPC/Stats/Soul.hpp"

namespace sl
{

ItemQuery::ItemQuery()
	:queryCategories(0)
	,queryType(QueryType::Invalid)
	,itemRefs()
	,selectedItem(itemRefs.end())
	,selectedInventory(inventories.end())
{
}

void ItemQuery::registerInventory(const std::string& name, Inventory *inventory, Soul *soul)
{
	auto& invent = inventories[name];
	invent.first = inventory;
	invent.second = soul;
}


void ItemQuery::addOrFilter(Item::Category category)
{
	if (queryType == QueryType::Invalid)
	{
		queryType = QueryType::Or;
		itemRefs.clear();
	}
	ASSERT(queryType == QueryType::Or);// can't mix OR/AND
	queryCategories |= category;
}

void ItemQuery::addAndFilter(Item::Category category)
{
	if (queryType == QueryType::Invalid)
	{
		queryType = QueryType::And;
		itemRefs.clear();
	}
	ASSERT(queryType == QueryType::And);// can't mix OR/AND
	queryCategories |= category;
}

void ItemQuery::queryInventory(const std::string& inventory)
{
	ASSERT(queryType != QueryType::Invalid);
	selectedInventory = inventories.find(inventory);
	ASSERT(selectedInventory != inventories.end());
	selectedInventory->second.first->queryItems(itemRefs, queryCategories);
	if (queryType == QueryType::And)
	{
		// remove items that only had some of the queryCategories but not all
		for (int i = 0; i < itemRefs.size(); ++i)
		{
			if (!itemRefs[i]->hasComponents(queryCategories))
			{
				std::swap(itemRefs[i], itemRefs.back());
				itemRefs.pop_back();
				--i; // to counteract the ++i
			}
		}
	}
	queryCategories = 0;
	queryType = QueryType::Invalid;
}

const std::vector<Inventory::ItemRef>& ItemQuery::getItems() const
{
	return itemRefs;
}

void ItemQuery::selectItem(int index)
{
	ASSERT(index >= 0 && index < itemRefs.size());
	selectedItem = std::next(itemRefs.begin(), index);
} 

void ItemQuery::giftItem(const std::string& target)
{
	Item item = selectedInventory->second.first->removeItem(*selectedItem);
	auto it = inventories.find(target);
	ASSERT(it != inventories.end());
	it->second.second->giftItem(std::move(item));
	reset();
}

void ItemQuery::consumeItem(const std::string& target)
{
	Item item = selectedInventory->second.first->removeItem(*selectedItem);
	auto it = inventories.find(target);
	ASSERT(it != inventories.end());
	it->second.second->consume(std::move(item));
	reset();
}

void ItemQuery::reset()
{
	itemRefs.clear();
	selectedItem = itemRefs.end();
}

Inventory::ItemRef ItemQuery::getSelectedItem() const
{
	ASSERT(selectedItem != itemRefs.end());
	return *selectedItem;
}


} // sl
