#ifndef SL_ITEMQUERY_HPP
#define SL_ITEMQUERY_HPP

#include "Items/Item.hpp"
#include "Items/Inventory.hpp"

namespace sl
{

class Soul;

class ItemQuery
{
public:
	ItemQuery();

	void registerInventory(const std::string& name, Inventory *inventory, Soul *soul);

	void addOrFilter(Item::Category category);

	void addAndFilter(Item::Category category);

	void queryInventory(const std::string& inventory);

	const std::vector<Inventory::ItemRef>& getItems() const;

	void selectItem(int index);

	void giftItem(const std::string& target);

	void consumeItem(const std::string& target);

	void reset();

	Inventory::ItemRef getSelectedItem() const;

private:
	//! Which categories to query for when doing Inventory things
	Item::Categories queryCategories;
	//! Whether the query categories are AND or OR
	enum class QueryType
	{
		Invalid,
		Or,
		And
	};
	QueryType queryType;
	//! ItemRefs to the last queried items
	std::vector<Inventory::ItemRef> itemRefs;
	std::map<std::string, std::pair<Inventory*, Soul*>> inventories;
	decltype(itemRefs)::iterator selectedItem;
	decltype(inventories)::iterator selectedInventory;
};

} // sl

#endif // SL_ITEMQUERY_HPP