#ifndef SL_OUTCOME_HPP
#define SL_OUTCOME_HPP

#include <map>
#include <queue>
#include <utility>

#include "NPC/Dialogue/Attribute.hpp"
#include "NPC/Dialogue/AttributeRange.hpp"
#include "Parsing/Binding/Bindings.hpp"
#include "Parsing/SyntaxTree/Statements/Statement.hpp"

namespace sl
{

class Outcome
{
public:
	Outcome(std::string id, std::unique_ptr<const ls::Statement> message, std::unique_ptr<const ls::Statement> result, std::unique_ptr<const ls::Statement> links, std::vector<std::pair<std::string, AttributeRange>>&& attributes, std::unique_ptr<const ls::Statement> availability, std::vector<std::unique_ptr<const Outcome>> outcomes, std::unique_ptr<const ls::Expression> priority);

	void execute(ls::Bindings& context) const;

	std::shared_ptr<const ls::Statement> getMessages() const;

	void getLinks(ls::Bindings& context) const;

	void getAttributeNames(std::vector<std::string>& output) const;

	bool isAvailable(ls::Bindings& context) const;

	int getPriority(ls::Bindings& context) const;

	float calculateDistance(const ls::Bindings& context, const std::map<std::string, Attribute>& npcAttributes, const std::vector<std::string>& selectedAttributes) const;

	const std::vector<std::unique_ptr<const Outcome>>& getOutcomes() const;

	const std::string& getIdentifier() const;

	void lint(ls::LintBindings& context) const;

private:

	std::string id;
	const std::shared_ptr<const ls::Statement> message;
	const std::unique_ptr<const ls::Statement> result;
	const std::unique_ptr<const ls::Statement> links;
	std::map<std::string, AttributeRange> attributes;
	const std::unique_ptr<const ls::Statement> availability;
	const std::vector<std::unique_ptr<const Outcome>> outcomes;
	std::unique_ptr<const ls::Expression> priority;
};

} // sl

#endif // SL_OUTCOME_HPP
