#ifndef SL_POPUP_HPP
#define SL_POPUP_HPP

#include "Engine/Graphics/Framerate.hpp"
#include "Engine/GUI/GUIWidget.hpp"
#include "NPC/FramerateSpec.hpp"
#include "NPC/NPCAppearance.hpp"
#include "Parsing/Binding/Bindings.hpp"

#include <string>


namespace sl
{

namespace gui
{
class DrawableContainer;
} // gui

class Popup : public gui::GUIWidget
{
public:
	static constexpr int WIDTH = 256;
	static constexpr int HEIGHT = 192;

	/**
	 * @param topLeft The top left of the widget.
	 * @param npc The NPCAppearance to read from - this MUST exist for as long as this object is in scope!
	 */
	Popup(const Vector2i& topLeft, const NPCAppearance& npc);

	/**
	 * Binds necessary functions to loliscript for changing the layers/animations/etc.
	 */
	void registerFunctions(ls::Bindings& context, const std::string& target);

	/**
	 * Loads a popup but WITHOUT setting category/action/face/framerate/repeats/etc
	 * This is to facilitate temporary animations mostly. 
	 */
	void load(const std::string& category, const std::string& action, const std::string& face, FramerateSpec framerateSpec = FramerateSpec(), bool repeats = true, bool preservesProgress = false);



	/**
	 * Sets AND reloads the popup based on the new values.
	 */
	void set(std::string category, std::string action, std::string face, FramerateSpec framerateSpec = FramerateSpec(), bool repeats = true, bool preservesProgress = false);

	/**
	 * Reloads based on the current settings. (Might also need to be called if the underlying NPCAppearance changes)
	 * @param preservesProgress Whether to preserve the old animation progress. This is useful for face changing mid-animations, etc
	 */
	void reload(bool preservesProgress);

	const std::string& getCategory() const { return category; }

	const std::string& getAction() const { return action; }

	const std::string& getFace() const { return face; }
private:

	void onUpdate(int x, int y);

	void initRenderingSetup();

	bool showing() const;

	const NPCAppearance& npc;
	std::string category;
	std::string action;
	std::string face;
	FramerateSpec framerateSpec;
	bool repeats;
	int timer;
	bool isAnimating;
	float fadeProgress;
	float fadeSpeed;
	gui::DrawableContainer *container;
	std::unique_ptr<Drawable> current;
	std::unique_ptr<Drawable> fadeout;
	sf::RenderTexture finalTexture;
	sf::RenderTexture bufferTexture;
	sf::Sprite bufferSprite;
};

} // sl

#endif // SL_POPUP_HPP