#ifndef SL_SCENARIO_HPP
#define SL_SCENARIO_HPP

#include <map>
#include <memory>
#include <string>

#include "NPC/Dialogue/Outcome.hpp"

namespace sl
{

class ItemQuery;

namespace test
{
class DialogueLintTest;
} // test

class Scenario
{
public:
	Scenario(const std::string& identifier, std::unique_ptr<const ls::Statement> title, std::unique_ptr<const ls::Statement> availability, std::unique_ptr<const ls::Statement> links, std::vector<std::unique_ptr<const Outcome>>& outcomes);

	/**
	 * Runs a Scenario and figures out which (available) Outcome best matches the input attributes.
	 */
	const Outcome& chooseOutcome(ls::Bindings& context, const std::map<std::string, Attribute>& npcAttributes) const;
	
	/**
	 * Gets the title of this Scenario
	 * @return The title
	 */
	std::string getTitle(ls::Bindings& context) const;

	bool isAvailable(ls::Bindings& context) const;

	/**
	 * @return A unique identifier for this scenario - should be the filename
	 */
	const std::string& getIdentifier() const;

	// @TODO: keep this here to only register stuff once? remove repeated registration once we add in objects to loliscript?
	void getLinks(const Outcome& outcome, ls::Bindings& context, ItemQuery& itemQuery, std::vector<std::pair<std::string, std::string>>& links) const;

	void lint(ls::LintBindings& context) const;

private:

	static const Outcome& chooseOutcomeImpl(ls::Bindings& context, const std::map<std::string, Attribute>& npcAttributes, const std::vector<std::unique_ptr<const Outcome>>& outcomes);
	

	std::unique_ptr<const ls::Statement> title;
	std::unique_ptr<const ls::Statement> availability;
	std::unique_ptr<const ls::Statement> links;
	std::vector<std::unique_ptr<const Outcome>> outcomes;
	std::string identifier;

	friend class test::DialogueLintTest;
};

} // sls

#endif // SL_SCENARIO_HPP
