#include "NPC/FamilyGen.hpp"

#include "Town/World.hpp"
#include "NPC/Stats/SoulGenerationSettings.hpp"
#include "NPC/ClothesDatabase.hpp"
#include "Utility/Memory.hpp"
#include "Utility/Random.hpp"

namespace sl
{

FamilyGen::FamilyGen(const SoulGenerationSettings& settings, World& world, Random& rng)
	:world(world)
	,rng(rng)
{
	SoulGenerationSettings parentsSettings;
	// to stop most couples from being interracial (since that's unlikely)
	parentsSettings.addNationality(settings.getNationality(rng));
	parentsSettings.setAgeRange(365 * 27, 365 * 36);
	parentsSettings.setMaleChance(0.f);
	mother = world.createSoul(rng, parentsSettings);
	mother->setType("adultfemale");
	mother->appearance.setHair("short");
	parentsSettings.setMaleChance(1.f);
	father = world.createSoul(rng, parentsSettings);
	father->setType("adultmale");
	father->appearance.setHair("short");
}


std::shared_ptr<Soul> FamilyGen::createChild(int minAge, int maxAge, float maleChance)
{
	//mother->mate(*father, rng);
	std::shared_ptr<Soul> kid = shared<Soul>(*mother, *father, rng, maleChance);
	ASSERT(kid);
	world.recordBirth(kid);
	const int age = rng.randomInclusive(minAge, maxAge);
	for (int i = 0; i < age; ++i)
	{
		kid->ageOneDay();
	}
	ClothesDatabase::get().dressLoliAppropriately(rng, kid->appearance);
	return kid;
}

std::shared_ptr<Soul> FamilyGen::getMother()
{
	return mother;
}

std::shared_ptr<Soul> FamilyGen::getFather()
{
	return father;
}

std::shared_ptr<Soul> FamilyGen::getParent()
{
	return rng.chance(0.5f) ? getMother() : getFather();
}

} // sl
