#include "NPC/FramerateSpec.hpp"

#include "NPC/AnimationDatabase.hpp"
#include "Utility/Assert.hpp"

namespace sl
{

FramerateSpec::FramerateSpec()
	:impl(Impl::Uninitialized)
	,framerate(0)
{
}

FramerateSpec::FramerateSpec(Framerate framerate)
	:impl(Impl::Framerate)
	,framerate(std::move(framerate))
{
}

FramerateSpec::FramerateSpec(std::string animId)
	:impl(Impl::AnimationDB)
	,framerate(-1)
	,animId(std::move(animId))
{
}


void FramerateSpec::set(Framerate framerate)
{
	ASSERT(impl == Impl::Uninitialized);
	impl = Impl::Framerate;
	this->framerate = std::move(framerate);
}

void FramerateSpec::set(std::string animId)
{
	ASSERT(impl == Impl::Uninitialized);
	impl = Impl::AnimationDB;
	this->animId = std::move(animId);
}

const Framerate& FramerateSpec::get(NPCSpriteLayer layer) const
{
	if (impl == Impl::AnimationDB)
	{
		const AnimationDatabase::Entry *entry = AnimationDatabase::get().get(animId);
		ASSERT(entry != nullptr);
		return entry->get(layer);
	}
	return framerate;
}

} // sl
