/**
* @section DESCRIPTION
* The specifications for a framerate. Can wrap raw-details or something looked up in the AnimationDatabase
*/
#ifndef SL_FRAMERATESPEC_HPP
#define SL_FRAMERATESPEC_HPP

#include "Engine/Graphics/Framerate.hpp"
#include "NPC/NPCSpriteLayers.hpp"

namespace sl
{

class FramerateSpec
{
public:
	FramerateSpec();

	FramerateSpec(Framerate framerate);

	FramerateSpec(std::string animId);


	/**
	 * Animates simply using provided rate
	 * @param animationSpeed How many game updates (frames) to wait in between animation frame changes
	 */
	void set(Framerate framerate);

	/**
	 * Animates based on animation details stored in AnimationDatabase
	 * @animId Key to look up in AnimationDatabase
	 */
	void set(std::string animId);

	const Framerate& get(NPCSpriteLayer layer) const;

private:
	enum class Impl
	{
		Uninitialized = 0,
		Framerate,
		AnimationDB
	};
	Impl impl;
	//! Only used if initialized without animId
	Framerate framerate;
	//! Only used if initialized without framerate
	std::string animId;
};

} // sl

#endif // SL_FRAMERATESPEC_HPP