#include "NPC/GroupAI/GameOfTag.hpp"

#include "NPC/NPC.hpp"
#include "Utility/Trig.hpp"
#include "Utility/TypeToString.hpp"

#include <algorithm>

namespace sl
{

DECLARE_TYPE_TO_STRING(GameOfTag)

GameOfTag::GameOfTag()
	:it(nullptr)
	,cooldown(0)
	,roundsPlayed(-1) // since setIt increases it
{
}

void GameOfTag::onUpdate()
{
	if (cooldown > 0)
	{
		--cooldown;
	}
	updateSpace();
}

bool GameOfTag::isPersistent() const
{
	return false;
}

GroupAI<GameOfTag>::Membership GameOfTag::addParticipant(NPC& participant)
{
	if (participants.empty())
	{
		setIt(participant);
	}
	participants.push_back(&participant);
	updateSpace();
	return GroupAI<GameOfTag>::Membership(*this, participant);
}

void GameOfTag::setIt(NPC& it)
{
	cooldown = 150;
	++roundsPlayed;
	// change the previous it's behaviour?
	this->it = &it;
	for (NPC *npc : participants)
	{
		// tell them to chase it?
	}
}

NPC* GameOfTag::getIt() const
{
	return it;
}

bool GameOfTag::isInGame(const NPC& npc) const
{
	return std::find(participants.cbegin(), participants.cend(), &npc) != participants.cend();
}

bool GameOfTag::isTaggingAllowedNow() const
{
	return (cooldown == 0);
}

int GameOfTag::getRoundsPlayed() const
{
	return roundsPlayed;
}

// private
void GameOfTag::updateSpace()
{
	if (!participants.empty())
	{
		// create a box around the players
		auto it = participants.cbegin();
		int minX, maxX, minY, maxY;
		minX = maxX = (*it)->getPos().x;
		minY = maxY = (*it)->getPos().y;
		for (++it; it != participants.cend(); ++it)
		{
			const int x = (*it)->getPos().x;
			const int y = (*it)->getPos().y;
			if (x < minX)
			{
				minX = x;
			}
			if (x > maxX)
			{
				maxX = x;
			}
			if (y < minY)
			{
				minY = y;
			}
			if (y > maxY)
			{
				maxY = y;
			}
		}
		// center activity at it
		pos.x = minX + (maxX - minX) / 2;
		pos.y = minY + (maxY - minY) / 2;
		// then make sure every point is at least minBoundary away from the circle boundary
		const float minBoundary = 192;
		radius = pointDistance(pos.x, pos.y, minX, minY) + minBoundary;
	}
}

} // sl
