#include "NPC/Locators/EntityLocator.hpp"

#include "Engine/Serialization/AutoDeserialize.hpp"
#include "Engine/Serialization/AutoSerialize.hpp"
#include "NPC/Behaviors/BehaviorContext.hpp"
#include "NPC/NPC.hpp"
#include "Utility/Assert.hpp"

namespace sl
{

EntityLocator::EntityLocator(const Entity::Type& type, LocatorHeuristic heuristic, int radius)
	:source(nullptr)
	, type(type)
	, heuristic(heuristic)
	, radius(radius)
{
}

EntityLocator::EntityLocator(const Entity& source, const Entity::Type& type, LocatorHeuristic heuristic, int radius)
	:source(&source)
	,type(type)
	,heuristic(heuristic)
	,radius(radius)
{
}


EntityLocator& EntityLocator::filter(std::function<bool(Entity*)> predicate)
{
	ASSERT(!this->predicate); // dunno why you'd do this, but if you do, then just make this generate a new function which captures the 2 and joins them
	this->predicate = std::move(predicate);
	return *this;
}

Target EntityLocator::findTarget() const
{
	ASSERT(source);
	const auto targets = findTargetsImpl(*source, type, heuristic, radius, predicate, 1);
	return !targets.empty() ? targets.front() : Target();
}

Target EntityLocator::findTarget(ai::BehaviorContext& context) const
{
	ASSERT(source == nullptr); // @TODO: figure out if it should use context.getNPC() vs *source in this case
	const auto targets = findTargetsImpl(context.getNPC(), type, heuristic, radius, predicate, 1);
	return !targets.empty() ? targets.front() : Target();
}

void EntityLocator::deserialize(Deserialize& in)
{
	AutoDeserialize::fields(in, source, type, radius);
	heuristic = static_cast<LocatorHeuristic>(in.u8("heuristic"));
}

void EntityLocator::serialize(Serialize& out) const
{
	ASSERT(!predicate);
	AutoSerialize::fields(out, source, type, radius);
	out.u8("heuristic", static_cast<uint8_t>(heuristic));
}

} // sl
