#ifndef SL_ENTITYLOCATOR_HPP
#define SL_ENTITYLOCATOR_HPP

#include "NPC/Locators/EntityLocatorCommon.hpp"
#include "NPC/Locators/TargetLocator.hpp"
#include "Engine/Entity.hpp"

#include <functional>

namespace sl
{

class EntityLocator : public TargetLocator
{
public:
	DECLARE_SERIALIZABLE_METHODS(EntityLocator)
	/**
	 * EntityLocator that takes its source Entity from ai::BehaviorContext to search from
	 * @param type The Type of Entity to search for
	 * @param heuristic The search strategy to use, detailed above.
	 * @param radius The maximium radius to search for IN WORLDSPACE (see comment at top of World.hpp). LOCATOR_UNBOUNDED_DISTANCE means search forever.
	 */
	EntityLocator(const Entity::Type& type, LocatorHeuristic heuristic = LocatorHeuristic::ExactlyClosest, int radius = LOCATOR_UNBOUNDED_DISTANCE);
	/**
	 * EntityLocator that uses an explicitly provided Entity to search from
	 * @param source The explicitly provided source entity to search from
	 * @param type The Type of Entity to search for
	 * @param heuristic The search strategy to use, detailed above.
	 * @param radius The maximium radius to search for (see comment at top of World.hpp). LOCATOR_UNBOUNDED_DISTANCE means search forever.
	 */
	EntityLocator(const Entity& source, const Entity::Type& type, LocatorHeuristic heuristic = LocatorHeuristic::ExactlyClosest, int radius = LOCATOR_UNBOUNDED_DISTANCE);

	EntityLocator& filter(std::function<bool(Entity*)> predicate);

	Target findTarget() const override;

	Target findTarget(ai::BehaviorContext& context) const override;

private:

	const Entity *source;
	Entity::Type type;
	LocatorHeuristic heuristic;
	int radius;
	std::function<bool(Entity*)> predicate;
};

} // sl

#endif // SL_ENTITYLOCATOR_HPP