#include "NPC/Locators/EntityMultiLocator.hpp"

#include "NPC/NPC.hpp"
#include "NPC/Behaviors/BehaviorContext.hpp"
#include "Utility/Assert.hpp"

namespace sl
{

EntityMultiLocator::EntityMultiLocator(const Entity::Type& type, int maxTargets, LocatorHeuristic heuristic, int radius)
	:source(nullptr)
	,type(type)
	,heuristic(heuristic)
	,radius(radius)
	,maxTargets(maxTargets)
{
}

EntityMultiLocator::EntityMultiLocator(const Entity& source, const Entity::Type& type, int maxTargets, LocatorHeuristic heuristic, int radius)
	:source(&source)
	,type(type)
	,heuristic(heuristic)
	,radius(radius)
	,maxTargets(maxTargets)
{
}


EntityMultiLocator& EntityMultiLocator::filter(std::function<bool(Entity*)> predicate)
{
	ASSERT(!this->predicate); // dunno why you'd do this, but if you do, then just make this generate a new function which captures the 2 and joins them
	this->predicate = std::move(predicate);
	return *this;
}

std::vector<Target> EntityMultiLocator::findTargets() const
{
	ASSERT(source);
	return findTargetsImpl(*source, type, heuristic, radius, predicate, maxTargets);
}

std::vector<Target> EntityMultiLocator::findTargets(ai::BehaviorContext& context) const
{
	ASSERT(source == nullptr); // @TODO: figure out if it should use context.getNPC() vs *source in this case
	return findTargetsImpl(context.getNPC(), type, heuristic, radius, predicate, maxTargets);
}

} // sl
