#ifndef SL_ENTITYMULTILOCATOR_HPP
#define SL_ENTITYMULTILOCATOR_HPP

#include "NPC/Locators/EntityLocatorCommon.hpp"
#include "NPC/Locators/MultiTargetLocator.hpp"
#include "Engine/Entity.hpp"

#include <functional>

namespace sl
{

class EntityMultiLocator : public MultiTargetLocator
{
public:
	/**
	 * EntityLocator that takes its source Entity from ai::BehaviorContext to search from
	 * @param type The Type of Entity to search for
	 * @param maxTargets. The maximum size to query for - simply returns the first maxTargets closest Targets
	 * @param heuristic The search strategy to use, detailed above.
	 * @param radius The maximium radius to search for IN WORLDSPACE (see comment at top of World.hpp). LOCATOR_UNBOUNDED_DISTANCE means search forever.
	 */
	EntityMultiLocator(const Entity::Type& type, int maxTargets = LOCATOR_NO_MAX_TARGETS, LocatorHeuristic heuristic = LocatorHeuristic::ExactlyClosest, int radius = LOCATOR_UNBOUNDED_DISTANCE);
	/**
	 * EntityLocator that uses an explicitly provided Entity to search from
	 * @param source The explicitly provided source entity to search from
	 * @param type The Type of Entity to search for
	 * @param maxTargets. The maximum size to query for - simply returns the first maxTargets closest Targets
	 * @param heuristic The search strategy to use, detailed above.
	 * @param radius The maximium radius to search for (see comment at top of World.hpp). LOCATOR_UNBOUNDED_DISTANCE means search forever.
	 */
	EntityMultiLocator(const Entity& source, const Entity::Type& type, int maxTargets = LOCATOR_NO_MAX_TARGETS, LocatorHeuristic heuristic = LocatorHeuristic::ExactlyClosest, int radius = LOCATOR_UNBOUNDED_DISTANCE);

	EntityMultiLocator& filter(std::function<bool(Entity*)> predicate);

	std::vector<Target> findTargets() const override;

	std::vector<Target> findTargets(ai::BehaviorContext& context) const override;

private:

	const Entity *source;
	Entity::Type type;
	LocatorHeuristic heuristic;
	int radius;
	std::function<bool(Entity*)> predicate;
	int maxTargets;
};

} // sl

#endif // SL_ENTITYMULTILOCATOR_HPP