#include "NPC/Locators/PathfindingTargetLocator.hpp"

#include "NPC/NPC.hpp"
#include "NPC/Behaviors/BehaviorContext.hpp"
#include "Utility/Assert.hpp"

namespace sl
{

PathfindingTargetLocator::PathfindingTargetLocator(const pf::NodePredicate& predicate, const pf::WeightFunc& heuristic, const std::string& weightMap, const std::function<void(std::deque<pf::Node>&)>& postProcess)
	:npc(nullptr)
	,predicate(predicate)
	,heuristic(heuristic)
	,weightMap(weightMap)
	,postProcess(postProcess)
{
}

PathfindingTargetLocator::PathfindingTargetLocator(NPC& npc, const pf::NodePredicate& predicate, const pf::WeightFunc& heuristic, const std::string& weightMap, const std::function<void(std::deque<pf::Node>&)>& postProcess)
	:npc(&npc)
	,predicate(predicate)
	,heuristic(heuristic)
	,weightMap(weightMap)
	,postProcess(postProcess)
{
}


Target PathfindingTargetLocator::findTarget() const
{
	ASSERT(npc);
	return npc->setTargetViaPathing(predicate, heuristic, weightMap, postProcess);
}

Target PathfindingTargetLocator::findTarget(ai::BehaviorContext& context) const
{
	ASSERT(npc == nullptr);
	return context.getNPC().setTargetViaPathing(predicate, heuristic, weightMap, postProcess);
}



} // sl
