#ifndef SL_PATHFINDINGTARGETLOCATOR_HPP
#define SL_PATHFINDINGTARGETLOCATOR_HPP

#include "Pathfinding/PathfindingCommon.hpp"
#include "NPC/Locators/TargetLocator.hpp"

// defined here since calling from other threads is about the only use-case, and so far this is the only place I can think of that needs it.
// if it's needed elsewhere, just move it there.
#define LOCK_NPC_OR_ABORT Ref<NPC, true> npc = context->lockNPC(); \
	do { \
		if (!npc) { \
			return true; \
		} \
	} while (false)

namespace sl
{

class NPC;

class PathfindingTargetLocator : public TargetLocator
{
public:
	/**
	 * Pathfinding target locator for use by BehaviorTrees only. It gets the NPC source from the BT context
	 * @param weightMap The weightmap to use for calculation - if empty then it uses the NPC's current one
	 */
	PathfindingTargetLocator(const pf::NodePredicate& predicate, const pf::WeightFunc& heuristic = pf::noHeuristic, const std::string& weightMap = std::string(), const std::function<void(std::deque<pf::Node>&)>& postProcess = std::function<void(std::deque<pf::Node>&)>());

	/**
	* Pathfinding target locator with a fixed NPC to use. Does not use BT context's
	* @param weightMap The weightmap to use for calculation - if empty then it uses the NPC's current one
	*/
	PathfindingTargetLocator(NPC& npc, const pf::NodePredicate& predicate, const pf::WeightFunc& heuristic = pf::noHeuristic, const std::string& weightMap = std::string(), const std::function<void(std::deque<pf::Node>&)>& postProcess = std::function<void(std::deque<pf::Node>&)>());


	Target findTarget() const override;

	Target findTarget(ai::BehaviorContext& context) const override;


private:

	NPC *npc;
	pf::NodePredicate predicate;
	pf::WeightFunc heuristic;
	std::string weightMap;
	std::function<void(std::deque<pf::Node>&)> postProcess;
};

} // sl

#endif // SL_PATHFINDINGTARGETLOCATOR_HPP