#ifndef SL_TARGET_HPP
#define SL_TARGET_HPP

#include "Engine/Ref.hpp"
#include "Engine/ScenePos.hpp"
#include "Utility/Vector.hpp"


namespace sl
{

class Entity;
class Scene;

namespace pf
{
class Path;
}

class Deserialize;
class Serialize;
class Target;
namespace AutoDeserialize
{
	void deserialize(Deserialize& in, Target& target);
} // AutoDeserialize

namespace AutoSerialize
{
	void serialize(Serialize& out, const Target& target);
} // AutoSerialize

class Target
{
public:
	Target();
	Target(const Target& other);
	Target(const ScenePos& scenePos);
	Target(const Vector2f& pos, Scene *scene);
	Target(int x, int y, Scene *scene);
	Target(Entity *other);
	Target(const pf::Path *path);

	void set(const Vector2f& pos, Scene *scene);
	void set(int x, int y, Scene *scene);
	void set(Entity *other);
	void set(const pf::Path *path);
	void unset();

	bool absorbPathData();

	int getX() const;
	int getY() const;
	Vector2<float> getPos() const;
	Scene* getScene() const;
	ScenePos getScenePos() const;

	Entity* getEntity() const;

	bool isCalculating() const;

	explicit operator bool() const;
	

private:
	enum Mode
	{
		None,
		Coordinates,
		EntityFollow,
		PathingResultFollow
	};

	Mode mode;
	Ref<Entity> other;
	ScenePos scenePos;
	const pf::Path *path;

	friend void AutoDeserialize::deserialize(Deserialize& in, Target& target);
	friend void AutoSerialize::serialize(Serialize& out, const Target& target);
};

} // sl

#endif