#ifndef SL_NPCAPPEARANCE_HPP
#define SL_NPCAPPEARANCE_HPP

#include <set>
#include <string>

#include "Engine/Graphics/ColorSwapID.hpp"
#include "NPC/Stats/DNA.hpp"
#include "NPC/ClothingLayers.hpp"
#include "NPC/ClothingType.hpp"

namespace sl
{

class Soul;
class Serialize;
class Deserialize;

namespace gui
{
class GalleryTest;
} // gui

class NPCAppearance
{
public:
	NPCAppearance();

	NPCAppearance(std::string type, int ageDays, const DNA& dna);

	/**
	 * For delay-initialization (since Soul needs to be more initialized before we can iniitalize this, which is a member of Soul)
	 */
	NPCAppearance& init(const Soul& soul);

	void serialize(Serialize& out) const;

	void deserialize(Deserialize& in);


	const std::string& getType() const;

	void setType(std::string type);

	float getAgeYears() const;

	void setAge(int ageInDays);

	const ColorSwapID& getBodySwapID() const;

	const ColorSwapID& getClothingSwapID(ClothingType clothingType) const;

	const ColorSwapID& getHairSwapID() const;

	const std::string& getHair() const;

	void setHair(std::string newHair);

	const std::string& getClothing(ClothingType clothingType) const;

	const std::string& getClothingColour(ClothingType clothingType) const;

	void setClothing(ClothingType clothingType, const std::string& newClothing);

	void setClothingColour(ClothingType clothingType, const std::string& newColour);

	std::string getResourcePath() const;

	const std::set<std::string>& getCumLayers() const;

	void addCumLayer(const std::string& layer);

	bool removeCumLayer(const std::string& layer);

	//! Which clothes the loli has on (@TODO: put this somewhere else? how do we handle clothes that are "left" on the scene? or are they still in the loli's inventory???)
	ClothingLayers::Object clothingLayers;



private:
	void resetSwaps();

	std::string type;
	ColorSwapID bodySwapID;

	std::set<std::string> cumLayers;
	std::string clothing[CLOTHING_TYPE_COUNT];
	std::string clothingColour[CLOTHING_TYPE_COUNT];
	mutable bool isClothingSwapIDValid[CLOTHING_TYPE_COUNT];
	mutable ColorSwapID cachedClothingSwapID[CLOTHING_TYPE_COUNT];

	std::string hair;
	mutable bool isHairSwapIDValid; // because dying hair in future?
	mutable ColorSwapID cachedHairSwapID;

	// duplicate with player
	int ageDays;
	DNA expressedDNA;

	friend class gui::GalleryTest;
};

} // sl

#endif // SL_NPCAPPEARANCE_HPP