#include "NPC/NPCState.hpp"

#include "Engine/Scene.hpp"
#include "NPC/NPC.hpp"
#include "Town/World.hpp"

namespace sl
{

NPCState::NPCState(NPC& owner)
	:target(owner.target)
	,pos(owner.pos)
	,owner(owner)
{
}

Vector2<float>& NPCState::getPos()
{
	return owner.pos;
}

void NPCState::changeState(std::unique_ptr<NPCState> newState)
{
	owner.state.changeState(std::move(newState));
}

NPCMoveResult NPCState::moveTowardsTarget()
{
	return owner.moveTowardsTarget();
}

void NPCState::setPathManager(const std::string& manager)
{
	owner.pathManager = owner.getScene()->getWorld()->getPathManager(manager);
}

void NPCState::loadWalkAnims()
{
	owner.loadWalkAnims();
}

void NPCState::enterCar(Car& car)
{
	owner.enterCar(car);
}

void NPCState::exitCar()
{
	owner.exitCar();
}

void NPCState::face(const Vector2f& posToFace)
{
	owner.face(posToFace);
}

void NPCState::setTarget(const Target& target)
{
	owner.setTarget(target);
}

Target NPCState::setTargetViaPathing(const pf::NodePredicate& targetPred, const pf::WeightFunc& heuristic, const std::string& weightMap)
{
	return owner.setTargetViaPathing(targetPred, heuristic, weightMap);
}

bool NPCState::reportCrime(int crimeId)
{
	const bool previouslyReported = owner.getSoul()->isCrimeReported(crimeId);
	if (!previouslyReported)
	{
		owner.getSoul()->setCrimeReported(crimeId, true);
	}
	owner.getWorld()->getPolice().reportCrime(crimeId);
	return !previouslyReported;
}

} // sl
