#include "NPC/NPCStatusGUI.hpp"

#include <sstream>

#include "Engine/Graphics/SpriteAnimation.hpp"
#include "Engine/GUI/Button.hpp"
#include "Engine/GUI/DrawableContainer.hpp"
#include "Engine/GUI/NormalBackground.hpp"
#include "Engine/GUI/Scrollbar.hpp"
#include "Engine/GUI/WidgetGrid.hpp"
#include "Engine/GUI/TextLabel.hpp"
#include "NPC/ClothesDatabase.hpp"
#include "NPC/LayeredSpriteLoader.hpp"
#include "NPC/Stats/Soul.hpp"
#include "NPC/SoulConjugator.hpp"
#include "Utility/DataToString.hpp"
#include "Utility/Memory.hpp"

// HACK STUFF
#include "NPC/NPC.hpp"
#include "Engine/Scene.hpp"
#include "Utility/Random.hpp"

namespace sl
{

const int statusGUIWidth = 256;

const int statusGUIHeight = 288;

NPCStatusGUI::NPCStatusGUI(int x, int y, Soul& soul)
	:GUIWidget(Rect<int>(x, y, statusGUIWidth, statusGUIHeight))
	,soul(soul)
{
	const gui::NormalBackground background(sf::Color::Black, sf::Color::White);

	applyBackground(background);

	const int buttonHeight = 24;
	const int tabs = 4;
	const int buttonWidth = box.width / tabs;
	int button = 0;
	auto addSwitchButton = [&](const std::string& name, Tab tab)
	{
		auto changeTabButton = gui::Button::createGenericButton(Rect<int>(box.left + button * buttonWidth, box.top, buttonWidth, buttonHeight), name, [tab,this]() {
			changeTab(tab);
		});
		addWidget(std::move(changeTabButton));
		++button;
	};
	addSwitchButton("Main", Tab::Main);
	addSwitchButton("Clothing", Tab::Clothing);
	addSwitchButton("Health", Tab::Health);
	addSwitchButton("Sexual", Tab::Sexual);
	//addSwitchButton("Mating", Tab::Mating);

	changeTab(Tab::Main);
}

void NPCStatusGUI::changeTab(Tab tab)
{
	for (GUIWidget *widget : substateWidgets)
	{
		removeWidget(*widget);
	}
	substateWidgets.clear();
	switch (tab)
	{
	case Tab::Main:
		{
			/*const Rect<int> nameBox(box.left + 32, box.top, 128, 32);
			auto name = std::make_unique<gui::TextLabel>(nameBox, SoulConjugator(soul).fullName(), gui::TextLabel::Left, gui::TextLabel::Middle, 0, 0);

			// preview image (part of name widget)
			auto npcPreview = unique<gui::DrawableContainer>(Rect<int>(box.left, box.top, 32, 32));
			npcPreview->setDrawable(LayeredSpriteLoader::create(soul.appearance, LayeredSpriteLoader::Config("walk", 32, 666, -4, -4), "south"));
			addSubstateWidget(std::move(npcPreview));

			addSubstateWidget(std::move(name));*/


			createStatDisplay(0, 0, "Age", formatAge(soul.getAgeDays()));
			createStatDisplay(0, 1, "Height", formatHeight(soul.getHeight()));
			createStatDisplay(0, 2, "Weight", formatWeight(soul.getWeight()));
			createStatDisplay(0, 3, "Nationality", soul.getNationality());
			//createStatDisplay(0, 4, "Wearing", soul.getClothing(ClothingType::Outfit) + " + " + soul.getClothing(ClothingType::Accessory));
		}
		break;
	case Tab::Health:
	{
		const float weightGrams = soul.getWeight() / 1000.f;
		const Soul::Status& status = soul.getStatus();
		createStatDisplay(0, 0, "Bodyfat", formatPercent(status.storedFat / weightGrams));
		createStatDisplay(0, 1, "Muscle", formatPercent(status.muscle / weightGrams));
		createStatDisplay(0, 2, "Hunger", formatPercent(status.hunger));
		createStatDisplay(0, 3, "Thirst", formatPercent(status.thirst));
		createStatDisplay(0, 4, "Nourishment", status.nourishment);
	}
	break;
	case Tab::Clothing:
		{
			ClothesDatabase& db = ClothesDatabase::get();
			int listLength = 0;
			for (const auto& clothes : db)
			{
				listLength += clothes.second.swaps.size();
			}
			
			auto clothesList = unique<gui::WidgetGrid>(0, 0, 1, listLength);
			int index = 0;
			const int listItemHeight = 32;
			for (const auto& clothes : db)
			{
				for (auto& swap : clothes.second.swaps)
				{
					const std::string clothSwapName = clothes.first + " (" + swap.first + ")";
					const ClothingType type = clothes.second.type;
					auto swapButton = gui::Button::createGenericButton(Rect<int>(0, index * listItemHeight, box.width - 16, listItemHeight), clothSwapName, [&, type](){
						soul.appearance.setClothing(type, clothes.first);
						soul.appearance.setClothingColour(type, swap.first);
						changeTab(Tab::Main);
					});
					clothesList->setWidget(0, index++, std::move(swapButton));
				}
			}
			auto scrollWidget = unique<gui::Scrollbar>(box);
			scrollWidget->setScrollArea(std::move(clothesList));
			addSubstateWidget(std::move(scrollWidget));
		}
		break;
	case Tab::Sexual:
		{
			createStatDisplay(0, 0, "Vaginal Circ", formatLength(soul.getVagTightness()));
			createStatDisplay(0, 1, "Anal Circ", formatLength(soul.getAssTightness()));
			createStatDisplay(0, 2, "Vaginal Tearing", soul.getStatus().vagDamage);
			createStatDisplay(0, 3, "Anal Tearing", soul.getStatus().assDamage);
		}
		break;
	case Tab::Mating:
		{
			//MASSIVE FUCKING HACK AHEAD JUST FOR THE DEMO
			Scene *physScene = static_cast<Scene*>(scene);
			std::list<NPC*> npcs;
			physScene->cull(npcs, Rect<int>(0, 0, physScene->getWidth(), physScene->getHeight()));
			NPC *soulsNPC = nullptr;
			std::vector<NPC*> potentialMates;
			for (NPC *npc : npcs)
			{
				Soul *npcSoul = npc->getSoul().get();
				if (npcSoul->getID() != soul.getID())
				{
					if (npcSoul->getPubertyDays() >= MENARCHE)
					{
						potentialMates.push_back(npc);
					}
				}
				else
				{
					soulsNPC = npc;
				}
			}
			ASSERT(soulsNPC);
			auto mateList = unique<gui::WidgetGrid>(0, 0, 2, potentialMates.size());
			int index = 0;
			const int listItemHeight = 32;
			for (NPC *mate : potentialMates)
			{
				// preview icon
				const int previewWidth = 32;
				auto npcPreview = unique<gui::DrawableContainer>(Rect<int>(0, index * listItemHeight, previewWidth, listItemHeight));
				npcPreview->setDrawable(LayeredSpriteLoader::create(LayeredSpriteLoader::Config(soul.appearance, "walk", 32, Vector2i(-4, -4)), "south"));
				
				
				// mate button
				auto swapButton = gui::Button::createGenericButton(Rect<int>(previewWidth, index * listItemHeight, box.width - 16, listItemHeight), SoulConjugator(*mate->getSoul()).fullName(), [this, mate, soulsNPC](){
					soulsNPC->getSoul()->mate(*scene->getWorld(), *mate->getSoul(), Random::get());
					mate->getSoul()->mate(*scene->getWorld(), soul, Random::get());

					//soulsNPC->loadPregnantAnims();
					//mate->loadPregnantAnims();

					changeTab(Tab::Main);
				});
				
				
				mateList->setWidget(0, index, std::move(npcPreview));
				mateList->setWidget(1, index, std::move(swapButton));
				++index;
			}
			auto scrollWidget = unique<gui::Scrollbar>(box);
			scrollWidget->setScrollArea(std::move(mateList));
			addSubstateWidget(std::move(scrollWidget));
		}
		break;
	}
}

void NPCStatusGUI::createStatDisplay(int col, int row, const std::string& name, float value)
{
	std::stringstream ss;
	ss << value;
	createStatDisplay(col, row, name, ss.str());
}

void NPCStatusGUI::createStatDisplay(int col, int row, const std::string& name, const std::string& value)
{
	const int initialXOffset = 24;
	const int initialYOffset = 32 + 16; // top + switch tabs
	const int colSize = 222;
	const int colGap = 16;
	const int rowSize = 16;
	const int rowGap = 4;
	const Rect<int> slotBox(box.left + initialXOffset + col * (colSize + colGap), box.top + initialYOffset + row * (rowSize + rowGap), colSize, rowSize);
	auto nameLabel = unique<gui::TextLabel>(Rect<int>(slotBox.left, slotBox.top, 128, slotBox.height), name, gui::TextLabel::Config().valign(gui::TextLabel::Middle));
	nameLabel->shrinkHorizontally();
	const int valueOffset = nameLabel->getBox().width;
	const Rect<int> valueBox(slotBox.left + valueOffset, slotBox.top, slotBox.width - valueOffset, slotBox.height);
	auto valueLabel = unique<gui::TextLabel>(valueBox, value, gui::TextLabel::Config().halign(gui::TextLabel::Right).valign(gui::TextLabel::Middle));
	addSubstateWidget(std::move(nameLabel));
	addSubstateWidget(std::move(valueLabel));
}

void NPCStatusGUI::addSubstateWidget(std::unique_ptr<gui::GUIWidget> widget)
{
	substateWidgets.push_back(widget.get());
	addWidget(std::move(widget));
}

} // sl
