#include "NPC/NPCTextBox.hpp"

#include "Engine/Graphics/SpriteText.hpp"
#include "Utility/Memory.hpp"
#include "Utility/TypeToString.hpp"

namespace sl
{

DECLARE_TYPE_TO_STRING(NPCTextBox)

NPCTextBox::NPCTextBox(const Entity& owner, std::string message)
	:Entity(owner.getPos().x, owner.getPos().y, getWidth(message), getHeight(message), "NPCTextBox")
	,link(*this)
	,lifetime(30*2.9)
	,owner(&owner)
	,offset(32.f)
	,message(std::move(message))
{
	auto text = unique<SpriteText>("default", getMask().getBoundingBox(), 0, 1, this->message);
	textBox = text.get();
	text->setDepth(-1000);
	addDrawable(std::move(text));
}

const Types::Type NPCTextBox::getType() const
{
	return makeType("NPCTextBox");
}

const std::string& NPCTextBox::getMessage() const
{
	return message;
}

// private
void NPCTextBox::onUpdate()
{
	setPos(owner->getPos() - Vector2f(getMask().getBoundingBox().width / 2, offset));
	if (--lifetime == 0)
	{
		destroy();
	}
	offset += 0.2f;
}

/*static*/ int NPCTextBox::getWidth(const std::string& message)
{
	// TODO make this not gross
	return 8 * message.size();
}

/*static*/ int NPCTextBox::getHeight(const std::string& message)
{
	return 12;
}

} // sl
