#include "NPC/NamedEntity.hpp"

#include "Engine/Scene.hpp"

namespace sl
{

NamedEntity::NamedEntity(DeserializeConstructor)
	:Entity(0, 0, 0, 0, "invalid - NamedEntity")
	,name("not deserialized yet - NamedEntity")
	,entity(nullptr)
{
}

NamedEntity::NamedEntity(const std::string& name, Entity *entity, const Vector2f& offset)
	:Entity(entity->getPos().x + offset.x, entity->getPos().y + offset.y, entity->getMask().getBoundingBox().width, entity->getMask().getBoundingBox().height, name)
	,name(std::move(name))
	,entity(entity)
{
}

void NamedEntity::onUpdate()
{
	//setPos(entity->getMask().getBoundingBox().topLeft());
	//if (scene != entity->getScene())
	//{
	//	entity->getScene()->transferEntity(this);
	//}

	// no need to tag something that disappears, right?
	if (!entity)
	{
		destroy();
	}
}

const Entity::Type NamedEntity::getType() const
{
	return name;
}

Entity* NamedEntity::getEntity()
{
	return entity.get();
}

std::string NamedEntity::serializationType() const
{
	return "NamedEntity";
}

bool NamedEntity::shouldSerialize() const
{
	return true;
}

void NamedEntity::serialize(Serialize& out) const
{
	out.startObject("NamedEntity");
	SAVEVAR(s, name);
	SAVEREF(entity);
	out.endObject("NamedEntity");
}

void NamedEntity::deserialize(Deserialize& in)
{
	in.startObject("NamedEntity");
	LOADVAR(s, name);
	LOADREF(entity);
	in.endObject("NamedEntity");
}

/*static*/void NamedEntity::tag(const std::string& name, Entity *entity, const Vector2f& offset)
{
	// but maybe only support static entities for now until we need to support more
	ASSERT(entity->isStatic());
	Scene *scene = entity->getScene();
	ASSERT(scene);
	// hopefully this isn't a performance issue for now
	scene->addEntityToList(new NamedEntity(name, entity, offset));
#ifdef SL_DEBUG
	// let's just make sure there aren't a shit-tonne of them in a scene then...
	std::list<Entity*> entitiesHere;
	scene->cull(entitiesHere, Rect<int>(scene->getWidth(), scene->getHeight()), name);
	ASSERT(entitiesHere.size() < 256);
#endif // SL_DEBUG
}

} // sl