#include "NPC/Region.hpp"

namespace sl
{

Region::Region(const Rect<int>& bounds, std::string regionType, int priority)
	:Entity(bounds.left, bounds.top, bounds.width, bounds.height, "Region")
	,regionType(std::move(regionType))
	,priority(priority)
	,suspiciousScenario(nullptr)
{
}

void Region::onUpdate()
{
}

const Entity::Type Region::getType() const
{
	return "Region";
}

const std::string& Region::getRegionType() const
{
	return regionType;
}

int Region::getPriority() const
{
	return priority;
}

const SuspiciousScenario* Region::getScenario() const
{
	return suspiciousScenario;
}

Region& Region::setSuspiciousScenario(const SuspiciousScenario *scenario)
{
	suspiciousScenario = scenario;
	return *this;
}

Region& Region::setRegionBounds(const Rect<int>& bounds)
{
	setCollisionMask(CollisionMask(0, 0, bounds.width, bounds.height));
	setPos(Vector2f(bounds.left, bounds.top));
	return *this;
}

void Region::deserialize(Deserialize& in)
{
	deserializeEntity(in);
	LOADVAR(s, regionType);
	LOADVAR(i, priority);
	const int width = in.i("width");
	const int height = in.i("height");
	setCollisionMask(CollisionMask(0, 0, width, height));

	// TODO serialize suspiciousScenario somehow?
}

void Region::serialize(Serialize& out) const
{
	serializeEntity(out);
	SAVEVAR(s, regionType);
	SAVEVAR(i, priority);
	// mask not (yet) saved via serializeEntity, so manually do it
	out.i("width", getMask().getBoundingBox().width);
	out.i("height", getMask().getBoundingBox().height);
}

} // sl