/**
 * @DESCRIPTION
 * 
 */
#ifndef SL_SCHEDULE_HPP
#define SL_SCHEDULE_HPP

#include "Engine/ScenePos.hpp"
#include "Utility/Time/GameTime.hpp"
#include "Utility/Time/TimeIntervalGenerator.hpp"

#include <functional>
#include <string>
#include <utility>
#include <vector>

namespace sl
{

class NPC;
class Soul;
class World;

namespace pf
{
class Path;
} // pf

class Schedule
{
public:
	static constexpr float unavailable = -1.f;
	static const std::pair<std::string, float> idle;

	/**
	 * @param time The time to search at
	 * @return The activiy id + priority with the highest priority at the given search time, or Schedule::idle if none are found
	 */
	std::pair<std::string, float> current(const Soul& soul, const GameTime& time) const;

	/**
	 * @param time The time to search at
	 * @return All activiy ids + priorities within the given search time. Does not include idle. Sorted with highest priority first.
	 */
	std::vector<std::pair<std::string, float>> allCurrent(const Soul& soul, const GameTime& time) const;

	class InterpolatedSchedule
	{
	public:
		struct Slot
		{
			std::string id;
			GameTime start;
			GameTime end;
			//bool interuptable;
		};
		void add(Slot slot);

		class Interpolation
		{
		public:
			float priority(const Soul& soul) const;

			ScenePos location(NPC& npc, const World& world) const;

			bool inBetween() const;

			float startContrib() const;
			
			float endContrib() const;

			std::shared_ptr<const pf::Path> path(NPC& npc, const World& world) const;

			std::string startId;
			std::string endId;
			float progress;
			
		};
		Interpolation interpolate(const GameTime& time) const;

	private:
	
		std::vector<Slot> slots;
	};
	InterpolatedSchedule computeSchedule(const Soul& soul, const GameTime& start, const GameTime& end) const;

	/**
	 * @param id The id to register
	 * @param priority The priority function for a given activity. This doubles as an availability check - simply return unavailable(-1.f) if something is unavailable.
	 */
	void registerActivity(std::string id, std::string location, std::unique_ptr<const TimeIntervalGenerator>&& time, std::function<float(const Soul&)> priority);

	void registerActivity(std::string id, std::string location, std::unique_ptr<const TimeIntervalGenerator>&& time, float fixedPriority);

	// TODO: reconsider whether this is worth exposing
	float getPriority(const std::string& id, const Soul& soul) const;

	// this too
	const std::string& getLocationId(const std::string& id) const;


private:

	struct Activity
	{
		// We might need to expose this if the btrees need it but it might be unnecessary when we use
		// this schedule thing more for background tasks too, then we can only use other priorities in the btree
		// for unpredictable events of other priorities.
		std::string id;
		std::string location;
		std::unique_ptr<const TimeIntervalGenerator> time;
		std::function<float(const Soul&)> priority;
	};
	std::vector<Activity> activities;
};

} // sl

#endif // SL_SCHEDULE_HPP