#include "NPC/ScheduleGeneration.hpp"

#include "Engine/Scene.hpp"
#include "NPC/Locators/EntityLocator.hpp"
#include "NPC/Locators/FixedTargetLocator.hpp"
#include "NPC/Locators/PathfindingTargetLocator.hpp"
#include "NPC/Stats/Soul.hpp"
#include "NPC/NPC.hpp"
#include "Pathfinding/Heuristics.hpp"
#include "Town/World.hpp"
#include "Utility/Time/RecurringTimeIntervalGenerator.hpp"
#include "Utility/Logger.hpp"
#include "Utility/Memory.hpp"

namespace sl
{

static constexpr int hours = 60 * 60;

bool registerLocations(Soul& soul, World& world)
{
	if (soul.getType() == "loli")
	{
		soul.setLocation("usual_park", unique<EntityLocator>("ParkMarker"));

		{
			Entity *desk = world.getSchoolRegistry().getDesk(soul.getID());
			if (desk == nullptr)
			{
				Logger::log(Logger::NPC, Logger::Debug) << " could not enroll NPC in school";
				return false;
			}
			soul.setKnownEntity("school_desk", desk);
		}
	}

	return true;
}

Schedule makeSchoolSchedule(NPC& npc, const World& world)
{
	Schedule schedule;
	
	// Travel time in seconds between two locations
	constexpr float expectedSlowestWalkSpeed = 1.5f;
	const float distPerSecond = expectedSlowestWalkSpeed / world.getTimeRate();
	auto travelTime = [&](const std::string& from, const std::string& to) -> int {
		const float dist = npc.getSoul()->locationPathCache.getPath(from, to, npc, *world.getPathManager("StaticGeometry"), NPC::weightsMap.at("npc_routine"), 5.f)->totalSceneDistance();
		return dist / distPerSecond;
	};

	// TODO: make them go to sleep before midnight once you let overflowing time periods apply to RecurringTimeInterval
	const int timeHomeToSchool = travelTime("bedroom", "school_desk");
	const int timeSchoolToPark = travelTime("school_desk", "usual_park");
	const int timeParkToHome = travelTime("usual_park", "bedroom");
	const int timeHomeToPark = travelTime("bedroom", "usual_park");
	// Weekdays
	schedule.registerActivity(
		"school",
		"school_desk",
		std::make_unique<RecurringTimeIntervalGenerator>(
			RecurringTimeInterval(
				TimeOfDay(9, 0),
				hours * 6,
				RecurringTimeInterval::AnyDayOfWeek)//Weekdays)
			),
		0.15f);
	schedule.registerActivity(
		"play_in_park",
		"usual_park",
		std::make_unique<RecurringTimeIntervalGenerator>(
			RecurringTimeInterval(
				TimeOfDay(15 * hours + timeSchoolToPark),
				9 * hours - timeSchoolToPark - timeParkToHome,
				RecurringTimeInterval::AnyDayOfWeek)//Weekdays)
			),
		0.15f);
	schedule.registerActivity(
		"sleep",
		"bedroom",
		std::make_unique<RecurringTimeIntervalGenerator>(
			RecurringTimeInterval(
				TimeOfDay(0, 0),
				9 * hours - timeHomeToSchool,
				RecurringTimeInterval::AnyDayOfWeek)//Weekdays)
			),
		0.15f);// (wtf? now the others take priority of this one too??)1000.f);//0.15f); // DEMO HACK

	//TODO: reneable DEMO HACK
	//// Weekend
	//schedule.registerActivity(
	//	"play_in_park_weekend",
	//	"usual_park",
	//	std::make_unique<RecurringTimeIntervalGenerator>(
	//		RecurringTimeInterval(
	//			TimeOfDay(10, 0),
	//			14 * hours - timeParkToHome,
	//			RecurringTimeInterval::Weekend)
	//		),
	//	0.15f);
	//schedule.registerActivity(
	//	"sleep_weekend",
	//	"bedroom",
	//	std::make_unique<RecurringTimeIntervalGenerator>(
	//		RecurringTimeInterval(
	//			TimeOfDay(0, 0),
	//			10 * hours - timeHomeToPark,
	//			RecurringTimeInterval::Weekend)
	//		),
	//	0.15f);

	return schedule;
}

} // sl