#include "NPC/SchoolRegistry.hpp"

#include "Engine/Serialization/AutoDeserialize.hpp"
#include "Engine/Serialization/AutoSerialize.hpp"
#include "NPC/NPC.hpp"

namespace sl
{

void SchoolRegistry::registerNPC(NPC& npc)
{
	const Soul::ID id = npc.getSoul()->getID();
	ASSERT(assignedDesks.count(id) == 0);
	desklessNPCs.insert(std::make_pair(id, &npc));
}

void SchoolRegistry::registerDesk(Entity *desk)
{
	freeDesks.push_back(desk);
}

Entity* SchoolRegistry::getDesk(Soul::ID id)
{
	auto assigned = assignedDesks.find(id);
	if (assigned == assignedDesks.end())
	{
		auto it = desklessNPCs.find(id);
		if (it == desklessNPCs.end())
		{
			ERROR("please register NPCs before trying to get their desks: id = " + std::to_string(id));
			return nullptr;
		}
		if (freeDesks.empty())
		{
			ERROR("no free desks");
			return nullptr;
		}
		// greedily assign to closest for now - hopefully this won't be an issue
		int closestIndex = 0;
		for (int i = 1; i < freeDesks.size(); ++i)
		{
			const float dist = worldDistance(*(it->second), *freeDesks[i]);
			if (dist < worldDistance(*(it->second), *freeDesks[closestIndex]))
			{
				closestIndex = i;
			}
		}
		assigned = assignedDesks.insert(std::make_pair(id, freeDesks[closestIndex])).first;
		freeDesks[closestIndex] = freeDesks.back();
		freeDesks.pop_back();
	}
	return assigned->second;
}


void SchoolRegistry::serialize(Serialize& out) const
{
	AutoSerialize::serialize(out, freeDesks);
	AutoSerialize::serialize(out, assignedDesks);
	AutoSerialize::serialize(out, desklessNPCs);
}

void SchoolRegistry::deserialize(Deserialize& in)
{
	AutoDeserialize::deserialize(in, freeDesks);
	AutoDeserialize::deserialize(in, assignedDesks);
	AutoDeserialize::deserialize(in, desklessNPCs);
}

} // sl
