#include "NPC/States/DisabledState.hpp"

#include "Engine/Game.hpp"
#include "Engine/Scene.hpp"
#include "Town/World.hpp"
#include "NPC/LayeredSpriteLoader.hpp"
#include "NPC/NPC.hpp"
#include "Player/Player.hpp"
#include "Utility/Memory.hpp"


namespace sl
{

DisabledState::DisabledState(NPC& owner)
	:NPCState(owner)
	,zzz(0)
{
	owner.clearDrawablesInSet("wiggle");
	owner.addDrawableToSet(
		"wiggle",
		LayeredSpriteLoader::create(
			LayeredSpriteLoader::Config(
				owner.getSoul()->appearance,
				"tied",
				32,
				Vector2i(16, 24)).animate(9),
			"wiggle",
			owner.getFacingDir() == East || owner.getFacingDir() == North ? false : true));
	owner.setDrawableSet("wiggle");
}

void DisabledState::update()
{
	const Soul& soul = *owner.getSoul();
	if (soul.isAlive())
	{
		if (soul.getStatus().chloroformTimer > 0)
		{
			if (zzz <= 0)
			{
				owner.makeTextBubble("Zzzz...");
				zzz = 300;
			}
			owner.setAnimationSpeed(0.f);
		}
		else
		{
			if (zzz <= 0)
			{
				if (soul.getFlags().get("gagged").asBool())
				{
					owner.makeTextBubble("Mmmmph!");
				}
				else
				{
					owner.makeTextBubble("Help!");
				}
				zzz = 120;
			}
			Random& rng = Random::get();
			if (rng.chance(0.1f))
			{
				owner.tryMoveRaycast(Vector2f(rng.randomFloat(-4.f, 4.f), rng.randomFloat(-4.f, 4.f)));
			}
			owner.setAnimationSpeed(rng.randomFloat(0.5f, 1.f));
		}
	}
	else
	{
		// nothing, really...
	}
}

void DisabledState::handleEvent(const Event& event)
{
}

} // sl
