#include "NPC/States/DraggedState.hpp"

#include "Engine/Game.hpp"
#include "NPC/LayeredSpriteLoader.hpp"
#include "NPC/NPC.hpp"
#include "Player/Player.hpp"
#include "Police/CrimeEntity.hpp"
#include "Town/World.hpp"
#include "Utility/Random.hpp"
#include "Utility/Memory.hpp"

namespace sl
{

DraggedState::DraggedState(NPC& owner, Player& player)
	:NPCState(owner)
	,player(player)
	,kidnappingCrime(nullptr)
	,t(0)
{
	LayeredSpriteLoader::load(owner, LayeredSpriteLoader::Config(owner.getSoul()->appearance, "dragged", 32, Vector2i(18, 28)).animate(3));
	auto applyDraggedDepthOffsets = [&](const std::string& key, int draggedOffset) {
		Drawable& drawable = *owner.getDrawable(key);
		drawable.setBottomPointOffset(drawable.getBottomPointOffset() + draggedOffset);
	};
	// to make sure when you're dragging her up she appears in front of you
	applyDraggedDepthOffsets("west", -1);
	applyDraggedDepthOffsets("east", -1);
	applyDraggedDepthOffsets("north", 1);
	applyDraggedDepthOffsets("south", -1);

	Random& rng = Random::get();
	float y = 0.f;
	float min = y;
	float max = y;
	for (int t = 0; t < n; ++t)
	{
		brownian[t] = y;
		if (min > y) min = y;
		if (max < y) max = y;
		y += rng.randomFromNormalDistribution(0.f, 1.f);
	}
	const float range = max - min;
	//std::cout << "B = ";
	for (int t = 0; t < n; ++t)
	{
		brownian[t] = (brownian[t] - min) / range;
		//std::cout << brownian[t], ", ";
	}
	//std::cout << std::endl;
	owner.setAnimationSpeed(0.f);
}

void DraggedState::update()
{

	//Random& rng = Random::get();
	//if (rng.chance(1.f / 60.f))
	//{
	//	owner.setAnimationSpeed(rng.randomFloat(1.f));
	//}
	kidnappingCrime->setPos(getPos());
	int& chloroTimer = owner.getSoul()->getStatus().chloroformTimer;
	if (chloroTimer > 0)
	{
		--chloroTimer;
		owner.setAnimationSpeed(0.f);
	}
	else
	{
		static int screamCount = 0;
		if (screamCount-- <= 0)
		{
			if (owner.getSoul()->getFear() < 0.3f)
			{
				owner.makeTextBubble(Random::get().choose("Help me!", "Someone get him off me!", "HEEELPP!!!"));
			}
			screamCount = 120;
		}
		kidnappingCrime->noiseImpulse();
		owner.setAnimationSpeed(brownian[t++ % n]);
	}
	owner.setPos(player.getPos());
	owner.setDrawableSet(player.getCurrentDrawableSet());
}

void DraggedState::onEnter()
{
	ASSERT(kidnappingCrime == nullptr);
	const auto& crimes = owner.getSoul()->getKnownCrimes();
	auto it = std::find_if(crimes.begin(), crimes.end(), [this](const std::pair<int, bool>& pair) {
		const Crime *crime = player.getWorld()->getPolice().getCriminalRecords().getCrime(pair.first);
		return crime->victim == owner.getSoul()->getID() && (crime->type == Crime::Type::Kidnapping);
	});
	ASSERT(it != crimes.end());
	kidnappingCrime = new CrimeEntity(getPos(), it->first, false, CrimeEntity::INDEFINITE);
	owner.getScene()->addEntityToList(kidnappingCrime);
}

void DraggedState::onExit()
{
	ASSERT(kidnappingCrime);
	kidnappingCrime->destroy();
}

} // sl
