#include "NPC/States/IdleState.hpp"

#include "Engine/Events/Event.hpp"
#include "NPC/NPC.hpp"
#include "NPC/Stealth/Noise.hpp"
#include "NPC/Locators/EntityLocator.hpp"
#include "NPC/Locators/FixedTargetLocator.hpp"
#include "NPC/Behaviors/LoliBehaviors.hpp"
#include "NPC/LayeredSpriteLoader.hpp"
#include "Player/Player.hpp"
#include "Town/Town.hpp"
#include "Utility/Random.hpp"

namespace sl
{

IdleState::IdleState(NPC& owner)
	:NPCState(owner)
	,off(Random::get().random(32) - 16, Random::get().random(32) - 16)
	,puddle(nullptr)
	,peeCooldown(0)
{
	loadWalkAnims();
	//const int rot = 90;
	//const int xoff = 6;
	//const int yoff = 12;
	//@ todo fix to work with layers (add in rotate param???)
	//const std::string fname = owner.getSoul()->getType() + "/walk/south.png";
	//auto loli = unique<SpriteAnimation>(fname, 24, 128, true, owner.getSoul()->getBodySwapID(), xoff, yoff);
	//loli->rotate(rot);
	//loli->setDepth(-100);
	//owner.addDrawableToSet("chloro", std::move(loli));

	const int rot = 90;
	const int xoff = 6;
	const int yoff = 12;
	//const std::string fname = owner.getSoul()->getType() + "/walk/" + owner.getSoul()->getClothing(ClothingType::Outfit) + "/south.png";
	// @layers
	LayeredSpriteLoader::Config config(owner.getSoul()->appearance, "walk", 32, Vector2i(16, 24));
	auto onSide = LayeredSpriteLoader::create(config, "down");//unique<SpriteAnimation>(fname, 24, 128, true, kidnapee->getColorSwapID(), xoff, yoff);
	onSide->rotate(rot);
	onSide->setDepth(-100);
	onSide->update(-21, 12);
	owner.addDrawableToSet("chloro", std::move(onSide));

	owner.setDrawableSet("chloro");
	owner.setDrawableSet(Random::get().choose({"north", "east", "west", "south", "south", "south"}));

	behavior = ai::testNode();
	//behavior = ai::captiveNode();
}

void IdleState::update()
{
	//const int randomMessage = Random::get().random(1000);
	//if (randomMessage < 5)
	//{
	//	owner.makeTextBubble("nyaaaa~");
	//}
	//else if(randomMessage < 7)
	//{
	//	owner.makeTextBubble("m-master~ pet me~");
	//}
	//else if (randomMessage < 10)
	//{
	//	owner.makeTextBubble("*purrr*  ^~^");
	//}

	owner.setAnimationSpeed(0.f);
	auto& chloroformTimer = owner.getSoul()->getStatus().chloroformTimer;
	if (chloroformTimer > 0)
	{
		--chloroformTimer;
		owner.setDrawableSet("chloro");
	}
	else
	{
		// to stop them from walking away while peeing!
#ifdef SL_DEBUG
		behavior->debugExecute(owner.getBehaviorContext());
#else
		behavior->execute(owner.getBehaviorContext());
#endif
	}

	//if (target)
	//{
	//	const NPCMoveResult result = moveTowardsTarget();
	//	if (result == NPCMoveResult::TargetReached)
	//	{
	//		findNewWaypoint();
	//	}
	//}
	//else
	//{
	//	findNewWaypoint();
	//}
#ifdef SL_DEBUG
	if (owner.debugDrawBehaviorTree)
	{
		behavior->drawAsTree(*owner.getScene(), pos, 400);//owner.getScene()->getMousePos(), 400);
	}
#endif // SL_DEBUG
	for (float dir = 0.f; dir < 360.f; dir += 12.f)
	{
		owner.canSee(pos + lengthdir(360.f, dir));
	}
}

void IdleState::onSceneEnter()
{
	setPathManager("StaticGeometry");
}

void IdleState::findNewWaypoint()
{
	const int townWidth = owner.getScene()->getWidth();
	const int townHeight = owner.getScene()->getHeight();
	do
	{
		target.set(townWidth / 4 + Random::get().random(townWidth / 2), townHeight / 4 + Random::get().random(townHeight / 2), owner.getScene());
	}
	while (!isSidewalk(static_cast<Town*>(owner.getScene())->getTileAt(target.getX(), target.getY())));
	//owner.getScene()->checkCollision(Rect<int>(target.getX() - 12, target.getY() - 12, 24, 24), "StaticGeometry") != nullptr);
}

} // sl