#include "NPC/States/LoliRoutineState.hpp"

#include "Engine/Game.hpp"
#include "Engine/Scene.hpp"
#include "Engine/Settings.hpp"
#include "Town/World.hpp"
#include "Pathfinding/PathManager.hpp"
#include "NPC/Behaviors/LoliBehaviors.hpp"
#include "NPC/Locators/FixedTargetLocator.hpp"
#include "NPC/NPC.hpp"
#include "Player/Player.hpp"
#include "Utility/Memory.hpp"

// TODO: remove
#ifndef SL_NOMULTITHREADED
	#include <thread>
#endif // !SL_NOMULTITHREADED
#include "NPC/ScheduleGeneration.hpp"

namespace sl
{

LoliRoutineState::LoliRoutineState(NPC& owner)
	:NPCState(owner)
{
	behavior = ai::routineNode();

	loadWalkAnims();

	owner.setDrawableSet("south");

	owner.setPathfindingWeightMap("npc_routine");
}

void LoliRoutineState::update()
{
	// This is here since if we run pathfinding before the 1st update
	// of the world, the interiors of the houses are for some reason impassable.
	// This might have to do with delay add/remove entities on WallGrid or something
	// TODO remove
	auto soul = owner.getSoul();
	if (soul->getType() == "loli" && !soul->getFlags().get("captive").asBool())
	{
		if (owner.getSoul()->scheduleValid == 2)
		{
#ifndef SL_NOMULTITHREADED
			std::thread scheduleThread([soul]() {
#endif // !SL_NOMULTITHREADED
				soul->schedule = makeSchoolSchedule(*soul->getNPC(), *soul->getNPC()->getWorld());
				soul->scheduleValid = 69;
#ifndef SL_NOMULTITHREADED
			});
			scheduleThread.detach();
#endif // !SL_NOMULTITHREADED
		}
		if (soul->scheduleValid <= 2)
		{
			soul->scheduleValid++;
			return;
		}
	}

	//Player *player = owner.getWorld()->getPlayer();
	//if (player->getScene() != owner.getScene())
	//{
	//	owner.getScene()->transferEntity(player);
	//	player->setPos(pos);

	//}
	owner.setAnimationSpeed(0.f);
#ifdef SL_DEBUG
	behavior->debugExecute(owner.getBehaviorContext());

	if (owner.debugDrawBehaviorTree && owner.getScene()->getGame()->getSettings().get("draw_btree"))
	{
		behavior->drawAsTree(*owner.getScene(), pos, 400);
	}
#else
	behavior->execute(owner.getBehaviorContext());
#endif // SL_DEBUG
}

void LoliRoutineState::handleEvent(const Event& event)
{
	if (event.getCategory() == Event::Category::NonBroadcasted && event.getName() == "update_drawables")
	{
		loadWalkAnims();
	}
}

void LoliRoutineState::onEnter()
{
	ASSERT(owner.getScene() != nullptr);
	setPathManager("StaticGeometry");
}

} // sl
