#include "NPC/States/SleepingState.hpp"

#include "Town/World.hpp"
#include "Engine/Events/Event.hpp"
#include "NPC/Behaviors/AcceptTalkWithPlayer.hpp"
#include "NPC/States/LoliRoutineState.hpp"
#include "NPC/NPC.hpp"
#include "NPC/Bed.hpp"
#include "Player/Player.hpp"
#include "Utility/Random.hpp"
#include "Utility/Memory.hpp"

namespace sl
{
// TODO: delete this entire class if we migrate fully to btree for activities.
SleepingState::SleepingState(NPC& owner, const GameTime& wakeupTime, Bed *bed)
	:NPCState(owner)
	,wakeupTime(wakeupTime)
	,bed(bed)
{
	behavior = unique<ai::AcceptTalkWithPlayer>("sleeping_begin");
}

void SleepingState::update()
{
	if (wakeupTime < owner.getScene()->getWorld()->getGameTime())
	{
		// @hack to stop non-kidnapped lolis from stockpiling like 100 liters of pee
		// either remove when non-kidnapped lolis have toilets, or when you decide to disallow them from drinking liquids
		owner.getSoul()->getStatus().bladderContent.fluid = 0;
		owner.getSoul()->getStatus().bladderContent.urobilin = 0;

		if (bed)
		{
			const int bedExitPositions = bed->getExitPositions().size();
			Vector2f exitPos;
			if (bedExitPositions > 1)
			{
				// get out of bed at a random valid bed exit position
				const int bedIndexOffset = Random::get().random(bedExitPositions);
				for (int bedIndex = 0; bedIndex < bedExitPositions; ++bedIndex)
				{
					const Vector2f p = bed->getExitPositions()[(bedIndexOffset + bedIndex) % bedExitPositions];
					const int w = owner.getMask().getBoundingBox().width;
					const int h = owner.getMask().getBoundingBox().height;
					Rect<int> exitArea(p.x - 2/w, p.y - h/2, w, h);
					if (owner.getScene()->checkCollision(exitArea, "StaticGeometry") == nullptr)
					{
						exitPos = bed->getExitPositions()[(bedIndexOffset + bedIndex) % bedExitPositions];
						break;
					}
				}
			}
			else
			{
				exitPos = bed->getExitPos(Random::get());
			}
			pos = exitPos;
			bed->setOccupant(nullptr);
		}
		changeState(unique<LoliRoutineState>(owner));
		owner.getSoul()->wakeup();
	}
}

void SleepingState::onEnter()
{
	if (bed)
	{
		bed->setOccupant(&owner);
		pos = bed->getPos() + Vector2f(10, 11);
	}
	loadWalkAnims();
	owner.setDrawableSet("north");
	owner.getSoul()->sleep();
}

void SleepingState::onExit()
{
	// @HACK why is this commented out?
	//if (bed)
	//{
	//	bed->setOccupant(nullptr);
	//	pos = bed->getExitPos(Random::get());
	//}
}

} // sl
