#include "NPC/States/ToddlerState.hpp"

#include "Engine/Events/Event.hpp"
#include "NPC/NPC.hpp"
#include "NPC/Behaviors/AcceptTalkWithPlayer.hpp"
#include "NPC/Behaviors/CustomCondition.hpp"
#include "NPC/Behaviors/Selector.hpp"
#include "NPC/LayeredSpriteLoader.hpp"
#include "Player/Player.hpp"
#include "Town/Town.hpp"
#include "Utility/Random.hpp"

namespace sl
{

ToddlerState::ToddlerState(NPC& owner)
	:NPCState(owner)
{
	owner.getSoul()->appearance.setHair("short");
	owner.getSoul()->appearance.setClothing(ClothingType::Panties, "diaper");
	
	loadWalkAnims();

	auto seq = unique<ai::Selector>();
	seq->addChild(unique<ai::AcceptTalkWithPlayer>("toddler/begin"));
	seq->addChild(unique<ai::CustomCondition>([](ai::BehaviorContext& context) {
		if (Random::get().chance(1 / 5.f))
		{
			context.getNPC().face(context.rng().choose({FacingDirection::West, FacingDirection::East}));
		}
		if (context.getNPC().getFacingDir() == FacingDirection::West)
		{
			if (context.getScene()->checkCollision(context.getNPC().getMask().getBoundingBox() + Vector2i(-2, 0), "StaticGeometry") != nullptr)
			{
				context.getNPC().face(FacingDirection::East);
			}
			else
			{
				context.getNPC().setPos(context.getNPC().getPos() - Vector2f(0.5f, 0.f));
			}
		}
		else
		{
			if (context.getScene()->checkCollision(context.getNPC().getMask().getBoundingBox() + Vector2i(2, 0), "StaticGeometry") != nullptr)
			{
				context.getNPC().face(FacingDirection::West);
			}
			else
			{
				context.getNPC().setPos(context.getNPC().getPos() + Vector2f(0.5f, 0.f));
			}
		}
		return true;
	}));
	behavior = std::move(seq);

	owner.face(FacingDirection::West);
}

void ToddlerState::update()
{
#ifdef SL_DEBUG
	behavior->debugExecute(owner.getBehaviorContext());
#else
	behavior->execute(owner.getBehaviorContext());
#endif

	// grow up
	if (owner.getSoul()->getAgeYears() > 3.5f)
	{
		auto& appearance = owner.getSoul()->appearance;
		appearance.setClothing(ClothingType::Outfit, "dress");
		appearance.setClothing(ClothingType::Panties, "plain");
		appearance.setHair(Random::get().choose("long", "pigtails", "short", "long", "long", "pigtails"));
		owner.initDefaultState();
	}
}

void ToddlerState::onEnter()
{
	setPathManager("StaticGeometry");
}

} // sl