#include "NPC/Stats/ColorPref.hpp"

#include "Utility/Assert.hpp"

namespace sl
{

static ColorPrefInfo colorInfo[COLOR_PREF_COUNT];

#define COLINIT(color, name, rgb) colorInfo[static_cast<int>(color)] = ColorPrefInfo((rgb), (name))
static void init()
{
	static bool initialized = false;
	if (!initialized)
	{
		COLINIT(ColorPref::Red, "red", sf::Color(225, 0, 0));
		COLINIT(ColorPref::Green, "green", sf::Color(0, 192, 0));
		COLINIT(ColorPref::Blue, "blue", sf::Color(0, 0, 192));
		COLINIT(ColorPref::Pink, "pink", sf::Color(255, 74, 133));
		COLINIT(ColorPref::White, "white", sf::Color(255, 255, 255));
		COLINIT(ColorPref::Black, "black", sf::Color(0, 0, 0));
		COLINIT(ColorPref::Purple, "purple", sf::Color(118, 10, 250));
		COLINIT(ColorPref::Orange, "orange", sf::Color(255, 80, 0));
		COLINIT(ColorPref::Yellow, "yellow", sf::Color(255, 204, 0));
		COLINIT(ColorPref::Cyan, "cyan", sf::Color(0, 255, 255));
		COLINIT(ColorPref::Brown, "brown", sf::Color(87, 64, 40));
		initialized = true;
	}
}
#undef COLINIT

const ColorPrefInfo& getInfo(ColorPref color)
{
	init();
	const int index = static_cast<int>(color);
	ASSERT(index >= 0 && index < COLOR_PREF_COUNT);
	return colorInfo[index];
}

ColorPref colorPrefFromString(const std::string& name)
{
	init();
	for (int i = 0; i < COLOR_PREF_COUNT; ++i)
	{
		if (colorInfo[i].name == name)
		{
			return static_cast<ColorPref>(i);
		}
	}
	ERROR("colour " + name + " does not exist");
	return ColorPref::Pink;
}

} // sl
