/**
 * @section DESCRIPTION
 * A collection of functions used to generate realistic lolis.
 */
#ifndef SL_LOLIGEN_HPP
#define SL_LOLIGEN_HPP

namespace sl
{

enum class Gender
{
	Female = 0,
	Male
};
enum class ActivityLevel
{
	Sedentary = 0,
	ModeratelyActive,
	Active
};

/**
 * @param ageDays The age of the person in days
 * @param gender The gender of the human
 * @return The expected mean weight in kg
 */
float expectedWeight(int ageDays, Gender gender);

/**
 * @param ageDays The age of the girl in days
 * @param gender The gender of the human
 * @return The expected mean height in cm
 */
float expectedHeight(int ageDays, Gender gender);

/**
 * @param ageDays The age of the person (same data for both genders!) in days
 * @return The expected mean bladder capacity in ml
 */
float bladderCapacity(int ageDays);

/**
 * @param ageDays age in days
 * @return Urine flow rate according to age
 */
float urineFlowRate(int ageDays, Gender gender);

/**
 * @param ageDays age in days
 * @param activityLevel Level of activity
 * @param gender The gender of the human
 * @return expected calorie expendenture per day (in kcal)
 */
float calorieRequirements(int ageDays, ActivityLevel activityLevel, Gender gender);

/**
 * @param ageDays age in days
 * @param gender The gender of the human
 * @return The ideal bodyfat % (as a ratio, ie 50% = 0.5f) for them
 */
float idealBodyfat(int ageDays, Gender gender);

/**
 * @param ageDays age in days
 * @param gender The gender of the human
 * @return The ideal muscle mass % (as a ratio, ie 50% = 0.5f) for them
 */
float idealMuscleMass(int ageDays, Gender gender);

} // sl

#endif // SL_LOLIGEN_HPP
