#include "NPC/Stats/Mood.hpp"

#include <algorithm>

#include "Engine/Serialization/Serialization.hpp"
#include "Engine/Serialization/Impl/SerializeVector.hpp"
#include "NPC/Stats/Soul.hpp"
#include "Utility/Math.hpp"

namespace sl
{

const Mood::ConveyedMoodV grief     {Mood::min, Mood::min, Mood::min};
const Mood::ConveyedMoodV loathing  {Mood::min, Mood::min, Mood::max};
const Mood::ConveyedMoodV terror    {Mood::min, Mood::max, Mood::min};
const Mood::ConveyedMoodV rage      {Mood::min, Mood::max, Mood::max};
const Mood::ConveyedMoodV relaxed   {Mood::max, Mood::min, Mood::min};
const Mood::ConveyedMoodV comfort   {Mood::max, Mood::min, Mood::max};
const Mood::ConveyedMoodV amazement	{Mood::max, Mood::max, Mood::min};
const Mood::ConveyedMoodV ecstasy	{Mood::max, Mood::max, Mood::max};

const float Mood::min = -100.f;
const float Mood::max = 100.f;

Mood::Mood()
{
	//std::fill(std::begin(rawData), std::end(rawData), 0.f);
}

Mood::Mood(const Soul& soul):Mood()
{
	// @todo initialize properly based on Soul
}


Mood::ConveyedMood Mood::getConveyedMood() const
{
	// @TODO decide the order to handle this when multiple have 0 distance - so do higher priority moods first
	ConveyedMood closest = static_cast<ConveyedMood>(0);
	float minDist = distance(closest);
	for (int i = 1; i <= static_cast<int>(ConveyedMood::Happy); ++i)
	{
		const ConveyedMood mood = static_cast<ConveyedMood>(i);
		const float dist = distance(mood);
		if (dist < minDist)
		{
			minDist = dist;
			closest = mood;
		}
	}
	return closest;
}


float Mood::getValue(Axis axis) const
{
	return rawToResult(rawData[axis]);
}

void Mood::setValue(Axis axis, float value)
{
	rawData[axis] = resultToRaw(std::clamp(value, -100.f, 100.f));
}

void Mood::adjustRawValue(Axis axis, float adjustment)
{
	rawData[axis] += adjustment;
}

void Mood::adjustValue(Axis axis, float adjustment)
{
	rawData[axis] = resultToRaw(getValue(axis) + adjustment);
}

void Mood::gravitateTowards(const Mood& baseline, float amplifier)
{
	for (int i = 0; i < axes; ++i)
	{
		const Axis axis = static_cast<Axis>(i);
		const float scalar = rawData[axis] < baseline.rawData[axis] ? 1.f : -1.f;
		adjustValue(axis, scalar * amplifier * 1.f);
		adjustRawValue(axis, scalar * amplifier * 3.f);
		if ((scalar > 0.f &&  rawData[axis] > baseline.rawData[axis]) || rawData[axis] < baseline.rawData[axis])
		{
			rawData[axis] = baseline.rawData[axis];
		}
	}
}

#define SQRDIF(axis) (tmp = (getValue(axis) - other.getValue(axis)), tmp*tmp)
float Mood::distance(const Mood& other) const
{
	float tmp, v = 0;
	for (int i = 0; i < axes; ++i)
	{
		v += SQRDIF(static_cast<Axis>(i));
	}
	return std::sqrt(v);
}
#undef SQRDIF

void Mood::serialize(Serialize& out) const
{
	AutoSerialize::serialize(out, rawData);
}

void Mood::deserialize(Deserialize& in)
{
	AutoDeserialize::deserialize(in, rawData);
}

/*static*/float Mood::resultToRaw(float x)
{
	// function is some arbitrary function that I got by toying around that results in the curve I want
	const float scalar = x >= 0.f ? 1.f : -1.f;
	// the +1.f is to avoid -inf log fuckery when taking the inverse in rawtoResult
	return scalar * 0.2f * expf(powf(scalar * x + 1.f, 0.48f)) + 1.f;
}

/*static*/float Mood::rawToResult(float x)
{
	const float scalar = x >= 0.f ? 1.f : -1.f;
	// assuming f(x) = 0.2 * e^0.48x, then f^-1(x) is this:
	return scalar * powf(logf(scalar * 5.f * x + 1.f), 2.083333333333333f);
}





// private

//TODO: why is the 5.f there??? should it be 0.f or something?
#define MDIF_NC(axis, amount, scalar) ret += std::fabsf(getValue(axis) - amount) * scalar
#define MDIF_UC(axis, amount, scalar) ret += std::min(5.f, getValue(axis) - amount) * scalar
#define MDIF_LC(axis, amount, scalar) ret -= std::max(5.f, getValue(axis) - amount) * scalar
float Mood::distance(ConveyedMood conveyedMood) const
{
	float ret = 0.f;
	/*switch (conveyedMood)
	{
	case ConveyedMood::Content:
		MDIF_NC(Happiness, 10,  0.3);
		MDIF_UC(Relaxation, 50, 0.7);
		break;
	case ConveyedMood::Anger:
		MDIF_UC(Anger, 50, 1.2);
		break;
	case ConveyedMood::Annoyed:
		MDIF_NC(Anger, 25, 0.7);
		MDIF_UC(Seriousness, 30, 0.3);
		break;
	case ConveyedMood::Bored:
		MDIF_LC(Surprise, -30, 0.4);
		MDIF_NC(Happiness, 0, 0.08);
		MDIF_NC(Relaxation, 0, 0.08);
		MDIF_NC(Arousal, 0, 0.08);
		MDIF_NC(Seriousness, 0, 0.08);
		MDIF_NC(Anger, 0, 0.08);
		break;
	case ConveyedMood::Sad:
		MDIF_NC(Happiness, -20, 1.2);
		break;
	case ConveyedMood::Depressed:
		MDIF_LC(Happiness, -50, 1.2);
		break;
	case ConveyedMood::Surprised:
		MDIF_UC(Surprise, 50, 1.2);
		break;
	case ConveyedMood::Fear:
		MDIF_LC(Relaxation, -60, 1.2);
		break;
	case ConveyedMood::Worried:
		MDIF_NC(Relaxation, -25, 1.2);
		break;
	case ConveyedMood::Happy:
		MDIF_NC(Happiness, 30, 1.2);
		break;
	case ConveyedMood::Ecstatic:
		MDIF_UC(Happiness, 65, 1.2);
		break;
	}*/
	return ret;
}
#undef MDIF_NC
#undef MDIF_UC
#undef MDIF_LC

} // sl
