#ifndef SL_MOOD_HPP
#define SL_MOOD_HPP

#include "Utility/Vector.hpp"

namespace sl
{

class Soul;
class Serialize;
class Deserialize;

class Mood
{
public:
	/**
	 * 100% neutral mood
	 */
	Mood();

	enum Axis
	{
		Pleasure = 0,
		Arousal,
		Dominance
		/*Happiness
		Relaxation,
		Seriousness,
		Anger,
		Surprise,*/
	};
	static constexpr int axes = 3;
	static const float min;
	static const float max;

	/**
	 * Constructs a mood matching a given soul's *default* mood (the one they tend towards)
	 */
	Mood(const Soul& soul);

	typedef Vector<float, 3> ConveyedMoodV;
	enum class ConveyedMood
	{
		Content = 0, // mild happy + relaxed
		Anger,       // anger
		Annoyed,     // slight anger + slight serious
		Bored,       // low surprise + neutral rest
		Sad,         // mild sad
		Depressed,   // sad
		Surprised,   // high surprise
		Fear,        // very low relaxed
		Worried,     // not relaxed
		Happy,       // med happy
		Ecstatic,    // high happy
	};
	ConveyedMood getConveyedMood() const;
	//The 8 extremes of the emotional system
	static const ConveyedMoodV grief;
	static const ConveyedMoodV loathing;
	static const ConveyedMoodV terror;
	static const ConveyedMoodV rage;
	static const ConveyedMoodV relaxed;
	static const ConveyedMoodV comfort;
	static const ConveyedMoodV amazement;
	static const ConveyedMoodV ecstasy;

	float getValue(Axis axis) const;

	void setValue(Axis axis, float value);

	void adjustRawValue(Axis axis, float adjustment);

	void adjustValue(Axis axis, float adjustment);

	void gravitateTowards(const Mood& baseline, float amplifier);

	float distance(const Mood& other) const;

	void serialize(Serialize& out) const;

	void deserialize(Deserialize& in);

	static float resultToRaw(float x);
	
	static float rawToResult(float x);

private:
	float distance(ConveyedMood conveyedMood) const;

	
	Vector<float, axes> rawData;
};

} // sl

#endif // SL_MOOD_HPP
