#include "NPC/Stats/NPCColorSwaps.hpp"

#include <map>

#include "Engine/Graphics/ColorSwapID.hpp"
#include "Engine/Graphics/ColorSwapLoader.hpp"
#include "Utility/Assert.hpp"

namespace sl
{

const ColorSwapID& toColorSwapID(DNA::HairColor hairColour)
{
	static std::map<DNA::HairColor, ColorSwapID> hairGeneMap;
	if (hairGeneMap.empty())
	{
		const std::map<std::string, ColorSwapID> jsonData = loadSwapsFile("data/genetics/hairswaps.json");

		hairGeneMap[DNA::HairColor::PlatinumBlonde] = jsonData.at("PlatinumBlonde");
		hairGeneMap[DNA::HairColor::Blonde] = jsonData.at("Blonde");
		hairGeneMap[DNA::HairColor::GoldenBlonde] = jsonData.at("GoldenBlonde");
		hairGeneMap[DNA::HairColor::LightBrownGreyish] = jsonData.at("LightBrownGreyish");
		hairGeneMap[DNA::HairColor::LightBrown] = jsonData.at("LightBrown");
		hairGeneMap[DNA::HairColor::DarkBrown] = jsonData.at("DarkBrown");
		hairGeneMap[DNA::HairColor::Black] = jsonData.at("Black");
		hairGeneMap[DNA::HairColor::Red] = jsonData.at("Red");
		hairGeneMap[DNA::HairColor::LightRed] = jsonData.at("LightRed");
	}
	ASSERT(!hairGeneMap.empty());
	return hairGeneMap[hairColour];
}

const ColorSwapID& toColorSwapID(DNA::SkinColor skinColour)
{
	static std::map<DNA::SkinColor, ColorSwapID> skinGeneMap;
	if (skinGeneMap.empty())
	{
		const std::map<std::string, ColorSwapID> jsonData = loadSwapsFile("data/genetics/skinswaps.json");

		skinGeneMap[DNA::SkinColor::Caucasian] = jsonData.at("Caucasian");
		skinGeneMap[DNA::SkinColor::Nordic] = jsonData.at("Nordic");
		skinGeneMap[DNA::SkinColor::Asian] = jsonData.at("Asian");
		skinGeneMap[DNA::SkinColor::Hispanic] = jsonData.at("Hispanic");
		skinGeneMap[DNA::SkinColor::Nigger] = jsonData.at("Nigger");
		skinGeneMap[DNA::SkinColor::Tropical] = jsonData.at("Tropical");
		skinGeneMap[DNA::SkinColor::Moreno] = jsonData.at("Moreno");
		skinGeneMap[DNA::SkinColor::Indian] = jsonData.at("Indian");
		skinGeneMap[DNA::SkinColor::Arabic] = jsonData.at("Arabic");
		skinGeneMap[DNA::SkinColor::Mediterranean] = jsonData.at("Mediterranean");
		skinGeneMap[DNA::SkinColor::SEAsian] = jsonData.at("SEAsian");
		skinGeneMap[DNA::SkinColor::ExtraNigger] = jsonData.at("ExtraNigger");
		skinGeneMap[DNA::SkinColor::Moreno2] = jsonData.at("Moreno2");
		skinGeneMap[DNA::SkinColor::DarkMoreno] = jsonData.at("DarkMoreno");
		skinGeneMap[DNA::SkinColor::DarkMoreno2] = jsonData.at("DarkMoreno2");
		skinGeneMap[DNA::SkinColor::DarkIndian] = jsonData.at("DarkIndian");
		skinGeneMap[DNA::SkinColor::DarkArabic] = jsonData.at("DarkArabic");
		skinGeneMap[DNA::SkinColor::DSEA] = jsonData.at("DSEA");
		skinGeneMap[DNA::SkinColor::GiggaNigga] = jsonData.at("GiggaNigga");
		skinGeneMap[DNA::SkinColor::DarkAsian] = jsonData.at("DarkAsian");
	}
	ASSERT(!skinGeneMap.empty());
	return skinGeneMap[skinColour];
}

const ColorSwapID& toColorSwapID(DNA::EyeColor eyeColour)
{
	static std::map<DNA::EyeColor, ColorSwapID> eyeGeneMap;
	if (eyeGeneMap.empty())
	{
		const std::map<std::string, ColorSwapID> jsonData = loadSwapsFile("data/genetics/eyeswaps.json");

		eyeGeneMap[DNA::EyeColor::Brown] = jsonData.at("Brown");
		eyeGeneMap[DNA::EyeColor::Blue] = jsonData.at("Blue");
		eyeGeneMap[DNA::EyeColor::Green] = jsonData.at("Green");
		eyeGeneMap[DNA::EyeColor::DarkBrown] = jsonData.at("DarkBrown");
		eyeGeneMap[DNA::EyeColor::Grey] = jsonData.at("Grey");
		eyeGeneMap[DNA::EyeColor::Hazel] = jsonData.at("Hazel");
		eyeGeneMap[DNA::EyeColor::LightBlue] = jsonData.at("LightBlue");
		eyeGeneMap[DNA::EyeColor::Cyan] = jsonData.at("Cyan");
		eyeGeneMap[DNA::EyeColor::Red] = jsonData.at("Red");
		eyeGeneMap[DNA::EyeColor::Black] = jsonData.at("Black");
	}
	ASSERT(!eyeGeneMap.empty());
	return eyeGeneMap[eyeColour];
}

} // sl
