#ifndef SL_NATIONALITYDATABASE_HPP
#define SL_NATIONALITYDATABASE_HPP

#include <map>
#include <memory>
#include <string>
#include <vector>

#include "NPC/Stats/DNADatabase.hpp"

namespace sl
{

class Random;

class NationalityDatabase
{
public:
	class Info
	{
	public:
		Info(DNADatabase::GenePool genePool)
			:genePool(genePool)
		{
		}

		DNADatabase::GenePool genePool;
		std::vector<std::string> firstNames;
		std::vector<std::string> lastNames;
	};
	typedef std::string Nationality;

	DNADatabase::GenePool getPoolFromNationality(const Nationality& nationality) const;

	const std::vector<Nationality>& getNationalities(DNADatabase::GenePool genePool) const;

	const Info& getInfo(const Nationality& nationality) const;

	bool exists(const Nationality& nationality) const;

	void load();



	static NationalityDatabase& get();

private:
	void loadGenePool(DNADatabase::GenePool genePool, const std::string& dir);
	void loadNationality(DNADatabase::GenePool genePool, const std::string& dir, const Nationality& nationality);



	std::map<DNADatabase::GenePool, std::vector<Nationality>> nationalities;
	std::map<Nationality, Info> info;



	static std::unique_ptr<NationalityDatabase> database;
};

const std::string& pickRandomName(Random& random, const NationalityDatabase::Info& info);

const std::string& pickRandomLastName(Random& random, const NationalityDatabase::Info& info);

} // sl

#endif // SL_NATIONALITYDATABASE_HPP