#include "NPC/Stats/Nutrition.hpp"

#include "Engine/Serialization/Serialization.hpp"

namespace sl
{

Nutrition::Nutrition()
	:starch(0.f)
	,fiber(0.f)
	,sugar(0.f)
	,omega3(0.f)
	,polyunsaturated(0.f)
	,monounsaturated(0.f)
	,saturated(0.f)
	,trans(0.f)
	,protein(0.f)
{
}

Nutrition::Nutrition(float calories, const Nutrition& other)
{
	const float ratio = calories / other.getTotalCalories();
	starch = ratio * other.starch;
	fiber = ratio * other.fiber;
	sugar = ratio * other.sugar;
	omega3 = ratio * other.omega3;
	polyunsaturated = ratio * other.polyunsaturated;
	monounsaturated = ratio * other.monounsaturated;
	saturated = ratio * other.saturated;
	trans = ratio * other.trans;
	protein = ratio * other.protein;
}

float Nutrition::getTotalCalories() const
{
	return 4.f * getTotalCarbs() + 4.f * getTotalProtein() + 9.f * getTotalFat();
}

float Nutrition::getTotalCarbs() const
{
	return starch + fiber + sugar;
}

float Nutrition::getTotalFat() const
{
	return omega3 + polyunsaturated + monounsaturated + saturated + trans;
}

float Nutrition::getTotalProtein() const
{
	return protein;
}

Nutrition& Nutrition::operator+=(const Nutrition& rhs)
{
	starch += rhs.starch;
	fiber += rhs.fiber;
	sugar += rhs.sugar;
	omega3 += rhs.omega3;
	polyunsaturated += rhs.polyunsaturated;
	monounsaturated += rhs.monounsaturated;
	saturated += rhs.saturated;
	trans += rhs.trans;
	protein += rhs.protein;
	return *this;
}

Nutrition& Nutrition::operator-=(const Nutrition& rhs)
{
	starch -= rhs.starch;
	fiber -= rhs.fiber;
	sugar -= rhs.sugar;
	omega3 -= rhs.omega3;
	polyunsaturated -= rhs.polyunsaturated;
	monounsaturated -= rhs.monounsaturated;
	saturated -= rhs.saturated;
	trans -= rhs.trans;
	protein -= rhs.protein;
	return *this;
}

namespace AutoSerialize
{
void serialize(Serialize& out, const Nutrition& nutrition)
{
	out.startObject("Nutrition");
	out.f("starch", nutrition.starch);
	out.f("fiber", nutrition.fiber);
	out.f("sugar", nutrition.sugar);
	out.f("omega3", nutrition.omega3);
	out.f("polyunsaturated", nutrition.polyunsaturated);
	out.f("monounsaturated", nutrition.monounsaturated);
	out.f("saturated", nutrition.saturated);
	out.f("trans", nutrition.trans);
	out.f("protein", nutrition.protein);
	out.endObject("Nutrition");
}
} // AutoSerialize

namespace AutoDeserialize
{
void deserialize(Deserialize& in, Nutrition& nutrition)
{
	in.startObject("Nutrition");
	nutrition.starch = in.f("starch");
	nutrition.fiber = in.f("fiber");
	nutrition.sugar = in.f("sugar");
	nutrition.omega3 = in.f("omega3");
	nutrition.polyunsaturated = in.f("polyunsaturated");
	nutrition.monounsaturated = in.f("monounsaturated");
	nutrition.saturated = in.f("saturated");
	nutrition.trans = in.f("trans");
	nutrition.protein = in.f("protein");
	in.endObject("Nutrition");
}
} // AutoDeserialize

} // sl
