#include "NPC/Stats/PersonalityModifiers.hpp"

#include "Engine/Serialization/AutoDefineObjectSerialization.hpp"

#include <algorithm>

namespace sl
{

PersonalityModifiers::PersonalityModifiers()
	:dominance(0.f)
	,openToCautious(0.f)
	,organizedToCareless(0.f)
	,extrovertToIntrovert(0.f)
	,friendlyToUnkind(0.f)
	,nervousToConfident(0.f)
{
}

PersonalityModifiers& PersonalityModifiers::operator+=(const PersonalityModifiers& rhs)
{
	dominance += rhs.dominance;
	openToCautious += rhs.openToCautious;
	organizedToCareless += rhs.organizedToCareless;
	extrovertToIntrovert += rhs.extrovertToIntrovert;
	friendlyToUnkind += rhs.friendlyToUnkind;
	nervousToConfident += rhs.nervousToConfident;
	return *this;
}

PersonalityModifiers& PersonalityModifiers::operator*=(float scalar)
{
	dominance *= scalar;
	openToCautious *= scalar;
	organizedToCareless *= scalar;
	extrovertToIntrovert *= scalar;
	friendlyToUnkind *= scalar;
	nervousToConfident *= scalar;
	return *this;
}

PersonalityModifiers& PersonalityModifiers::clamp()
{
	dominance = std::clamp(dominance, -1.f, 1.f);
	openToCautious = std::clamp(openToCautious, -1.f, 1.f);
	organizedToCareless = std::clamp(organizedToCareless, -1.f, 1.f);
	extrovertToIntrovert = std::clamp(extrovertToIntrovert, -1.f, 1.f);
	friendlyToUnkind = std::clamp(friendlyToUnkind, -1.f, 1.f);
	nervousToConfident = std::clamp(nervousToConfident, -1.f, 1.f);
	return *this;
}

// non-members
PersonalityModifiers operator+(const PersonalityModifiers& lhs, const PersonalityModifiers& rhs)
{
	PersonalityModifiers result(lhs);
	result += rhs;
	return result;
}

PersonalityModifiers operator*(const PersonalityModifiers& obj, float scalar)
{
	PersonalityModifiers result(obj);
	result *= scalar;
	return result;
}

PersonalityModifiers operator*(float scalar, const PersonalityModifiers& obj)
{
	return obj * scalar;
}

DEF_SERIALIZE_OBJ_FIELDS(PersonalityModifiers,
	&PersonalityModifiers::dominance,
	&PersonalityModifiers::openToCautious,
	&PersonalityModifiers::organizedToCareless,
	&PersonalityModifiers::extrovertToIntrovert,
	&PersonalityModifiers::friendlyToUnkind,
	&PersonalityModifiers::nervousToConfident)

} // sl