#ifndef SL_PERSONALITYMODIFIERS_HPP
#define SL_PERSONALITYMODIFIERS_HPP

#include "Engine/Serialization/AutoDeclareObjectSerialization.hpp"

namespace sl
{

class PersonalityModifiers
{
public:
	PersonalityModifiers();

	PersonalityModifiers& operator+=(const PersonalityModifiers& rhs);

	PersonalityModifiers& operator*=(float scalar);

	PersonalityModifiers& clamp();

	//! Dominant vs Submissive. 1 = fully dominant, -1 = fully submissive
	float dominance;
	//	see: https://en.wikipedia.org/wiki/Big_Five_personality_traits
	// note: everything in [-1, 1]
	//! Openness to experience: (inventive/curious vs. consistent/cautious)
	float openToCautious;
	//! Conscientiousness: (efficient/organized vs. easy-going/careless)
	float organizedToCareless;
	//! Extraversion: (outgoing/energetic vs. solitary/reserved)
	float extrovertToIntrovert;
	//! Agreeableness: (friendly/compassionate vs. cold/unkind)
	float friendlyToUnkind;
	//! Neuroticism: (sensitive/nervous vs. secure/confident)
	float nervousToConfident;
};

extern PersonalityModifiers operator+(const PersonalityModifiers& lhs, const PersonalityModifiers& rhs);

extern PersonalityModifiers operator*(const PersonalityModifiers& obj, float scalar);

extern PersonalityModifiers operator*(float scalar, const PersonalityModifiers& obj);

DEC_SERIALIZE_OBJ(PersonalityModifiers)

} // sl

#endif // SL_PERSONALITYMODIFIERS_HPP