#ifndef SL_PREFERENCES_HPP
#define SL_PREFERENCES_HPP

#include "NPC/Stats/FlavorProfile.hpp"
#include "NPC/Stats/ColorPref.hpp"

#include <map>
#include <string>
#include <vector>

namespace sl
{

class Deserialize;
class Serialize;
class Soul;

class Preferences
{
public:
	Preferences(Soul& soul);


	void deserialize(Deserialize& in);

	void serialize(Serialize& out) const;


	struct FlavorResponse
	{
		FlavorProfile flavor;
		float response; // should be in range [-1, 1]
		int datapoints;
	};

	float calculateResponse(const FlavorProfile& flavor) const;

	void setFoodPreference(const FlavorProfile& flavor, float response);

	void tryFood(const FlavorProfile& food);
	
	ColorPref getFavoriteColor() const;

	float getColorPreference(ColorPref color) const;

private:
	float calculateResponseBySimilarityMethod(const FlavorProfile& flavor) const;

	float calculateResponseByLERPing(const FlavorProfile& flavor) const;
	
	float flavorDistance(const FlavorProfile& a, const FlavorProfile& b) const;
	
	float foodAdventurous() const;

	float tasteMalleability() const;

	void computeColorPreferences();



	std::vector<FlavorResponse> flavors;

	float colors[COLOR_PREF_COUNT];

	float colorExperiences[COLOR_PREF_COUNT];

	float adventerous; // @TODO derived this from other stats?

	Soul& soul;
};

} // sl

#endif // SL_PREFERENCES_HPP