/**
 * @section DESCRIPTION
 * The meaningful data stored within kidnappable NPCs. This is what makes a loli unique. I am NOT documenting this doxygen-style until later on.
 */
#ifndef SL_SOUL_HPP
#define SL_SOUL_HPP

#include <array>
#include <cstdint>
#include <string>
#include <map>
#include <memory>
#include <set>

#include "Items/Inventory.hpp"
#include "Parsing/Binding/Flags.hpp"
#include "NPC/Stats/Associations.hpp"
#include "NPC/Stats/GeneticStats.hpp"
#include "NPC/Stats/DNA.hpp"
#include "NPC/Stats/Mood.hpp"
#include "NPC/Stats/PersonalityModifiers.hpp"
#include "NPC/Locators/TargetLocator.hpp"
#include "NPC/LifeEvents.hpp"
#include "NPC/Stats/Nutrition.hpp"
#include "NPC/Stats/Preferences.hpp"
#include "NPC/LocationPathCache.hpp"
#include "NPC/NPCAppearance.hpp"
#include "NPC/Schedule.hpp"
#include "NPC/Urine.hpp"

namespace sl
{

class Entity;
class NPC;
class Random;
class SoulGenerationSettings;
class BuildingLevel;
class DNADatabase;
class PlayerStats;
class World;

// based on Kinsey scale
enum class Sexuality : uint8_t
{
	Asexual,
	Straight,
	AlmostEntirelyStraight,
	MostlyStraight,
	Bisexual,
	MostlyGay,
	AlmostEntirelyGay,
	Gay
};

const int MENARCHE = 450;

class Soul
{
public:
	typedef uint64_t ID;

	Soul(Deserialize& in);
	/**
	 * Generate a Soul randomly from thin air
	 * @param random The RNG to use
	 * @param settings The parameters to use during generation
	 */
	Soul(Random& random, const SoulGenerationSettings& settings);

	/**
	 * Generate a Soul from a mother and father, sharing both of their traits
	 * according to Mendelian inheritance mostly.
	 * @param mother The mother of the child
	 * @param father The father of the child
	 * @param random The RNG to use
	 * #param maleChance The chance that it picks the father's Y chromosome instead of his X
	 */
	Soul(const Soul& mother, const Soul& father, Random& random, float maleChance = 0.5f);


	void serialize(Serialize& out) const;

	void deserialize(Deserialize& in);

	ID getID() const;

	/**
	 * Ideally, only call this from NPC
	 */
	void setNPC(NPC *npc);

	NPC* getNPC();

	const NPC* getNPC() const;

	const ls::Flags& getFlags() const;

	ls::Flags& getFlags();

	void mate(World& world, Soul& partner, Random& random);

	std::shared_ptr<Soul> giveBirth(World& world);

	/**
	 * Updates the long-term physical status of the loli. Ages her and makes her grow.
	 */
	void ageOneDay();

	/**
	 * Updates current (short-term) physical status (THIS DOES NOT INCREASE AGE)
	 * @param timeSkip Whether to do stuff like bladder and such. Otherwise if you time-skip many days, the bladders are huge.
	 */
	void hourlyUpdate(World& world, Random& rng, bool timeSkip);



	// ---------------------------------------------------------------------------
	// ------------------------------- appearance --------------------------------
	const std::string& getType() const;

	void setType(const std::string& newType);

	NPCAppearance appearance;

	// ---------------------------------------------------------------------------
	// --------------------------- appearance / main -----------------------------
	const std::string& getFirstName() const;

	const std::string& getLastName() const;

	/**
	 * @return The NPC's age in years
	 */
	float getAgeYears() const;

	/**
	 * @return The NPC's age in days
	 */
	int getAgeDays() const;

	/**
	 * @return The girl's height in cm
	 */
	int getHeight() const;

	/**
	 * @return The girl's weight in kg
	 */
	int getWeight() const;

	const DNA& getDNA() const;

	const std::string& getNationality() const;



	// ---------------------------------------------------------------------------
	// -------------------------------- stats ------------------------------------
	/**
	 * @return Strength (I guess needed for when you use force and they fight back)
	 */
	int strength() const;
	/**
	 * @return Luck (could play a small part in many things maybe)
	 */
	int luck() const;
	/**
	 * @return Intelligence
	 */
	int intelligence() const;
	/**
	 * @return Willpower (how easy they are to mind-break)
	 */
	int willpower() const;
	/**
	 * @return Charisma. How easily they can influence others.
	 */
	int charisma() const;
	/**
	 * @return Dexterity (could affect stuff like how fast they run when they try to get away)
	 */
	int dexterity() const;
	/**
	 * @return Physical attractiveness
	 */
	int beauty() const;
	


	// ---------------------------------------------------------------------------
	// ----------------------------------- personality ---------------------------
	/**
	 * @return Gullibility (ease of persuasion) 0 to 1
	 */
	float gullibility() const;
	/**
	 * @return How dominant the loli is. 1 is fully dominant, -1 is fully submissive.
	 */
	float dominance() const;
	/**
	 * @return Openness to experience: (inventive/curious [-1] vs. consistent/cautious [1])
	 */
	float openToCautious() const;
	/**
	* @return  Conscientiousness: (efficient/organized vs. easy-going/careless)
	*/
	float organizedToCareless() const;
	/**
	* @return Openness to experience: (inventive/curious [-1] vs. consistent/cautious [1])
	*/
	float extrovertToIntrovert() const;
	/**
	* @return Openness to experience: (inventive/curious [-1] vs. consistent/cautious [1])
	*/
	float friendlyToUnkind() const;
	/**
	* @return Openness to experience: (inventive/curious [-1] vs. consistent/cautious [1])
	*/
	float nervousToConfident() const;



	// ---------------------------------------------------------------------------
	// ----------------------------------- mood ----------------------------------
	void affectMood(Mood::Axis moodAxis, float value);

	// Status wrappers?
	// Mental ones
	float getHope() const;
	float getWillingness() const;
	float getTrust() const;
	float getFear() const;
	float getLove() const;
	void setFear(float fear);



	// ---------------------------------------------------------------------------
	// ---------------------------- physical status ------------------------------
	class Status;
	const Status& getStatus() const;
	Status& getStatus();

	bool isAlive() const;

	void kill();

	/**
	* Returns how nourished a loli is. range is [-1, 1]
	* 0 is normal, 1 is over-fed and -1 is malnourished
	* @return nourishment level
	*/
	float getNourishment() const;

	/**
	* 0.f = completely dehydrated, 1.f = completely hydrated
	* Let's say 0.5f is normal.
	* @return How hydrated the loli is.
	*/
	float getHydration() const;

	/**
	 * Removes urine from the bladder equivalent to peeing for 1 second.
	 * @return The amount of urine released + the amount of salts
	 */
	Urine urinate();

	float getUrinaryUrges() const;

	void sleep();

	void wakeup();








	// This stuff is not in a category right now. Dunno what category to put it in.


	void giftItem(Item item);

	void consume(Item item);

	Inventory items;

	// TODO: remove.
	int scheduleValid = 0;

	Schedule::InterpolatedSchedule getInterpolatedSchedule(const GameTime& time);

	const Schedule& getSchedule() const;

	

	// ---------------------------------------------------------------------------
	// -------------------------------- relations --------------------------------
	bool playerKnows(const std::string& fact) const;

	void setPlayerKnowledge(const std::string& fact, bool value = true);

	const TargetLocator* getLocation(const std::string& locationName) const;
	
	/**
	 * Set a location. Cannot have the same name as any knownEntity.
	 */
	void setLocation(const std::string& locationName, std::unique_ptr<TargetLocator> location);

	Entity* getKnownEntity(const std::string& entityName);

	const Entity* getKnownEntity(const std::string& entityName) const;

	void setKnownEntity(const std::string& entityName, Entity *entity);

	// TODO: is this the right place?
	LocationPathCache locationPathCache;

	/**
	 * @return Whether the crime was reported
	 */
	bool isCrimeReported(int crimeId) const;

	bool isCrimeKnown(int crimeId) const;

	void setCrimeReported(int crimeId, bool reported);

	std::vector<int> getUnreportedCrimes() const;

	/**
	 * @return All crimes this NPC knows about, in the format <id, reported>
	 */
	const std::map<int, bool>& getKnownCrimes() const;

	/**
	 * @return The number of NPCs registered to {@param npcName}. if == 1, call getNPC(), if > 1 call getNPCs().
	 */
	int hasNPC(const std::string& nameName) const;

	ID getNPC(const std::string& npcName) const;

	void setNPC(const std::string& npcName, ID id);

	const std::vector<ID>& getNPCs(const std::string& npcCategory) const;

	void addNPC(const std::string& npcCategory, ID id);

	float computeSuspicionOfPlayer() const;

	/**
	 * Record noticing the player doing something suspicious.
	 * @param id The activtiy type/id
	 * @param level How suspicious the activity is to them. 0 = not at all, 1 = very.
	 */
	void suspectPlayer(const std::string& id, float level);

	void shiftNPCRelationship(ID id, float amount);

	float getNPCRelationship(ID id) const;

	void setOpinion(const std::string& thing, float value);

	void shiftOpinion(const std::string& thing, float value);

	float getOpinion(const std::string& thing) const;

	Associations associations;

	Preferences preferences;

	LifeEvents lifeEvents;



	// ---------------------------------------------------------------------------
	// ---------------------------------- sexual ---------------------------------
	// for males, this is the performing on a female
	// @TODO figure out wtf to do for lesbian sex. use this anyway?
	// or maybe those acts that are ambiguous don't check the receiver's
	// sex skill level?
	enum class SexType
	{
		//! loli getting fucked by *
		Vaginal = 0,
		//! loli getting fucked by *
		Anal,
		// loli sucking on/licking * (TODO: separate ones for cunnilingus when that is implemented?)
		Oral,
		// loli jerking off *
		Handjob,
		// loli getting fucked by *
		Thighsex,
		// loli giving * a footjob
		Footjob,
		// loli rubbing herself [use this for her rubbing other lolis too]
		Masturbation,
		// * rubbing loli [likely only used for count, not ability]
		Molestation,
		// * licking loli [likely only used for count, not ability]
		OralReceive,

		COUNT
	};
	static constexpr int SexType_Count = static_cast<int>(SexType::COUNT);
	/**
	* @return Breast size. Nourishment/age/genetics will affect actual size.
	*/
	int oppai() const;
	/**
	* @return Hours until giving birth. 0 if about to give birth, -1 if not pregnant
	*/
	int getHoursUntilPregnancy() const;
	/**
	* @return How far (in days) the girl is into puberty
	*/
	int getPubertyDays() const;
	/**
	* @return How far (relativley) the girl is into puberty. 0 = not start, 1 = done
	*/
	float getPuberty() const;

	int getVagTightness() const;

	int getAssTightness() const;

	float getFertility() const;

	/**
	 * @return The effective sexual arousal. From 0 to 1
	 */
	float getArousal() const;

	/**
	 * @param diff The amount to change arousal's raw factor by. Should be linear in the lower regions.
	 */
	void arouse(float diff);

	/**
	 * Sexually stimulates by the given amount.
	 * @param amount How much to stimulate by. Is affected by arousal/etc
	 */
	void stimulate(float amount);

	/**
	 * @return How close she is to orgasming. 0 = start, 1 = orgasming
	 */
	float getOrgasm() const;

	/**
	 * Experience an orgasm. Must be ready to have one.
	 * @param cause The sex act that caused the orgasm
	 */
	void haveOrgasm(SexType cause);

	/**
	 * @param cause The cause (sex act) to query for
	 * @return Orgasm count for a given cause
	 */
	int getOrgasmCount(SexType cause) const;

	/**
	 * @return Total orgasm count
	 */
	int getTotalOrgasmCount() const;

	/**
	 * Adds sex experience for a given act.
	 * Takes into account SexType cooldown, and if scenario is non-null, then scenario cooldown too
	 * @param type The SexType to add experience to. Also adds a cooldown to
	 * @param scenario If non-null, then the scenario to add a cooldown to
	 * @param xp The amount of xp (before cooldowns) to add
	 * @param dominance How dominant the action was. -1 = fully submissive, 0 = neutral, 1 = fully dominant
	 */
	void addSexExp(SexType type, const std::string *scenaro, float xp, float dominance);

	/**
	 * @return the RAW (uncurved) sex experience for JUST the SexType given
	 */
	float getSexExp(SexType type) const;

	void incrementSex(SexType type);

	int getSexCount(SexType type) const;

	int getTotalSexCount() const;

	/**
	 * NOTE: this is not the raw ability, but the curved effective ability
	 *       it has also been affected by other modifiers like related SexTypes
	 * @param type The sexual act category
	 * @param dominance 1 for fully dominant act, -1 for fully submissive act
	 * @return the ability (0 to 1) with 0 being terrible and 1 being perfect at which she should perform it at
	 */
	float getSexAbility(SexType type, float dominance) const;

	float getMaxSexExp() const { return 10000.f; }

	/**
	 * @param girth The circmference in cm of the object to insert
	 * @return the ratio of object to vagina. ie 0.5 = only takes up 50% of the vagina, 1.1 = need 10% more vag to fit object
	 */
	float checkVaginalFit(float girth) const;

	/**
	 * Attempts to perform a vaginal insertion and does necessary stretching/damage when necessary.
	 * @param girth The circmference in cm of the object to insert
	 * @return On failure, 0.f. On sucess, the ratio of object to vagina. ie 0.5 = only takes up 50% of the vagina, 1.1 = need 10% more vag to fit object
	 */
	float performVaginalInsertion(float girth);

	/**
	 * @param girth The circmference in cm of the object to insert
	 * @return the ratio of object to anus. ie 0.5 = only takes up 50% of the anus, 1.1 = need 10% more anus to fit object
	 */
	float checkAnalFit(float girth) const;

	/**
	 * Attemots to perform a anal insertion and does necessary stretching/damage when necessary.
	 * @param girth The circmference in cm of the object to insert
	 * @return On failure, 0.f. On sucess, the ratio of object to anal. ie 0.5 = only takes up 50% of the anus, 1.1 = need 10% more anus to fit object
	 */
	float performAnalInsertion(float girth);

private:
	/**
	 * Sets all the statistics randomly (given the constraints of already-set DNA)
	 * @param random The Random number generator to use
	 */
	void randomlyGenerate(Random& random);

	/**
	 * @return The age at which puberty should start. Computed from various things like nourishment/etc
	 */
	int pubertyAge() const;

	float computePubertalContributionToArousal() const;

	float computeArousalContributionToVaginalSize() const;

	/**
	 * @return The base vaginal circumference before any stretching given the current loli's stats
	 */
	float computeVagCircumference() const;

	/**
	 * @return The base vaginal circumference before any stretching given the current loli's stats
	 */
	float computeAssCircumference() const;

	void adjustMoodTowardsDefault(float rate);

	// to keep appearance up to date
	void setAgeInDays(int newAge);


	ID id;
	NPC *npc;
	std::string firstName;
	std::string lastName;
	std::string nationality;
	std::string type;
	ls::Flags flags;
	std::set<std::string> playerKnowledge;
	DNA motherDNA;
	DNA fatherDNA;
	DNA expressedDNA; // just motherDNA | fatherDNA
	GeneticStats geneticStats;
	uint64_t motherID;
	uint64_t fatherID;
	public:
	class Status
	{
	public:
		static const int digestionTime = 7; // 7 hours
		static const int spermLifespan = 5; // 5 days

		// Current status (physical)
		//! negative would be underweight, positive would be fat (-100 to 100)
		float nourishment;
		//! Stored extra fat in grams
		float storedFat;
		//! Muscle mass in grams
		float muscle;
		//! The loli's health. (0 to maxhp)
		float hp;
		//! The loli's max hp
		float maxhp;
		//! The loli's hunger (0 to 1)
		float hunger;
		//! The loli's thirst (0 to 1)
		float thirst;
		//! energy to do stuff (0 to 1)
		float energy;
		//! how is this different from energy? short-term?
		float stamina;
		//! urine content
		Urine bladderContent;
		//! Urinary urges. Increases in relation to bladderContent
		float urinaryUrge;
		//! Amount of water currently being digested
		uint16_t waterInDigestiveSystem;
		//! Amount of food being currently digested
		Nutrition digestingFood[digestionTime];
		//! Index of the oldest imbibed food
		int8_t digestionIndex;
		//! Energy storage
		Nutrition storedFood;


		bool isSleeping;


		int chloroformTimer;

		// Current status (mental)
		//! mind-broken yet? (0 to 1)
		float hope;
		//!  (-1 to 1)
		float trust;
		//! fear of the player (0 to 1)
		float fear;
		//! love of the player (-1 to 1)
		float love;
		Mood defaultMood;
		Mood currentMood;
		//! Cooldown for movement in one direction along a mood axis (axis*2 = positive direction, axis*2 + 1 = negative direction)
		float moodCooldown[Mood::axes * 2];

		PersonalityModifiers personalityModifiers;

		// Sexual status
		//! the desire to do or avoid sexual things. (-1 to 1)
		float sexDesires;
		//! tearing done to vagina. will gradually repair only when not over a certain threshold. will repair quicker when less damage. (0 to 100)
		int vagDamage;
		//! tearing done to anus. will gradually repair only when not over a certain threshold. will repair quicker when less damage. (0 to 100)
		int assDamage;
		//! any form of long-term stretching of a vagina.
		float vagStretching;
		//! any form of long-term stretching of an ass
		float assStretching;
		//! Amount of sperm (in billions), by ejaculator, indexed by day, which rotates
		std::map<ID, std::array<float, spermLifespan>> spermInWomb;
		//! Index of the current day in the circular womb buffer
		int spermIndex;
		//! How many hours until ovulation
		int hoursToOvulation;
		//! How many hours until menstration
		int hoursToMenstration;

		int hour;
		//! Current (random) horniness modifier, changes per hour. (0 to 1)
		float hornyModifier;
		//! Current (random) horniness modifier change, this is how much hornyModifier changes per hour
		float hornyModifierChange;
		//! Current sexual arousal level. Before impact of hornyModifier/etc.
		float arousal;
		//! How close she is to orgasming. 0 to 1.
		float orgasm;
	};private:
	//! The current status of the NPC. If this is null then the NPC has died I guess
	std::unique_ptr<Status> status;

	// Physical dimensions
	//! Age in days.
	uint32_t ageInDays;
	//! height in cm
	float height;
	//! weight in kg
	float weight;

	// Reproductive/Sexual stats
	//! Times orgasmed
	int orgasmCount[SexType_Count];
	//! Times done a given sexual act (at any point during the sex session, not just the finale)
	int sexCount[SexType_Count];
	//! Ability to perform a given sex act
	float sexSkillType[SexType_Count];
	//! Cooldown associated with a sex act. Goes down hourly and daily
	float sexCooldownByAct[SexType_Count];
	//! Cooldown associated with a scenario. Goes down hourly and daily
	std::map<std::string, float> sexCooldownByScenario;
	//! Ability to be dominant in sex
	float sexSkillDom;
	//! Ability to be submissive in sex
	float sexSkillSub;
	//! actual vaginal tightness when fully aroused. circumference in cm.
	float vagTightness;
	//! actual vaginal tightness when fully aroused. circumference in cm.
	float assTightness;
	//! how far long in puberty the loli is in days. malnutrition can delay/hamper this..  (0 to 365*5)
	uint32_t puberty;
	Sexuality sexuality;
	int currentOppai;
	//! The baby the NPC is currently carrying (null if not pregnant)
	std::unique_ptr<Soul> baby;
	//! how long until baby is born in HOURS (-1 = not pregnant)
	int16_t pregnant;


	std::map<std::string, std::unique_ptr<TargetLocator>> locations;
	std::map<std::string, Entity*> knownEntities;
	std::map<std::string, std::vector<ID>> knownNPCs;
	std::map<int, bool> knownCrimes;
	public: // TODO: for serialization
	struct SuspiciousActivityExplanation
	{
		float suspiciousness;
		float explanationLevel;
	};
	private:
	//! Activities that the player has done that are suspicious to this NPC.
	std::map<std::string, SuspiciousActivityExplanation> suspiciousActivities;
	//! Raw relationship value. 
	std::map<Soul::ID, float> relationships;
	//! Raw opinion values
	std::map<std::string, float> opinions;


	// TODO: multiple ones? does that make any sense???
	Schedule schedule;
	//! The (most) current interpolated schedule.
	Schedule::InterpolatedSchedule interpolatedSchedule;


	//! The next available ID tag to use. Also the count of the number of current Souls created.
	static uint64_t souls;


	friend class DNADatabase;
	// @todo remove this later? this is here to create the player with DNA similar to a loli.
	friend class BuildingLevel;

	friend std::shared_ptr<PlayerStats> createPlayer(World& world, Random& rng);

	friend class World; // for serializing the souls static var -- is there a better way?

	// TODO: HACK remove (used to access schedule)
	friend class LoliRoutineState;
};

}// End of sl namespace

#endif
