#include "NPC/Stats/SoulGenerationSettings.hpp"

#include "NPC/Stats/NationalityDatabase.hpp"
#include "Utility/Random.hpp"

namespace sl
{

SoulGenerationSettings::SoulGenerationSettings()
	:natConfig(NationalityConfig::CompletelyRandom)
	,minAge(4 * 365)
	,maxAge(14 * 365)
	,maleChance(0.5f)
{
}

const std::string& SoulGenerationSettings::getNationality(Random& rng) const
{
	if (natConfig == NationalityConfig::FixedNationalities)
	{
		return nationalities.get(rng);
	}
	const DNADatabase::GenePool pool = natConfig == NationalityConfig::FixedPools ?
		pools.get(rng) : static_cast<DNADatabase::GenePool>(rng.random(DNADatabase::GENE_POOL_SIZE)); 
	return rng.choose(NationalityDatabase::get().getNationalities(pool));
}

int SoulGenerationSettings::getAge(Random& rng) const
{
	return rng.randomInclusive(minAge, maxAge);
}

bool SoulGenerationSettings::isMale(Random& rng) const
{
	return rng.chance(maleChance);
}


SoulGenerationSettings& SoulGenerationSettings::addNationality(const std::string& nationality, double frequency)
{
	ASSERT(natConfig == NationalityConfig::CompletelyRandom || natConfig == NationalityConfig::FixedNationalities);
	natConfig = NationalityConfig::FixedNationalities;
	nationalities.insert(nationality, frequency);
	return *this;
}

SoulGenerationSettings& SoulGenerationSettings::addGenePool(DNADatabase::GenePool pool, double frequency)
{
	ASSERT(natConfig == NationalityConfig::CompletelyRandom || natConfig == NationalityConfig::FixedPools);
	natConfig = NationalityConfig::FixedPools;
	pools.insert(pool, frequency);
	return *this;
}

SoulGenerationSettings& SoulGenerationSettings::setAgeRange(int min, int max)
{
	ASSERT(min <= max);
	ASSERT(min >= 0);
	minAge = min;
	maxAge = max;
	return *this;
}

SoulGenerationSettings& SoulGenerationSettings::setMaleChance(float newMaleChance)
{
	ASSERT(newMaleChance >= 0.f && newMaleChance <= 1.f);
	maleChance = newMaleChance;
	return *this;
}

} // sl
