#ifndef SL_SOULGENERATIONSETTINGS_HPP
#define SL_SOULGENERATIONSETTINGS_HPP

#include <string>

#include "NPC/Stats/DNADatabase.hpp"
#include "Utility/ProportionList.hpp"

namespace sl
{

class Random;

class SoulGenerationSettings
{
public:
	enum class NationalityConfig
	{
		CompletelyRandom,
		FixedPools,
		FixedNationalities
	};
	
	SoulGenerationSettings();


	const std::string& getNationality(Random& rng) const;

	int getAge(Random& rng) const;

	bool isMale(Random& rng) const;


	SoulGenerationSettings& addNationality(const std::string& nationality, double frequency = 1.0);

	SoulGenerationSettings& addGenePool(DNADatabase::GenePool pool, double frequency = 1.0);

	SoulGenerationSettings& setAgeRange(int min, int max);

	SoulGenerationSettings& setMaleChance(float newMaleChance);
	
private:
	NationalityConfig natConfig;
	ProportionList<DNADatabase::GenePool> pools;
	ProportionList<std::string> nationalities;
	int minAge;
	int maxAge;
	float maleChance;
};

} // sl

#endif // SL_SOULGENERATIONSETTINGS_HPP