#include "NPC/Stealth/Noise.hpp"

#include "Engine/Scene.hpp"
#include "Utility/Trig.hpp"
#include "Utility/TypeToString.hpp"

namespace sl
{

DECLARE_TYPE_TO_STRING(Noise)

const float Noise::Level::innocent = 0.1f;
const float Noise::Level::concerning = 0.4f;
const float Noise::Level::serious = 1.f;

Noise::Noise(const Vector2f& pos, float concernLevel, float radius)
	:Entity(pos.x, pos.y, 1, 1, "Noise")
	,radius(1)
	,goalRadius(radius)
	,concernLevel(concernLevel)
{
}

float Noise::getConcernLevel() const
{
	return concernLevel;
}

float Noise::getLoudness() const
{
	return goalRadius / ((radius/goalRadius)*(radius/goalRadius));
}


const Types::Type Noise::getType() const
{
	return Entity::makeType("Noise");
}

/*	  Entity private methods	  */
void Noise::onUpdate()
{
	radius += 5.f;
	if (radius >= goalRadius)
	{
		this->destroy();
	}
	CollisionMask newMask(pos.x, pos.y, radius * 2, radius * 2);
	newMask.setOrigin(radius, radius);
	setCollisionMask(std::move(newMask));
	sf::Color col(0, 0, 0);
	auto& channel = concernLevel >= Level::serious ? col.r : (concernLevel >= Level::concerning ? col.g : col.b);
	channel = std::min(255, (int)getLoudness() * 10);
	scene->debugDrawCircle("draw_noise", pos, radius, col);
}

} // sl
