#include "NPC/Stealth/SuspiciousScenario.hpp"

namespace sl
{

SuspiciousScenario::SuspiciousScenario(std::string type, float baseSuspicion, const SuspiciousScenario *parent)
	:type(std::move(type))
	,baseSuspicion(baseSuspicion)
	,parent(parent)
{
}

SuspiciousScenario& SuspiciousScenario::addModifier(SuspicionModifier modifier)
{
	modifiers.push_back(std::move(modifier));
	return *this;
}

float SuspiciousScenario::getSuspicion(const SuspicionContext& context) const
{
	const float parentSuspicion = parent ? parent->getSuspicion(context) : 0.f;
	float suspicion = baseSuspicion;
	for (const SuspicionModifier& modifier : modifiers)
	{
		suspicion *= modifier(context);
	}
	return std::max(
		suspicion + 0.5f * parentSuspicion,
		0.5f * suspicion + parentSuspicion);
}

} // sl