#include "NPC/UrinePuddle.hpp"

#include <cmath>

#include <SFML/Graphics/CircleShape.hpp>

#include "Engine/Graphics/SFDrawable.hpp"
#include "Utility/Memory.hpp"
#include "Utility/Trig.hpp"


namespace sl
{

UrinePuddle::UrinePuddle(int x, int y, const Urine& urine)
	:Entity(x, y, 6, 6, "UrinePuddle")
	,timer(1000000)
	,urine(urine)
	,puddle(nullptr)
{
	auto drawable = unique<SFDrawable<sf::CircleShape>>(); 
	drawable->setDepth(1); // below player
	puddle = &drawable->getData();
	addDrawable(std::move(drawable));

	updateFluid();
}

void UrinePuddle::addFluid(const Urine& toAdd)
{
	urine += toAdd;
	updateFluid();
}

const Types::Type UrinePuddle::getType() const
{
	return Entity::makeType("UrinePuddle");
}

/*	  Entity private methods	  */
void UrinePuddle::onUpdate()
{
	if (--timer <= 0)
	{
		this->destroy();
	}
}

void UrinePuddle::updateFluid()
{
	if (urine.fluid > 0)
	{
		const float puddleHeight = 0.1f; // 0.1cm high (ml = cm^3)
		const float area = urine.fluid / puddleHeight;
		const float radius = sqrtf(area / pi);
		const float cmPerPixel = 5.f;
		const float radiusPx = radius / cmPerPixel;
		const float diameterPx = radiusPx * 2.f;
	
		puddle->setRadius(radiusPx);
		puddle->setOrigin(radiusPx, radiusPx);
		mask.setOrigin(radiusPx, radiusPx);
		mask.set(pos.x, pos.y, diameterPx, diameterPx);

		const float urobilinPerMg = (float)urine.urobilin / urine.fluid;
		// 1ug/ml is is normal, and min urine production is norm/3, so let's have it go from 1ug-3ug as the range
		const float salinity = std::clamp((urobilinPerMg - 1.f) / 2.f, 0.01f, 0.99f);
		puddle->setFillColor(sf::Color(220 + salinity * 25,
									   220 - salinity * 55,
									   160 - 110 * salinity,
									   100 + 55 * salinity));
	}
}

} // sl
