#include "Parsing/Binding/Bindings.hpp"

#include <iostream>
#include <sstream>
#include <utility>


#include "Parsing/Binding/BoundNumber.hpp"
#include "Parsing/Errors/RuntimeError.hpp"
// for exec()
#include "Parsing/Parsers/StatementParser.hpp"
// for perist: stuff
#include "Parsing/Parsers/FileParser.hpp"
// for random() and randomf()
#include <algorithm>
#include "Utility/Assert.hpp"
#include "Utility/Increment.hpp"
#include "Utility/ProportionList.hpp"
#include "Utility/Random.hpp"
// for enums
#include "NPC/Stats/Soul.hpp"
#include "NPC/ClothingType.hpp"



#include "Utility/Logger.hpp"
#include "Utility/Memory.hpp"

namespace sl
{
namespace ls
{

class PersistantRandom
{
public:
	int random(const Arguments& args)
	{
		VERIFY_ARG_COUNT(args, 1, 2);
		const int n = args[0].asInt();
		ASSERT(n > 0);
		if (args.count() == 1)
		{
			return getAutomaticId(n, args.getContext().getCurrentNode());
		}
		else
		{
			return getManualId(n, args[1].asString());
		}
	}
	// TODO: manual id here too later
	Value choose(const Arguments& args)
	{
		const int n = getAutomaticId(args.count() - 1, args.getContext().getCurrentNode());
		return args[n];
	}

	// TODO: serialize?

private:
	int getManualId(int n, const std::string& id)
	{
		static std::map<std::string, std::vector<int>> rng;
		ASSERT(n > 0);
		auto it = rng.find(id);
		if (it == rng.end())
		{
			it = rng.insert(std::make_pair(std::move(id), std::vector<int>(n))).first;
			std::generate(it->second.begin(), it->second.end(), Increment<int>());
			Random::get().shuffle(it->second.begin(), it->second.end());
		}
		const int ret = it->second.back();
		it->second.pop_back();
		if (it->second.empty())
		{
			rng.erase(it);
		}
		return ret;
	}

	int getAutomaticId(int n, const SyntaxTreeNode *node)
	{
		const size_t curHash = FileParser::get().getHash(node->getFilename());
		auto hash = hashes.find(node->getFilename());
		auto it = hash == hashes.end() || hash->second != curHash
		        ? rng.end()
		        : rng.find(node->getUniqueId());
		if (it == rng.end())
		{
			hashes[node->getFilename()] = curHash;
			// insert a default one not a n-elem one for efficiency, since we might have old ones in the case of hash-mismatch
			auto ir = rng.insert(std::make_pair(node->getUniqueId(), std::vector<int>()));
			it = ir.first;
			it->second.resize(n);
			std::generate(it->second.begin(), it->second.end(), Increment<int>());
			// shuffle so we can just pop_back() for efficiency gains later
			Random::get().shuffle(it->second.begin(), it->second.end());
		}
		const int ret = it->second.back();
		it->second.pop_back();
		if (it->second.empty())
		{
			rng.erase(it);
		}
		return ret;
	}
	
	//! <id, numbers left>
	std::map<std::string, std::vector<int>> rng;
	//! Info about stored hash - if the file hash doesn't match, clear rng <filename, hash>
	std::map<std::string, size_t> hashes;
};

// TODO: store an instance somewhere else and serialize it
static PersistantRandom persistantRandom;

Bindings::Bindings()
	:currentNode(nullptr)
{
	localVars.push(std::map<std::string, Value>());

	constants["true"] = Value(true);
	constants["false"] = Value(false);
#define REG_ENUMCLASS(className, enumName) constants[#enumName] = Value(static_cast<int>(className::enumName))
	// Soul::SexType
	REG_ENUMCLASS(Soul::SexType, Vaginal);
	REG_ENUMCLASS(Soul::SexType, Anal);
	REG_ENUMCLASS(Soul::SexType, Oral);
	REG_ENUMCLASS(Soul::SexType, Handjob);
	REG_ENUMCLASS(Soul::SexType, Thighsex);
	REG_ENUMCLASS(Soul::SexType, Footjob);
	REG_ENUMCLASS(Soul::SexType, Masturbation);
	REG_ENUMCLASS(Soul::SexType, Molestation);
	REG_ENUMCLASS(Soul::SexType, OralReceive);
	// ClothingType
	REG_ENUMCLASS(ClothingType, Outfit);
	REG_ENUMCLASS(ClothingType, Accessory);
	REG_ENUMCLASS(ClothingType, Panties);
	REG_ENUMCLASS(ClothingType, Bra);
	REG_ENUMCLASS(ClothingType, Socks);
#undef REG_ENUMCLASS

	registerFunction("local", "exec", [this](const Arguments& args) {
		if (args.count() > 1)
		{
			// do param passing here?
		}
		FileParser::get().readRawLoliscriptFile(args[0].asString()).execute(*this);
	});
	registerFunction("local", "random", Random::get(), &Random::random);
	registerFunction("local", "randomf", [](const Arguments& args) -> float {
		VERIFY_ARG_COUNT(args, 1, 2);
		const float r = (args.count() == 1)
			          ? Random::get().randomFloat(args[0].asFloat())
			          : Random::get().randomFloat(args[0].asFloat(), args[1].asFloat());
		return r;
	});
	registerFunction("local", "varf", [](float x, OptionalParam<float> diff) -> float {
		return diff.hasValue()
			? x + Random::get().randomFloat(-diff.get(), diff.get())
			: Random::get().randomFloat(-x, x);
	});
	registerFunction("local", "withChance", Random::get(), &Random::chance);
	registerFunction("local", "choose", [](const Arguments& args) -> Value {
		return args[Random::get().random(args.count())];
	});
	registerFunction("local", "choosew", [](const Arguments& args) -> Value {
		LSASSERT(args.count() % 2 == 0);
		ProportionList<Value> choices;
		for (int i = 0; i < args.count(); i += 2)
		{
			choices.insert(args[i], args[i + 1].asFloat());
		}
		return choices.get(Random::get());
	});
	// TODO: figure out why I need a lambda here here - it seems obj + method registration can't work with const Arguments& params?
	// Offending registration: registerFunction("persist", "random", persistantRandom, &PersistanceRandom::random);
	// note: doesn't work even if you make it return a Value. same goes for choose()
	registerFunction("persist", "random", [](const Arguments& args) -> int {
		return persistantRandom.random(args);
	});
	registerFunction("persist", "choose", [](const Arguments& args) -> Value {
		return persistantRandom.choose(args);
	});
	// ternary operator as a function
	registerFunction("local", "trn", [](bool cond, Value a, Value b) -> Value {
		return cond ? a : b;
	});
	// TODO: re-add. apparently doesn't compile on linux.
	//registerFunction("local", "floor", floorf);
	//registerFunction("local", "ceil", ceilf);
	//registerFunction("local", "round", roundf);
	registerFunction("local", "min", [](Value a, Value b) -> Value {
		return a < b ? a : b;
	});
	registerFunction("local", "max", [](Value a, Value b) -> Value {
		return a < b ? b : a;
	});
	registerFunction("local", "pow", powf);
	registerFunction("local", "asBool", [](bool x) { return x; });
	registerFunction("local", "asInt", [](int x) { return x; });
	registerFunction("local", "asFloat", [](float x) { return x; });
	registerFunction("local", "asString", [](std::string x) { return x; });
}

void Bindings::setVar(const std::string& scope, const std::string& name, const Value& value)
{
	if (scope == "local")
	{
		localVars.top()[name] = value;
	}
	else
	{
		try
		{
			variables.at(scope).at(name)->set(value);
		}
		catch (const std::exception& e)
		{
			std::stringstream ss;
			ss << "Error encountered setting " << scope << "." << name << ":\n\t" << e.what() << std::endl;
			throw RuntimeError(ss.str());
		}
	}
}

Value Bindings::getVar(const std::string& scope, const std::string& name) const
{
	if (scope == "local")
	{
		auto cit = constants.find(name);
		if (cit != constants.end())
		{
			return cit->second;
		}
		else
		{
			auto lit = localVars.top().find(name);
			if (lit != localVars.top().end())
			{
				return lit->second;
			}
		}
	}
	Value value;
	try
	{
		value = variables.at(scope).at(name)->get();
	}
	catch (const std::exception& e)
	{
		std::stringstream ss;
		ss << "Error encountered getting " << scope << "." << name << ":\n\t" << e.what() << std::endl;
		throw RuntimeError(ss.str());
	}
	return std::move(value);
}

bool Bindings::variableExists(const std::string& scope, const std::string& name) const
{
	auto scopeIterator = variables.find(scope);
	if (scopeIterator != variables.end())   //  scope exists
	{
		auto nameIterator = scopeIterator->second.find(name);
		return (nameIterator != scopeIterator->second.end());   //  return if name exists in scope
	}
	return false;
}

void Bindings::registerVar(const std::string& scope, const std::string& name, std::unique_ptr<BoundVar> var)
{
	auto scopeIterator = variables.find(scope);
	if (scopeIterator != variables.end())   //  scope exists
	{
		auto nameIterator = scopeIterator->second.find(name);
		if (nameIterator != scopeIterator->second.end())   //  variable also exists
		{
			//  maybe report some kind of warning/error/something?
			Logger::log(Logger::Parsing, Logger::Debug) << "Redefinition of [" << scope << ":" << name << "]" << std::endl;
			nameIterator->second = std::move(var);
		}
		else	//  Add variable, scope already exists
		{
			scopeIterator->second.insert(std::make_pair(name, std::move(var)));
		}
	}
	else	//  Scope doesn't even exist, add both.
	{
		variables[scope][name] = std::move(var);
	}
}

bool Bindings::functionExists(const std::string& scope, const std::string& name) const
{
	auto scopeIterator = functions.find(scope);
	if (scopeIterator != functions.end())   //  scope exists
	{
		auto nameIterator = scopeIterator->second.find(name);
		return (nameIterator != scopeIterator->second.end());   //  return if name exists in scope
	}
	return false;
}

Value Bindings::evaluateFunction(const std::string& scope, const std::string& name, const Parameters& parameters) const
{
	//std::cout << "evaling " << scope << ":" << name << parameters.getFormattedSignature() << std::endl;
	auto scopeIterator = functions.find(scope);
	if (scopeIterator != functions.end())
	{
		auto nameIterator = scopeIterator->second.find(name);
		if (nameIterator != scopeIterator->second.end())
		{
			return nameIterator->second(Arguments(*this, parameters));
		}
		else
		{
			std::stringstream ss;
			ss << "Can't evaluate '" << scope << ":" << name << "(" << parameters.size() << ")"
			   << "' because '" << name << "' was not found in scope '" << scope << "'";
			throw RuntimeError(ss.str());
		}
	}
	else
	{
		std::stringstream ss;
		ss << "Can't evaluate '" << scope << ":" << name << "(" << parameters.size() << ")"
		   << "' because scope '" << scope << "' doesn't even exist";
		throw RuntimeError(ss.str());
	}
}

void Bindings::unregisterScope(const std::string& scope)
{
	auto scopeIterator = variables.find(scope);
	if (scopeIterator != variables.end())
	{
		scopeIterator->second.clear();
	}
	else
	{
		Logger::log(Logger::Parsing, Logger::Serious) << "Trying to unregister scope [" << scope << ":*] but scope doesn't even exist in the first place. :(" << std::endl;
	}
}

void Bindings::unregisterVar(const std::string& scope, const std::string& name)
{
	auto scopeIterator = variables.find(scope);
	if (scopeIterator != variables.end())   //  scope exists
	{
		auto nameIterator = scopeIterator->second.find(name);
		if (nameIterator != scopeIterator->second.end())   //  variable also exists
		{
			scopeIterator->second.erase(nameIterator);
		}
		else
		{
			Logger::log(Logger::Parsing, Logger::Serious) << "Trying to unregister var [" << scope << ":" << name << "] but var wasn't registered in the first place." << std::endl;
		}
	}
	else
	{
		Logger::log(Logger::Parsing, Logger::Serious) << "Trying to unregister var [" << scope << ":" << name << "] but scope doesn't even exist in the first place. :(" << std::endl;
	}
}

#ifdef SL_DEBUG
void Bindings::print()
{
	Logger::log(Logger::Parsing, Logger::Debug) << "Printing bound context vars:" << std::endl;
	for (auto it = variables.cbegin(); it != variables.cend(); ++it)
	{
		for (auto it2 = it->second.cbegin(); it2 != it->second.cend(); ++it2)
		{
			Logger::log(Logger::Parsing, Logger::Debug) << "\t" << it->first << "." << it2->first << " = " << it2->second->get().asString() << std::endl;
		}
	}
}
#endif

const SyntaxTreeNode* Bindings::getCurrentNode() const
{
	return currentNode;
}

} // ls

void VERIFY_ARG_COUNT(const ls::Arguments& args, int desiredCount)
{
	if (args.count() != desiredCount)
	{
		std::stringstream ss;
		ss << "Found " << args.count() << " arguments, but function has " << desiredCount << " parameters";
		throw ls::RuntimeError(ss.str());
	}
}
void VERIFY_ARG_COUNT(const ls::Arguments& args, int minParamCount, int maxParamCount)
{
	if (args.count() < minParamCount || args.count() > maxParamCount)
	{
		std::stringstream ss;
		ss << "Found " << args.count() << " arguments, but function only accepts " << minParamCount << " to " << maxParamCount << " parameters";
		throw ls::RuntimeError(ss.str());
	}
}

} // sl
