#include "Parsing/Binding/LintBindings.hpp"

#include "Parsing/Binding/Bindings.hpp"
#include "Utility/Assert.hpp"

namespace sl
{
namespace ls
{

LintBindings::LintBindings()
	:bindings(nullptr)
	,currentFile("no file set")
	,currentLine(-1)
{
}

void LintBindings::addVariable(const std::string& scope, const std::string& name)
{
	variables.insert(std::make_pair(scope, name));
}

void LintBindings::addFunction(const std::string& scope, const std::string& name)
{
	functions.insert(std::make_pair(scope, name));
}

void LintBindings::checkVariable(const std::string& scope, const std::string& name)
{
	if (variables.count(std::make_pair(scope, name)) == 0 &&
	    (bindings == nullptr || !bindings->variableExists(scope, name)))
	{
		errors.push_back(Error{Error::Type::Variable, scope, name, currentFile, currentLine});
	}
}

void LintBindings::checkFunction(const std::string& scope, const std::string& name)
{
	if (functions.count(std::make_pair(scope, name)) == 0 &&
	    (bindings == nullptr || !bindings->functionExists(scope, name)))
	{
		errors.push_back(Error{Error::Type::Function, scope, name, currentFile, currentLine});
	}
}

void LintBindings::setBindings(const Bindings& bindings)
{
	this->bindings = &bindings;
}

void LintBindings::markFile(const std::string& file)
{
	currentFile = file;
}

void LintBindings::markLine(int line)
{
	currentLine = line;
}

const std::vector<LintBindings::Error>& LintBindings::getErrors() const
{
	return errors;
}

void LintBindings::Error::print(std::ostream& os) const
{
	ASSERT(type == Type::Function || type == Type::Variable);
	os << file << ":" << line << " | " << (type == Type::Variable ? "variable '" : "function '") << name << "' does not exist in scope '" << scope << "'";
}

} // ls
} // sl