#ifndef SL_LS_LINTBINDINGS_HPP
#define SL_LS_LINTBINDINGS_HPP

#include <set>
#include <string>
#include <vector>

namespace sl
{
namespace ls
{

class Bindings;

class LintBindings
{
public:
	LintBindings();

	void addVariable(const std::string& scope, const std::string& name);

	void addFunction(const std::string& scope, const std::string& name);

	void checkVariable(const std::string& scope, const std::string& name);

	void checkFunction(const std::string& scope, const std::string& name);

	void setBindings(const Bindings& bindings);

	void markFile(const std::string& file);

	void markLine(int line);

	struct Error
	{
		void print(std::ostream& os) const;

		enum class Type
		{
			Variable,
			Function
		};
		Type type;
		std::string scope;
		std::string name;
		std::string file;
		int line;
	};
	
	const std::vector<Error>& getErrors() const;
	
private:
	const Bindings *bindings;
	std::set<std::pair<std::string, std::string>> variables;
	std::set<std::pair<std::string, std::string>> functions;
	std::vector<Error> errors;
	std::string currentFile;
	int currentLine;
};

} // ls
} // sl

#endif // SL_LS_LINTBINDINGS_HPP