#ifndef SL_LS_OPTIONALPARAM_HPP
#define SL_LS_OPTIONALPARAM_HPP

#include "Parsing/Binding/Value.hpp"
#include "Parsing/Binding/ValueConversion.hpp"
#include "Utility/Assert.hpp"

namespace sl
{
namespace ls
{

template <typename T>
class OptionalParam
{
public:
	// Only here because AutomaticFunctionWrapping requires automatically converted types to have default constructors in convertFunctorArgs()
	OptionalParam() = default;

	OptionalParam(Value value)
		:value(std::move(value))
	{
	}

	Value getValue() const
	{
		ASSERT(hasValue());
		return value;
	}

	Value getValue(const T& defaultValue) const
	{
		return hasValue()
			? value
			: Value::make(defaultValue);
	}

	bool hasValue() const
	{
		return value.getType() != Value::Type::Uninitialized;
	}

	T get() const
	{
		ASSERT(hasValue());
		return convert<T>(getValue());
	}

	T get(const T& defaultValue) const
	{
		return hasValue()
			? get()
			: defaultValue;
	}

private:
	Value value;
};

namespace avc
{

template <typename T>
struct ValueConverter<OptionalParam<T>>
{
	static OptionalParam<T> convert(const Value& value)
	{
		return OptionalParam<T>(value);
	}
};

} // avc

} // ls
} // sl

#endif // SL_LS_OPTIONALPARAM_HPP