#ifndef SL_LS_PARAMETERS_HPP
#define SL_LS_PARAMETERS_HPP

#include "Parsing/SyntaxTree/Expressions/Expression.hpp"
#include "Parsing/Binding/Value.hpp"

#include <string>
#include <vector>
#include <memory>

namespace sl
{
namespace ls
{

class Bindings;

class Parameters
{
public:
	Parameters() = default;

	template <typename... Expressions>
	Parameters(std::unique_ptr<const Expression> expression, Expressions... expressions)
	{
		addParameters(std::move(expression), std::forward(expressions)...);
	}

	template <typename... Expressions>
	void addParameters(std::unique_ptr<const Expression> expression, Expressions... expressions)
	{
		addParameter(std::move(expression));
		addParameters(expressions...);
	}
	
	void addParameters() {}

	void addParameter(std::unique_ptr<const Expression> expParam);

	Value getParameter(const Bindings& context, int parameterIndex) const;

	int size() const;

private:

	std::vector<std::unique_ptr<const Expression>> expressions;
};

} // ls
} // sl

#endif // SL_LS_PARAMETERS_HPP
