#ifndef SL_LS_VALUECONVERSION_HPP
#define SL_LS_VALUECONVERSION_HPP

#include "Parsing/Binding/Value.hpp"

#include <string>

// TODO: move (for enum conversion)
#include "NPC/ClothingType.hpp"
#include "NPC/Stats/Soul.hpp"

namespace sl
{
namespace ls
{

// TODO: move?
#define LSASSERT_MSG(x, msg) do { \
		if (!(x)) { \
			throw ::sl::ls::RuntimeError(std::string("Loliscript assertion failed: ") + msg); \
		} \
	} while (0)
#define LSASSERT(x) LSASSERT_MSG(x, #x)

namespace avc
{

// this is a class so we can partially specialize it, unlike functions (Damn you C++...)
template <typename T>
struct ValueConverter
{
	static T convert(const Value&)
	{
		static_assert(false, "Unsupported Value Type");
		return T();
	}
};
template <typename T>
struct ValueConverter<T*>
{
	static T* convert(const Value& value)
	{
		return value.asPtr<T>();
	}
};
template <>
struct ValueConverter<Value>
{
	static const Value& convert(const Value& value)
	{
		return value;
	}
};
#define VALUE_DEFINE_CONVERT_AS(T, method) template <> \
struct ValueConverter<T> { \
	static T convert(const Value& value) { \
		return value.method(); \
	} \
}
VALUE_DEFINE_CONVERT_AS(bool, asBool);
VALUE_DEFINE_CONVERT_AS(int, asInt);
VALUE_DEFINE_CONVERT_AS(float, asFloat);
VALUE_DEFINE_CONVERT_AS(double, asFloat);
VALUE_DEFINE_CONVERT_AS(std::string, asString);
#undef VALUE_DEFINE_CONVERT_AS

// TODO: where to put enums?
#define VALUE_DEFINE_CONVERT_ENUM(Enum, size) template <> \
struct ValueConverter<Enum> { \
static Enum convert(const Value& value) { \
	const int typeIndex = value.asInt(); \
	LSASSERT(typeIndex >= 0 && typeIndex < size); \
	return static_cast<Enum>(typeIndex); \
} \
}
VALUE_DEFINE_CONVERT_ENUM(ClothingType, CLOTHING_TYPE_COUNT);
VALUE_DEFINE_CONVERT_ENUM(Soul::SexType, Soul::SexType_Count);
#undef VALUE_DEFINE_CONVERT_ENUM

} // avc

template <typename T>
static inline T convert(const Value& value)
{
	return avc::ValueConverter<T>::convert(value);
}


} // ls
} // sl

#endif // SL_LS_VALUECONVERSION_HPP