#include "Parsing/Errors/RuntimeError.hpp"

#include "Parsing/SyntaxTree/Statements/Statement.hpp"

#include <sstream>

namespace sl
{
namespace ls
{

RuntimeError::RuntimeError(const std::string& info, const Statement *statement)
	:origInfo(info)
	,annotated(false)
{
	annotate(statement);
}

const char * RuntimeError::what() const throw()
{
	return formattedInfo.c_str();
}

void RuntimeError::annotate(const Statement* statement)
{
	std::stringstream ss;
	ss << "Runtime interpreter error ";
	if (statement)
	{
		ss << "[" << statement->getFilename() << ":" << statement->getLine() << "]";
		annotated = true;
	}
	else
	{
		ss << "[no file info]";
	}
	ss << " : " << origInfo;
	formattedInfo = ss.str();
}

bool RuntimeError::isAnnotated() const
{
	return annotated;
}

} // ls
} // sl