#include "Parsing/Parsers/FileParser.hpp"

#include "Parsing/Errors/SyntaxError.hpp"
#include "Parsing/Parsers/DialogueParser.hpp"
#include "Parsing/Parsers/ExpressionParser.hpp"
#include "Parsing/Parsers/StatementParser.hpp"
#include "Utility/FileUtil.hpp"

#include <functional>
#include <numeric>

namespace sl
{
namespace ls
{

static size_t hashLines(const std::vector<std::string>& lines)
{
	const std::string allAsOne = std::accumulate(lines.begin(), lines.end(), std::string());
	return std::hash<std::string>()(allAsOne);
}

const Statement& FileParser::readRawLoliscriptFile(const std::string& loliscriptName)
{
	const std::string fullFilename = "resources/dlg/" + loliscriptName + ".ls";
	auto it = cachedLoliscript.find(fullFilename);
	// TODO: have a flag to disable cached check/reloads (to allow on-the-fly changing)
	if (it == cachedLoliscript.end())
	{
		ASSERT(cachedLoliscript.count(fullFilename) == 0);
		std::vector<std::string> lines = fileToLines(fullFilename.c_str());
		if (lines.empty())
		{
			throw SyntaxError("Loliscript file (" + fullFilename + ") does not exist or is empty");
		}
		ExpressionParser expressionParser(fullFilename);
		it = cachedLoliscript.insert(std::make_pair(fullFilename, parseStrings(expressionParser, lines, 0, 0, fullFilename))).first;
		auto ret = hashes.insert(std::make_pair(fullFilename, hashLines(lines)));
		ASSERT(ret.second);
	}
	return *it->second;
}

const Scenario& FileParser::readDialogueFile(const std::string& dialogueName)
{
	const std::string fullFilename = fullPathFromDialogueIdentifier(dialogueName);
	auto it = cachedDialogue.find(fullFilename);
	// TODO: have a flag to disable cached check/reloads (to allow on-the-fly changingKO
	if (it == cachedDialogue.end())
	{
		ASSERT(cachedDialogue.count(fullFilename) == 0);
		std::vector<std::string> lines = fileToLines(fullFilename.c_str());
		if (lines.empty())
		{
			throw SyntaxError("Dialogue file (" + fullFilename + ") does not exist or is empty");
		}
		ExpressionParser expressionParser(fullFilename);
		it = cachedDialogue.insert(std::make_pair(fullFilename, parseDialogueStrings(expressionParser, dialogueName, lines))).first;
		auto ret = hashes.insert(std::make_pair(fullFilename, hashLines(lines)));
		ASSERT(ret.second);
	}
	return *it->second;
}

size_t FileParser::getHash(const std::string& filename) const
{
	auto it = hashes.find(filename);
	ASSERT(it != hashes.end());
	return it->second;
}

/*static*/FileParser& FileParser::get()
{
	static FileParser instance;
	return instance;
}

} // ls
} // sl