/**
 * @DESCRIPTION Wrapper around dialogue/loliscript parsing.
 * This has additional funcitonality like caching and hashing files.
 * The hashing is needed so we can remove old data associated with persist: calls upon file changes
 */
#ifndef SL_LS_FILEPARSER_HPP
#define SL_LS_FILEPARSER_HPP

#include "NPC/Dialogue/Scenario.hpp"
#include "Parsing/SyntaxTree/Statements/Statement.hpp"

#include <memory>
#include <map>
#include <string>

namespace sl
{
namespace ls
{

class FileParser
{
public:
	/**
	 * @param filename WITHOUT the extension and WITHOUT the path to the dialogue files (adds .ls)
	 * @return 
	 */
	const Statement& readRawLoliscriptFile(const std::string& loliscriptName);

	/**
	 * @param filename WITHOUT the extension and WITHOUT the path to the dialogue files
	 * @param 
	 */
	const Scenario& readDialogueFile(const std::string& dialogueName);

	/**
	 * @param filename The FULL filename WITH extension
	 */
	size_t getHash(const std::string& fullFilename) const;

	static FileParser& get();

private:
	std::map<std::string, size_t> hashes;
	std::map<std::string, std::unique_ptr<const Statement>> cachedLoliscript;
	std::map<std::string, std::unique_ptr<const Scenario>> cachedDialogue;
};

} // ls
} // sl

#endif // SL_LS_FILEPARSER_HPP