#include "Parsing/Parsers/ParamParser.hpp"

#include "Parsing/Parsers/ExpressionParser.hpp"
#include "Parsing/Errors/SyntaxError.hpp"

#include "Utility/Logger.hpp" 

namespace sl
{
namespace ls
{

std::unique_ptr<const Parameters> parseFunctionParameters(ExpressionParser& expressionParser, const char *c)
{
	int depthLevel = 0;
	const char *startOfParam = c;
	auto params = std::make_unique<Parameters>();
	bool isInLiteral = false;
	for (; *c != '\0'; ++c)
	{
		if (isInLiteral)
		{
			// don't worry about there being a null-terminator before this is ended since
			// that would've been caught earlier
			if (*c == '"' && *(c - 1) != '\\') // safe since never in string literal in first char
			{
				isInLiteral = false;
			}
			continue;
		}
		else
		{
			if (*c == '"')
			{
				isInLiteral =  true;
			}
			else if (*c == '(')
			{
				++depthLevel;
			}
			else if (*c == ')')
			{
				if (--depthLevel < 0)
				{
					std::stringstream ss;
					ss << "Unbalanced parenthesis in function parameters '"
					   << c
					   << "' - too many )s";
					throw SyntaxError(ss.str());
				}
			}
			else if (*c == ',' && depthLevel == 0)	//	move onto the next param
			{
				std::string buffer(startOfParam, c - startOfParam);
				//std::cout << "!arg is {" << buffer << "}" << std::endl;
				params->addParameter(expressionParser.parseEntireExpression(buffer.c_str()));
				startOfParam = c + 1;
			}
		}
	}
	if (depthLevel > 0)
	{
		std::stringstream ss;
		ss << "Unbalanced parenthesis in function parameters '"
			<< c
			<< "' - too many (s";
		throw SyntaxError(ss.str());
	}
	std::string buffer(startOfParam,  c - startOfParam);
	Logger::log(Logger::Parsing) << "~arg is {" << buffer << "}" << std::endl;
	if (!buffer.empty())
	{
		params->addParameter(expressionParser.parseEntireExpression(buffer.c_str()));
	}
	//Logger::log(Logger::Parsing) << "paramlist created: " << params->getFormattedSignature() << std::endl;
	return params;
}

} // ls
} // sl