#include "Parsing/Parsers/ParserTester.hpp"

#include <iostream>
#include <string>
#include <vector>
#include <sstream>

#include "Parsing/Parsers/StatementParser.hpp"
#include "Parsing/Parsers/ExpressionParser.hpp"
#include "Parsing/SyntaxTree/Expressions/Literal.hpp"
#include "Parsing/Errors/SyntaxError.hpp"
#include "Parsing/Errors/RuntimeError.hpp"
#include "Tests/ParserTestUtil.hpp"
#include "Utility/Logger.hpp"
#include "Utility/Memory.hpp"

/*
	TODO:	 fix a ++ b crashing
*/

namespace sl
{
namespace ls
{

void useParser()
{
	Logger logger;
	Logger::setDefaultLogger(&logger);
	logger.setMinimumPriority(Logger::Parsing, Logger::Debug);

	std::cout << "Welcome to the parsing tester!" << std::endl;
	std::string input;
	Bindings context;
	test::bindTestFunctions(context);
	//context.registerFunction("local", "printTheThing", print_the_thing);
	for ( ; ; )
	{
		try
		{
			std::cout << ">";
			std::getline(std::cin, input);
			ExpressionParser parser("ParserTester");
			if (input.compare(0, 4, "eval") == 0)
			{
				std::unique_ptr<Expression> exp = parser.parseEntireExpression(input.c_str() + 4);
				std::cout << exp->evaluate(context).asString() << std::endl;
			}
			else if (input.compare(0, 4, "exec") == 0)
			{
				std::vector<std::string> lines;
				std::cout << "Enter lines to get executed. Type \"end\" to finish." << std::endl;
				for(int line = 1; ; ++line)
				{
					for (int i = 1000; i > line; i /= 10)
					{
						std::cout << " ";
					}
					std::cout << line << "|";
					std::getline(std::cin, input);
					if (input == "end")
					{
						executeStrings(lines, context);
						break;
					}
					else
					{
						lines.push_back(input);
					}
				}
			}
			else if (input.compare(0, 6, "concat") == 0)
			{
				auto s = parser.parseAsConcatenatedStrings(input.c_str() + 6);
				std::cout << "Concat result: " << s->evaluate(context).asString() << std::endl;;
			}
			else if (input.compare(0, 3, "run") == 0)
			{
			}
			else if (input.compare(0, 4, "exit") == 0)
			{
				break;
			}
			else
			{
				std::cout << "Invalid input. Try: 'eval <expression>', 'exec', 'run <filename>', or 'exit'" << std::endl;
			}
		}
		catch (SyntaxError& see)
		{
			std::cout << "Syntax Error Occured:\n" << see.what() << std::endl;
		}
		catch (RuntimeError& riee)
		{
			std::cout << "Runtime Error Occured:\n" << riee.what() << std::endl;
		}
	}
}

} // ls
} // sl
