#include "Parsing/SyntaxTree/Expressions/Arithmetic.hpp"

namespace sl
{
namespace ls
{

/*		Addition			*/
Addition::Addition(const std::string& filename, int uniqueId, std::unique_ptr<const Expression> lhs, std::unique_ptr<const Expression> rhs)
	:Expression(filename, uniqueId)
	,lhs(std::move(lhs))
	,rhs(std::move(rhs))
{
}

Value Addition::onEvaluate(const Bindings& context) const
{
	return lhs->evaluate(context) + rhs->evaluate(context);
}

void Addition::lint(LintBindings& context) const
{
	lhs->lint(context);
	rhs->lint(context);
}



/*		Subtraction			*/
Subtraction::Subtraction(const std::string& filename, int uniqueId, std::unique_ptr<const Expression> lhs, std::unique_ptr<const Expression> rhs)
	:Expression(filename, uniqueId)
	,lhs(std::move(lhs))
	,rhs(std::move(rhs))
{
}

Value Subtraction::onEvaluate(const Bindings& context) const
{
	return lhs->evaluate(context) - rhs->evaluate(context);
}

void Subtraction::lint(LintBindings& context) const
{
	lhs->lint(context);
	rhs->lint(context);
}




/*		Multiplication		*/
Multiplication::Multiplication(const std::string& filename, int uniqueId, std::unique_ptr<const Expression> lhs, std::unique_ptr<const Expression> rhs)
	:Expression(filename, uniqueId)
	,lhs(std::move(lhs))
	,rhs(std::move(rhs))
{
}

Value Multiplication::onEvaluate(const Bindings& context) const
{
	return lhs->evaluate(context) * rhs->evaluate(context);
}

void Multiplication::lint(LintBindings& context) const
{
	lhs->lint(context);
	rhs->lint(context);
}



/*		Division		*/
Division::Division(const std::string& filename, int uniqueId, std::unique_ptr<const Expression> lhs, std::unique_ptr<const Expression> rhs)
	:Expression(filename, uniqueId)
	,lhs(std::move(lhs))
	,rhs(std::move(rhs))
{
}

Value Division::onEvaluate(const Bindings& context) const
{
	return lhs->evaluate(context) / rhs->evaluate(context);
}

void Division::lint(LintBindings& context) const
{
	lhs->lint(context);
	rhs->lint(context);
}



/*		Unary minus		*/
UnaryMinus::UnaryMinus(const std::string& filename, int uniqueId, std::unique_ptr<const Expression> expression)
	:Expression(filename, uniqueId)
	,expression(std::move(expression))
{
}

Value UnaryMinus::onEvaluate(const Bindings& context) const
{
	return Value(-1) * expression->evaluate(context);
}

void UnaryMinus::lint(LintBindings& context) const
{
	expression->lint(context);
}

} // ls
} // sl
