#ifndef SL_LS_ARITHMETIC_HPP
#define SL_LS_ARITHMETIC_HPP

#include "Parsing/SyntaxTree/Expressions/Expression.hpp"

#include <memory>

namespace sl
{
namespace ls
{
class Addition : public Expression
{
public:
	Addition(const std::string& filename, int uniqueId, std::unique_ptr<const Expression> lhs, std::unique_ptr<const Expression> rhs);

	Value onEvaluate(const Bindings& context) const override;

	void lint(LintBindings& context) const override;

private:
	const std::unique_ptr<const Expression> lhs;
	const std::unique_ptr<const Expression> rhs;
};




class Subtraction : public Expression
{
public:
	Subtraction(const std::string& filename, int uniqueId, std::unique_ptr<const Expression> lhs, std::unique_ptr<const Expression> rhs);

	Value onEvaluate(const Bindings& context) const override;

	void lint(LintBindings& context) const override;

private:
	const std::unique_ptr<const Expression> lhs;
	const std::unique_ptr<const Expression> rhs;
};




class Multiplication : public Expression
{
public:
	Multiplication(const std::string& filename, int uniqueId, std::unique_ptr<const Expression> lhs, std::unique_ptr<const Expression> rhs);

	Value onEvaluate(const Bindings& context) const override;

	void lint(LintBindings& context) const override;

private:
	const std::unique_ptr<const Expression> lhs;
	const std::unique_ptr<const Expression> rhs;
};




class Division : public Expression
{
public:
	Division(const std::string& filename, int uniqueId, std::unique_ptr<const Expression> lhs, std::unique_ptr<const Expression> rhs);

	Value onEvaluate(const Bindings& context) const override;

	void lint(LintBindings& context) const override;

private:
	const std::unique_ptr<const Expression> lhs;
	const std::unique_ptr<const Expression> rhs;
};




class UnaryMinus : public Expression
{
public:
	UnaryMinus(const std::string& filename, int uniqueId, std::unique_ptr<const Expression> expression);

	Value onEvaluate(const Bindings& context) const override;

	void lint(LintBindings& context) const override;

private:
	const std::unique_ptr<const Expression> expression;
};

} // ls
} // sl

#endif // SL_LS_ARITHMETIC_HPP