#include "Parsing/SyntaxTree/Expressions/Logical.hpp"

namespace sl
{
namespace ls
{

/*			Logical AND			*/
LogicalAnd::LogicalAnd(const std::string& filename, int uniqueId, std::unique_ptr<const Expression> lhs, std::unique_ptr<const Expression> rhs)
	:Expression(filename, uniqueId)
	,lhs(std::move(lhs))
	,rhs(std::move(rhs))
{
}


Value LogicalAnd::onEvaluate(const Bindings& context) const
{
	return Value(lhs->evaluate(context).asBool() && rhs->evaluate(context).asBool());
}

void LogicalAnd::lint(LintBindings& context) const
{
	lhs->lint(context);
	rhs->lint(context);
}



/*			Logical OR			*/
LogicalOr::LogicalOr(const std::string& filename, int uniqueId, std::unique_ptr<const Expression> lhs, std::unique_ptr<const Expression> rhs)
	:Expression(filename, uniqueId)
	,lhs(std::move(lhs))
	,rhs(std::move(rhs))
{
}

Value LogicalOr::onEvaluate(const Bindings& context) const
{
	return Value(lhs->evaluate(context).asBool() || rhs->evaluate(context).asBool());
}

void LogicalOr::lint(LintBindings& context) const
{
	lhs->lint(context);
	rhs->lint(context);
}




/*			Logical NOT		*/
Inverse::Inverse(const std::string& filename, int uniqueId, std::unique_ptr<const Expression> expression)
	:Expression(filename, uniqueId)
	,expression(std::move(expression))
{
}

Value Inverse::onEvaluate(const Bindings& context) const
{
	return Value(!expression->evaluate(context).asBool());
}

void Inverse::lint(LintBindings& context) const
{
	expression->lint(context);
}

} // ls
} // sl
