#include "Parsing/SyntaxTree/Statements/Block.hpp"

#include "Parsing/Binding/Bindings.hpp"

namespace sl
{
namespace ls
{

Block::Block(std::string filename, int line)
	:Statement(std::move(filename), line)
{
}

Block::Block(std::string filename, int line, std::vector<std::unique_ptr<Statement>> statements)
	:Statement(std::move(filename), line)
	,statements(std::move(statements))
{
}

void Block::onExecute(Bindings& context) const
{
	for (const std::unique_ptr<Statement>& statement : statements)
	{
		statement->execute(context);
	}
}

void Block::lint(LintBindings& context) const
{
	context.markLine(getLine());
	for (const auto& statement : statements)
	{
		statement->lint(context);
	}
}

} // ls
} // sl
