#include "Parsing/SyntaxTree/Statements/ChanceProbability.hpp"

#include "Utility/Assert.hpp"
#include "Utility/Mapping.hpp"
#include "Utility/Random.hpp"

#include <numeric>

namespace sl
{
namespace ls
{

ChanceProbability::ChanceProbability(std::string filename, int line)
	:Statement(std::move(filename), line)
{
}

void ChanceProbability::addCase(std::unique_ptr<const Expression> probability, std::unique_ptr<const Statement> body)
{
	cases.push_back(std::make_pair(std::move(probability), std::move(body)));
}

void ChanceProbability::onExecute(Bindings& context) const
{
	// only eval twice in case the expressions are mutable? I'm not sure if they can be, either way, this (can be) more efficient
	const std::vector<float> probabilities = mapVec(cases, [&context](const auto& casePair) -> float {
		return casePair.first->evaluate(context).asFloat();
	});
	const float sum = std::accumulate(probabilities.begin(), probabilities.end(), 0.f);
	float x = Random::get().randomFloat(sum);
	for (int i = 0; i < probabilities.size(); ++i)
	{
		if (x <= probabilities[i] + std::numeric_limits<float>::epsilon())
		{
			cases[i].second->execute(context);
			return;
		}
		x -= probabilities[i];
	}
	ERROR("Should not be reached. Floating point fuckery?");
}

void ChanceProbability::lint(LintBindings& context) const
{
	context.markLine(getLine());
	for (const auto& probabilityCase : cases)
	{
		probabilityCase.first->lint(context);
		probabilityCase.second->lint(context);
	}
}

} // ls
} // sl
