#include "Parsing/SyntaxTree/Statements/If.hpp"

namespace sl
{
namespace ls
{

If::If(std::string filename, int line, std::unique_ptr<const Expression> condition, std::unique_ptr<const Statement> onTrue, std::unique_ptr<const Statement> onFalse)
	:Statement(std::move(filename), line)
	,condition(std::move(condition))
	,onTrue(std::move(onTrue))
	,onFalse(std::move(onFalse))
{
}

void If::onExecute(Bindings& context) const
{
	if (condition->evaluate(context).asBool())
	{
		onTrue->execute(context);
	}
	else if (onFalse)
	{
		onFalse->execute(context);
	}
}

void If::lint(LintBindings& context) const
{
	context.markLine(getLine());
	condition->lint(context);
	onTrue->lint(context);
	if (onFalse)
	{
		onFalse->lint(context);
	}
}

} // ls
} // sl
