#include "Parsing/SyntaxTree/Statements/Statement.hpp"

#include "Parsing/Errors/RuntimeError.hpp"

namespace sl
{
namespace ls
{

Statement::Statement(std::string filename, int line)
	:SyntaxTreeNode(filename, line)
	,line(line)
{
}

void Statement::execute(Bindings& context) const
{
	// Only annotate on the deepest (ie most accurate) Statement.
	Bindings::Mark mark(context, this);
	try
	{
		onExecute(context);
	}
	catch (RuntimeError& rie)
	{
		// Only annotate the deepest statement, to give the most accurate results
		// and also to not have long, ugly, recursive and largely redundant annotations.
		if (!rie.isAnnotated())
		{
			rie.annotate(this);
		}
		throw rie;
	}
}

int Statement::getLine() const
{
	return line;
}

} // ls
} // sl
