#include "Parsing/SyntaxTree/Statements/SwitchCase.hpp"

namespace sl
{
namespace ls
{

SwitchCase::SwitchCase(std::string filename, int line, std::unique_ptr<const Expression> expression)
	:Statement(std::move(filename), line)
	,expression(std::move(expression))
{
}

void SwitchCase::addCase(std::unique_ptr<const Expression> expression, std::unique_ptr<const Statement> body)
{
	cases.push_back(std::make_pair(std::move(expression), std::move(body)));
}

bool SwitchCase::setDefault(std::unique_ptr<const Statement> body)
{
	if (defaultStatement == nullptr)
	{
		defaultStatement = std::move(body);
		return true;
	}
	return false;
}

void SwitchCase::onExecute(Bindings& context) const
{
	const Value value = expression->evaluate(context);
	for (const auto& casePair : cases)
	{
		if (value == casePair.first->evaluate(context))
		{
			casePair.second->execute(context);
			return;
		}
	}
	if (defaultStatement)
	{
		defaultStatement->execute(context);
	}
}

void SwitchCase::lint(LintBindings& context) const
{
	context.markLine(getLine());
	for (const auto& casePair : cases)
	{
		casePair.first->lint(context);
		casePair.second->lint(context);
	}
}

} // ls
} // sl
