#include "Parsing/SyntaxTree/Statements/WhileLoop.hpp"

namespace sl
{
namespace ls
{

WhileLoop::WhileLoop(std::string filename, int line, std::unique_ptr<const Expression> condition, std::unique_ptr<const Statement> body)
	:Statement(std::move(filename), line)
	,condition(std::move(condition))
	,body(std::move(body))
{
}


void WhileLoop::onExecute(Bindings& context) const
{
	while (condition->evaluate(context).asBool())
	{
		body->execute(context);
	}
}

void WhileLoop::lint(LintBindings& context) const
{
	context.markLine(getLine());
	condition->lint(context);
	body->lint(context);
}

} // ls
} // sl

