#include "Pathfinding/CarConfig.hpp"

#include "Engine/Scene.hpp"
#include "Town/Cars/Car.hpp"
#include "Utility/Assert.hpp"
#include "Utility/Trig.hpp"

namespace sl
{
namespace pf
{

Scene *CarConfig::town = nullptr;
float CarConfig::halfTownTileSize = 8.f;
const float CarConfig::speedToRealSpeed[4] = {
	-2.f, 3.f, 8.f, 12.f
};
const float CarConfig::angleToRealAngle[16] = {
	0.f,
	22.5f,
	45.f,
	67.5f,
	90.f,
	112.5f,
	135.f,
	157.5f,
	180.f,
	202.5f,
	225.f,
	247.5f,
	270.f,
	292.5f,
	315.f,
	337.5f
};

CarConfig::CarConfig(Vector2i pos, Speed speed, Angle angle)
	:pos(pos)
	,speed(speed)
	,angle(angle)
{
}

NodeID CarConfig::toNodeID() const
{
	return NodeID(pos.x / 2, pos.y / 2, town);
}

float CarConfig::distance(const CarConfig& rhs) const
{
	const float posDiff = halfTownTileSize * pointDistance(pos.x, pos.y, 2.f * rhs.pos.x, 2.f * rhs.pos.y);
	const float speedDiff = speedToRealSpeed[static_cast<int>(speed)] - speedToRealSpeed[static_cast<int>(rhs.speed)];
	const float a1 = static_cast<float>(angle);
	const float a2 = static_cast<float>(rhs.angle);
	const float adiffabs = a1 - a2;
	const float adiff = std::min(adiffabs, 16.f - adiffabs);
	// TODO: verify that these are even good ratios
	return posDiff + 0.5f*(speedDiff*speedDiff) + (adiff*adiff*adiff);
}

float CarConfig::distance(const NodeID& rhs) const
{
	return halfTownTileSize * pointDistance(pos.x, pos.y, 2.f * rhs.getX(), 2.f * rhs.getY());
}

/*static*/CarConfig CarConfig::carToConfig(const Car& car)
{
	Speed speed = Speed::REVERSE;
	if (car.getSpeed() > speedToRealSpeed[2])
	{
		speed = Speed::FAST;
	}
	else if (car.getSpeed() > speedToRealSpeed[1])
	{
		speed = Speed::MEDIUM;
	}
	else if (car.getSpeed() >= 0.f)
	{
		speed = Speed::SLOW;
	}
	const Angle angle = static_cast<Angle>(static_cast<int>(car.getAngle() * 16 / 180.f) % 16);
	return CarConfig(Vector2i(car.getPos()) / halfTownTileSize, speed, angle);
}

/*static*/ void CarConfig::setTown(Scene *newTown)
{
	// there should only be one town. if that changes, CarConfig must take the town as an argument.
	ASSERT(town == nullptr || newTown == nullptr);
	town = newTown;
	halfTownTileSize = town->getTileSize() / 2.f;
}

} // pf
} // sl