#ifndef SL_PF_GRIDPATHMANAGER_HPP
#define SL_PF_GRIDPATHMANAGER_HPP

#include <initializer_list>
#include <string>
#include <vector>
#include <deque>
#include <functional>
#include <map>
#include <mutex>
#include <thread>

#include "Pathfinding/PathGrid.hpp"
#include "Pathfinding/PathManager.hpp"

namespace sl
{

class Scene;
class Target;

namespace pf
{

class GridPathManager : public PathManager
{
public:
	GridPathManager();

	//---------------------  PathManager  ---------------------
	std::shared_ptr<Path> getPath(const PathSpec& spec, Priority priority) override;

	bool cancelCalculation(std::shared_ptr<Path>& path) override;

	void cancelAllCalculations() override;

	NodeID getRandomID() const override;

	NodeID getClosestNodeID(const Target& target) const override;

	Node getNodeFromID(NodeID id) const override;

	void update(int64_t timestamp) override;

	//-------------------  GridPathManager  -------------------
	void registerGrid(Scene *scene, std::shared_ptr<PathGrid> pathGrid);


#ifdef SL_DEBUG
	std::vector<std::map<Vector2i, std::set<Vector2i>>> exctractGraphsAsAdjlists(Scene *scene) const;
#endif // SD_DEBUG

private:
#ifndef SL_NOMULTITHREADED
	struct Args
	{
		Args(std::shared_ptr<Path> path, const PathSpec& spec)
			:path(path)
			,spec(spec)
		{
		}
		std::shared_ptr<Path> path;
		PathSpec spec;
	};


	
		
	class PathThread
	{
	public:
		PathThread(GridPathManager& manager);

		PathThread(PathThread&& other);

		PathThread& operator=(PathThread&& rhs);


		~PathThread();

		bool refresh();

		void cancel();

	private:
		PathThread(const PathThread&) = delete;
		PathThread& operator=(const PathThread&) = delete;

		void calculatePathLoop(int id);
		
		GridPathManager *man;
		std::thread thread;
		std::unique_ptr<std::mutex> lock;
	};
	std::vector<PathThread> calculationThreads;
	std::deque<Args> calculationQueue;
	std::mutex calQueueLock;
#endif // SL_NOMULTITHREADED

	bool findPathImpl(const PathSpec& spec, Path& output) const;


	std::map<Scene*, std::vector<std::shared_ptr<PathGrid>>> pathGrids;

	friend class PathThread;
};

} // pf
} // sl

#endif // SL_PF_GRIDPATHMANAGER_HPP
