#include "Pathfinding/Heuristics.hpp"

#include <cmath>

#include "Engine/Scene.hpp"

namespace sl
{
namespace pf
{

ManhattanDistance::ManhattanDistance(const NodeID& target, float factor)
	:target(target)
	,factor(factor)
{
}

float ManhattanDistance::operator()(const NodeID& node, ai::BehaviorContext*) const
{
	const sf::Vector3i targetWorldCoords = node.getScene()->getWorldCoords();
	const sf::Vector3i nodeWorldCoords = node.getScene()->getWorldCoords();
	const int xdiff = fabsf((node.getX() + nodeWorldCoords.x) - (target.getX() + targetWorldCoords.x));
	const int ydiff = fabsf((node.getY() + nodeWorldCoords.y) - (target.getY() + targetWorldCoords.y));
	const int zdiff = fabsf(nodeWorldCoords.z - targetWorldCoords.z);
	return factor * (xdiff + ydiff + zdiff);
}

} // pf
} // sl
