#include "Pathfinding/Node.hpp"

#include "Engine/Serialization/AutoDefineObjectSerialization.hpp"
#include "Engine/Scene.hpp"

namespace sl
{
namespace pf
{

Node::Node(const NodeID& id)
	:Node(id.getX(), id.getY(), id.getScene())
{
}

Node::Node(int x, int y, Scene *scene)
	:pos((x + 0.5f) * scene->getTileSize(), (y + 0.5f) * scene->getTileSize())
	,scene(scene)
	,checker(nullptr)
{
}

Node::Node(int x, int y, Scene *scene, const NodeAvailabilityChecker *checker)
	:pos((x + 0.5f) * scene->getTileSize(), (y + 0.5f) * scene->getTileSize())
	,scene(scene)
	,checker(checker)
{
}

bool Node::isAvailable() const
{
	return checker == nullptr || checker->getAvailability();
}

int Node::getX() const
{
	return pos.x;
}

int Node::getY() const
{
	return pos.y;
}

const Vector2f& Node::getPos() const
{
	return pos;
}

Scene* Node::getScene() const
{
	return scene;
}

ScenePos Node::getScenePos() const
{
	return ScenePos(*getScene(), getPos());
}

NodeID Node::getID() const
{
	return NodeID(pos.x / scene->getTileSize(), pos.y / scene->getTileSize(), scene);
}

} // pf

DEF_SERIALIZE_OBJ_CTOR(pf::Node, &pf::Node::getID)

} // sl