/**
 * @section DESCRIPTION
 * A Node in the client-facing part of the pathfinding system. NodeID has a succinct representation of this.
 * This is shown usually only after someone has calculated a path, and that path will be a list of this class.
 */
#ifndef SL_PF_NODE_HPP
#define SL_PF_NODE_HPP

#include "Engine/Serialization/AutoDeclareObjectSerialization.hpp"
#include "Engine/ScenePos.hpp"
#include "Pathfinding/NodeAvailabilityChecker.hpp"
#include "Pathfinding/NodeID.hpp"
#include "Utility/Vector.hpp"

namespace sl
{

class Scene;

namespace pf
{

class Node
{
public:
	Node(const NodeID& id);
	// x/y are in PATHFINDING (tile) space!
	Node(int x, int y, Scene *scene);
	Node(int x, int y, Scene *scene, const NodeAvailabilityChecker *checker);


	bool isAvailable() const;
	
	// x/y are in SCENE space!
	int getX() const;
	int getY() const;
	const Vector2f& getPos() const;
	Scene *getScene() const;
	ScenePos getScenePos() const;
	NodeID getID() const;

private:
	Vector2f pos;
	Scene *scene;
	const NodeAvailabilityChecker *checker;
};

} // pf

DEC_SERIALIZE_OBJ_CTOR(pf::Node)

} // sl

#endif // SL_PF_NODE_HPP