#ifndef SL_PF_NODEID_HPP
#define SL_PF_NODEID_HPP

#include "Engine/Serialization/AutoDeclareObjectSerialization.hpp"
#include "Utility/Vector.hpp"

namespace sl
{

class Scene;

namespace pf
{

class NodeID
{
public:
	NodeID(Vector2i pos, Scene *scene);

	NodeID(int x, int y, Scene *scene/* = nullptr*/);

	// inlined for performance reasons (substantial CPU usage in pathfinding)
	inline bool operator==(const NodeID& rhs) const
	{
		return x == rhs.x && y == rhs.y && scene == rhs.scene;
	}

	inline bool operator!=(const NodeID& rhs) const
	{
		return x != rhs.x || y != rhs.y || scene != rhs.scene;
	}

	inline bool operator<(const NodeID& rhs) const
	{
		if (x < rhs.x)
		{
			return true;
		}
		else if (x == rhs.x)
		{
			if (y < rhs.y)
			{
				return true;
			}
			else if (y == rhs.y)
			{
				return scene < rhs.scene;
			}
		}
		return false;
		//return x < rhs.x || (x == rhs.x && y < rhs.y) || (x == rhs.x && y == rhs.y && scene < rhs.scene);
	}

	/**
	 * @return x coordinate *in pathfinding space* (likely tilespace)
	 */
	inline int getX() const
	{
		return x;
	}

	/**
	* @return y coordinate *in pathfinding space* (likely tilespace)
	*/
	inline int getY() const
	{
		return y;
	}

	Vector2f scenePos() const;

	inline Scene* getScene() const
	{
		return scene;
	}

private:
	int x;
	int y;
	Scene *scene;
};

} // pf

DEC_SERIALIZE_OBJ_CTOR(pf::NodeID)

} // sl

#endif // SL_PF_NODEID_HPP