#ifndef SL_PF_PATHMANAGER_HPP
#define SL_PF_PATHMANAGER_HPP

#include <functional>
#include <map>
#include <memory>
#include <string>

#include "Pathfinding/PathfindingCommon.hpp"
#include "Pathfinding/Node.hpp"
#include "Pathfinding/NodeID.hpp"
#include "Pathfinding/Path.hpp"
#include "Utility/Vector.hpp"

namespace sl
{

class Target;

namespace pf
{

class PathManager
{
public:
	virtual ~PathManager();

	enum class Priority
	{
		//! Blocking priority.
		Realtime = 0,
		//! Highest non-blocking priority. (cops, things by the player, etc)
		High,
		//! Average non-blocking priority. (general purpose)
		Medium,
		//! Low non-blocking priority. (stuff that really isn't important)
		Low
	};
	
	struct PathSpec
	{
		PathSpec(const NodeID& from, const NodePredicate& to, const WeightMap *weights, const WeightFunc& heuristic, ai::BehaviorContext *context, const std::function<void(std::deque<Node>&)>& postProcess = std::function<void(std::deque<Node>&)>())
			:from(from)
			,to(to)
			,weights(weights)
			,heuristic(heuristic)
			,context(context)
			,postProcess(postProcess)
		{
			ASSERT(weights != nullptr);
			ASSERT(heuristic);
			// postProcess optional
		}
		
		NodeID from;
		NodePredicate to;
		// TODO: keep this by-value? should we ensure it exists? I mean if this goes out of scope, the BehaviorContext will probably not work either.
		const WeightMap *weights;
		WeightFunc heuristic;
		ai::BehaviorContext *context;
		std::function<void(std::deque<Node>&)> postProcess;
	};
	// TODO: remove the default priority argument
	virtual std::shared_ptr<Path> getPath(const PathSpec& spec, Priority priority = Priority::Medium);

	virtual std::shared_ptr<Path> getPath(NodeID from, NodeID to);

	virtual bool cancelCalculation(std::shared_ptr<Path>& path) = 0;
	
	virtual void cancelAllCalculations() = 0;

	virtual NodeID getRandomID() const = 0;

	virtual NodeID getClosestNodeID(const Target& target) const = 0;

	virtual Node getNodeFromID(NodeID id) const = 0;

	virtual void update(int64_t timestamp) {}
};

} // pf
} // sl

#endif // SL_PF_PATHMANAGER_HPP
