#ifndef SL_PF_PATHFIND_HPP
#define SL_PF_PATHFIND_HPP

#include <deque>
#include <functional>
#include <map>
#include <memory>
#include <string>
#include <vector>

#include "Pathfinding/CarConfig.hpp"
#include "Pathfinding/PathfindingCommon.hpp"
#include "Pathfinding/NodeID.hpp"
#include "Pathfinding/Vertex.hpp"

namespace sl
{

class Scene;

namespace pf
{

class PathGrid;

extern bool findShortestPath(std::deque<const Vertex*>& destination, const std::vector<std::unique_ptr<Vertex>>& source, const Vertex *start, const Vertex *end);

extern bool testPath(const std::vector<std::unique_ptr<Vertex>>& source, const Vertex *start, const Vertex *end);

extern bool findShortestPathOnGrid(std::deque<NodeID>& output, const NodeID& start, const NodePredicate& isEnd,
	const std::map<Scene*, std::vector<std::shared_ptr<PathGrid>>>& grids, const WeightMap& weights, const Portals& portals,
	const WeightFunc& heuristic, ai::BehaviorContext *context);

extern bool findShortestCarPath(std::deque<CarConfig>& output, const CarConfig& start, const NodePredicate& isEnd,
                                std::function<float(const CarConfig&, ai::BehaviorContext *context)> heuristic, ai::BehaviorContext *context);

} // pf
} // sl

#endif // SL_PF_PATHFIND_HPP
