#ifndef SL_PF_PRECOMPUTEDPATHGRID_HPP
#define SL_PF_PRECOMPUTEDPATHGRID_HPP

#include <cstdint>
#include <string>

#include "Pathfinding/PathGrid.hpp"
#include "Utility/Grid.hpp"
#include "Utility/Rect.hpp"

namespace sl
{

class Scene;

namespace pf
{

class PrecomputedPathGrid : public PathGrid
{
public:
	PrecomputedPathGrid(Scene &scene, std::string type);

	/**
	 * Recomputes the walkability checks in a certain area
	 * Note: assumes you're either computing small amounts, or are computing against an efficiently-queryable thing
	 *       this means it does not batch together collision queries and will repeatedly query per-tile.
	 *       This should be a valid assumption though, as things not in efficiently-queryable datastructures would
	 *       likely be things that move and thus would not deserve being precomputed. Use RealTimePathGrid for those instead.
	 * @param area The area in scene-space (not tiles) to compute around.
	 */
	void recompute(const Rect<int>& area);

	//---------------------  PathGrid  ---------------------
	bool isSolid(int x, int y) const override;

	bool canGoRight(int x, int y) const override;

	bool canGoLeft(int x, int y) const override;

	bool canGoUp(int x, int y) const override;

	bool canGoDown(int x, int y) const override;

	int getWidth() const override;

	int getHeight() const override;

	int getTileSize() const override;

#ifdef SL_DEBUG
	void debugSaveImage(const std::string& sceneType) const;
#endif // SL_DEBUG

private:
	// the actual checks for when we recompute walkability
	bool isSolidImpl(int x, int y) const;

	bool canGoRightImpl(int x, int y) const;

	bool canGoDownImpl(int x, int y) const;

	const int tileSize;
	static constexpr uint8_t SOLID =  1;
	static constexpr uint8_t LEFT  =  2;
	static constexpr uint8_t RIGHT =  4;
	static constexpr uint8_t UP    =  8;
	static constexpr uint8_t DOWN  = 16;
	Grid<uint8_t> grid;
	Scene& scene;
	std::string type;
};

} // pf
} // sl

#endif // SL_PF_PRECOMPUTEDPATHGRID_HPP
