#ifndef SL_PF_REALTIMECACHEDPATHGRID_HPP
#define SL_PF_REALTIMECACHEDPATHGRID_HPP

#include <string>

#include "Pathfinding/PathGrid.hpp"
#include "Utility/Grid.hpp"
#include "Utility/Rect.hpp"

namespace sl
{
class Scene;
namespace pf
{

class RealTimeCachedPathGrid : public PathGrid
{
public:
	// @todo potentially allow for multiple types
	RealTimeCachedPathGrid(Scene& scene, const std::string& type);

	//---------------------  PathGrid  ---------------------
	bool isSolid(int x, int y) const override;

	bool canGoRight(int x, int y) const override;

	bool canGoLeft(int x, int y) const override;

	bool canGoUp(int x, int y) const override;

	bool canGoDown(int x, int y) const override;

	int getWidth() const override;

	int getHeight() const override;

	int getTileSize() const override;

	void update(int64_t newTimestamp) override;

private:
	bool check(int x, int y) const;

	Scene& scene;
	std::string type;
	const int tileSize;
	const int halfTileSize;
	const int quarterTileSize;
	int64_t updateTime;
	int64_t timestamp;
	mutable Rect<int> queryRect;
	struct CacheStatus
	{
		CacheStatus(bool hit, int64_t timestamp)
			:hit(hit)
			,timestamp(timestamp)
		{
		}
		bool hit;
		int64_t timestamp;
	};
	// this splits the tiles into 4 smaller ones then does math on those
	// so (4, 5) -> (5, 5) needs tiles (8, 10), (9, 10) and (10, 10)
	// since odd tiles are inter-tile tiles
	mutable Grid<CacheStatus> cacheStatus;
};

} // pf
} // sl

#endif // SL_PF_REALTIMECACHEDPATHGRID_HPP
