#include "Pathfinding/RealTimePathGrid.hpp"

#include "Engine/Scene.hpp"

namespace sl
{
namespace pf
{

static constexpr float gap = 0.4f;

RealTimePathGrid::RealTimePathGrid(Scene& scene, const std::string& type)
	:scene(scene)
	,type(type)
	,tileSize(scene.getTileSize())
{
}

bool RealTimePathGrid::isSolid(int x, int y) const
{
	return scene.checkCollision(Rect<int>((x + gap) * tileSize, (y + gap) * tileSize, (1 - 2 * gap) * tileSize, (1 - 2 * gap) * tileSize), type) != nullptr;
}

bool RealTimePathGrid::canGoRight(int x, int y) const
{
	return scene.checkCollision(Rect<int>((x + gap) * tileSize, (y + gap) * tileSize, (2 - 2 * gap) * tileSize, (1 - 2 * gap) * tileSize), type) == nullptr;
}

bool RealTimePathGrid::canGoLeft(int x, int y) const
{
	return canGoRight(x - 1, y);
}

bool RealTimePathGrid::canGoUp(int x, int y) const
{
	return canGoDown(x, y - 1);
}

bool RealTimePathGrid::canGoDown(int x, int y) const
{
	return scene.checkCollision(Rect<int>((x + gap) * tileSize, (y + gap) * tileSize, (1 - 2 * gap) * tileSize, (2 - 2 * gap) * tileSize), type) == nullptr;
}

int RealTimePathGrid::getWidth() const
{
	return scene.getWidth() / tileSize;
}

int RealTimePathGrid::getHeight() const
{
	return scene.getHeight() / tileSize;
}

int RealTimePathGrid::getTileSize() const
{
	return tileSize;
}

} // pf
} // sl
