#include "Pathfinding/Vertex.hpp"

#include <cmath>

// @TILESIZE
const int ts = 16;

namespace sl
{
namespace pf
{

Vertex::Vertex(int x, int y)
	:x(x)
	,y(y)
	,circle(8)
{
	circle.setFillColor(sf::Color::Transparent);
	circle.setOutlineColor(sf::Color::Red);
	circle.setOutlineThickness(2);
	circle.setOrigin(8, 8);
	circle.setPosition(x * ts/* + 16*/, y * ts/* + 16*/);
}

void Vertex::addVertex(const Vertex *vertex, float weight)
{
	for (const std::pair<const Vertex*, int>& vertexWeightPair : edges)
	{
		if (vertex == vertexWeightPair.first)
		{
			return ;
		}
	}
	int distance = weight * sqrt((this->x - vertex->x) * (this->x - vertex->x) + (this->y - vertex->y) * (this->y - vertex->y));
	edges.push_back(std::make_pair(vertex, distance));
	lines.push_back(sf::VertexArray(sf::Lines, 2));
	lines.back()[0] = sf::Vertex(sf::Vector2f(x * ts/* + 16*/, y * ts/* + 16*/), sf::Color::Red);
	lines.back()[1] = sf::Vertex(sf::Vector2f(vertex->x * ts/* + 16*/, vertex->y * ts/* + 16*/), sf::Color::Green);
}

void Vertex::linkVertices(Vertex *vertex, float weight)
{
	bool addToThis = true;
	bool addToOther = true;
	for (const std::pair<const Vertex*, int>& vertexWeightPair : edges)
	{
		if (vertex == vertexWeightPair.first)
		{
			addToThis = false;
			break;
		}
	}
	for (const std::pair<const Vertex*, int>& vertexWeightPair : vertex->edges)
	{
		if (this == vertexWeightPair.first)
		{
			addToOther = false;
			break;
		}
	}
	int distance = weight * sqrt((this->x - vertex->x) * (this->x - vertex->x) + (this->y - vertex->y) * (this->y - vertex->y));
	if (addToThis)
	{
		edges.push_back(std::make_pair(vertex, distance));
	}
	if (addToOther)
	{
		vertex->edges.push_back(std::make_pair(this, distance));
	}
}

void Vertex::draw(RenderTarget& target) const
{
	target.draw(circle);
	for (const sf::VertexArray& line : lines)
	{
		target.draw(line);
	}
}

} // pf
} // sl
