#ifndef SL_PF_VERTEXBASEDPATHMANAGER_HPP
#define SL_PF_VERTEXBASEDPATHMANAGER_HPP

#include <deque>
#include <map>
#include <memory>
#include <mutex>
#include <thread>
#include <vector>

#include "Engine/Graphics/RenderTarget.hpp"
#include "Pathfinding/PathManager.hpp"
#include "Pathfinding/Vertex.hpp"

namespace sl
{

class Serialize;
class Deserialize;

namespace pf
{

class VertexBasedPathManager : public PathManager
{
public:
	VertexBasedPathManager(int tileSize);



	//---------------------  PathManager  ---------------------
	std::shared_ptr<Path> getPath(NodeID from, NodeID to) override;

	bool cancelCalculation(std::shared_ptr<Path>& path) override;

	void cancelAllCalculations() override;

	NodeID getRandomID() const override;

	NodeID getClosestNodeID(const Target& target) const override;

	Node getNodeFromID(NodeID id) const override;


	//---------------  VertexBasedPathManager  ----------------
	//bool pathExists(NodeID from, NodeID to) const;

	//void registerVertices(NodeID id, Vertex *vertex);
	void addDirectedEdge(NodeID from, NodeID to, float weight = 1);

	void addDirectedConditionalEdge(NodeID from, NodeID to, const NodeAvailabilityChecker *checker, float weight = 1);

	void draw(RenderTarget& target) const;

	void serialize(Serialize& out) const;

	void deserialize(Deserialize& in);

private:
	VertexBasedPathManager(const VertexBasedPathManager&);
	VertexBasedPathManager& operator=(const VertexBasedPathManager&);

	struct Args
	{
		Args(std::shared_ptr<Path> path, NodeID from, NodeID to)
			:path(path)
			,from(from)
			,to(to)
		{
		}
		std::shared_ptr<Path> path;
		NodeID from;
		NodeID to;
	};
#ifndef SL_NOMULTITHREADED
	void calculatePathLoop(int id);
#endif // SL_NOMULTITHREADED
	bool getPath(std::deque<Node>& results, NodeID from, NodeID to);


	std::map<NodeID, Vertex*> vertices;
	std::vector<std::unique_ptr<Vertex>> vertexList;

	std::map<const Vertex*, std::map<const Vertex*, const NodeAvailabilityChecker*>> conditionalEdges;

	mutable std::vector<std::pair<sf::VertexArray, const NodeAvailabilityChecker*>> condedges;

	std::map<std::pair<NodeID, NodeID>, std::deque<Node>> computedPaths;

	int tileSize;

#ifndef SL_NOMULTITHREADED
	std::thread calculationThread;
	std::deque<Args> calculationQueue;
	std::mutex lock;
	std::mutex calQueueLock;
	bool calculatingPaths;
#endif
};

} // pf
} // sl

#endif // SL_PF_VERTEXBASEDPATHMANAGER_HPP
