#ifndef SL_PLAYERACTION_HPP
#define SL_PLAYERACTION_HPP

#include <memory>
#include <string>

namespace sl
{

class Drawable;
class NPC;
class Player;

class PlayerAction
{
public:
	virtual ~PlayerAction() {}

	virtual std::string getName() const = 0;

	virtual std::string getShortName() const
	{
		return getName();
	}

	virtual std::unique_ptr<Drawable> getIcon() const = 0;

	enum class Usability
	{
		Useable,
		NoTarget,
		NoneLeft
	};
	/**
	 * Checks if the action is useable on the given target. Target=nullptr means no target.
	 */
	virtual Usability canUse(Player& player, NPC *target) const = 0;

	static constexpr float SUCCESS = 1.f;
	static constexpr float FAILURE = -1.f;
	/**
	 * Uses the action on the given target. Target=nullptr means no target.
	 * Assumes that canUse(player, target) is true
	 * @return a number in [0, 1) if the action is in progress, 1 (SUCCESS) if it completed successfully, and -1 (FAILURE) if it failed.
	 */
	virtual float use(Player& player, NPC *target) = 0;

	virtual void abort() {}
	/**
	 * Override this to return true if your use() invokes a state change on Player
	 * If this returns false then the client state will do its own default state change
	 * @return Whether to skip default state change upon action completion
	 */
	virtual bool hasCustomFinishStateChange() const { return false; }

	// TODO: other kinds of targets (lines, etc) like in ItemUseAction
	virtual float usageRadius() const { return 24.f; }
};

} // sl

#endif // SL_PLAYERACTION_HPP