#include "Player/Actions/PlayerDropItemAction.hpp"

#include "Items/Item.hpp"
#include "Items/ItemPickup.hpp"
#include "Items/ItemToDrawables.hpp"
#include "Player/Player.hpp"

namespace sl
{

PlayerDropItemAction::PlayerDropItemAction(int itemType)
	:itemType(itemType)
{
}

std::string PlayerDropItemAction::getName() const
{
	return "Drop " + Item(itemType).getName();
}

std::string PlayerDropItemAction::getShortName() const
{
	return "Drop";
}

std::unique_ptr<Drawable> PlayerDropItemAction::getIcon() const
{
	return itemToDrawable(itemType, false);
}

PlayerAction::Usability PlayerDropItemAction::canUse(Player& player, NPC *target) const
{
	return query(player).empty() ? Usability::NoneLeft : Usability::Useable;
}

float PlayerDropItemAction::use(Player& player, NPC *target)
{
	if (canUse(player, target) == Usability::Useable)
	{
		auto available = query(player);
		ASSERT(!available.empty());
		auto& items = player.getStats().soul->items;
		Item item = items.removeItem(available.front());
		Entity *pickup = new ItemPickup(player.getPos(), std::move(item));
		player.getScene()->addEntityToList(pickup);
		return SUCCESS;
	}
	return FAILURE;
}

// private
std::vector<Inventory::ItemRef> PlayerDropItemAction::query(Player& player) const
{
	std::vector<Inventory::ItemRef> hits;
	auto& items = player.getStats().soul->items;
	items.queryItems(hits, itemType);
	return hits;
}

} // sl