#include "Player/Actions/PlayerGrabAction.hpp"

#include "Engine/Graphics/Sprite.hpp"
#include "NPC/States/DraggedState.hpp"
#include "NPC/NPC.hpp"
#include "Player/Player.hpp"
#include "Player/States/PlayerCarryingState.hpp"
#include "Police/CrimeUtil.hpp"

namespace sl
{

std::string PlayerGrabAction::getName() const
{
	return "Grab";
}

PlayerAction::Usability PlayerGrabAction::canUse(Player& player, NPC *target) const
{
	return target ? Usability::Useable : Usability::NoTarget;
}

std::unique_ptr<Drawable> PlayerGrabAction::getIcon() const
{
	return unique<Sprite>("actions/grab.png");
}

float PlayerGrabAction::use(Player& player, NPC *target)
{
	// TODO: timer?
	ASSERT(target);
	const int crimeId = commitCrime(*target, Crime::Type::Kidnapping);
	target->getSoul()->setCrimeReported(crimeId, false);
	target->changeState(unique<DraggedState>(*target, player));
	player.getState().changeState(unique<PlayerCarryingState>(player, *target));
	return 0.5f; // to make it so this state-change doesn't get overriden by the PerformingAction -> Walking one
}

} // sl