#include "Player/Actions/PlayerPunchAction.hpp"

#include "Engine/Graphics/Sprite.hpp"
#include "NPC/Blood.hpp"
#include "NPC/LayeredSpriteLoader.hpp"
#include "NPC/NPC.hpp"
#include "Player/Player.hpp"
#include "Police/CrimeUtil.hpp"

namespace sl
{

PlayerPunchAction::PlayerPunchAction()
	:progress(0.f)
	,hit(false)
{
}

std::string PlayerPunchAction::getName() const
{
	return "Punch";
}

std::unique_ptr<Drawable> PlayerPunchAction::getIcon() const
{
	return std::make_unique<Sprite>("actions/punch.png");
}

PlayerAction::Usability PlayerPunchAction::canUse(Player& player, NPC *target) const
{
	return Usability::Useable;
}

float PlayerPunchAction::use(Player& player, NPC *target)
{
	if (progress == 0.f)
	{
		LayeredSpriteLoader::load(
			player,
			LayeredSpriteLoader::Config(
				player.getStats().soul->appearance,
				"punch",
				32,
				Vector2i(18, 28)).animate(
					std::vector<Framerate::Frame>(
						{
							{0, 4},
							{1, 3},
							{2, 2},
							{3, 5},
							{1, 2}
						}),
					false));
		// TODO: remove when north/south animations are done
		if (player.facing() == North || player.facing() == West)
		{
			player.setDrawableSet("west");
		}
		else
		{
			player.setDrawableSet("east");
		}
	}
	const float progressIncr = 1.f / 18.f;
	progress = std::clamp(progress + progressIncr, 0.f, 1.f);
	if (target && progress > 0.5f && !hit)
	{
		const int n = Random::get().randomInclusive(3, 6);
		const Vector2f toTarget = (target->getPos() - player.getPos()).normalized();
		for (int i = 0; i < n; ++i)
		{	
			const Vector2f veloc = 4.f * toTarget + lengthdir(4, Random::get().random(360));
			const float s = Random::get().randomFloat(1);
			target->getScene()->addEntityToList(new Blood(target->getPos().x, target->getPos().y, veloc.x * s, veloc.y * s));
		}
		hit = true;
		target->tryMoveRaycast(toTarget * 7.f);
		commitCrime(*target, Crime::Type::Assault);
		target->getSoul()->setFear(target->getSoul()->getFear() + 0.15f);
		const float newFear = target->getSoul()->getFear();
		if (newFear > 0.6f)
		{
		}
		else if (newFear > 0.25f)
		{
			target->makeTextBubble("P-Please don't... I'll listen...");
		}
		else
		{
			target->makeTextBubble("Ow! Get off me!");
		}
		
		//target->getSoul()->affectMood(Mood::Axis::Pleasure, -0.25f);
		//target->getSoul()->affectMood(Mood::Axis::Dominance, -0.25f);
		//target->getSoul()->affectMood(Mood::Axis::Arousal, 0.25f);
	}
	if (progress == SUCCESS)
	{
		abort();
		return SUCCESS;
	}
	return progress;
}

void PlayerPunchAction::abort()
{
	progress = 0.f;
	hit = false;
}

} // sl