#include "Player/Actions/PlayerStalkItemAction.hpp"

#include "Items/ItemToDrawables.hpp"
#include "NPC/LayeredSpriteLoader.hpp"
#include "NPC/NPC.hpp"
#include "Player/Player.hpp"
#include "Town/World.hpp"


// @REMOVE
#include <iostream>
#include "Utility/DataToString.hpp"

namespace sl
{

PlayerStalkItemAction::PlayerStalkItemAction(Item::Type itemType)
	:item(itemType)
	,progress(0.f)
{
}

std::string PlayerStalkItemAction::getName() const
{
	return "Observe using " + item.getName();
}

std::string PlayerStalkItemAction::getShortName() const
{
	return "Observe";
}

std::unique_ptr<Drawable> PlayerStalkItemAction::getIcon() const
{
	return itemToDrawable(item.getType(), false);
}

PlayerAction::Usability PlayerStalkItemAction::canUse(Player& player, NPC *target) const
{
	return target ? Usability::Useable : Usability::NoTarget;
}

float PlayerStalkItemAction::use(Player& player, NPC *target)
{
	if (progress == 0.f)
	{
		// TODO: some kind of drawable paramterization inside data/items.json
		// also make the animation system less shit.
		LayeredSpriteLoader::load(
			player,
			LayeredSpriteLoader::Config(
				player.getStats().soul->appearance,
				"photo",
				32,
				Vector2i(18, 28)).animate(std::vector<int>({ 3, 3, 3, 5, 5, 6, 4, 13 }), false));
		// TODO: remove when north/south animations are done
		if (player.facing() == North || player.facing() == West)
		{
			player.setDrawableSet("west");
		}
		else
		{
			player.setDrawableSet("east");
		}
	}
	ASSERT(target);
	const float progressIncr = 1.f / item.getStalkStats()->timer;
	progress = std::clamp(progress + progressIncr, 0.f, 1.f);
	if (progress == SUCCESS)
	{
		if (item.getStalkStats()->photo)
		{
			// TODO: visual indication of a photo taken (flash?)
			std::cout << "Picture taken!" << std::endl;
			player.getStats().stalkingData.stalk(
				target->getSoul()->getID(),
				player.getWorld()->getGameTime(),
				NPCPhoto{
					NPCPhoto::getColorFromSurroundings(target),
					target->getSoul()->appearance,
					"talking",
					"default",
					"default"});
		}
		else
		{
			player.getStats().stalkingData.stalk(target->getSoul()->getID(), player.getWorld()->getGameTime().date);
		}
		std::cout << "Stalked " << target->getSoul()->getFirstName() << " " << target->getSoul()->getFirstName() << " (aged " << formatAge(target->getSoul()->getAgeDays(), true) << ")" << std::endl;
		progress = 0.f;
		return SUCCESS;
	}
	return progress;
}

void PlayerStalkItemAction::abort()
{
	progress = 0.f;
}

float PlayerStalkItemAction::usageRadius() const
{
	return item.getStalkStats()->radius;
}

} // sl