#include "Player/Actions/PlayerSubdueItemAction.hpp"

#include "Items/Item.hpp"
#include "Items/ItemToDrawables.hpp"
#include "NPC/States/DisabledState.hpp"
#include "NPC/States/DraggedState.hpp"
#include "NPC/NPC.hpp"
#include "Player/Player.hpp"
#include "Player/States/PlayerCarryingState.hpp"
#include "Police/CrimeUtil.hpp"

namespace sl
{

PlayerSubdueItemAction::PlayerSubdueItemAction(int itemType)
	:itemType(itemType)
	,progress(0.f)
{
}

std::string PlayerSubdueItemAction::getName() const
{
	return "Subdue with " + Item(itemType).getName();
}

std::string PlayerSubdueItemAction::getShortName() const
{
	return "Subdue";
}

std::unique_ptr<Drawable> PlayerSubdueItemAction::getIcon() const
{
	return itemToDrawable(itemType, false);
}

PlayerAction::Usability PlayerSubdueItemAction::canUse(Player& player, NPC *target) const
{
	// TODO: check uses, etc.
	return target ? Usability::Useable : Usability::NoTarget;
}

float PlayerSubdueItemAction::use(Player& player, NPC *target)
{
	ASSERT(target);
	progress = std::clamp(progress + 0.05f, 0.f, 1.f);
	// TODO: non-linear progress, etc. maybe a way to hit a button to hit her?
	if (progress == SUCCESS)
	{
		const int crimeId = commitCrime(*target, Crime::Type::Kidnapping);
		target->getSoul()->setCrimeReported(crimeId, false);
		const SubdueItem *s = Item(itemType).getSubdueStats();
		ASSERT(s);
		switch (s->successState)
		{
		case SubdueItem::ResultingState::Chloroformed:
			target->getSoul()->getStatus().chloroformTimer = 30 * 60; // 1 minute in-game
			target->changeState(unique<DraggedState>(*target, player));
			player.getState().changeState(unique<PlayerCarryingState>(player, *target));
			break;
		case SubdueItem::ResultingState::Tied:
			target->changeState(unique<DisabledState>(*target));
			break;
		}
		progress = 0.f;
		return SUCCESS;
	}
	return progress;
}

void PlayerSubdueItemAction::abort()
{
	progress = 0.f;
}

bool PlayerSubdueItemAction::hasCustomFinishStateChange() const
{
	const SubdueItem *s = Item(itemType).getSubdueStats();
	ASSERT(s);
	switch (s->successState)
	{
	case SubdueItem::ResultingState::Chloroformed:
		return true;
	case SubdueItem::ResultingState::Tied:
		return false;
	}
	return false;
}

} // sl