#include "Player/Actions/PlayerThrowItemAction.hpp"

#include "Items/Item.hpp"
#include "Items/ItemToDrawables.hpp"
#include "NPC/ThrownItem.hpp"
#include "Player/Player.hpp"

namespace sl
{

PlayerThrowItemAction::PlayerThrowItemAction(int itemType)
	:itemType(itemType)
{
}

std::string PlayerThrowItemAction::getName() const
{
	return "Throw " + Item(itemType).getName();
}

std::string PlayerThrowItemAction::getShortName() const
{
	return "Throw";
}

std::unique_ptr<Drawable> PlayerThrowItemAction::getIcon() const
{
	return itemToDrawable(itemType, false);
}

PlayerAction::Usability PlayerThrowItemAction::canUse(Player& player, NPC *target) const
{
	return target || query(player).empty() ? Usability::NoneLeft : Usability::Useable;
}

float PlayerThrowItemAction::use(Player& player, NPC *target)
{
	if (canUse(player, target) == Usability::Useable)
	{
		auto available = query(player);
		ASSERT(!available.empty());
		auto& items = player.getStats().soul->items;
		Item item = items.removeItem(available.front());
		ThrownItem *thrownItem = new ThrownItem(
			player.getPos(),
			player.getPos() + 128.f * facingDirToVector(player.facing()),
			std::move(item));
		player.getScene()->addEntityToList(thrownItem);
		return SUCCESS;
	}
	return FAILURE;
}

// private
std::vector<Inventory::ItemRef> PlayerThrowItemAction::query(Player& player) const
{
	std::vector<Inventory::ItemRef> hits;
	auto& items = player.getStats().soul->items;
	items.queryItems(hits, itemType);
	return hits;
}

} // sl