#include "Player/PlayerGen.hpp"

#include "Town/World.hpp"
#include "NPC/Stats/Soul.hpp"
#include "NPC/Stats/DNA.hpp"
#include "NPC/Stats/SoulGenerationSettings.hpp"
#include "Player/PlayerStats.hpp"
#include "Utility/Memory.hpp"

namespace sl
{

int playerGenPool = 0;

std::shared_ptr<PlayerStats> createPlayer(World& world, Random& rng)
{
	
	SoulGenerationSettings soulSettings;
	soulSettings.addGenePool(static_cast<DNADatabase::GenePool>(playerGenPool));
	//soulSettings.addNationality("english");
	auto stats = shared<PlayerStats>(world.createSoul(rng, soulSettings));

	stats->soul->setType("player");
	stats->soul->appearance.setClothing(ClothingType::Outfit, "tshirtjeans");
	stats->soul->appearance.setHair("short");

	// stub inventory stuff
	Inventory& items = stats->soul->items;
#define STUB_ADD(type, count) for (int i = 0; i < count; ++i, items.addItem(Item(Item::strToType(type))));
	STUB_ADD("candy", 10);
	STUB_ADD("rope", 2);
	STUB_ADD("chloroform", 3);
	STUB_ADD("bandage", 1);
	STUB_ADD("chocolatebar", 5);
	STUB_ADD("lollipop", 4);
	STUB_ADD("can", 10);
	STUB_ADD("binoculars", 1);
	STUB_ADD("camera", 1);
#undef STUB_ADD
	
	//stats->hotbar.addItem(items.addItem(Item(Item::strToType("rope"))));
	//stats->hotbar.addItem(items.addItem(Item(Item::strToType("chloroform"))));
	//stats->hotbar.addItem(items.addItem(Item(Item::strToType("can"))));

	return std::move(stats);
}

} // sl
