#include "Player/States/PlayerPerformingActionState.hpp"

#include "Engine/Graphics/SpriteAnimation.hpp"
#include "Player/Player.hpp"
#include "Player/States/PlayerWalkState.hpp"
#include "Utility/Assert.hpp"

namespace sl
{

PlayerPerformingActionState::PlayerPerformingActionState(Player& player, NPC *target, std::unique_ptr<PlayerAction> action)
	:PlayerState(player)
	,target(target)
	,action(std::move(action))
{
}

void PlayerPerformingActionState::update()
{
	const float progression = action->use(player, target);
	if (progression == PlayerAction::FAILURE || progression >= PlayerAction::SUCCESS)
	{
		if (!action->hasCustomFinishStateChange())
		{
			changeState(unique<PlayerWalkState>(player, std::move(action)));
		}
	}
	else
	{
		progressDial->setFrame(static_cast<int>(progression * (progressDial->getFrameCount() - 1)));
	}
}

void PlayerPerformingActionState::onEnter()
{
	auto dial = unique<SpriteAnimation>("progress_dial.png", 16, 0, false);
	dial->setOrigin(Vector2i(0.f, 32.f));
	progressDial = dial.get();
	player.addDrawable(std::move(dial));
}

void PlayerPerformingActionState::onExit()
{
	player.removeNonSetDrawable(progressDial);
	progressDial = nullptr;
}

} // namespace sl
