#include "Police/CrimeEntity.hpp"

#include "Police/Crime.hpp"
#include "Police/CriminalRecords.hpp"
#include "Town/World.hpp"
#include "Utility/TypeToString.hpp"

namespace sl
{


// TODO: serialize?
DECLARE_TYPE_TO_STRING(CrimeEntity)

CrimeEntity::CrimeEntity(const Vector2f& pos, int crimeId, bool constantNoise, int lifetime)
	:Entity(pos.x, pos.y, 64, 64, "CrimeEntity")
	,crimeId(crimeId)
	,constantNoise(constantNoise)
	,lifetime(lifetime)
{
	mask.setOrigin(32, 32);
}

const Types::Type CrimeEntity::getType() const
{
	return Entity::makeType("CrimeEntity");
}

void CrimeEntity::noiseImpulse()
{
}

void CrimeEntity::report()
{
	getWorld()->getPolice().reportCrime(crimeId);
}

int CrimeEntity::getCrimeId() const
{
	return crimeId;
}


// private
void CrimeEntity::onUpdate()
{
	if (constantNoise)
	{
		noiseImpulse();
	}
	if (lifetime != INDEFINITE)
	{
		if (--lifetime <= 0)
		{
			destroy();
		}
	}
}

} // sl