#include "Police/CriminalRecords.hpp"

#include "Engine/Serialization/AutoDeserialize.hpp"
#include "Engine/Serialization/AutoSerialize.hpp"

namespace sl
{

void CriminalRecords::recordCrime(const Crime& crime)
{
	crimes.insert(std::make_pair(crime.id, std::make_pair(crime, false)));
}

const Crime* CriminalRecords::reportCrime(int crimeId)
{
	auto it = crimes.find(crimeId);
	if (it != crimes.end())
	{
		const Crime& crime = it->second.first;
		const int victimID = crime.victim;
		if (!it->second.second)
		{
			it->second.second = true;
			return &crime;
		}
	}
	return nullptr;
}

const Crime* CriminalRecords::getCrime(int crimeId) const
{
	auto it = crimes.find(crimeId);
	return it != crimes.end() ? &it->second.first : nullptr;
}


std::map<uint64_t, std::vector<Crime>> CriminalRecords::getCrimesForMenu() const
{
	// should this logic even by done here?
	std::map<uint64_t, std::vector<Crime>> byVictim;
	for (const std::pair<int, std::pair<Crime, bool>>& pair : crimes)
	{
		const Crime& crime = pair.second.first;
		byVictim[crime.victim].push_back(crime);
	}
	return byVictim;
}

namespace AutoSerialize
{
static inline void serialize(Serialize& out, const Crime& crime)
{
	out.u8("type", static_cast<uint8_t>(crime.type));
	out.u64("victim", crime.victim);
	out.i("id", crime.id);
	out.b("consensual", crime.consensual);
}
}
namespace AutoDeserialize
{
static inline void deserialize(Deserialize& in, Crime& crime)
{
	crime.type = static_cast<Crime::Type>(in.u8("type"));
	crime.victim = in.u64("victim");
	crime.id = in.i("id");
	crime.consensual = in.b("consensual");
}
}

void CriminalRecords::serialize(Serialize& out) const
{
	out.u64("currentCrimeId", Crime::currentID);
	AutoSerialize::serialize(out, crimes);
}

void CriminalRecords::deserialize(Deserialize& in)
{
	Crime::currentID = in.u64("currentCrimeId");
	AutoDeserialize::deserialize(in, crimes);
}

} // sl