#ifndef SL_CRIMINALRECORDS_HPP
#define SL_CRIMINALRECORDS_HPP

#include <functional>
#include <map>
#include <vector>

#include "Police/Crime.hpp"

namespace sl
{

class Serialize;
class Deserialize;

class CriminalRecords
{
public:
	/**
	 * Records the commiting of a crime being commited by the player. This does not mean the police know about it yet.
	 * For them to know about it someone must report the crime first.
	 * @param crime The crime that was just committed
	 */
	void recordCrime(const Crime& crime);

	/**
	 * Reports a crime as being committed. Must previously have actually been comitted by the player
	 * @param crimeID The ID off the crime being reported
	 * @return A pointer to the crime if it is a newly reported crime, null otherwise
	 */
	const Crime* reportCrime(int crimeId);

	/**
	 * @param crimeId The crime id
	 * @return The crime associated with the id, if it has been recorded/exists
	 */
	const Crime* getCrime(int crimeId) const;

	/**
	 * @return The crimes committed by the player, indexed by who comitted them
	 */
	std::map<uint64_t, std::vector<Crime>> getCrimesForMenu() const;

	void serialize(Serialize& out) const;

	void deserialize(Deserialize& in);

private:
	//! Key: crimeID, Value: <Corresponding crime, Reported?>
	std::map<int, std::pair<Crime, bool>> crimes;

};

} // sl

#endif // SL_CRIMINALRECORDS_HPP