#ifndef SL_POLICEFORCE_HPP
#define SL_POLICEFORCE_HPP

#include "NPC/Locators/Target.hpp"
#include "Police/CriminalRecords.hpp"
#include "Utility/IntrusiveMultiList.hpp"

namespace sl
{

class Entity;
class Town;

class PoliceForce
{
public:
	enum SeverityThresholds : int
	{
		None = 0,
		Lookout = 10,
		Pursuing = 500,
		Manhunt = 1000 // lol
	};

	PoliceForce();

	void setTown(Town& town);

	void update();

	/**
	 * Updates the last known location to the player's location.
	 * Maybe if we need we could also have one that takes arguments.
	 */
	void updateLastKnownLocation();

	const Target& getLastKnownLocation() const;

	/**
	 * @return age (in frames) since the player's last location was updated
	 */
	int getLastKnownLocationAge() const;


	SeverityThresholds getSeverity() const;

	/**
	 * @return The crime reported if it was unknown, or nullptr if someone had already reported it.
	 */
	const Crime* reportCrime(int crimeID);

	void recordCrime(const Crime& crime);

	const CriminalRecords& getCriminalRecords() const;

	void serialize(Serialize& out) const;

	void deserialize(Deserialize& in);

private:

	int severity;
	Target lastKnownLocation;
	Town *town;
	IntrusiveMultiList<Entity> cops;
	CriminalRecords records;
	int updatesSincePlayerSeen;
};

} // sl

#endif // SL_POLICEFORCE_HPP