#ifndef SL_TEST_ASSOCIATIONSTEST_HPP
#define SL_TEST_ASSOCIATIONSTEST_HPP

#include "Tests/Test.hpp"

#include "NPC/Stats/Associations.hpp"
#include "NPC/Stats/Concepts.hpp"

namespace sl
{
namespace test
{

class AssociationsTest : public Test
{
public:
	bool run(std::ostream& err) override
	{
		// TODO: test other stuff like markov
		return runShortestPathTest(err);
	}
private:
	bool runShortestPathTest(std::ostream& err) const
	{
		Concepts c = Concepts::makeEmpty();
		const std::string x = "x";
		const std::string y = "y";
		const std::string z = "z";
		const std::string u = "u";
		const std::string v = "v";
		const std::string w = "w";
		c.registerConcept(x);
		c.registerConcept(y);
		c.registerConcept(z);
		c.registerConcept(u);
		c.registerConcept(v);
		c.registerConcept(w);
		Associations a(c);
		a.associate(x, y, 0.5f);
		a.associate(x, w, 0.1f);
		a.associate(x, u, 0.2f);
		a.associate(y, w, 0.1f);
		a.associate(y, v, 0.9f);
		a.associate(y, z, 0.7f);
		a.associate(z, u, 0.9f);
		a.associate(z, v, 0.3f);
		a.associate(u, v, 0.001f);
		a.associate(v, w, 0.8f);

		std::map<std::string, std::map<std::string, float>> shortest;
		for (int i = 0; i < c.getConceptCount(); ++i)
		{
			shortest.insert(std::make_pair(c.getConcept(i), a.shortestDistance(i, 0.1f, std::set<int>(), Associations::allEdges)));
		}
		int failed = 0;
		for (int i = 0; i < c.getConceptCount(); ++i)
		{
			for (int j = 0; j < c.getConceptCount(); ++j)
			{
				const std::string& from = c.getConcept(i);
				const std::string& to = c.getConcept(j);
				try
				{
					CHECK_EQ_FLOAT(shortest.at(from).at(to), shortest.at(to).at(from));
				}
				catch (const std::exception& e)
				{
					err << "No results between " << from << " and " << to << "; exception: " << e.what() << std::endl;
					return false;
				}
			}
		}
#define ASSOC_TEST_CASE(from, to, expected) try \
	{ \
		const float dist = shortest.at(from).at(to); \
		CHECK_EQ_FLOAT(dist, (expected)); \
	} \
	catch (const std::exception& e) \
	{ \
		err << "Case from " << from << " to " << to << " failed due to exception: " << e.what() << std::endl; \
		return false; \
	} \

	ASSOC_TEST_CASE(x, u, 0.5*0.7*0.9)
	ASSOC_TEST_CASE(y, w, 0.8*0.9)
	ASSOC_TEST_CASE(w, u, 0.9*0.7*0.9*0.8)

#undef ASSOC_TEST_CASE
		return true;
	}
};

} // test
} // sl

#endif // SL_TEST_TILESERIALIZETEST_HPP