#ifndef SL_TEST_AUTODEFINEOBJECTSERIALIZATIONTEST_HPP
#define SL_TEST_AUTODEFINEOBJECTSERIALIZATIONTEST_HPP

#include "Engine/Serialization/AutoDefineObjectSerialization.hpp"
#include "Engine/Serialization/BinaryDeserialize.hpp"
#include "Engine/Serialization/BinarySerialize.hpp"
#include "Tests/Test.hpp"

#include <string>
#include <map>
#include <vector>

namespace sl
{
namespace test
{

class Foo
{
public:
	Foo() : x(-1), y(-2) {}
	Foo(int x, int y) : x(x), y(y) {}

	bool operator==(const Foo& rhs) const
	{
		return x == rhs.x && y == rhs.y;
	}

	int getX() const { return x; }
	int x, y;
};
		
class Bar
{
public:
	bool operator==(const Bar& rhs) const
	{
		return x == rhs.x && y == rhs.y && z == rhs.z;
	}

	int x;
	std::vector<float> y;
	std::map<std::string, Foo> z;
};

} //  test

DEF_SERIALIZE_OBJ_CTOR(test::Foo, &test::Foo::getX, &test::Foo::y)
DEF_SERIALIZE_OBJ_FIELDS(test::Bar, &test::Bar::x, &test::Bar::y, &test::Bar::z)

namespace test
{

class AutoDefineObjectSerializationTest : public Test
{
public:
	bool run(std::ostream& err) override
	{
		BinarySerialize out("autodef_serialization_test");
		Bar bar;
		bar.x = -7;
		bar.y = { 1.f, 3.f, 3.f, 7.f };
		bar.z["one"] = Foo(6, 9);
		bar.z["two"] = Foo(1, 3);
		AutoSerialize::serialize(out, bar);
		out.finish();
		std::map<std::string, std::function<std::unique_ptr<Serializable>()>> ptrAllocators;
		BinaryDeserialize in("autodef_serialization_test", ptrAllocators);
		Bar result;
		CHECK(!(result == bar));
		AutoDeserialize::deserialize(in, result);
		in.finish();
		return bar == result;
	}
};

} // test
} // sl

#endif // SL_TEST_AUTODEFINEOBJECTSERIALIZATIONTEST_HPP