#ifndef SL_TEST_CONTAINERSERIALIZETEST_HPP
#define SL_TEST_CONTAINERSERIALIZETEST_HPP

#include "Engine/Serialization/AutoDeserialize.hpp"
#include "Engine/Serialization/AutoSerialize.hpp"
#include "Engine/Serialization/BinaryDeserialize.hpp"
#include "Engine/Serialization/BinarySerialize.hpp"
#include "Tests/Test.hpp"

#include <string>
#include <map>

namespace sl
{
namespace test
{

class ContainerSerializeTest : public Test
{
public:
	bool run(std::ostream& err) override
	{
		std::map<std::pair<std::string, std::string>, std::map<int, std::vector<int>>> dickSizesTakenAtAge, result;
		dickSizesTakenAtAge[std::make_pair("Amanda", "Jones")] = {
			std::make_pair(4, std::vector<int>({5, 6}))
		};
		dickSizesTakenAtAge[std::make_pair("Jenny", "Campbell")] = {
			std::make_pair(9, std::vector<int>({ 5, 5, 4 })),
			std::make_pair(10, std::vector<int>({5, 6, 7}))
		};
		dickSizesTakenAtAge[std::make_pair("Hannah", "Brooks")] = {
			std::make_pair(7, std::vector<int>({ 3, 2 })),
			std::make_pair(8, std::vector<int>({ 3, 4, 5 })),
			std::make_pair(9, std::vector<int>({ 6, 5, 6, 4 })),
			std::make_pair(10, std::vector<int>({ 6, 7, 8, 7 })),
		};
		constexpr int x_init = 1337;
		constexpr int y_init = 666;
		constexpr int z_init = 9;
		BinarySerialize out("container_test");
		AutoSerialize::serialize(out, dickSizesTakenAtAge);
		AutoSerialize::fields(out, x_init, y_init, z_init);
		out.finish();
		std::map<std::string, std::function<std::unique_ptr<Serializable>()>> ptrAllocators;
		int x = 0;
		int y = 0;
		int z = 0;
		BinaryDeserialize in("container_test", ptrAllocators);
		AutoDeserialize::deserialize(in, result);
		AutoDeserialize::fields(in, x, y, z);
		in.finish();
		CHECK(dickSizesTakenAtAge == result);
		CHECK_EQ(x, x_init);
		CHECK_EQ(y, y_init);
		CHECK_EQ(z, z_init);
		return true;
	}
};

} // test
} // sl

#endif // SL_TEST_CONTAINERSERIALIZETEST_HPP