#ifndef SL_TEST_CONVERGENTSERIESTEST_HPP
#define SL_TEST_CONVERGENTSERIESTEST_HPP

#include "Tests/Test.hpp"

#include "Utility/Functions/ConvergentSeries.hpp"

namespace sl
{
namespace test
{

class ConvergentSeriesTest : public Test
{
public:
	bool run(std::ostream& err) override
	{
		constexpr int n = 10'000;
#define TEST_FUNCTION(fn) do \
{ \
	std::cout << "Testing case :" << #fn << std::endl; \
	const auto f = std::move(fn); \
	float sum = 0.f; \
	bool reached = false; \
	for (int i = 1; i <= n; ++i) \
	{ \
		sum += f(i); \
		if (!reached && sum >= 0.999f) \
		{ \
			std::cout << " - reached 0.999 at n = " << i << std::endl; \
			reached = true; \
		} \
	} \
	CHECK_EQ_FLOAT_EPS(sum, 1.f, 0.001f); \
} while (false)

		TEST_FUNCTION(geometricSeries);
		TEST_FUNCTION(baselSeries);
		TEST_FUNCTION(FiniteSeries([](int x) { return 100.f - std::sqrt(x); }));
		TEST_FUNCTION(FiniteSeries([](int x) { return 1.f - std::sin(x / 23.f); }, 1337));
		return true;
	}
};

} // test
} // sl

#endif // SL_TEST_CONVERGENTSERIESTEST_HPP