#ifndef SL_TEST_GRAPHCLUSTERINGTEST_HPP
#define SL_TEST_GRAPHCLUSTERINGTEST_HPP

#include "Tests/Test.hpp"
#include "Utility/GraphClustering.hpp"

namespace sl
{
namespace test
{

class GraphClusteringTest : public Test
{
public:
	bool run(std::ostream& err) override
	{
		Matrix<float> adj(n, n, 0.f);
		auto edge = [&adj](int u, int v, float w) {
			adj(u, v) = w;
			adj(v, u) = w;
		};
		edge(0, 6, 0.3f);
		edge(0, 5, 0.4f);
		edge(0, 9, 0.5f);
		edge(1, 2, 0.3f);
		edge(1, 3, 0.25f);
		edge(2, 3, 0.3f);
		edge(2, 6, 0.1f);
		edge(2, 4, 0.3f);
		edge(5, 7, 0.1f);
		edge(6, 8, 0.4f);
		edge(7, 8, 0.6f);
		edge(7, 9, 0.1f);
		
		const ClusterParameters paramCases[] = {
			{3, 0.1f},
			{4, 0.1f}
		};
		for (const ClusterParameters& params : paramCases)
		{
			const auto clusters = weightedDirectedClusters(adj, params.clusterSize, params.threshold);
			if (!verifyClusters(err, adj, params, clusters))
			{
				return false;
			}
			const auto mergedClusters = mergeLeftoversGreedily(adj, params.clusterSize, params.threshold, clusters);
			if (!verifyClusters(err, adj, params, mergedClusters))
			{
				return false;
			}
		}
		return true;
	}
private:
	struct ClusterParameters
	{
		int clusterSize = 3;
		float threshold = 0.1f;
	};

	bool verifyClusters(std::ostream& err, const Matrix<float>& adj, const ClusterParameters& params, const std::vector<std::vector<int>>& clusters) const
	{
		std::vector<int> foundCount(n, 0);
		std::cout << "Verifying clusters: " << std::endl;
		for (int i = 0; i < clusters.size(); ++i)
		{
			CHECK(!clusters[i].empty());
			CHECK_LEQ(clusters[i].size(), params.clusterSize);
			std::cout << "Cluster " << i << " =  { " << clusters[i][0];
			for (int j = 1; j < clusters[i].size(); ++j)
			{
				std::cout << ", " << clusters[i][j];
			}
			std::cout << "}" << std::endl;
			for (int u : clusters[i])
			{
				++foundCount[u];
				// TODO: proper check to see if they meet the threshold
				//for (int v : clusters[i])
				//{
				//	if (u != v)
				//	{
				//		std::cout << "adj(" << u << ", " << v << ") = " << adj(u, v) << std::endl;
				//		CHECK_GEQ(adj(u, v), threshold);
				//	}
				//}
			}
		}
		for (int count : foundCount)
		{
			CHECK_EQ(count, 1);
		}
		return true;
	}

	static constexpr int n = 10;
};

} // test
} // sl

#endif // SL_TEST_GRAPHCLUSTERINGTEST_HPP