/**
 * @section DESCRIPTION Not actually a test. Just takes the contents of loli/middle/___/hair/___
 * and copies them to the kinder/elem folders with the correct offsets, sot hat you only need to edit /middle/ for the
 * small sprites' hair.
 */
#ifndef SL_TEST_HAIRPIPELINE_HPP
#define SL_TEST_HAIRPIPELINE_HPP

#include "Tests/Test.hpp"
#include "Utility/Assert.hpp"

#include <SFML/Graphics/Image.hpp>

#include <experimental/filesystem>
#include <map>
#include <string>
#include <vector>

namespace sl
{
namespace test
{

class HairPipeline : public Test
{
public:
	bool run(std::ostream& err) override
	{
		struct HairSwapInfo
		{
			std::map<std::string, std::vector<int>> offsets;
			std::string dir;
		};
		HairSwapInfo swaps[2];
		swaps[0].dir = "kinder";
		swaps[0].offsets["up.png"]   = { 1, 2, 1, 2 };
		swaps[0].offsets["side.png"] = { 1, 1, 1, 1 };
		swaps[0].offsets["down.png"] = { 1, 2, 1, 2 };
		swaps[1].dir = "elem";
		swaps[1].offsets["up.png"]   = { 1, 1, 1, 1 };
		swaps[1].offsets["side.png"] = { 1, 1, 1, 1 };
		swaps[1].offsets["down.png"] = { 1, 1, 1, 1 };
		namespace fs = std::experimental::filesystem;
		const std::string middlePath = makePath("middle");
		bool success = true;
		for (const fs::directory_entry& file : fs::recursive_directory_iterator(middlePath))
		{
			if (file.path().extension() == ".png")
			{
				const std::string fullPath = file.path().string();
				const std::string terminalPath(std::next(std::begin(fullPath), middlePath.size()), std::end(fullPath));
				const std::string fullDir = fs::path(file.path()).remove_filename().string();
				const std::string terminalDir(std::next(std::begin(fullDir), middlePath.size()), std::end(fullDir));
				std::cout << "IN: " << fullPath << std::endl;
				sf::Image image;
				if (!image.loadFromFile(fullPath))
				{
					err << "Could not load '" << fullPath << "'" << std::endl;
					success = false;
				}
				sf::Image newImage;
				const int w = image.getSize().x;
				const int h = image.getSize().y;
				for (const HairSwapInfo& swap : swaps)
				{
					newImage.create(w, h, sf::Color::Transparent);
					ASSERT(w == 32 * 4 && h == 32);
					auto offsets = swap.offsets.find(file.path().filename().string());
					if (offsets == swap.offsets.end())
					{
						err << "Unknown hair file '" << file.path() << "'" << std::endl;
						return false;
					}
					ASSERT(offsets->second.size() == 4);
					for (int offset = 0; offset < 4; ++offset)
					{
						for (int y = 0; y < h; ++y)
						{
							for (int x = 0; x < 32; ++x)
							{
								const int px = x + 32 * offset;
								const int py = y + offsets->second[offset];
								if (py >= 0 && py < h)
								{
									newImage.setPixel(px, py, image.getPixel(px, y));
								}
							}
						}
					}
					const std::string newDir = makePath(swap.dir) + terminalDir;
					//std::cout << "OUTDIR: " << newDir << std::endl;
					fs::create_directory(newDir);
					const std::string newFile = makePath(swap.dir) + terminalPath;
					std::cout << "OUT: " << newFile << std::endl;
					newImage.saveToFile(newFile);
				}
			}
		}
		return success;
	}
private:
	static std::string makePath(const std::string& dir)
	{
		return "resources\\img\\loli\\" + dir + "\\walk\\hair\\";
	}
};

} // test
} // sl

#endif // SL_TEST_HAIRPIPELINE_HPP