#ifndef SL_TEST_MULTIVARIATELINEARINTERPOLATIONTEST_HPP
#define SL_TEST_MULTIVARIATELINEARINTERPOLATIONTEST_HPP

#include "Tests/Test.hpp"

#include "Utility/MultivariateLinearInterpolation.hpp"

namespace sl
{
namespace test
{

// TODO: move so we can use it elsewhere?
template <typename T, unsigned int N>
std::ostream& operator<<(std::ostream& os, const Vector<T, N>& v)
{
	os << "(";
	for (int i = 0; i < N; ++i)
	{
		if (i > 0)
		{
			os << ", ";
		}
		os << v[i];
	}
	os << ")";
	return os;
}

class MultivariateLinearInterpolationTest : public Test
{
public:
	bool run(std::ostream& err) override
	{
		const std::vector<Vector2f> pts = {
			Vector2f(0.f, 0.f),
			Vector2f(0.f, 1.f),
			Vector2f(1.f, 1.f),
			Vector2f(1.f, 0.f),
			Vector2f(0.5f, 0.5f)
		};
		typedef Vector<float, 5> Vector5f;
		const std::vector<Vector5f> values = {
			Vector5f(1.f, 0.f, 0.f, 0.f, 0.f),
			Vector5f(0.f, 1.f, 0.f, 0.f, 0.f),
			Vector5f(0.f, 0.f, 1.f, 0.f, 0.f),
			Vector5f(0.f, 0.f, 0.f, 1.f, 0.f),
			Vector5f(0.f, 0.f, 0.f, 0.f, 1.f)
		};
		// check absolute
		for (int i = 0; i < 5; ++i)
		{
			CHECK_EQ(values[i], voronoiLerp(pts, values, pts[i], 0.f, 0.f));
		}
		// check distance
		for (float step = 0.05f; step < 1.f; step += 0.1f)
		{
			const Vector5f lerpOnX = voronoiLerp(pts, values, Vector2f(step, 0.5f), 0.f, 1.0f);
			CHECK_EQ(lerpOnX.at<0>(), lerpOnX.at<1>());
			CHECK_EQ(lerpOnX.at<2>(), lerpOnX.at<3>());
			const Vector5f lerpOnY = voronoiLerp(pts, values, Vector2f(step, 0.5f), 0.f, 1.0f);
			CHECK_EQ(lerpOnY.at<0>(), lerpOnY.at<1>());
			CHECK_EQ(lerpOnY.at<2>(), lerpOnY.at<3>());
		}
		// check dot product
		for (float step = 0.05f; step < 1.f; step += 0.1f)
		{
			const Vector5f d1 = voronoiLerp(pts, values, Vector2f(step, step), 1.f, 0.f);
			CHECK_EQ_FLOAT(d1.at<1>(), 0.f);
			CHECK_EQ_FLOAT(d1.at<3>(), 0.f);
			CHECK_NEQ(d1.at<4>(), 0.f);
			if (step < 0.5f)
			{
				CHECK_NEQ(d1.at<0>(), 0.f);
				CHECK_EQ(d1.at<2>(), 0.f);
			}
			else
			{
				CHECK_EQ(d1.at<0>(), 0.f);
				CHECK_NEQ(d1.at<2>(), 0.f);
			}
			const Vector5f d2 = voronoiLerp(pts, values, Vector2f(step, 1.f - step), 1.f, 0.f);
			CHECK_EQ_FLOAT(d2.at<0>(), 0.f);
			CHECK_EQ_FLOAT(d2.at<2>(), 0.f);
			CHECK_NEQ(d2.at<4>(), 0.f);
			if (step < 0.5f)
			{
				CHECK_NEQ(d2.at<1>(), 0.f);
				CHECK_EQ(d2.at<3>(), 0.f);
			}
			else
			{
				CHECK_EQ(d2.at<1>(), 0.f);
				CHECK_NEQ(d2.at<3>(), 0.f);
			}
		}
		// TODO: more tests
		//CHECK_EQ(Vector5f(0.f, 0.f, 0.f, 0.f, 0.f),voronoiLerp(pts, values, Vector2f(), 0.f, 1.f));
		return true;
	}
};

} // test
} // sl

#endif // SL_TEST_MULTIVARIATELINEARINTERPOLATIONTEST_HPP 