#ifndef SL_TEST_MULTIVARIATEVORONOITEST_HPP
#define SL_TEST_MULTIVARIATEVORONOITEST_HPP

#include "Tests/Test.hpp"

#include "Utility/MultivariateVoronoi.hpp"

namespace sl
{
namespace test
{

// TODO: move this to another file and generalize?
std::ostream& operator<<(std::ostream& os, const std::vector<int>& container)
{
	os << "{";
	for (int i = 0; i < container.size(); ++i)
	{
		if (i > 0)
		{
			os << ", ";
		}
		os << container[i];
	}
	os << "}";
	return os;
}

class MultivariateVoronoiTest : public Test
{
public:
	bool run(std::ostream& err) override
	{
		const std::vector<Vector2f> pts1 = {
			Vector2f(140, 137),
			Vector2f(343, 85),
			Vector2f(579, 136),
			Vector2f(747, 98),
			Vector2f(73, 307),
			Vector2f(305, 379),
			Vector2f(519, 315),
			Vector2f(740, 389),
			Vector2f(119, 520),
			Vector2f(319, 592),
			Vector2f(486, 518),
			Vector2f(642, 595),
			Vector2f(834, 463)
		};
		const std::vector<std::vector<int>> adj1 = {
			{0, 1, 4, 5},
			{0, 1, 2, 5, 6},
			{1, 2, 3, 6, 7},
			{2, 3, 7},
			{0, 4, 5, 8},
			{0, 1, 4, 5, 6, 8, 9, 10},
			{1, 2, 5, 6, 7, 10},
			{2, 3, 6, 7, 10, 11, 12},
			{4, 5, 8, 9},
			{5, 8, 9, 10},
			{5, 6, 7, 9, 10, 11},
			{7, 10, 11, 12},
			{7, 11, 12}
		};
		for (int i = 0; i < pts1.size(); ++i)
		{
			std::vector<int> neighbors = computeVoronoiNeighbors(pts1, pts1[i]);
			ASSERT(neighbors[0] == i);
			std::sort(neighbors.begin(), neighbors.end());
			CHECK_EQ(neighbors, adj1[i]);
		}
		std::vector<Vector<float, 3>> pts2;
		std::map<Vector<float, 3>, int> pts2ToIndex;
		for (int x = 0; x < 3; ++x)
		{
			for (int y = 0; y < 3; ++y)
			{
				for (int z = 0; z < 3; ++z)
				{
					const Vector<float, 3> vec(x, y, z);
					pts2ToIndex.insert(std::make_pair(vec, pts2.size()));
					pts2.push_back(vec);
				}
			}
		}
		for (int i = 0; i < pts2.size(); ++i)
		{
			// add neighboring cells
			std::vector<int> adj2;
			adj2.push_back(i);
			if (pts2[i].at<0>() > 0.f)
			{
				adj2.push_back(pts2ToIndex.at(pts2[i] - Vector<float, 3>(1.f, 0.f, 0.f)));
			}
			if (pts2[i].at<0>() < 2.f)
			{
				adj2.push_back(pts2ToIndex.at(pts2[i] + Vector<float, 3>(1.f, 0.f, 0.f)));
			}
			if (pts2[i].at<1>() > 0.f)
			{
				adj2.push_back(pts2ToIndex.at(pts2[i] - Vector<float, 3>(0.f, 1.f, 0.f)));
			}
			if (pts2[i].at<1>() < 2.f)
			{
				adj2.push_back(pts2ToIndex.at(pts2[i] + Vector<float, 3>(0.f, 1.f, 0.f)));
			}
			if (pts2[i].at<2>() > 0.f)
			{
				adj2.push_back(pts2ToIndex.at(pts2[i] - Vector<float, 3>(0.f, 0.f, 1.f)));
			}
			if (pts2[i].at<2>() < 2.f)
			{
				adj2.push_back(pts2ToIndex.at(pts2[i] + Vector<float, 3>(0.f, 0.f, 1.f)));
			}
			std::sort(adj2.begin(), adj2.end());

			// check
			std::vector<int> neighbors = computeVoronoiNeighbors(pts2, pts2[i]);
			std::sort(neighbors.begin(), neighbors.end());
			CHECK_EQ(neighbors, adj2);
		}
		return true;
	}
};

} // test
} // sl

#endif // SL_TEST_MULTIVARIATEVORONOITEST_HPP