#ifndef SL_TEST_PARSEREXPRESSIONTEST_HPP
#define SL_TEST_PARSEREXPRESSIONTEST_HPP

#include "Parsing/Binding/Bindings.hpp"
#include "Parsing/Parsers/ExpressionParser.hpp"
#include "Parsing/SyntaxTree/Expressions/Literal.hpp"
#include "Parsing/Errors/SyntaxError.hpp"
#include "Parsing/Errors/RuntimeError.hpp"
#include "Tests/ParserTestUtil.hpp"
#include "Tests/Test.hpp"

namespace sl
{
namespace test
{

class ParserExpressionTest : public Test
{
public:
	bool run(std::ostream& err) override
	{
		using ls::Value;
		int failed = 0;
		int total = 0;
		ls::Bindings context;
		bindTestFunctions(context);

		auto testcase = [&](const char *line, auto rawExpected) {
			const Value expected = Value::make(rawExpected);
			++total;
			std::unique_ptr<ls::Expression> expression;
			try
			{
				expression = ls::ExpressionParser("ParserExpressionTest").parseEntireExpression(line);
			}
			catch (const ls::SyntaxError& see)
			{
				err << "Syntax error: " << see.what() << std::endl;
				++failed;
				return;
			}
			try
			{
				const Value result = expression->evaluate(context);
				if (result != expected)
				{
					err << "Failed test case: \"" << line << "\". Expected result: " << expected.asString() << ". Found result: " << result.asString() << std::endl;
					return;
				}
			}
			catch (const ls::RuntimeError& ree)
			{
				err << "Runtime Error: " << ree.what() << std::endl;
				++failed;
				return;
			}
		};
		if (failed != 0)
		{
			err << "Failed " << failed << " / " << total << " test cases." << std::endl;
			return false;
		}

		testcase("(5 + 3) / 8", 1);
		testcase("fp_add(5, 3)", 8);
		testcase("mp_add(-1)", 8);
		testcase("fp_add(5, 3) == mp_add(-1)", true);
		testcase("fp_add(5, 3) != mp_add(0)", true);
		testcase("counter()", "0");
		testcase("counter()", "1");
		testcase("(1 < 2) != (2 < 1)", true);
		testcase("merge(\"little\", id(\" gi\") + \"rl\") + \"s\"", "little girls");
		testcase("lam_add(3.5) > 13 + 23", false);
		testcase("id(5 + 3)", 8);

		return true;
	}
};

} // test
} // sl

#endif // SL_TEST_PARSEREXPRESSIONTEST_HPP