#include "Tests/ParserTestUtil.hpp"

namespace sl
{
namespace test
{

int fp_add(int x, int y)
{
	return x + y;
}

class Foo
{
public:
	Foo(int x)
		:x(x)
		,counter(0)
	{}

	float add(float y) const
	{
		return x + y;
	}

	std::string count()
	{
		return std::to_string(counter++);
	}

	int x;
	int counter;
};

class IdentityValue
{
public:
	ls::Value operator()(ls::Value value) const
	{
		return value;
	}
};

static Foo foo(9);

void bindTestFunctions(ls::Bindings& context)
{
	// tests function pointers
	context.registerFunction("local", "fp_add", fp_add);
	// tests methods
	context.registerFunction("local", "mp_add", foo, &Foo::add);
	// tests std::string return and non-const method
	context.registerFunction("local", "counter", foo, &Foo::count);
	// tests lambdas (ie functors) presumably const
	const int x = 3;
	context.registerFunction("local", "lam_add", [x](float y) { return x + y; });
	// tests std::string and Value as parameters
	context.registerFunction("local", "merge", [](std::string lhs, ls::Value rhs) {
		return lhs + rhs.asString();
	});
	// tests void-return and Arguments& (ie loliscript) argument style
	context.registerFunction("local", "print_args", [](const ls::Arguments& args) {
		std::cout << "[";
		for (int i = 0; i < args.count(); ++i)
		{
			if (i != 0)
			{
				std::cout << ", ";
			}
			std::cout << args[i].asString();
		}
		std::cout << "]";
	});
	// tests Value return and non-lambda functor
	context.registerFunction("local", "id", IdentityValue());
}

} // test
} // sl

namespace std
{

std::ostream& operator<<(std::ostream& os, const sl::ls::Value& value)
{
	os << "{Type: ";
	switch (value.getType())
	{
	case sl::ls::Value::Type::Uninitialized:
		os << "Uninitialized}";
		return os;
	case sl::ls::Value::Type::Bool:
		os << "Bool";
		break;
	case sl::ls::Value::Type::Int:
		os << "Int";
		break;
	case sl::ls::Value::Type::Float:
		os << "Float";
		break;
	case sl::ls::Value::Type::String:
		os << "String";
		break;
	case sl::ls::Value::Type::Ptr:
		os << "Ptr";
		break;
	}
	os << "Value: " << value.asString() << "}";
	return os;
}

} // std
