#ifndef SL_TEST_REFTEST_HPP
#define SL_TEST_REFTEST_HPP

#include "Tests/Test.hpp"

#include "Engine/Ref.hpp"
#include "Engine/Refable.hpp"

namespace sl
{
namespace test
{

// can't be a member or we can't prototype this function...
struct TestRefable
{
	TestRefable()
	{
		static int y = 0;
		x = y++;
	}

	const Refable& getAliveRef() const
	{
		return refable;
	}

	int x;
	Refable refable;
};

// for printing here only
template <bool O>
std::ostream& operator<<(std::ostream& os, const Ref<TestRefable, O>& ref)
{
	os << "Ref<";
	if (ref)
	{
		os << "TestRef[" << ref->x << "]";
	}
	else
	{
		os << "null";
	}
	os << ">";
	if (O)
	{
		os << "{locked}";
	}
	return os;
}
class RefTest : public Test
{
public:
	bool run(std::ostream& err) override
	{
		TestRefable t1, t2;
		Ref<TestRefable> n1, n2, r1(&t1), r2(&t2);

		CHECK(!n1);
		CHECK(r1);
		CHECK(t1.refable.canDestroy());
		{
			auto l = r1.lock();
			CHECK(!t1.refable.canDestroy());
		}
		CHECK(t1.refable.canDestroy());
		CHECK_EQ(&t1, r1.get());
		CHECK_NEQ(&t2, r1.get());
		CHECK_EQ(n1, n2);
		CHECK_NEQ(r1, r2);
		CHECK_NEQ(r1, n1);
		{
			CHECK(!n2);
			TestRefable temp;
			n2 = Ref<TestRefable>(&temp);
			CHECK(n2);
		}
		CHECK(!n2);

		return true;
	}
};

} // test
} // sl

#endif // SL_TEST_REFTEST_HPP