#ifndef SL_TEST_SERIALIZABLETEST_HPP
#define SL_TEST_SERIALIZABLETEST_HPP

#include "Engine/Serialization/BinaryDeserialize.hpp"
#include "Engine/Serialization/BinarySerialize.hpp"
#include "Engine/Serialization/Serializable.hpp"
#include "Tests/Test.hpp"
#include "Utility/Memory.hpp"

#include <string>
#include <map>

namespace sl
{
namespace test
{

class SerializableTest : public Test
{
public:
	bool run(std::ostream& err) override
	{
		std::map<std::string, std::function<std::unique_ptr<Serializable>()>> allocators;
		allocators["Foo"] = []() {
			return unique<Foo>("uninitialized");
		};
		BinarySerialize out("serializable_test");
		auto foo1 = unique<Foo>("Foo1");
		auto foo2 = unique<Foo>("Foo2");
		foo1->other = foo2.get();
		foo2->other = foo1.get();
		out.saveSerializable("foo1", foo1.get());
		out.saveSerializable("foo2", foo2.get());
		//foo3->other = foo3.get();
		out.finish();
		BinaryDeserialize in("serializable_test", std::move(allocators));
		std::unique_ptr<Foo> foo1_back((Foo*)in.loadSerializable("foo1").release());
		std::unique_ptr<Foo> foo2_back((Foo*)in.loadSerializable("foo2").release());
		in.finish();
		bool success = true;
		if (foo1->name != foo1_back->name)
		{
			err << "foo1->name (" << foo1->name << ") != foo1_back->name (" << foo1_back->name << ")" << std::endl;
			success = false;
		}
		if (foo2->name != foo2_back->name)
		{
			err << "foo2->name (" << foo1->name << ") != foo2_back->name (" << foo1_back->name << ")" << std::endl;
			success = false;
		}
		if (foo1_back->other != foo2_back.get() || foo2_back->other != foo1_back.get())
		{
			err << "foo1_back->other != foo2_back || foo2_back->other != foo1_back\n";
			success = false;
		}
		return success;
	}
private:
	class Foo : public Serializable
	{
	public:
		Foo(std::string name)
			:other(nullptr)
			,name(std::move(name))
		{
		}

		std::string serializationType() const override
		{
			return "Foo";
		}

		bool shouldSerialize() const override
		{
			return true;
		}

		void serialize(Serialize& out) const override
		{
			out.ptr("other", other);
			out.s("name", name);
		}

		void deserialize(Deserialize& in) override
		{
			other = static_cast<Foo*>(in.ptr("other"));
			name = in.s("name");
		}

		Foo *other;
		std::string name;
	};
};

} // test
} // sl

#endif // SL_TEST_SERIALIZABLETEST_HPP