#include "Tests/TestRunner.hpp"

#include "Tests/AssociationsTest.hpp"
#include "Tests/AutoDefineObjectSerializationTest.hpp"
#include "Tests/ContainerSerializeTest.hpp"
#include "Tests/ConvergentSeriesTest.hpp"
#include "Tests/DialogueLintTest.hpp"
#include "Tests/GraphClusteringTest.hpp"
#include "Tests/HairPipeline.hpp"
#include "Tests/MultivariateLinearInterpolationTest.hpp"
#include "Tests/MultivariateVoronoiTest.hpp"
#include "Tests/ParserExpressionTest.hpp"
#include "Tests/RefTest.hpp"
#include "Tests/SerializableTest.hpp"
#include "Tests/TileSerializeTest.hpp"
#include "Tests/Test.hpp"
#include "Utility/Assert.hpp"
#include "Utility/Memory.hpp"

#include <map>

namespace sl
{
namespace test
{

bool runTests(std::ostream& err, const std::vector<std::string>& testsToRun)
{
#define REGISTER_TEST(test) ASSERT(tests.insert(std::make_pair((#test), unique<test>())).second)
	static std::map <std::string, std::unique_ptr<Test>> tests;
	if (tests.empty())
	{
		REGISTER_TEST(AssociationsTest);
		REGISTER_TEST(AutoDefineObjectSerializationTest);
		REGISTER_TEST(ContainerSerializeTest);
		REGISTER_TEST(ConvergentSeriesTest);
		REGISTER_TEST(DialogueLintTest);
		REGISTER_TEST(GraphClusteringTest);
		//REGISTER_TEST(HairPipeline);
		REGISTER_TEST(MultivariateLinearInterpolationTest);
		REGISTER_TEST(MultivariateVoronoiTest);
		REGISTER_TEST(RefTest);
		REGISTER_TEST(ParserExpressionTest);
		REGISTER_TEST(SerializableTest);
		REGISTER_TEST(TileSerializeTest);
	}
	int failed = 0;
	if (testsToRun.empty())
	{
		for (const auto& test : tests)
		{
			if (!test.second->run(err))
			{
				err << "Failed test '" << test.first << "'" << std::endl;
				++failed;
			}
		}
	}
	else
	{
		for (const std::string& test : testsToRun)
		{
			auto it = tests.find(test);
			if (it != tests.end())
			{
				if (!it->second->run(err))
				{
					err << "Failed test '" << test << "'" << std::endl;
					++failed;
				}
			}
			else
			{
				err << "Test '" << test << "' does not exist. Skipping." << std::endl;
				++failed;
			}
		}
	}
	if (failed != 0)
	{
		err << "Failed " << failed << " / " << testsToRun.size() << " tests." << std::endl;
	}
	return failed == 0;
}

bool runAlwaysTests(std::ostream& err)
{
	//return runTests(err, { "HairPipeline" });
	//return runTests(err, { "TileSerializeTest" });
	return runTests(err, { });
}

} // test
} // sl