#ifndef SL_TEST_TILESERIALIZETEST_HPP
#define SL_TEST_TILESERIALIZETEST_HPP

#include "Engine/Serialization/BinaryDeserialize.hpp"
#include "Engine/Serialization/BinarySerialize.hpp"
#include "Engine/Serialization/TileSerialization.hpp"
#include "Tests/Test.hpp"
#include "Utility/Random.hpp"

namespace sl
{
namespace test
{

class TileSerializeTest : public Test
{
public:
	bool run(std::ostream& err) override
	{
		Grid<TileType> tilesOrig(256, 256);
		auto it = tilesOrig.begin();
		const std::pair<TileType, int> makeInARow[] = {
			{Grass, 128},
			{Dirt, 127},
			{SidewalkE, 129},
			{ParkPathS, 128*128},
			{Grass, 128*128 - 1},
			{Intersection, 128*128 + 1},
			{Dirt, 5265}
		};
		for (int i = 0, n = 256 + Random::get().random(1024); i < n; ++i)
		{
			*(it++) = static_cast<TileType>(Random::get().random(NumOfTiles));
		}
		for (const auto& inARow : makeInARow)
		{
			for (int i = 0; i < inARow.second; ++i)
			{
				*(it++) = inARow.first;
			}
		}
		TileType cur;
		int left = 0;
		while (it != tilesOrig.end())
		{
			if (left <= 0)
			{
				left = 8 + Random::get().random(256);
				cur = static_cast<TileType>(Random::get().random(NumOfTiles));
			}
			*(it++) = cur;
		}
		BinarySerialize out("tile_test");
		serializeTiles(out, tilesOrig);
		out.finish();
		Grid<TileType> tilesBack;
		std::map<std::string, std::function<std::unique_ptr<Serializable>()>> ptrAllocators;
		BinaryDeserialize in("tile_test", ptrAllocators);
		deserializeTiles(in, tilesBack);
		bool passed = true;
		for (int y = 0; y < tilesOrig.getHeight(); ++y)
		{
			for (int x = 0; x < tilesOrig.getWidth(); ++x)
			{
				if (tilesOrig(x, y) != tilesBack(x, y))
				{
					err << "different at (" << x << ", " << y << ") - " << tilesOrig(x, y) << " != " << tilesBack(x, y) << std::endl;
					passed = false;
				}
			}
		}
		return passed;
	}
};

} // test
} // sl

#endif // SL_TEST_TILESERIALIZETEST_HPP