#include "Town/Cars/CarControls.hpp"

namespace sl
{

CarControls::CarControls()
	:isTurningRight(false)
	,isTurningLeft(false)
	,isPedalDown(false)
	,isBackPedalDown(false)
	,isBrakeDown(false)
	,cruiseSpeed(true)
{
}


void CarControls::turnLeft()
{
	isTurningLeft = true;
}

void CarControls::turnRight()
{
	isTurningRight = true;
}

void CarControls::accelerate()
{
	isPedalDown = true;
}

void CarControls::brake()
{
	isBrakeDown = true;
}

void CarControls::reverse()
{
	isBackPedalDown = true;
}

void CarControls::toggleCruiseSpeed()
{
	cruiseSpeed = !cruiseSpeed;
}

void CarControls::reset()
{
	isTurningLeft = false;
	isTurningRight = false;
	isPedalDown = false;
	isBackPedalDown = false;
	isBrakeDown = false;
}

float CarControls::getWheelRotation() const
{
	float position = 0.f;
	if (isTurningLeft)
	{
		position += 1.f;
	}
	else if (isTurningRight)
	{
		position -= 1.f;
	}
	return position;
}

float CarControls::getPedalPosition() const
{
	return isPedalDown ? 1.f : 0.f;
}

bool CarControls::isReverse() const
{
	return isBackPedalDown;
}

bool CarControls::isBraking() const
{
	return isBrakeDown;
}

bool CarControls::isSpeedLimited() const
{
	return cruiseSpeed;
}

} // sl
