#include "Town/Cars/CarSpecs.hpp"

#include "Utility/Assert.hpp"

namespace sl
{

int CarSpecs::initialized = 0;
CarSpecs CarSpecs::specs[NumOfModels];


const CarSpecs& CarSpecs::getSpecs(Model model)
{
	ASSERT(model != NumOfModels);
	return specs[model];
}

CarSpecs::CarSpecs()
{
	//  Make sure this doesn't execute until we're on the last element!
	//  (To make sure that std::string::operator= isn't invoked on an instance that
	//   hasn't been constructed properly yet, which would cause a segfault.)
	if (initialized < NumOfModels)
	{
		++initialized;
	}
	if (initialized == NumOfModels)
	{
		init();
	}
}

void CarSpecs::init()
{
	// TODO: externalize all car data
	specs[Van].modelName                      = "Van";
	specs[Van].turnSpeed                      = 0.5f;
	specs[Van].turnMax                        = 6.f;
	specs[Van].acceleration                   = 0.1f;
	specs[Van].accelerationReverse            = 0.05f;
	specs[Van].brakes                         = 0.2f;
	specs[Van].topSpeed                       = 7.f;
	specs[Van].topSpeedReverse                = -2.5f;
	specs[Van].grip                           = 0.3f;
	specs[Van].filename                       = "van";
	specs[Van].trunkCapacity                  = 16;
	specs[Van].passangerCapacity              = 1;

	specs[IceCreamTruck].modelName            = "Ice Cream Truck";
	specs[IceCreamTruck].turnSpeed            = 0.4f;
	specs[IceCreamTruck].turnMax              = 5.f;
	specs[IceCreamTruck].acceleration         = 0.1f;
	specs[IceCreamTruck].accelerationReverse  = 0.05f;
	specs[IceCreamTruck].brakes               = 0.2f;
	specs[IceCreamTruck].topSpeed             = 6.f;
	specs[IceCreamTruck].topSpeedReverse      = -3.f;
	specs[IceCreamTruck].grip                 = 0.2f;
	specs[IceCreamTruck].filename             = "icecreamtruck";
	specs[IceCreamTruck].trunkCapacity        = 4;
	specs[IceCreamTruck].passangerCapacity    = 1;

	specs[CopCar].modelName                   = "Cop car";
	specs[CopCar].turnSpeed                   = 0.5f;
	specs[CopCar].turnMax                     = 5.f;
	specs[CopCar].acceleration                = 0.125f;
	specs[CopCar].accelerationReverse         = 0.05f;
	specs[CopCar].brakes                      = 0.3f;
	specs[CopCar].topSpeed                    = 11.f;
	specs[CopCar].topSpeedReverse             = -3.5f;
	specs[CopCar].grip                        = 0.2f;
	specs[CopCar].filename                    = "copcar";
	specs[CopCar].trunkCapacity               = 2;
	specs[CopCar].passangerCapacity           = 3;

	specs[SportsCar].modelName                = "Sportscar";
	specs[SportsCar].turnSpeed                = 0.6f;
	specs[SportsCar].turnMax                  = 6.f;
	specs[SportsCar].acceleration             = 0.15f;
	specs[SportsCar].accelerationReverse      = 0.05f;
	specs[SportsCar].brakes                   = 0.3f;
	specs[SportsCar].topSpeed                 = 14.f;
	specs[SportsCar].topSpeedReverse          = -3.5f;
	specs[SportsCar].grip                     = 0.3f;
	specs[SportsCar].filename                 = "sportscar";
	specs[SportsCar].trunkCapacity            = 1;
	specs[SportsCar].passangerCapacity        = 1;

	specs[Sedan].modelName                    = "Sedan";
	specs[Sedan].turnSpeed                    = 1.f;
	specs[Sedan].turnMax                      = 8.f;
	specs[Sedan].acceleration                 = 0.1f;
	specs[Sedan].accelerationReverse          = 0.05f;
	specs[Sedan].brakes                       = 0.2f;
	specs[Sedan].topSpeed                     = 9.f;
	specs[Sedan].topSpeedReverse              = -2.5f;
	specs[Sedan].grip                         = 0.3f;
	specs[Sedan].filename                     = "sedan";
	specs[Sedan].trunkCapacity                = 2;
	specs[Sedan].passangerCapacity            = 3;

	specs[LuxerySedan].modelName              = "Luxery Sedan";
	specs[LuxerySedan].turnSpeed              = 1.f;
	specs[LuxerySedan].turnMax                = 8.f;
	specs[LuxerySedan].acceleration           = 0.1f;
	specs[LuxerySedan].accelerationReverse    = 0.05f;
	specs[LuxerySedan].brakes                 = 0.2f;
	specs[LuxerySedan].topSpeed               = 10.f;
	specs[LuxerySedan].topSpeedReverse        = -2.5f;
	specs[LuxerySedan].grip                   = 0.25f;
	specs[LuxerySedan].filename               = "luxerysedan";
	specs[LuxerySedan].trunkCapacity          = 3;
	specs[LuxerySedan].passangerCapacity      = 4;

	specs[Schoolbus].modelName                = "Schoolbus";
	specs[Schoolbus].turnSpeed                = 0.5f;
	specs[Schoolbus].turnMax                  = 4.f;
	specs[Schoolbus].acceleration             = 0.1f;
	specs[Schoolbus].accelerationReverse      = 0.05f;
	specs[Schoolbus].brakes                   = 0.2f;
	specs[Schoolbus].topSpeed                 = 5.f;
	specs[Schoolbus].topSpeedReverse          = -3.f;
	specs[Schoolbus].grip                     = 0.3f;
	specs[Schoolbus].filename                 = "schoolbus";
	specs[Schoolbus].trunkCapacity            = 0;
	specs[Schoolbus].passangerCapacity        = 32;

	specs[Supercar].modelName                 = "Supercar";
	specs[Supercar].turnSpeed                 = 0.9f;
	specs[Supercar].turnMax                   = 7.f;
	specs[Supercar].acceleration              = 0.35f;
	specs[Supercar].accelerationReverse       = 0.1f;
	specs[Supercar].brakes                    = 0.7f;
	specs[Supercar].topSpeed                  = 19.f;
	specs[Supercar].topSpeedReverse           = -5.f;
	specs[Supercar].grip                      = 0.5f;
	specs[Supercar].filename                  = "supercar";
	specs[Supercar].trunkCapacity             = 0;
	specs[Supercar].passangerCapacity         = 1;
}

}
