#include "Town/Cars/TireSkid.hpp"

#include "Engine/Graphics/Sprite.hpp"
#include "Engine/Scene.hpp"
#include "Utility/Assert.hpp"
#include "Utility/Trig.hpp"

namespace sl
{

TireSkid::TireSkid(float x, float y, bool bloodbath)
	:Entity(x, y, 0, 0, "TireSkid")
	,next(nullptr)
	,sprite(nullptr)
	,lifetime(90)
	,bloodbath(bloodbath)
{
}

void TireSkid::join(const TireSkid *newNext)
{
	if (!next)
	{
		next = newNext;
		int gap = 6;
		int skidsNeededToJoin = pointDistance(pos.x, pos.y, next->pos.x, next->pos.y) / gap;
		int dir = pointDirection(pos.x, pos.y, next->pos.x, next->pos.y);
		TireSkid *a = this, *b = this;
		for (int i = 1; i < skidsNeededToJoin + 1; ++i)
		{
			ASSERT(a->sprite == nullptr);
			a->sprite = new Sprite(bloodbath ? "skidmark_bloody.png" : "skidmark.png", 16, 16);
			a->addDrawable(std::unique_ptr<Drawable>(a->sprite));
			a->sprite->rotate(dir);
			a->sprite->setDepth(100);	//	so it appears under everything
			b = new TireSkid(pos.x + lengthdirX(gap * i, dir), pos.y + lengthdirY(gap * i, dir), bloodbath);
			a->next = b;
			a = b;
			scene->addEntityToList(b);
		}
		ASSERT(b->sprite == nullptr);
		b->sprite = new Sprite(bloodbath ? "skidmark_bloody.png" : "skidmark.png", 16, 16);
		b->addDrawable(std::unique_ptr<Drawable>(b->sprite));
		b->sprite->rotate(dir);
		b->sprite->setDepth(100);	//	so it appears under everything
	}
}

const Types::Type TireSkid::getType() const
{
	return Entity::makeType("TireSkid");
}

/*	  private/protected   */
void TireSkid::onUpdate()
{
	if (--lifetime < 0)
	{
		this->destroy();
	}
}

}
