#include "Town/Container.hpp"

#include "Engine/Graphics/Sprite.hpp"
#include "Utility/TypeToString.hpp"

namespace sl
{

DECLARE_TYPE_TO_STRING(Container)

Container::Container(const Vector2f& pos)
	:Entity(pos.x, pos.y, 30, 24, "Container")
	,closed(false)
{
	setClosed(true);
}

const Types::Type Container::getType() const
{
	return Entity::makeType("Container");
}

bool Container::isStatic() const
{
	return true;
}

void Container::serialize(Serialize& out) const
{
	serializeEntity(out);
}

void Container::deserialize(Deserialize& in)
{
	deserializeEntity(in);
}

void Container::setClosed(bool closed)
{
	if (closed != this->closed)
	{
		clearNonSetDrawables();
		addDrawable(std::make_unique<Sprite>(closed ? "dumpster_closed.png" : "dumpster_open.png", 1, 17));
		this->closed = closed;
	}
}

bool Container::hasNPCs() const
{
	return !NPCs.empty();
}

std::shared_ptr<Soul> Container::takeNPC()
{
	if (NPCs.empty())
	{
		return nullptr;
	}
	auto npc = std::move(NPCs.back());
	NPCs.pop_back();
	return npc;
}

void Container::addNPC(std::shared_ptr<Soul> npc)
{
	NPCs.push_back(std::move(npc));
}

void Container::onUpdate()
{
}

} // sl