#include "Town/DeserializationAllocators.hpp"

#include "Engine/Graphics/DrawableCollection.hpp"
#include "Engine/Graphics/Sprite.hpp"
#include "Engine/Graphics/SpriteAnimation.hpp"
#include "Engine/Serialization/DeserializeConstructor.hpp"
#include "Engine/Serialization/Serializable.hpp"
#include "Engine/Door.hpp"
#include "Engine/Game.hpp"
#include "Home/Internet/Computer.hpp"
#include "Home/BuildingLevel.hpp"
#include "Home/CellDoor.hpp"
#include "Home/OpenableDoor.hpp"
#include "Home/PlayerBed.hpp"
#include "Items/ItemPickup.hpp"
#include "NPC/Locators/EntityLocator.hpp"
#include "NPC/Locators/FixedTargetLocator.hpp"
#include "NPC/Bed.hpp"
#include "NPC/NamedEntity.hpp"
#include "NPC/NPC.hpp"
#include "NPC/Region.hpp"
#include "NPC/ThrownItem.hpp"
#include "Player/Player.hpp"
#include "Town/Cars/Car.hpp"
#include "Town/Container.hpp"
#include "Town/LocationMarker.hpp"
#include "Town/RoadSegment.hpp"
#include "Town/Town.hpp"
#include "Town/TrafficLight.hpp"
#include "Town/StaticGeometry.hpp"
#include "Town/World.hpp"
#include "Utility/Memory.hpp"

namespace sl
{

std::map<std::string, std::function<std::unique_ptr<Serializable>()>> deserializationAllocators(Game *game)
{
#define CUSTOM_ALLOC(Type, ...) std::make_pair(#Type, [] { return unique<Type>(__VA_ARGS__); })
#define DESER_ALLOC(Type) std::make_pair(#Type, [] { return unique<Type>(deserializeConstructor); })
	return std::map<std::string, std::function<std::unique_ptr<Serializable>()>>({
		// Entities
		DESER_ALLOC(NPC),
		CUSTOM_ALLOC(Player, 0, 0, nullptr),
		CUSTOM_ALLOC(StaticGeometry, 0, 0, 0, 0, nullptr),
		DESER_ALLOC(Car),
		CUSTOM_ALLOC(Container, Vector2f()),
		DESER_ALLOC(Door),
		CUSTOM_ALLOC(Bed, Rect<int>()),
		DESER_ALLOC(NamedEntity),
		DESER_ALLOC(ThrownItem),
		CUSTOM_ALLOC(LocationMarker, -1, -1, ""),
		CUSTOM_ALLOC(OpenableDoor, -1, -1, OpenableDoor::DoorType::Regular),
		//DESER_ALLOC(CellDoor),
		CUSTOM_ALLOC(PlayerBed, -1, -1),
		CUSTOM_ALLOC(TrafficLight, -1, -1, TrafficLight::Direction::North, false),
		CUSTOM_ALLOC(Computer, -1, -1),
		CUSTOM_ALLOC(Region, Rect<int>(), "", 0),
		DESER_ALLOC(RoadSegment),
		DESER_ALLOC(ItemPickup),

		// Scenes
		std::make_pair("World", [game] {
			return unique<World>(game, game->getWindowWidth(), game->getWindowHeight());
		}),
		std::make_pair("BuildingLevel", [game] {
			return unique<BuildingLevel>(game, 1, 1, 0, 0, Rect<int>(), 1.f, false);
		}),
		std::make_pair("Town", [game] {
			return unique<Town>(game, 1, 1, game->getWindowWidth(), game->getWindowHeight());
		}),

		// Drawables
		CUSTOM_ALLOC(DrawableCollection),
		DESER_ALLOC(Sprite),

		// Locators
		CUSTOM_ALLOC(EntityLocator, ""),
		CUSTOM_ALLOC(FixedTargetLocator, Target())
	});
#undef DEF_ALLOC
}

} // sl