#include "Town/FourWayTrafficController.hpp"

#include "Town/TrafficLight.hpp"

namespace sl
{

FourWayTrafficController::FourWayTrafficController(TrafficLight& north, TrafficLight& east, TrafficLight& west, TrafficLight& south)
	:Entity()
	,north(north)
	,east(east)
	,west(west)
	,south(south)
	,greenTime(90)
	,yellowTime(30)
	,state(0)
	,alarm(greenTime + yellowTime)
{
	// straight north/south
	states.push_back(State(true, false, false, false, false, false, true, false));
	// left north/south
	states.push_back(State(false, true, false, false, false, false, false, true));
	// staight west/east
	states.push_back(State(false, false, true, false, true, false, false, false));
	// left north/south
	states.push_back(State(false, false, false, true, false, true, false, false));
}

const Types::Type FourWayTrafficController::getType() const
{
	return "FourWayTrafficController";
}

void FourWayTrafficController::onUpdate()
{
	--alarm;
	if (alarm == yellowTime)
	{
		const State& currentState = states[state];
		const State& nextState = states[(state + 1) % states.size()];
		// if the next one is green, don't bother going to yellow to avoid
		// a green->yellow->green change because that is fucking stupid
		if (!nextState.northStraight)
		{
			north.setColour(currentState.northStraight ? TrafficLight::Yellow : TrafficLight::Red);
		}
		if (!nextState.northLeft)
		{
			north.setLeftColour(currentState.northLeft ? TrafficLight::Yellow : TrafficLight::Red);
		}
		if (!nextState.eastStraight)
		{
			east.setColour(currentState.eastStraight ? TrafficLight::Yellow : TrafficLight::Red);
		}
		if (!nextState.eastLeft)
		{
			east.setLeftColour(currentState.eastLeft ? TrafficLight::Yellow : TrafficLight::Red);
		}
		if (!nextState.westStraight)
		{
			west.setColour(currentState.westStraight ? TrafficLight::Yellow : TrafficLight::Red);
		}
		if (!nextState.westLeft)
		{
			west.setLeftColour(currentState.westLeft ? TrafficLight::Yellow : TrafficLight::Red);
		}
		if (!nextState.southStraight)
		{
			south.setColour(currentState.southStraight ? TrafficLight::Yellow : TrafficLight::Red);
		}
		if (!nextState.southLeft)
		{
			south.setLeftColour(currentState.southLeft ? TrafficLight::Yellow : TrafficLight::Red);
		}
	}
	else if (alarm == 0)
	{
		state = (state + 1) % states.size();

		const State& currentState = states[state];

		north.setColour(currentState.northStraight ? TrafficLight::Green : TrafficLight::Red);
		north.setLeftColour(currentState.northLeft ? TrafficLight::Green : TrafficLight::Red);
		east.setColour(currentState.eastStraight ? TrafficLight::Green : TrafficLight::Red);
		east.setLeftColour(currentState.eastLeft ? TrafficLight::Green : TrafficLight::Red);
		west.setColour(currentState.westStraight ? TrafficLight::Green : TrafficLight::Red);
		west.setLeftColour(currentState.westLeft ? TrafficLight::Green : TrafficLight::Red);
		south.setColour(currentState.southStraight ? TrafficLight::Green : TrafficLight::Red);
		south.setLeftColour(currentState.southLeft ? TrafficLight::Green : TrafficLight::Red);

		alarm = greenTime + yellowTime;
	}
}

}