#ifndef SL_FOURWAYTRAFFICCONTROLLER_HPP
#define SL_FOURWAYTRAFFICCONTROLLER_HPP

#include "Engine/Entity.hpp"

namespace sl
{

class TrafficLight;

class FourWayTrafficController : public Entity
{
public:
	/**
	 * Constructs a controller for a 4-way traffic light.
	 * @param a One of the lights to control
	 * @param b One of the lights to control
	 * @param c One of the lights to control
	 * @param d One of the lights to control
	 */
	FourWayTrafficController(TrafficLight& north, TrafficLight& east, TrafficLight& west, TrafficLight& south);


	const Types::Type getType() const override;

private:
	void onUpdate() override;

	/**
	 * Represents a given state for all four lights to be in
	 */
	class State
	{
	public:
		State(bool northStraight, bool northLeft, bool eastStraight, bool eastLeft, bool westStraight, bool westLeft, bool southStraight, bool southLeft)
			:northStraight(northStraight)
			,northLeft(northLeft)
			,eastStraight(eastStraight)
			,eastLeft(eastLeft)
			,westStraight(westStraight)
			,westLeft(westLeft)
			,southStraight(southStraight)
			,southLeft(southLeft)
		{
		}

		const bool northStraight;
		const bool northLeft;
		const bool eastStraight;
		const bool eastLeft;
		const bool westStraight;
		const bool westLeft;
		const bool southStraight;
		const bool southLeft;
	};


	//! The controlled lights
	TrafficLight &north;
	TrafficLight &east;
	TrafficLight &west;
	TrafficLight &south;
	//! All states the controller will cycle through
	std::vector<State> states;
	//!	The amount of time the lights should stay green for
	const int greenTime;
	//! The amount of time the lights should stay yellow for (only matters if next state has them red)
	const int yellowTime;
	//! Which state we're currently in (indexed into states)
	int state;
	//! A countdown in frames until we hit 0 then we go to the next state
	int alarm;
};

}

#endif // SL_FOURWAYTRAFFICCONTROLLER_HPP