#ifndef SL_TOWNGEN_BLOCKGENERATOR_HPP
#define SL_TOWNGEN_BLOCKGENERATOR_HPP

#include <utility>
#include <vector>

#include "NPC/Stats/SoulGenerationSettings.hpp"
#include "Town/Generation/BuildingGenerator.hpp"
#include "Town/Generation/FillerPlacement.hpp"
#include "Town/Generation/RequiredBuildings.hpp"
#include "Utility/Rect.hpp"

namespace sl
{
namespace towngen
{

// TODO: should this go here?
enum class BlockType
{
	Forest = 0,
	Residential,
	PoorResidential,
	Commercial,
	School,
	Park // TODO: remove? keep? places 1 per park. this is partially to mask a bug where parks weren't being created which causes scheduling crashes
};

class BlockGenerator
{
public:
	BlockGenerator(BlockOfBuildings::FenceType fenceType, SoulGenerationSettings npcSettings);

	void addFiller(std::unique_ptr<YardBlueprint> filler, FillerPlacement fillerPlacement = FillerPlacement::Above);

	BuildingGenerator& buildings();

	void generate(Random& rng, const Rect<int>& zoneArea, std::vector<BlockOfBuildings>& buildings, std::vector<std::tuple<int, int, int>>& roadSplitLocations, RequiredCountList *requiredBuildings) const;

private:

	FillerPlacement fillerPlacement;
	BuildingGenerator buildingGenerator;
	std::vector<std::pair<FillerPlacement, std::unique_ptr<YardBlueprint>>> fillers;
	BlockOfBuildings::FenceType fenceType;
	SoulGenerationSettings npcSettings;
};

} // towngen
} // sl

#endif // SL_TOWNGEN_BLOCKGENERATOR_HPP
