#include "Town/Generation/BlockGeneratorCreation.hpp"

#include "Town/Generation/ParkingLotGenerator.hpp"
#include "Town/Generation/YardBlueprint.hpp"
#include "Town/Generation/YardObjectPrototype.hpp"
#include "Utility/Memory.hpp"

namespace sl
{
namespace towngen
{

void makeBlockGenerators(std::map<BlockType, BlockGenerator>& generators, RequiredCountList& requiredBuildings, int tileSize)
{
	//-------------------------  Small Garage (Attached)  ------------------------
	/*std::shared_ptr<BuildingBlueprint> smallAttachedGarage(new BuildingBlueprint(BuildingBlueprint::create(BuildingBlueprint::OutsideType::SmallGarage)));
	std::shared_ptr<BuildingAttachment> garbageCan(YardObjectPrototype::create(YardObjectPrototype::Premade::Trashcan));*/
	//----------------------------------------------------------------------------
	//------------------------------  Residential  -------------------------------
	//----------------------------------------------------------------------------
	auto parkFiller = unique<YardBlueprint>("park filler", 8, 8, YardBlueprint::FillType::ParkAbove);
	SoulGenerationSettings residentialNPCSettings;
	residentialNPCSettings
		.addGenePool(DNADatabase::GenePool::Caucasian, 0.35)
		.addGenePool(DNADatabase::GenePool::NordicCaucasian, 0.20)
		.addGenePool(DNADatabase::GenePool::MediterraneanCaucasian, 0.10)
		.addGenePool(DNADatabase::GenePool::Asian, 0.20)
		.addGenePool(DNADatabase::GenePool::Hispanic, 0.01)
		.addGenePool(DNADatabase::GenePool::Tropical, 0.01)
		.addGenePool(DNADatabase::GenePool::Nigger, 0.01)
		.addGenePool(DNADatabase::GenePool::Indian, 0.05)
		.addGenePool(DNADatabase::GenePool::SouthEastAsian, 0.02)
		.addGenePool(DNADatabase::GenePool::Arabic, 0.05);
	BlockGenerator& residential = generators.insert(std::make_pair(BlockType::Residential, BlockGenerator(BlockOfBuildings::FenceType::Wooden, std::move(residentialNPCSettings)))).first->second;
	residential.addFiller(std::move(parkFiller), FillerPlacement::Above);
	//------------------------------  Basic House  ----------------------------
	{
		auto smallAttachedGarage = unique<BuildingBlueprint>(BuildingBlueprint::create(BuildingBlueprint::OutsideType::SmallGarage));
		std::shared_ptr<BuildingAttachment> garbageCan(YardObjectPrototype::create(YardObjectPrototype::Premade::Trashcan));
		auto basicHouse = unique<YardBlueprint>("basic house", 10, 6, YardBlueprint::FillType::LightTrees);

		BuildingBlueprint house(BuildingBlueprint::create(BuildingBlueprint::OutsideType::HouseBasic));
		//ConditionalPrototypeLimit *garageLimit = house.createNewLimit(1);
		//BuildingBlueprint::AttachmentConfig garageConfig(smallAttachedGarage, 0.1f, garageLimit);
		//house.addAttachment(garageConfig, BuildingBlueprint::Left | BuildingBlueprint::Right);
		basicHouse->addBuilding(std::move(house));

		residential.buildings().addBlueprint(BuildingGenerator::YardConfig(std::move(basicHouse)), 2);
	}

	//--------------------------  Bigger Basic House  -------------------------
	{
		auto smallAttachedGarage = unique<BuildingBlueprint>(BuildingBlueprint::create(BuildingBlueprint::OutsideType::SmallGarage));
		std::shared_ptr<BuildingAttachment> garbageCan(YardObjectPrototype::create(YardObjectPrototype::Premade::Trashcan));
		auto biggerBasicHouse = unique<YardBlueprint>("bigger basic house", 10, 6, YardBlueprint::FillType::LightTrees);

		BuildingBlueprint house(BuildingBlueprint::create(BuildingBlueprint::OutsideType::HouseBiggerBasic));
		//ConditionalPrototypeLimit *garageLimit = house.createNewLimit(1);
		//BuildingBlueprint::AttachmentConfig garageConfig(smallAttachedGarage, 0.3f, garageLimit);
		//house.addAttachment(garageConfig, BuildingBlueprint::Left | BuildingBlueprint::Right);
		//house.addAttachment(BuildingBlueprint::AttachmentConfig(garbageCan, 0.4f), BuildingBlueprint::Bottom);
		biggerBasicHouse->addBuilding(std::move(house));

		residential.buildings().addBlueprint(BuildingGenerator::YardConfig(std::move(biggerBasicHouse)), 4);
	}

	//--------------------------  Middle Class House  -------------------------
	{
		auto richHouse = unique<YardBlueprint>("middle class house", 16, 8, YardBlueprint::FillType::LightTrees);
		BuildingBlueprint house(BuildingBlueprint::create(BuildingBlueprint::OutsideType::HouseMiddleClass));
		richHouse->addBuilding(std::move(house));

		residential.buildings().addBlueprint(BuildingGenerator::YardConfig(std::move(richHouse)), 4);
	}

	//--------------------------  Middle Class House  -------------------------
	{
		auto richHouse = unique<YardBlueprint>("middle class house", 16, 8, YardBlueprint::FillType::LightTrees);
		BuildingBlueprint house(BuildingBlueprint::create(BuildingBlueprint::OutsideType::HouseMiddleClassTall));
		richHouse->addBuilding(std::move(house));

		residential.buildings().addBlueprint(BuildingGenerator::YardConfig(std::move(richHouse)), 3);
	}

	////---------------------------------  Park  --------------------------------
	//{
	//	auto park = unique<YardBlueprint>("park", 32, 12, YardBlueprint::FillType::Park);

	//	residential.buildings().addBlueprint(BuildingGenerator::YardConfig(std::move(park)), 1);

	//	requiredBuildings.insert(std::make_pair("park", CreatedRequired(1)));
	//}

	////--------------------------------  School  -------------------------------
	//{
	//	auto school = unique<YardBlueprint>("school", 32, 20, YardBlueprint::FillType::None);
	//	BuildingBlueprint house(BuildingBlueprint::create(BuildingBlueprint::OutsideType::School));
	//	school->addBuilding(std::move(house));

	//	residential.buildings().addBlueprint(BuildingGenerator::YardConfig(std::move(school), 100), 1);

	//	requiredBuildings.insert(std::make_pair("school", CreatedRequired(2)));
	//}

	////----------------------------  School (small) ----------------------------
	//{
	//	auto school = unique<YardBlueprint>("school", 24, 10, YardBlueprint::FillType::None);
	//	BuildingBlueprint house(BuildingBlueprint::create(BuildingBlueprint::OutsideType::SchoolSmall));
	//	school->addBuilding(std::move(house));

	//	residential.buildings().addBlueprint(BuildingGenerator::YardConfig(std::move(school), 100), 1);
	//}

	//----------------------------------------------------------------------------
	//----------------------------  Poor Residential  ----------------------------
	//----------------------------------------------------------------------------
	SoulGenerationSettings poorResidentialNPCSettings;
	poorResidentialNPCSettings
		.addGenePool(DNADatabase::GenePool::Caucasian, 0.14)
		.addGenePool(DNADatabase::GenePool::NordicCaucasian, 0.05)
		.addGenePool(DNADatabase::GenePool::MediterraneanCaucasian, 0.05)
		.addGenePool(DNADatabase::GenePool::Asian, 0.01)
		.addGenePool(DNADatabase::GenePool::Hispanic, 0.2)
		.addGenePool(DNADatabase::GenePool::Tropical, 0.1)
		.addGenePool(DNADatabase::GenePool::Nigger, 0.2)
		.addGenePool(DNADatabase::GenePool::Indian, 0.1)
		.addGenePool(DNADatabase::GenePool::SouthEastAsian, 0.1)
		.addGenePool(DNADatabase::GenePool::Arabic, 0.1);
	BlockGenerator& poorResidential = generators.insert(std::make_pair(BlockType::PoorResidential, BlockGenerator(BlockOfBuildings::FenceType::Chainlink, std::move(poorResidentialNPCSettings)))).first->second;
	poorResidential.addFiller(unique<YardBlueprint>("park filler", 8, 8, YardBlueprint::FillType::ParkAbove), FillerPlacement::Above);
	//------------------------------  Basic House  ----------------------------
	{
		auto smallAttachedGarage = unique<BuildingBlueprint>(BuildingBlueprint::create(BuildingBlueprint::OutsideType::SmallGarage));
		std::shared_ptr<BuildingAttachment> garbageCan(YardObjectPrototype::create(YardObjectPrototype::Premade::Trashcan));
		auto basicHouse = unique<YardBlueprint>("basic house", 10, 6, YardBlueprint::FillType::LightTrees);

		BuildingBlueprint house(BuildingBlueprint::create(BuildingBlueprint::OutsideType::HouseBasic));
		/*	ConditionalPrototypeLimit *garageLimit = house.createNewLimit(1000);
		BuildingBlueprint::AttachmentConfig garageConfig(smallAttachedGarage, 0.92f, garageLimit);
		house.addAttachment(garageConfig, BuildingBlueprint::Left | BuildingBlueprint::Right);*/
		basicHouse->addBuilding(std::move(house));

		poorResidential.buildings().addBlueprint(BuildingGenerator::YardConfig(std::move(basicHouse)), 10);
	}

	//--------------------------  Bigger Basic House  -------------------------
	{
		auto smallAttachedGarage = unique<BuildingBlueprint>(BuildingBlueprint::create(BuildingBlueprint::OutsideType::SmallGarage));
		std::shared_ptr<BuildingAttachment> garbageCan(YardObjectPrototype::create(YardObjectPrototype::Premade::Trashcan));
		auto biggerBasicHouse = unique<YardBlueprint>("bigger basic house", 10, 6, YardBlueprint::FillType::LightTrees);

		BuildingBlueprint house(BuildingBlueprint::create(BuildingBlueprint::OutsideType::HouseBiggerBasic));
		//ConditionalPrototypeLimit *garageLimit = house.createNewLimit(1);
		//BuildingBlueprint::AttachmentConfig garageConfig(smallAttachedGarage, 0.3f, garageLimit);
		//house.addAttachment(garageConfig, BuildingBlueprint::Left | BuildingBlueprint::Right);
		//house.addAttachment(BuildingBlueprint::AttachmentConfig(garbageCan, 0.4f), BuildingBlueprint::Bottom);
		biggerBasicHouse->addBuilding(std::move(house));

		poorResidential.buildings().addBlueprint(BuildingGenerator::YardConfig(std::move(biggerBasicHouse)), 7);
	}

	//---------------------------  Abandonded House  --------------------------
	{
		std::shared_ptr<BuildingBlueprint> smallAttachedGarage = shared<BuildingBlueprint>(BuildingBlueprint::create(BuildingBlueprint::OutsideType::SmallGarage));
		std::shared_ptr<BuildingAttachment> garbageCan(YardObjectPrototype::create(YardObjectPrototype::Premade::Trashcan));
		auto abandonedHouse = unique<YardBlueprint>("abandonded house", 8, 6, YardBlueprint::FillType::LightTrees, 6);

		BuildingBlueprint house(BuildingBlueprint::create(BuildingBlueprint::OutsideType::AbandonedHouse));
		//for (int i = 0; i < 2; ++i)
		//{
		//	house.addAttachment(BuildingBlueprint::AttachmentConfig(garbageCan, 0.5f), BuildingBlueprint::Left | BuildingBlueprint::Right | BuildingBlueprint::Bottom);
		//}
		abandonedHouse->addBuilding(std::move(house));

		poorResidential.buildings().addBlueprint(BuildingGenerator::YardConfig(std::move(abandonedHouse)), 7);
	}

	//---------------------------------  Park  --------------------------------
	{
		auto park = unique<YardBlueprint>("park", 32, 12, YardBlueprint::FillType::Park);

		//poorResidential.buildings().addBlueprint(BuildingGenerator::YardConfig(std::move(park)), 1);
	}



	//-----------------------------  Rich House  ------------------------------
	//residential.addBlueprint(unique<YardBlueprint>(5, 3));


	//-------------------------------------------------------------------------
	//-----------------------------  Commercial  ------------------------------
	//-------------------------------------------------------------------------
	SoulGenerationSettings commercialNPCSettings;
	commercialNPCSettings
		.addGenePool(DNADatabase::GenePool::Caucasian, 0.15)
		.addGenePool(DNADatabase::GenePool::NordicCaucasian, 0.05)
		.addGenePool(DNADatabase::GenePool::MediterraneanCaucasian, 0.10)
		.addGenePool(DNADatabase::GenePool::Asian, 0.15)
		.addGenePool(DNADatabase::GenePool::Hispanic, 0.08)
		.addGenePool(DNADatabase::GenePool::Tropical, 0.05)
		.addGenePool(DNADatabase::GenePool::Nigger, 0.15)
		.addGenePool(DNADatabase::GenePool::Indian, 0.10)
		.addGenePool(DNADatabase::GenePool::SouthEastAsian, 0.10)
		.addGenePool(DNADatabase::GenePool::Arabic, 0.07);
	BlockGenerator& commercial = generators.insert(std::make_pair(BlockType::Commercial, BlockGenerator(BlockOfBuildings::FenceType::Chainlink, std::move(commercialNPCSettings)))).first->second;
	// create parking lot fillers of layers 1 to 6
	for (int layers = 1; layers <= 6; ++layers)
	{
		const int parkingLotHeight = ParkingLotGenerator::getHeight(layers);
		auto parkingLot = unique<YardBlueprint>(std::to_string(layers) + "-layer parking lot", 6, parkingLotHeight, YardBlueprint::FillType::None, parkingLotHeight);
		parkingLot->addContentGenerator(Vector2i(0, 0), unique<ParkingLotGenerator>(layers, false));
		commercial.addFiller(std::move(parkingLot), FillerPlacement::Below);
	}
	// create rear parking lot fillers of layers 1 to 4
	for (int layers = 1; layers <= 2; ++layers)
	{
		const int parkingLotHeight = ParkingLotGenerator::getHeight(layers);
		auto parkingLot = unique<YardBlueprint>(std::to_string(layers) + "-layer rear parking lot", 6, parkingLotHeight, YardBlueprint::FillType::None, parkingLotHeight);
		parkingLot->addContentGenerator(Vector2i(0, 0), unique<ParkingLotGenerator>(layers, true));
		commercial.addFiller(std::move(parkingLot), FillerPlacement::Above);
	}
	//-------------------------------  Loli-Co  -------------------------------
	{
		auto lolico = unique<YardBlueprint>("lolico", 14, 8, YardBlueprint::FillType::None, 10);
		BuildingBlueprint house("buildings/lolicotechoutside2.png", Vector2i(1, 64), Vector2i(176, 113) / tileSize);
		lolico->addBuilding(std::move(house));
		lolico->addObject(StaticEntityPrototype::Dumpster, 0, 2);

		commercial.buildings().addBlueprint(BuildingGenerator::YardConfig(std::move(lolico)), 5);

		requiredBuildings.insert(std::make_pair("lolico", CreatedRequired(1)));
	}

	//-----------------------------  Home Grown  ------------------------------
	{
		auto homeGrown = unique<YardBlueprint>("homegrown", 10, 6, YardBlueprint::FillType::None, 8);
		BuildingBlueprint house("buildings/homegrown.png", Vector2i(1, 45), Vector2i(120, 48) / tileSize);
		homeGrown->addBuilding(std::move(house));
		homeGrown->addObject(StaticEntityPrototype::Dumpster, 0, 1);

		commercial.buildings().addBlueprint(BuildingGenerator::YardConfig(std::move(homeGrown)), 5);

		requiredBuildings.insert(std::make_pair("homegrown", CreatedRequired(1)));
	}

	//-----------------------------  Candy Shop  ------------------------------
	{
		auto candyShop = unique<YardBlueprint>("candyshop", 8, 6, YardBlueprint::FillType::None, 8);
		BuildingBlueprint house(BuildingBlueprint::create(BuildingBlueprint::OutsideType::CandyShop));
		candyShop->addBuilding(std::move(house));

		commercial.buildings().addBlueprint(BuildingGenerator::YardConfig(std::move(candyShop)), 1);

		requiredBuildings.insert(std::make_pair("candyshop", CreatedRequired(1)));
	}

	//----------------------------  Old Apartment  ----------------------------
	{
		std::unique_ptr<YardBlueprint> oldApartment = unique<YardBlueprint>("old apartment", 14, 8);
		BuildingBlueprint house(BuildingBlueprint::create(BuildingBlueprint::OutsideType::OldApartment));
		oldApartment->addBuilding(std::move(house));
		oldApartment->addObject(StaticEntityPrototype::Dumpster, 1, 3);

		commercial.buildings().addBlueprint(BuildingGenerator::YardConfig(std::move(oldApartment)), 25);
	}




	//-------------------------------------------------------------------------
	//-------------------------------  School  --------------------------------
	//-------------------------------------------------------------------------
	BlockGenerator& schoolSettings = generators.insert(std::make_pair(BlockType::School, BlockGenerator(BlockOfBuildings::FenceType::Chainlink, SoulGenerationSettings()))).first->second;

	//--------------------------------  School  -------------------------------
	{
		auto school = unique<YardBlueprint>("school", 32, 28, YardBlueprint::FillType::None);
		BuildingBlueprint house(BuildingBlueprint::create(BuildingBlueprint::OutsideType::School));
		school->addBuilding(std::move(house));

		schoolSettings.buildings().addBlueprint(BuildingGenerator::YardConfig(std::move(school), 100), 1);

		//requiredBuildings.insert(std::make_pair("school", CreatedRequired(2)));
	}

	//----------------------------  School (small) ----------------------------
	{
		auto school = unique<YardBlueprint>("school", 24, 18, YardBlueprint::FillType::None);
		BuildingBlueprint house(BuildingBlueprint::create(BuildingBlueprint::OutsideType::SchoolSmall));
		school->addBuilding(std::move(house));

		schoolSettings.buildings().addBlueprint(BuildingGenerator::YardConfig(std::move(school), 100), 1);
	}


	//-------------------------------------------------------------------------
	//--------------------------------  Park  ---------------------------------
	//-------------------------------------------------------------------------
	BlockGenerator& parkSettings = generators.insert(std::make_pair(BlockType::Park, BlockGenerator(BlockOfBuildings::FenceType::None, SoulGenerationSettings()))).first->second;

	//---------------------------------  Park  --------------------------------
	{
		auto park = unique<YardBlueprint>("park", 16, 12, YardBlueprint::FillType::Park);

		parkSettings.buildings().addBlueprint(BuildingGenerator::YardConfig(std::move(park)), 1);
	}
}

} // towngen
} // sl