#ifndef SL_TOWNGEN_BLOCKOFBUILDINGS_HPP
#define SL_TOWNGEN_BLOCKOFBUILDINGS_HPP

#include <memory>
#include <vector>

#include "Town/Generation/RequiredBuildings.hpp"
#include "Town/Generation/FillerPlacement.hpp"
#include "Town/Generation/StaticEntityPrototype.hpp"
#include "Town/Generation/YardBlueprint.hpp"

namespace sl
{

class MinimapStaticEntityData;
class Random;
class Town;

namespace towngen
{

class BlockOfBuildings
{
public:
	enum class FenceType
	{
		None,
		Wooden,
		Chainlink,
		Hedge
	};

	BlockOfBuildings(int x, int y, FenceType fenceType, const SoulGenerationSettings& npcSettings);

	BlockOfBuildings(BlockOfBuildings&& other);
	// @HACK since std::vector on early versions of GCC doesn't support move semantics on push_back (how fucking useless...)
	BlockOfBuildings(BlockOfBuildings& other);

	BlockOfBuildings& operator=(BlockOfBuildings&& rhs);


	int getX() const;

	int getY() const;

	int buildings() const;

	int getWidth() const;

	int getHeight() const;

	void add(const YardBlueprint& blueprint);

	void setFiller(const YardBlueprint& blueprint, FillerPlacement fillerPlacement);

	void remove(RequiredCountList *required = nullptr);

	void clear(RequiredCountList *required = nullptr);

	void generate(Random& rng, Town& town, Grid<TileType>& tiles, int tileSize, map::MapStaticEntityData& minimapStaticEntityData, bool& isPlayerGenerated);

	const YardBlueprint& last() const;



	/**
	 * Extends the block vertically, giving each house extra yard space
	 * @param rng The random number generator to use
	 * @param height The new height to extend the block by
	 */
	void extendVertically(Random& rng, int height);

	/**
	 * Extends the block horizontally, giving the width to random yards
	 * @param rng The random number generator to use
	 * @param width The width in tiles
	 */
	void extendHorizontally(Random& rng, int width);

	void move(int x, int y);

private:
	//BlockOfBuildings(const BlockOfBuildings&);
	BlockOfBuildings& operator=(const BlockOfBuildings&);
	//class Yard
	//{
	//public:
	//	/**
	//	 * @param blueprint The blueprint of the yard
	//	 */
	//	explicit Yard(YardBlueprint& blueprint);


	//	void generate(const Vector2i& pos, Random& rng, Town& town);

	//	void extend(int left, int right, int top, int bottom);

	//	int getWidth() const;

	//	int getHeight() const;

	//private:
	//	//!The x position of the blueprint within the yard
	//	int x;
	//	//!The y position of the blueprint within the yard
	//	int y;
	//	//!The width of the yard (always >= blueprint's width)
	//	int width;
	//	//!The height of the yard (always >= blueprint's height)
	//	int height;
	//	//!The blueprint of the yard
	//	YardBlueprint *blueprint;
	//};
	StaticEntityPrototype::Type horizFenceType() const;

	StaticEntityPrototype::Type vertFenceType() const;



	int x;
	int y;
	int width;
	int height;
	std::vector<YardBlueprint> blueprints;
	std::unique_ptr<YardBlueprint> filler;
	FillerPlacement fillerPlacement;
	FenceType fenceType;
	const SoulGenerationSettings& npcSettings;


	static const int FENCE_WIDTH = 1;
	static const int FENCE_HEIGHT = 1;
};

} // towngen
} // sl

#endif // SL_TOWNGEN_BLOCKOFBUILDINGS_HPP
