#ifndef SL_TOWNGEN_BUILDINGATTACHMENT_HPP
#define SL_TOWNGEN_BUILDINGATTACHMENT_HPP

#include "Utility/Vector.hpp"

namespace sl
{

namespace map
{
	class MapStaticEntityData;
} // map

class ColorSwapID;
class Random;
class SoulGenerationSettings;
class Town;

namespace towngen
{

class BuildingAttachment
{
public:
	virtual ~BuildingAttachment() {}

	/**
	 * Generate the building attachment in the target town
	 * @param pos The position (in worldspace) of the top left of the attachment
	 * @param rng The random number generator to use (if necessary)
	 * @param town The town to generate the attachment in
	 * @param minimapStaticEntityData for updating the minimap with icons for any static objects placed
	 * @param isPlayerGenerator (@todo pls refactor by adding in an upper limit and having a separate house-type for the player)
	 * @param npcSettings Settings for generating NPCs within the place
	 * @param forceSwap The palette-swap to force (if nullptr, attachments are free to use whatever) (this is to ensure attached buildings share a colour scheme)
	 */
	virtual void generate(const Vector2i& pos, Random& rng, Town& town, map::MapStaticEntityData& minimapStaticEntityData, bool& isPlayerGenerated, const SoulGenerationSettings& npcSettings, const ColorSwapID *forceSwap = nullptr) const = 0;

	/*
	 * @return The width (in worldspace) of the attachment
	 */
	virtual int getWidth() const = 0;

	/*
	 * @return The height (in worldspace) of the attachment
	 */
	virtual int getHeight() const = 0;
};

} // towngen
}// sl

#endif // SL_TOWNGEN_BUILDINGATTACHMENT_HPP