#include "Town/Generation/BuildingGenerator.hpp"

#include "Utility/Assert.hpp"
#include "Utility/Grid.hpp"
#include "Utility/Random.hpp"

#include <algorithm>

#include <iostream>
#include <fstream>

namespace sl
{
namespace towngen
{

BuildingGenerator::BuildingGenerator()
	:minWidth(0)
	,maxWidth(0)
	,minHeight(0)
	,maxHeight(0)
{
}

//void BuildingGenerator::load(const std::string& filename)
//{
//	std::ifstream filein(filename.c_str());
//
//	if (filein.is_open())
//	{
//		int maxWidth = 0;
//		std::vector<std::string> lines;
//
//		maxWidth = 0;
//		while (filein)
//		{
//			lines.push_back(std::string());
//			std::getline(filein, lines.back());
//			if (lines.back().empty())
//			{
//				lines.pop_back();
//			}
//			else if (lines.size() > maxWidth * 2)
//			{
//				maxWidth = lines.size() / 2;
//			}
//		}
//		Grid<unsigned short> grid(maxWidth, lines.size());
//		for (int i = 0; i < lines.size(); ++i)
//		{
//			for (int j = 0; j < lines[i].size(); ++j)
//			{
//				//grid.get(j, i) = (lines[i][j] << 8) | lines[i][++j];
//			}
//		}
//
//		filein.close();
//	}
//	else
//	{
//		std::cerr << "BuildingGenerator: could not load " << filename << std::endl;
//	}
//}

void BuildingGenerator::addBlueprint(YardConfig&& config, int weight)
{
	ASSERT(weight > 0);
	const int yardWidth = config.blueprint->getWidth();
	const int yardHeight = config.blueprint->getHeight();
	if (buildings.empty())
	{
		minWidth = yardWidth;
		maxWidth = yardWidth;
		minHeight = yardHeight;
		maxHeight = yardHeight;
	}
	else
	{
		if (minWidth > yardWidth)
		{
			minWidth = yardWidth;
		}
		if (maxWidth < yardWidth)
		{
			maxWidth = yardWidth;
		}
		if (minHeight > yardHeight)
		{
			minHeight = yardHeight;
		}
		if (maxHeight < yardHeight)
		{
			maxHeight = yardHeight;
		}
	}
	buildings.insert(std::move(config), weight);
}

int BuildingGenerator::getRandomBuildings(Random& random, BlockOfBuildings& output, int maxAllowedWidth, int maxAllowedHeight, RequiredCountList *required) const
{
	const int FENCE_WIDTH = 1;
	const int FENCE_HEIGHT = 1;
	const YardBlueprint* temp;
	int width = FENCE_WIDTH;
	int maxHeightUsed = minHeight;
	ASSERT(maxAllowedWidth == -1 || maxAllowedWidth >= minWidth);
	ASSERT(maxAllowedHeight == -1 || maxAllowedHeight >= minHeight);
	ASSERT(!buildings.empty());

	if (!buildings.empty())
	{
		while (width < maxAllowedWidth - minWidth - FENCE_WIDTH)
		{
			for ( ; ; )
			{
				temp = &(getRandomBuilding(random, maxAllowedWidth - FENCE_WIDTH - width, maxAllowedHeight, required));
				if (maxAllowedHeight == -1 || temp->getHeight() + FENCE_HEIGHT <= maxAllowedHeight)
				{
					output.add(*temp);
					width += temp->getWidth() + FENCE_WIDTH;
					if (temp->getHeight() > maxHeightUsed)
					{
						maxHeightUsed = temp->getHeight();
					}
					break;
				}
			}
		}
		if (width >= maxAllowedWidth)
		{
			width -= output.last().getWidth() + FENCE_WIDTH;
			output.remove(required);
		}
		//  Add the leftover stuff to give the other ones bigger yards
		if (output.buildings() != 0)
		{
			const int widthToAdd = maxAllowedWidth - width;
			if (widthToAdd > 0)
			{
				output.extendHorizontally(random, widthToAdd);
				width += widthToAdd;
			}
			ASSERT(width == maxAllowedWidth);
		}
		else
		{
			// maybe the smallest building could fit...
			// @TODO
		}
	}
	return maxHeightUsed + FENCE_HEIGHT;
}

int BuildingGenerator::getMinWidth() const
{
	return minWidth;
}

int BuildingGenerator::getMaxWidth() const
{
	return maxWidth;
}

int BuildingGenerator::getMinHeight() const
{
	return minHeight;
}

int BuildingGenerator::getMaxHeight() const
{
	return maxHeight;
}

const YardBlueprint& BuildingGenerator::getRandomBuilding(Random& random, int maxAllowedWidth, int maxAllowedHeight, RequiredCountList *required) const
{
	const YardBlueprint *temp = nullptr;
	if (maxAllowedWidth == -1)
	{
		maxAllowedWidth = maxWidth;
	}
	if (maxAllowedHeight == -1)
	{
		maxAllowedHeight = maxHeight;
	}
	ASSERT(maxAllowedWidth == -1 || maxAllowedWidth >= minWidth);
	ASSERT(maxAllowedHeight == -1 || maxAllowedHeight >= minHeight);
	// if we have requirements, finish those first
	if (required && !required->empty())
	{
		const auto req = std::find_if(required->begin(), required->end(),
			[&](RequiredCountList::value_type& p) -> bool
		{
			if (p.second.created >= p.second.required)
			{
				return false;
			}
			const auto yard = std::find_if(buildings.cbegin(), buildings.cend(),
				[&](const ProportionList<YardConfig>::Element& config) -> bool
			{
				const auto& blueprint = *config.data.blueprint;
				return blueprint.getWidth() <= maxAllowedWidth
					&& blueprint.getHeight() <= maxAllowedHeight
					&& blueprint.getName() == p.first;
			});
			if (yard != buildings.cend())
			{
				temp = yard->data.blueprint.get();
				return true;
			}
			return false;
		});
		ASSERT(req == required->end() || temp != nullptr);
	}
	// otherwise take houses randomly
	if (!temp)
	{
		do
		{
			temp = buildings.get(random).blueprint.get();
		}
		while ((maxAllowedWidth != -1 && temp->getWidth() > maxAllowedWidth) ||
			   (maxAllowedHeight != -1 && temp->getHeight() > maxAllowedHeight));
	}
	ASSERT(temp != nullptr);
	if (required)
	{
		(*required)[temp->getName()].created++;
	}
	return *temp;
}

}//towngen namespace
}//sl namespace
