#ifndef SL_TOWNGEN_BUILDINGGENERATOR_HPP
#define SL_TOWNGEN_BUILDINGGENERATOR_HPP

#include <memory>
#include <string>
#include <utility>
#include <vector>

#include "Town/Generation/YardBlueprint.hpp"
#include "Town/Generation/BlockOfBuildings.hpp"
#include "Town/Generation/RequiredBuildings.hpp"
#include "Utility/ProportionList.hpp"

namespace sl
{

class Random;

namespace towngen
{

class BuildingGenerator
{
public:
	class YardConfig
	{
	public:
		YardConfig(std::unique_ptr<YardBlueprint>&& blueprint, int requiredCount = 0, int maxPerBlock = -1)
			:blueprint(std::move(blueprint))
			,requiredCount(requiredCount)
			,maxPerBlock(maxPerBlock)
		{
		}
		YardConfig(YardConfig&& other)
			:blueprint(std::move(other.blueprint))
			,requiredCount(other.requiredCount)
			,maxPerBlock(other.maxPerBlock)
		{
		}

		std::unique_ptr<YardBlueprint> blueprint;
		int requiredCount;
		int maxPerBlock;

		// @hack to compile on GCC
		YardConfig(YardConfig& other)
			:blueprint(std::move(other.blueprint))
			,requiredCount(other.requiredCount)
			,maxPerBlock(other.maxPerBlock)
		{
		}
	private:
		//YardConfig(const YardConfig&);
		YardConfig& operator=(const YardConfig&);
	};


	BuildingGenerator();

	// wtf as this from?
	//void load(const std::string& filename);

	//void clear();

	void addBlueprint(YardConfig&& blueprint, int weight = 1);

	int getRandomBuildings(Random& random, BlockOfBuildings& output, int maxAllowedWidth, int maxAllowedHeight = -1, RequiredCountList *required = nullptr) const;


	int getMinWidth() const;
	int getMaxWidth() const;
	int getMinHeight() const;
	int getMaxHeight() const;


private:

	const YardBlueprint& getRandomBuilding(Random& random, int maxAllowedWidth = -1, int maxAllowedHeight = -1, RequiredCountList *required = nullptr) const;



	ProportionList<YardConfig> buildings;
	int minWidth;
	int maxWidth;
	int minHeight;
	int maxHeight;
};

}//towngen namespace
}//sl namespace

#endif // SL_TOWNGEN_BUILDINGGENERATOR_HPP
