#ifndef SL_TOWNGEN_CONDITIONALPROTOTYPE_HPP
#define SL_TOWNGEN_CONDITIONALPROTOTYPE_HPP

#include <vector>
#include "Town/Generation/EntityPrototype.hpp"
#include "Town/Generation/StaticEntityPrototype.hpp"

namespace sl
{

class Scene;
class Random;

namespace towngen
{

class ConditionalPrototypeLimit;

class ConditionalPrototype
{
public:
	/** 
	 * @param prototype The prototype to bind to this conditional generator
	 * @param limit The spawn limit count to use
	 * @param probability The probability that stuff will generate on each call to generate()
	 */
	ConditionalPrototype(const StaticEntityPrototype& prototype, ConditionalPrototypeLimit *limit, float probability);

	/**
	 * Attempts to generate the bound StaticEntityPrototype
	 * Sucess is dependant on the probability and the spawn limit
	 * @param rng The random generator ro use
	 * @param scene The level to generate into
	 * @param tileSize the width/height of a tile in pixels
	 * @param offset The offset in tiles to create the prototype at
	 */
	void generate(Random& rng, Scene& scene, int tileSize, map::MapStaticEntityData& minimapStaticEntityData, const Vector2i& offset = Vector2i(0, 0));


private:

	StaticEntityPrototype prototype;

	ConditionalPrototypeLimit *limit;

	float probability;
};

}	//	end towngen namespace
}	//	end sl namespace

#endif // SL_TOWNGEN_CONDITIONALPROTOTYPE_HPP