#include "Town/Generation/Interiors/EditingUndo.hpp"

#include "Engine/Entity.hpp"

namespace sl
{
namespace towngen
{

EditingUndo::EditingUndo(const HomeObjectPrototypeInstance& instance, const Rect<int>& bounds)
	:Entity(bounds.left, bounds.top, bounds.width, bounds.height, "EditingUndo")
	,instance(instance)
{
}


void EditingUndo::addEntityToUndo(Entity *entity)
{
	entitiesToDelete.push_back(entity);
}

void EditingUndo::addHeightMapToUndo(HeightMap *map)
{
	heightMapsToDelete.push_back(map);
}

void EditingUndo::addUndoFunc(const std::function<void()>& func)
{
	toExecute.push_back(func);
}

void EditingUndo::undo()
{
	for (Entity *entity : entitiesToDelete)
	{
		entity->destroy();
	}
	entitiesToDelete.clear();
	for (HeightMap *map : heightMapsToDelete)
	{
		delete map;
	}
	heightMapsToDelete.clear();
	for (const auto& func : toExecute)
	{
		func();
	}
	toExecute.clear();
	destroy();
}


const HomeObjectPrototypeInstance& EditingUndo::getInstance() const
{
	return instance;
}

// private
void EditingUndo::onUpdate()
{
}

const Entity::Type EditingUndo::getType() const
{
	return "EditingUndo";
}

} // towngen
} // sl