#include "Town/Generation/Interiors/HomeTileDatabase.hpp"

#include <SFML/Graphics/Rect.hpp>

#include <fstream>

#include "Utility/Assert.hpp"
#include "Utility/JsonUtil.hpp"

namespace sl
{
namespace towngen
{


void HomeTileDatabase::reload()
{
	std::string err;
	json11::Json data = readJsonFile("data/gendata/hometiles.json", err);
	if (err.empty())
	{
		for (const json11::Json& tilesheet : data.array_items())
		{
			const json11::Json::object& sheetVars = tilesheet.object_items();
			const std::string& fname = sheetVars.at("filename").string_value();
			const int tileSize = sheetVars.at("tilesize").int_value();
			const json11::Json::array& tiles = sheetVars.at("tiles").array_items();
			for (const json11::Json& tile : tiles)
			{
				const json11::Json::object& vars = tile.object_items();
				const Vector2i offset(vars.at("x").int_value(), vars.at("y").int_value());
				const std::string& id = vars.at("id").string_value();
				if (!prototypes.insert(std::make_pair(id, Prototype(fname, offset, tileSize))).second)
				{
					ERROR("duplicate tile definition for " + id);
				}
			}
		}
	}
	else
	{
		ERROR(err);
	}
}

const HomeTileDatabase::Prototype* HomeTileDatabase::getPrototype(const std::string& id) const
{
	auto it = prototypes.find(id);
	if (it != prototypes.end())
	{
		return &it->second;
	}
	return nullptr;
}

HomeTileDatabase::Data::const_iterator HomeTileDatabase::begin() const
{
	return prototypes.cbegin();
}

HomeTileDatabase::Data::const_iterator HomeTileDatabase::end() const
{
	return prototypes.cend();
}

int HomeTileDatabase::size() const
{
	return prototypes.size();
}


/*static*/ HomeTileDatabase& HomeTileDatabase::get()
{
	static HomeTileDatabase database;
	return database;
}

HomeTileDatabase::HomeTileDatabase()
{
	reload();
}


//-------------------------------- Prototype ----------------------------------
HomeTileDatabase::Prototype::Prototype(const std::string& fname, const Vector2i& offset, int tileSize)
	:fname(fname)
	,offset(offset)
	,tileSize(tileSize)
{
}

const std::string& HomeTileDatabase::Prototype::getFilename() const
{
	return fname;
}

sf::IntRect HomeTileDatabase::Prototype::getTextureRect() const
{
	return sf::IntRect(offset.x * tileSize, offset.y * tileSize, tileSize, tileSize);
}


} // towngen
} // sl
