#ifndef SL_TOWNGEN_PARKPATHGEN_HPP
#define SL_TOWNGEN_PARKPATHGEN_HPP

#include <vector>

#include "Town/TownTiles.hpp"
#include "Utility/Grid.hpp"
#include "Utility/Rect.hpp"
#include "Utility/Vector.hpp"

namespace sl
{

class Random;
class Town;

namespace towngen
{

class ParkPathGen
{
public:
	enum class DirectionalTendency
	{
		NorthWest,
		NorthEast,
		SouthWest,
		SouthEast,
		DrunkenWalk
	};
	enum class Direction
	{
		North,
		East,
		West,
		South
	};


	ParkPathGen(Rect<int> bounds, Random& rng);


	/**
	 * Adds a walker to the generator
	 */
	void walk(DirectionalTendency dirTend, Direction dir, int x, int y);

	/**
	 * Creates the path around an object (eg Fountain) and sprawls paths from it
	 */
	void sprawlFromFountain(int x, int y);

	/**
	 * Starts the path simulation which ends when all walkers are consumed
	 */
	void generate();


	/**
	 * Adds in an entrance at a random spot on the bottom
	 * @return The entrance create's starting position (in tiles)
	 */
	Vector2i addEntrance();

	Vector2i addTopEntrance();

	/**
	 * Extracts the tiles out of a simulated path generator
	 */
	void extractTiles(Grid<TileType>& grid, Town& town) const;


private:

	class Walker
	{
	public:
		Walker(Grid<bool>& pathGrid, Random& rng, DirectionalTendency dirTend, Direction dir, int x, int y);

		void walk();

		bool isAlive() const;


	private:
		Grid<bool> *pathGrid;
		Random *rng;
		DirectionalTendency tendency;
		Direction direction;
		int x;
		int y;
		bool alive;
	};


	Rect<int> bounds;
	Grid<bool> pathGrid;
	Random& rng;
	std::vector<Walker> walkers;
};

} // towngen
} // sl

#endif // SL_TOWNGEN_PARKPATHGEN_HPP
