#include "Town/Generation/PathGenUtil.hpp"

namespace sl
{
namespace towngen
{

void cleanUpClumps(Grid<bool>& pathGrid)
{
	using namespace dirmasks;

	Grid<uint8_t> masks(pathGrid.getWidth(), pathGrid.getHeight(), 0);

	const int width = pathGrid.getWidth() - 1;
	const int height = pathGrid.getHeight() - 1;
	for (int y = 1; y < height; ++y)
	{
		for (int x = 1; x < width; ++x)
		{
			if (pathGrid.at(x, y))
			{
				uint8_t mask = 0;
				if (pathGrid.at(x - 1, y))
				{
					mask |= west;
				}
				if (pathGrid.at(x + 1, y))
				{
					mask |= east;
				}
				if (pathGrid.at(x, y - 1))
				{
					mask |= north;
				}
				if (pathGrid.at(x, y + 1))
				{
					mask |= south;
				}
				masks.at(x, y) = mask;
			}
		}
	}
	for (int y = 1; y < height; ++y)
	{
		for (int x = 1; x < width; ++x)
		{
			if ((masks.at(x, y) == eastwestsouth && masks.at(x, y + 1) == northeastwest) ||
				(masks.at(x, y) == northeastsouth && masks.at(x + 1, y) == northsouthwest) ||
				(masks.at(x, y) == northeastwestsouth && ((masks.at(x + 1, y + 1) && masks.at(x - 1, y - 1)) || (masks.at(x + 1, y - 1) && masks.at(x - 1, y + 1)))))
			{
				pathGrid.at(x, y) = false;
			}
		}
	}
}

void extractParkPathTiles(Grid<TileType>& output, const Vector2i& offset, const Grid<bool>& source, const Rect<int>& sourceRegion)
{
	using namespace dirmasks;

	for (int y = sourceRegion.top; y < sourceRegion.bottom(); ++y)
	{
		for (int x = sourceRegion.left; x < sourceRegion.right(); ++x)
		{
			if (source.at(x, y))
			{
				uint8_t mask = 0;
				if (x > 0 && source.at(x - 1, y))
				{
					mask |= west;
				}
				if (x < source.getWidth() - 1 && source.at(x + 1, y))
				{
					mask |= east;
				}
				if (y > 0 && source.at(x, y - 1))
				{
					mask |= north;
				}
				if (y < source.getHeight() - 1 && source.at(x, y + 1))
				{
					mask |= south;
				}

				TileType& tile = output.at(offset.x + x - sourceRegion.left, offset.y + y - sourceRegion.top);
				TileType before = tile;
				switch (mask)
				{
				case north:
					tile = ParkPathN;
					break;
				case east:
					tile = ParkPathE;
					break;
				case west:
					tile = ParkPathW;
					break;
				case south:
					tile = ParkPathS;
					break;
				case northsouth:
					tile = ParkPathNS;
					break;
				case eastwest:
					tile = ParkPathEW;
					break;
				case northeast:
					tile = ParkPathNE;
					break;
				case northwest:
					tile = ParkPathNW;
					break;
				case eastsouth:
					tile = ParkPathES;
					break;
				case westsouth:
					tile = ParkPathWS;
					break;
				case northeastwest:
					tile = ParkPathNEW;
					break;
				case northsouthwest:
					tile = ParkPathNWS;
					break;
				case northeastsouth:
					tile = ParkPathNES;
					break;
				case eastwestsouth:
					tile = ParkPathEWS;
					break;
				case northeastwestsouth:
					tile = ParkPathNEWS;
					break;
				}
			}
			//grid.at(bounds.left + x - 1, bounds.top + y - 1) = TileType::ParkPathNEWS;
		}
	}
}

void extractFixedPathTiles(Grid<TileType>& output, TileType tile, const Vector2i& offset, const Grid<bool>& source, const Rect<int>& sourceRegion)
{
	for (int y = sourceRegion.top; y < sourceRegion.bottom(); ++y)
	{
		for (int x = sourceRegion.left; x < sourceRegion.right(); ++x)
		{
			if (source.at(x, y))
			{
				output.at(offset.x + x - sourceRegion.left, offset.y + y - sourceRegion.top) = tile;
			}
		}
	}
}

} // towngen
} // sl