#ifndef SL_TOWNGEN_PATHGENUTIL_HPP
#define SL_TOWNGEN_PATHGENUTIL_HPP

#include "Town/TownTiles.hpp"
#include "Utility/Grid.hpp"
#include "Utility/Rect.hpp"

#include <cstdint>

namespace sl
{
namespace towngen
{

namespace dirmasks
{
static constexpr uint8_t north = 1;
static constexpr uint8_t east = 1 << 1;
static constexpr uint8_t west = 1 << 2;
static constexpr uint8_t south = 1 << 3;
static constexpr uint8_t northsouth = north | south;
static constexpr uint8_t eastwest = east | west;
static constexpr uint8_t northeast = north | east;
static constexpr uint8_t northwest = north | west;
static constexpr uint8_t eastsouth = east | south;
static constexpr uint8_t westsouth = west | south;
static constexpr uint8_t northeastwest = north | east | west;
static constexpr uint8_t northsouthwest = north | south | west;
static constexpr uint8_t northeastsouth = north | east | south;
static constexpr uint8_t eastwestsouth = east | west | south;
static constexpr uint8_t northeastwestsouth = north | east | west | south;
} // dirmasks

/**
 * Does some post-processing after generation and removes ugly clumps of paths
 */
extern void cleanUpClumps(Grid<bool>& pathGrid);

/**
 * Using source as where the paths are, sets tiles in output with positions offset from offset. ie pos(output) = pos(source) + offset
 * @param output Tile output
 * @param offset The offset to use when indexing into output
 * @param source path structure (true = path, false = no path)
 * @param sourceRegion The area within source to generate from
 */
extern void extractParkPathTiles(Grid<TileType>& output, const Vector2i& offset, const Grid<bool>& source, const Rect<int>& sourceRegion);

extern void extractFixedPathTiles(Grid<TileType>& output, TileType tile, const Vector2i& offset, const Grid<bool>& source, const Rect<int>& sourceRegion);

} // towngen
} // sl

#endif // SL_TOWNGEN_PATHGENUTIL_HPP