#ifndef SL_TOWNGEN_ROADVERTEX_HPP
#define SL_TOWNGEN_ROADVERTEX_HPP

#include "Pathfinding/NodeID.hpp"
#include "Town/Generation/TownGenCommon.hpp"

namespace sl
{

class Scene;
class Town;

namespace towngen
{

class RoadVertex
{
public:
	RoadVertex(int x, int y)
		:north(nullptr)
		,east(nullptr)
		,west(nullptr)
		,south(nullptr)
		,x(x)
		,y(y)
	{
	}

	void buildPathing(Town& town) const;

	RoadVertex *north;
	RoadVertex *east;
	RoadVertex *west;
	RoadVertex *south;

private:
	//         |  nl   nr  |
	//_________| = = = = = |________
	//         = nwo   neo =
	//     wu  =   nw ne   = eu
	//     wd  =   sw se   = ed
	//_________= swo   seo =________
	//         | = = = = = |
	//         |  sl  sr   |
	pf::NodeID sr(Scene *town) const;
	pf::NodeID sl(Scene *town) const;
	pf::NodeID nr(Scene *town) const;
	pf::NodeID nl(Scene *town) const;
	pf::NodeID eu(Scene *town) const;
	pf::NodeID ed(Scene *town) const;
	pf::NodeID wu(Scene *town) const;
	pf::NodeID wd(Scene *town) const;
	pf::NodeID nw(Scene *town) const;
	pf::NodeID ne(Scene *town) const;
	pf::NodeID sw(Scene *town) const;
	pf::NodeID se(Scene *town) const;
	pf::NodeID nwo(Scene *town) const;
	pf::NodeID neo(Scene *town) const;
	pf::NodeID swo(Scene *town) const;
	pf::NodeID seo(Scene *town) const;


	int x;
	int y;
};

} // towngen
} // sl

#endif // SL_TOWNGEN_ROADVERTEX_HPP