#include "Town/Generation/StaticEntityPrototype.hpp"


#include "Engine/Graphics/Sprite.hpp"
#include "Town/Container.hpp"
#include "Town/StaticGeometry.hpp"
#include "Town/Map/MapStaticEntityData.hpp"
#include "Engine/Scene.hpp"
#include "Utility/Assert.hpp"
#include "Utility/Memory.hpp"
#include "Utility/Random.hpp"

namespace sl 
{
namespace towngen
{

StaticEntityPrototype::StaticEntityPrototype(int x, int y, Type type)
	:x(x)
	,y(y)
	,type(type)
{
}

void StaticEntityPrototype::generate(Random& rng, Scene& scene, int tileSize, map::MapStaticEntityData& minimapStaticEntityData, const Vector2i& offset) const
{
	const float sx = (x + offset.x) * tileSize;
	const float sy = (y + offset.y) * tileSize;
	switch (type)
	{
		case Tree:
		{
			std::stringstream fname;
			fname << "tree" /*<< rng.random(4)*/ << ".png";
			// this temporary is to avoid an internal compiler error on MSVC++ (2015)
			const std::string fnameStr = fname.str();
			const int w = 10;
			const int h = 10;
			const int sw = 48;
			const int sh = 64;
			const int xr = 6;
			const int yr = 6;
			const int xo = (tileSize - w - xr) / 2;
			const int yo = (tileSize - h - yr) / 2;
			const int treeX = (x + offset.x) * tileSize + xo + xr / 2 - Random::get().random(xr);
			const int treeY = (y + offset.y) * tileSize + yo + yr / 2 - Random::get().random(yr);
			scene.addImmediately(new StaticGeometry(treeX, treeY, w, h, unique<Sprite>(fnameStr.c_str(), 19, sh - h)), StorageType::InGridNoUpdate);
			minimapStaticEntityData.set(x + offset.x, y + offset.y, map::MapStaticEntityData::StaticEntity::Tree);
		}
		break;
		case FenceH:
		{
			scene.addImmediately(new StaticGeometry(sx - 8, sy + 10, 16, 4, unique<Sprite>("fence_horiz.png", 0, 12)), StorageType::InGridNoUpdate);
			minimapStaticEntityData.set(x + offset.x, y + offset.y, map::MapStaticEntityData::StaticEntity::Fence);
		}
		break;
		case FenceV:
		{
			scene.addImmediately(new StaticGeometry(sx + 6, sy, 4, 16, unique<Sprite>("fence_vert.png", 6, 16)), StorageType::InGridNoUpdate);
			minimapStaticEntityData.set(x + offset.x, y + offset.y, map::MapStaticEntityData::StaticEntity::Fence);
		}
		break;
		case ChainLinkFenceH:
		{
			scene.addImmediately(new StaticGeometry(sx - 8, sy + 10, 16, 4, unique<Sprite>("chainlink_fence_horiz.png", 0, 12)), StorageType::InGridNoUpdate);
			minimapStaticEntityData.set(x + offset.x, y + offset.y, map::MapStaticEntityData::StaticEntity::Fence);
		}
		break;
		case ChainLinkFenceV:
		{
			scene.addImmediately(new StaticGeometry(sx + 6, sy, 4, 16, unique<Sprite>("chainlink_fence_vert.png", 6, 16)), StorageType::InGridNoUpdate);
			minimapStaticEntityData.set(x + offset.x, y + offset.y, map::MapStaticEntityData::StaticEntity::Fence);
		}
		break;
		case Dumpster:
		{
			scene.addImmediately(new Container(Vector2f(sx + 1, sy)), StorageType::InGridNoUpdate);
			// TODO: make the type system hierarchial...
			scene.addImmediately(new StaticGeometry(sx, sy, 32, 16, nullptr), StorageType::InGridNoUpdate);
			minimapStaticEntityData.set(x + offset.x, y + offset.y, map::MapStaticEntityData::StaticEntity::Fence);
			minimapStaticEntityData.set(x + offset.x + 1, y + offset.y, map::MapStaticEntityData::StaticEntity::Fence);
		}
		break;
		default:
		{
			ERROR("Unhandled Type");
		}
		break;
	}
}

StaticEntityPrototype::Type StaticEntityPrototype::getType() const
{
	return type;
}

int StaticEntityPrototype::getX() const
{
	return x;
}

int StaticEntityPrototype::getY() const
{
	return y;
}

static const Vector2i sizes[6] = {
	Vector2i(1, 1),
	Vector2i(1, 1),
	Vector2i(1, 1),
	Vector2i(1, 1),
	Vector2i(1, 1),
	Vector2i(2, 1)
};

Rect<int> StaticEntityPrototype::box() const
{
	return Rect<int>(x, y, sizes[type].x, sizes[type].y);
}

/*static*/const Vector2i& StaticEntityPrototype::size(Type type)
{
	return sizes[type];
}

} // towngen
} // sl
