#include "Town/Generation/YardObjectPrototype.hpp"

#include "Engine/Graphics/Sprite.hpp"
#include "Town/StaticGeometry.hpp"
#include "Town/Town.hpp"

namespace sl
{
namespace towngen
{

YardObjectPrototype::YardObjectPrototype(const Rect<int> bounds, const std::string& fname, const Vector2i& imageOrigin)
	:bounds(bounds)
	,fname(fname)
	,imageOrigin(imageOrigin)
{
}


void YardObjectPrototype::generate(const Vector2i& pos, Random& rng, Town& town, map::MapStaticEntityData& minimapStaticEntityData, bool& isPlayerGenerated, const SoulGenerationSettings& npcSettings, const ColorSwapID *forceSwap) const
{
	auto sprite = unique<Sprite>(fname.c_str(), forceSwap ? *forceSwap : ColorSwapID(), imageOrigin.x, imageOrigin.y);
	StaticGeometry *obj = new StaticGeometry(pos.x + bounds.left, pos.y + bounds.top, bounds.width, bounds.height, std::move(sprite));
	town.addEntityToGrid(obj);
}

int YardObjectPrototype::getWidth() const
{
	return bounds.width;
}

int YardObjectPrototype::getHeight() const
{
	return bounds.height;
}

/*static*/ std::unique_ptr<YardObjectPrototype> YardObjectPrototype::create(Premade config)
{
	switch (config)
	{
		case Premade::Trashcan:
			return unique<YardObjectPrototype>(Rect<int>(14, 14), "trashcan.png", Vector2i(1, 10));
	}
	ERROR("not implemented");
	return nullptr;
}

} // towngen
} // sl
