#include "Town/LoadBasedFourWayTrafficController.hpp"

#include <vector>

#include "Town/TrafficLight.hpp"
#include "Utility/Random.hpp"

// todo: replace with constexpr function when you upgrade VS to 2013
#define straight(d) (2 * (d))
#define left(d) (2 * (d) + 1)

namespace sl
{

//int straight(TrafficLight::Direction direction)
//{
//	return 2 * direction;
//}
//
//int left(TrafficLight::Direction direction)
//{
//	return 2 * direction + 1;
//}

const int LoadBasedFourWayTrafficController::conflicts[8][4] = {
	// straight(TrafficLight::North)
	{
		straight(TrafficLight::East),
		left(TrafficLight::South),
		left(TrafficLight::West),
		straight(TrafficLight::West)
	},
	// left(TrafficLight::North)
	{
		left(TrafficLight::West),
		straight(TrafficLight::East),
		left(TrafficLight::East),
		straight(TrafficLight::South)
	},
	// straight(TrafficLight::East)
	{
		straight(TrafficLight::South),
		left(TrafficLight::West),
		left(TrafficLight::North),
		straight(TrafficLight::North)
	},
	// left(TrafficLight::East)
	{
		straight(TrafficLight::South),
		left(TrafficLight::North),
		left(TrafficLight::South),
		straight(TrafficLight::West)
	},
	// straight(TrafficLight::West)
	{
		straight(TrafficLight::North),
		left(TrafficLight::East),
		left(TrafficLight::South),
		straight(TrafficLight::South)
	},
	// left(TrafficLight::West)
	{
		straight(TrafficLight::North),
		left(TrafficLight::South),
		left(TrafficLight::North),
		straight(TrafficLight::East)
	},
	// straight(TrafficLight::South)
	{
		straight(TrafficLight::West),
		left(TrafficLight::North),
		left(TrafficLight::East),
		straight(TrafficLight::East)
	},
	// left(TrafficLight::South)
	{
		straight(TrafficLight::West),
		left(TrafficLight::East),
		left(TrafficLight::West),
		straight(TrafficLight::North)
	}
};

// since this array up there depends on a specific ordering so we need to check no one fucked up the enum order or straight()/left() something
static_assert(straight(TrafficLight::North) == 0, "the above conflict array probably has issues");
static_assert(left(TrafficLight::North)     == 1, "the above conflict array probably has issues");
static_assert(straight(TrafficLight::East)  == 2, "the above conflict array probably has issues");
static_assert(left(TrafficLight::East)      == 3, "the above conflict array probably has issues");
static_assert(straight(TrafficLight::West)  == 4, "the above conflict array probably has issues");
static_assert(left(TrafficLight::West)      == 5, "the above conflict array probably has issues");
static_assert(straight(TrafficLight::South) == 6, "the above conflict array probably has issues");
static_assert(left(TrafficLight::South)     == 7, "the above conflict array probably has issues");

bool greenStraight(unsigned char flag, TrafficLight::Direction direction)
{
	return (flag & (1 << straight(direction))) != 0;
}

bool greenLeft(unsigned char flag, TrafficLight::Direction direction)
{
	return (flag & (1 << left(direction))) != 0;
}

LoadBasedFourWayTrafficController::LoadBasedFourWayTrafficController(TrafficLight *north, TrafficLight *east, TrafficLight *west, TrafficLight *south)
	:Entity()
	,greenTime(90)
	,yellowTime(30)
	,state(0)
	,alarm(greenTime + yellowTime)
	,bestFlag(0)
{
	ASSERT(north != nullptr);
	ASSERT(east != nullptr);
	ASSERT(west != nullptr);
	ASSERT(south  != nullptr);
	lights[0] = north;
	lights[1] = east;
	lights[2] = west;
	lights[3] = south;
}


const Types::Type LoadBasedFourWayTrafficController::getType() const
{
	return Entity::makeType("LoadBasedFourWayTrafficController");
}

/*					private						*/
void LoadBasedFourWayTrafficController::onUpdate()
{
	--alarm;
	if (alarm == yellowTime)
	{
		const unsigned char lastFlag = bestFlag;
		computeBestFlag();
		for (int i = 0; i < 4; ++i)
		{
			const TrafficLight::Direction dir = static_cast<TrafficLight::Direction>(i);
			// only change to yellow if the next one is red and the current is green
			if (greenStraight(lastFlag, dir) && !greenStraight(bestFlag, dir))
			{
				lights[i]->setColour(TrafficLight::Yellow);
			}
			if (greenLeft(lastFlag, dir) && !greenLeft(bestFlag, dir))
			{
				lights[i]->setLeftColour(TrafficLight::Yellow);
			}
		}
	}
	else if (alarm == 0)
	{	
		// assign results of config to all traffic lights
		for (int i = 0; i < 4; ++i)
		{
			const TrafficLight::Direction dir = static_cast<TrafficLight::Direction>(i);
			lights[i]->setColour(greenStraight(bestFlag, dir) ? TrafficLight::Green : TrafficLight::Red);
			lights[i]->setLeftColour(greenLeft(bestFlag, dir) ? TrafficLight::Green : TrafficLight::Red);
		}
		
		// reset timer
		alarm = greenTime + yellowTime;
	}
}

void LoadBasedFourWayTrafficController::computeBestFlag()
{
	// compute best config to maximize traffic flow
	bestFlag = 0;
	int bestValue = 0;
	bool visited[8];
	std::vector<unsigned char> bestFlags;
	for (unsigned char flag = 1; ; ++flag)
	{
		bool isValid = true;
		for (bool& b : visited)
		{
			b = false;
		}
		for (int i = 0; i < 8 && isValid; ++i)
		{
			if (flag && (1 << i) != 0)
			{
				visited[i] = true;
				for (int j = 0; j < 4; ++j)
				{
					if (visited[conflicts[i][j]])
					{
						isValid = false;
						break;
					}
				}
			}
		}
		int value = 0;
		for (const TrafficLight *light : lights)
		{
			value += light->getStraightLoad();
			value += light->getLeftLoad();
		}
		
		if (isValid && value >= bestValue)
		{
			bestFlags.push_back(flag);
			bestValue = value;
		}

		// fuck overflow
		if (flag == 255)
		{
			break;
		}
	}
	// pick a random best flag if multiple same-weight ones are found
	if (!bestFlags.empty())
	{
		bestFlag = bestFlags[Random::get().random(bestFlags.size())];
	}
}

} // sl
