#include "Town/LocationMarker.hpp"

#include <memory>

#include "Engine/Graphics/Sprite.hpp"


namespace sl
{

LocationMarker::LocationMarker(int x, int y, const Entity::Type& markerName)
	:LocationMarker(x, y, markerName, "")
{
}

LocationMarker::LocationMarker(int x, int y, const Entity::Type& markerName, const std::string& fname)
	:Entity(x, y, 1, 1, markerName)
	,markerName(markerName)
	,fname(fname)
	,icon(getPos(), sf::Color::Transparent)
{
	initIcon();
}


const Entity::Type LocationMarker::getType() const
{
	return markerName;
}

map::Icon& LocationMarker::editIcon()
{
	return icon;
}


// private
void LocationMarker::initIcon()
{
	icon.setLabel(markerName);
	if (!fname.empty())
	{
		icon.setDetailedIcon(map::LOD::Minimap, std::make_unique<Sprite>(("map/" + fname + "_icon_mm.png").c_str(), 5, 5));
		icon.setDetailedIcon(map::LOD::Map, std::make_unique<Sprite>(("map/" + fname + "_icon.png").c_str(), 8, 8));
	}
}

void LocationMarker::onUpdate()
{
}

void LocationMarker::onEnter()
{
	icon.setScene(scene);
}

void LocationMarker::deserialize(Deserialize& in)
{
	deserializeEntity(in);
	LOADVAR(s, markerName);
	LOADVAR(s, fname);
	initIcon();
	forceTypeChangeForDeserialization(in, markerName);
}

void LocationMarker::serialize(Serialize& out) const
{
	serializeEntity(out);
	SAVEVAR(s, markerName);
	SAVEVAR(s, fname);
}

} // sl
