#include "Town/Map/Icon.hpp"

#include <SFML/Graphics/RenderTarget.hpp>
#include <SFML/Graphics/Sprite.hpp>
#include <SFML/Graphics/Texture.hpp>

#include "Engine/Scene.hpp"
#include "Town/Map/Map.hpp"
#include "Utility/Vector.hpp"

namespace sl
{
namespace map
{

Icon::Icon(const Vector2f& followPos, sf::Color color, Scene *scene)
	:scene(nullptr)
	,followPos(followPos)
	,visible(true)
	,onScreen(false)
	,link(*this)
{
	if (scene)
	{
		setScene(scene);
	}

	dot.setFillColor(color);
	dot.setSize(sf::Vector2f(1.f, 1.f));
	//dot.setOrigin(1, 1);
}


void Icon::show()
{
	visible = true;
}

void Icon::hide()
{
	visible = false;
}

Icon& Icon::setScene(Scene *scene)
{
	this->scene = scene;
	if (scene && scene->getMap())
	{
		scene->getMap()->registerIcon(*this);
	}
	return *this;
}

Icon& Icon::setColor(const sf::Color& color)
{
	dot.setFillColor(color);
	return *this;
}


Icon& Icon::setDetailedIcon(LOD lod, std::unique_ptr<Sprite> icon)
{
	icons[static_cast<int>(lod)] = std::move(icon);
	return *this;
}

Icon& Icon::setLabel(std::string label)
{
	text = std::move(label);
	return *this;
}

const Vector2f& Icon::getPos() const
{
	return followPos;
}

const std::string& Icon::getLabel() const
{
	return text;
}


/*			private				*/

void Icon::update(const Rect<float>& mapGUIRegion, const Rect<float>& visibleRegion, float zoom, int tileSize)
{
	const Vector2f posMapSpace = followPos / tileSize;
	const Vector2f posGUISpace = visibleRegion.mapTo(posMapSpace, mapGUIRegion);

	dot.setPosition(posGUISpace.x, posGUISpace.y);
	for (auto& icon : icons)
	{
		if (icon)
		{
			icon->update(posGUISpace.x, posGUISpace.y);
		}
	}
	onScreen = visibleRegion.containsPoint(posMapSpace);
}

void Icon::draw(sf::RenderTarget& target, LOD lod) const
{
	if (visible && onScreen)
	{
		const auto& detailedIcon = icons[static_cast<int>(lod)];
		if (detailedIcon)
		{
			detailedIcon->draw(target);
		}
		else
		{
			target.draw(dot);
		}
	}
}

} // map
} // sl
