#ifndef SL_MAP_ICON_HPP
#define SL_MAP_ICON_HPP

#include <SFML/Graphics/RectangleShape.hpp>

#include <memory>

#include "Engine/Graphics/Sprite.hpp"
#include "Town/Map/LOD.hpp"
#include "Utility/IntrusiveList.hpp"
#include "Utility/Vector.hpp"

namespace sf
{
class RenderTarget;
class Sprite;
} // sf

namespace sl
{

class Scene;

namespace map
{

class Icon
{
public:
	
	Icon(const Vector2f& followPos, sf::Color color = sf::Color::White, Scene *scene = nullptr);

	void show();

	void hide();

	Icon& setScene(Scene *scene);

	Icon& setColor(const sf::Color& color);

	Icon& setDetailedIcon(LOD lod, std::unique_ptr<Sprite> icon);

	Icon& setLabel(std::string label);

	const Vector2f& getPos() const;

	const std::string& getLabel() const;
	
private:
	// Called by Map
	/**
	 * Updates an icon relative to its map.
	 * @param mapGUIpos The top left position (in screen/GUI coordinates) of the map / area it occupies
	 * @param visibleRegion The area rendered by the map In MAPSPACE coordinates
	 * @param zoom The zoom level of the map. 1x = 1tile = 1pixel, 2x = 1tile = 2px, etc
	 * @param tileSize How many pixels (worldspace coords) are in a tile
	 */
	void update(const Rect<float>& mapGUIRegion, const Rect<float>& visibleRegion, float zoom, int tileSize);

	void draw(sf::RenderTarget& target, LOD lod) const;

	Scene *scene;
	const Vector2f& followPos;
	sf::RectangleShape dot;
	std::unique_ptr<Sprite> icons[3];
	bool visible;
	bool onScreen;
	IntrusiveListNode<Icon> link;
	std::string text;

	friend class Map; // for  update() mostly
};

} // map
} // sl

#endif // SL_MAP_ICON_HPP