#include "Town/Map/Map.hpp"

#include "Town/Generation/TownGenMinimap.hpp"
#include "Town/Map/Icon.hpp"
#include "Utility/Trig.hpp"

namespace sl
{
namespace map
{

const int rimThickness = 1;

Map::Map(/*const MapData& tiles, */int tileSize, const Vector2i& minimapSize, const Vector2i& mapSize)
	:minimapBounds(minimapSize.x, minimapSize.y)
	,mapBounds(mapSize.x, mapSize.y)
	,drawableBounds(minimapBounds)
	,tileSize(tileSize)
	,lod(LOD::Minimap)
{
	//setData(tiles);
	//towngen::createMapTexture(tiles).saveToFile("minimap_output.png");
	sprite.setColor(sf::Color(255, 255, 255, 164));;
}

void Map::update(int x, int y)
{
	// @todo figure out why -1 is needed
	sprite.setPosition(x + rimThickness, y + rimThickness - 1);
	//getBounds().left = x;
	//getBounds().top = y;
	drawableBounds.left = x;
	drawableBounds.top = y;
	drawableBounds.width = getBounds().width * getZoomLevel();
	drawableBounds.height = getBounds().height * getZoomLevel();
	for (Icon& icon : icons)
	{
		icon.update(drawableBounds.to<float>(), visibleRegion(), getZoomLevel(), tileSize);
	}
}

const Rect<int>& Map::getCurrentBounds() const
{
	return drawableBounds;
}

void Map::draw(sf::RenderTarget& target) const
{
	target.draw(sprite);
	for (const Icon& icon : icons)
	{
		icon.draw(target, lod);
	}
}

void Map::setPositionInMap(float x, float y)
{
	centerPos.x = x;
	centerPos.y = y;
	// @todo figure out why + 1 is needed
	// @todo @hack
	// TODO: stop at edges?
	//if (lod == LOD::Minimap)
	//{
	const Rect<float> visible = visibleRegion();// *(float)getZoomLevel();
	sprite.setTextureRect(sf::IntRect(visible.left, visible.top, visible.width, visible.height));
	//}
	//else
	//{
	//	sprite.setTextureRect(sf::IntRect(0, 0, 319, 191));
	//}
}

void Map::registerIcon(Icon& icon)
{
	icons.insert(icon.link);
}

void Map::setData(const MapData& mapData)
{
	setData(towngen::createMapTexture(mapData));
}

void Map::setData(const sf::Image& raw)
{
	texture.loadFromImage(raw);
	sprite.setTexture(texture);
}

sf::Image Map::makeImage() const
{
	return texture.copyToImage();
}

void Map::setLOD(LOD lod)
{
	this->lod = lod;
	const float scale = getZoomLevel();
	sprite.setScale(scale, scale);
}

LOD Map::getLOD() const
{
	return lod;
}

Icon* Map::getIconAt(const Vector2f& posMapSpace)
{
	for (Icon& icon : icons)
	{
		// @todo variable sized icons
		const Vector2f iconPosMapSpace = icon.getPos() / tileSize;
		if (pointDistance(posMapSpace, iconPosMapSpace) <= 4.f)
		{
			return &icon;
		}
	}
	return nullptr;
}

Vector2f Map::worldspaceFromMapspace(const Vector2f& pos) const
{
	return pos * tileSize;
}

Vector2f Map::mapSpaceFromDrawableSpace(const Vector2f& posDrawable) const
{
	return drawableBounds.to<float>().mapTo(posDrawable, visibleRegion());
	//const Vector2f posMap = (posDrawable - drawableBounds.topLeft()) / getZoomLevel();
	//return posMap + visibleRegion().topLeft();
	//return (1.f / getZoomLevel()) *
	//	getBounds().to<float>().mapTo(
	//		pos - drawableBounds.topLeft().to<float>(),
	//		visibleRegion() * (float)getZoomLevel());
}

Rect<float> Map::visibleRegion() const
{
	Rect<float> mapBounds = getBounds().to<float>();
	//mapBounds.left += (texture.getSize().x - (mapBounds.size().x / getZoomLevel())) * centerPos.x;
	//mapBounds.top  += (texture.getSize().y - (mapBounds.size().y / getZoomLevel())) * centerPos.y;
	mapBounds.left += (centerPos.x * texture.getSize().x) - (mapBounds.size().x / 2.f);
	mapBounds.top  += (centerPos.y * texture.getSize().y) - (mapBounds.size().y / 2.f);
	return mapBounds;
	//return Rect<float>(
	//	centerPos.x - mapBounds.width / 2 + rimThickness,
	//	centerPos.y - mapBounds.height / 2 + rimThickness,
	//	mapBounds.width - 2 * rimThickness + 1,
	//	mapBounds.height - 2 * rimThickness + 1);
}

int Map::getZoomLevel() const
{
	if (lod == LOD::Minimap)
	{
		return 1;
	}
	return 2;// zoomLevel;
}

// private
Rect<int>& Map::getBounds()
{
	if (lod == LOD::Minimap)
	{
		return minimapBounds;
	}
	return mapBounds;
}

const Rect<int>& Map::getBounds() const
{
	if (lod == LOD::Minimap)
	{
		return minimapBounds;
	}
	return mapBounds;
}

float Map::getScale() const
{
	return tileSize / getZoomLevel();
}

} // map
} // sl
