#ifndef SL_MAP_MAP_HPP
#define SL_MAP_MAP_HPP

#include <SFML/Graphics/CircleShape.hpp>
#include <SFML/Graphics/Sprite.hpp>
#include <SFML/Graphics/Texture.hpp>

#include "Engine/Graphics/Drawable.hpp"
#include "Town/Map/LOD.hpp"
#include "Utility/IntrusiveList.hpp"
#include "Utility/Vector.hpp"

namespace sl
{
namespace map
{

class MapData;
class Icon;

class Map : public Drawable
{
public:
	Map(/*const MapData& mapData, */int tileSize, const Vector2i& minimapSize, const Vector2i& mapSize);


	void update(int x, int y) override;

	const Rect<int>& getCurrentBounds() const override;

	void draw(sf::RenderTarget& target) const override;

	void setPositionInMap(float x, float y);

	void registerIcon(Icon& icon);

	void setData(const MapData& mapData);

	void setData(const sf::Image& raw);

	sf::Image makeImage() const;
	
	void setLOD(LOD lod);
	
	LOD getLOD() const;

	/**
	 * @param pos The query position (in map space)
	 * @return THe icon drawn there OR null if none exist
	 */
	Icon* getIconAt(const Vector2f& posMapSpace);

	Vector2f worldspaceFromMapspace(const Vector2f& pos) const;

	Vector2f mapSpaceFromDrawableSpace(const Vector2f& pos) const;

	/**
	 * @return The visible region IN MAP SPACE
	 */
	Rect<float> visibleRegion() const;

	int getZoomLevel() const;
	
private:

	Rect<int>& getBounds();

	const Rect<int>& getBounds() const;

	float getScale() const;

	
	
	sf::Texture texture;
	sf::Sprite sprite;
	Rect<int> minimapBounds;
	Rect<int> mapBounds;
	Rect<int> drawableBounds;
	IntrusiveList<Icon> icons;
	const int tileSize;
	LOD lod;
	Vector2f centerPos;
};

} // map
} // sl

#endif // SL_MAP_MAP_HPP
