#include "Town/Map/MapGUI.hpp"

#include "Engine/GUI/DrawableContainer.hpp"
#include "Engine/GUI/TextLabel.hpp"
#include "Town/Map/Map.hpp"
#include "Town/Map/Icon.hpp"
#include "Town/Town.hpp"
#include "Town/LocationMarker.hpp"

namespace sl
{
namespace map
{

MapGUI::MapGUI(Map& map, Town& town, const Rect<int>& box)
	:GUIWidget(box)
	,state(State::Default)
	,map(map)
	,town(town)
	,icons()
{
	addWidget(unique<gui::DrawableContainer>(box, &map));

	auto hoverTextWidget = unique<gui::TextLabel>(Rect<int>(100, 16));
	hoverText = hoverTextWidget.get();
	addWidget(std::move(hoverTextWidget));

	namespace ph = std::placeholders;
	registerMouseInputEventWithMousePos(Key::GUIClick, GUIWidget::InputEventType::Pressed, std::bind(&MapGUI::onLeftClick, this, ph::_1, ph::_2));

	map.setPositionInMap(0.5f, 0.5f);
}


void MapGUI::onLeftClick(int x, int y)
{
	switch (state)
	{
	case State::Default:
	//	{
	//		Icon *icon = map.getIconAt(Vector2f(x, y));
	//		if (icon)
	//		{
	//		
	//		}
	//	}
	//	break;
	//case State::AddMarker:
		{
			const Vector2f locPos = map.worldspaceFromMapspace(map.mapSpaceFromDrawableSpace(Vector2f(x, y)));
			LocationMarker *homeMarker = new LocationMarker(locPos.x, locPos.y, "Custom Marker", "custom");
			homeMarker->editIcon().setLabel("user-defined");
			town.addEntityToList(homeMarker);
			icons.insert(homeMarker);
			
			changeState(State::Default);
		}
		break;
	}
}

void MapGUI::onMouseHover(int x, int y)
{
	Icon *icon = map.getIconAt(map.mapSpaceFromDrawableSpace(Vector2f(x, y)));
	if (icon)
	{
		hoverText->setString(icon->getLabel());
		hoverText->move(x, y - 12);
	}
	else
	{
		hoverText->setString("");
		hoverText->move(-100, -100);
	}
}

void MapGUI::changeState(State newState)
{
	state = newState;
	// change gui here?
}

//Vector2f MapGUI::guiToMapSpace(int x, int y) const
//{
//	return Rect<int>(0, 0, scene->getViewWidth(), scene->getViewHeight()).mapTo(Vector2i(x, y), map.getCurrentBounds());
//}

} // map
} // sl
