#include "Town/Map/MapStaticEntityData.hpp"

#include "Engine/Entity.hpp"
#include "Utility/Rect.hpp"

namespace sl
{
namespace map
{

//@todo replace with std::round when that becomes available (MSVC++2013)
int shittyRound(float x)
{
	const int integerPart = x;
	const float floatPart = x - integerPart;
	if (floatPart >= 0.5f)
	{
		return integerPart + 1;
	}
	else
	{
		return integerPart;
	}
}

MapStaticEntityData::MapStaticEntityData(int width, int height, int tileSize)
	:tileSize(tileSize)
	,data(width, height, StaticEntity::None)
{
}


void MapStaticEntityData::set(const Entity& entity, StaticEntity type)
{
	const Rect<int>& bounds = entity.getMask().getBoundingBox();
	const float tileSizeF = tileSize;
	const int t1x = shittyRound(bounds.left / tileSizeF);
	const int t1y = shittyRound(bounds.top / tileSizeF);
	int t2x = shittyRound((bounds.left + bounds.width) / tileSizeF);
	int t2y = shittyRound((bounds.top + bounds.height) / tileSizeF);
	// make sure everything's at least 1 pixel wide/high
	if (t2x == t1x)
	{
		++t2x;
	}
	if (t2y == t1y)
	{
		++t2y;
	}
	for (int y = t1y; y < t2y; ++y)
	{
		for (int x = t1x; x < t2x; ++x)
		{
			set(x, y, type);
		}
	}
}

void MapStaticEntityData::set(const Rect<int>& boundsInTiles, StaticEntity type)
{
	for (int y = 0; y < boundsInTiles.height; ++y)
	{
		for (int x = 0; x < boundsInTiles.width; ++x)
		{
			set(boundsInTiles.left + x, boundsInTiles.top + y, type);
		}
	}
}

void MapStaticEntityData::set(int x, int y, StaticEntity type)
{
	data.at(x, y) = type;
}

MapStaticEntityData::StaticEntity MapStaticEntityData::get(int x, int y) const
{
	return data.at(x, y);
}

int MapStaticEntityData::getWidth() const
{
	return data.getWidth();
}

int MapStaticEntityData::getHeight() const
{
	return data.getHeight();
}

} // map
} // sl