#include "Town/RoadSegment.hpp"

#include "Town/Road.hpp"
#include "Utility/Assert.hpp"
#include "Utility/TypeToString.hpp"

namespace sl
{

DECLARE_TYPE_TO_STRING(RoadSegment)

RoadSegment::RoadSegment(DeserializeConstructor)
	:Entity(0, 0, 0, 0, "RoadSegment")
{
}

RoadSegment::RoadSegment(const Rect<int>& bounds, int magnet, Orientation orientation, const Road& road, const Road *crossRoad)
	:Entity(bounds.left, bounds.top, bounds.width, bounds.height, "RoadSegment")
	,magnet(magnet)
	,orientation(orientation)
	,road(&road)
	,crossRoad(crossRoad)
{
}


std::string RoadSegment::getName() const
{
	std::string ret(road->getName());
	if (crossRoad)
	{
		ret += " by ";
		ret += crossRoad->getName();
	}
	return std::move(ret);
}

int RoadSegment::getMagnet() const
{
	return magnet;
}

const Vector2f& RoadSegment::getDirection() const
{
	static const Vector2f directions[4] = {
		Vector2f(0.f, -1.f), // North
		Vector2f(1.f, 0.f), // East
		Vector2f(-1.f, 0.f), // West
		Vector2f(0.f, 1.f)  // South
	};
	return directions[static_cast<int>(orientation)];
}

RoadSegment::Orientation RoadSegment::getOrientation() const
{
	return orientation;
}

// private
const Types::Type RoadSegment::getType() const
{
	return Entity::makeType("RoadSegment");
}

void RoadSegment::onUpdate()
{
	//switch (orientation)
	//{
	//case Orientation::North:
	//	scene->debugDrawRect("draw_btree", mask.getBoundingBox(), sf::Color::Red, sf::Color(255, 0, 0, 128));
	//	break;
	//case Orientation::East:
	//	scene->debugDrawRect("draw_btree", mask.getBoundingBox(), sf::Color::Green, sf::Color(0, 255, 0, 128));
	//	break;
	//case Orientation::West:
	//	scene->debugDrawRect("draw_btree", mask.getBoundingBox(), sf::Color::Blue, sf::Color(0, 0, 255, 128));
	//	break;
	//case Orientation::South:
	//	scene->debugDrawRect("draw_btree", mask.getBoundingBox(), sf::Color::Magenta, sf::Color(255, 0, 255, 128));
	//	break;
	//}
}

void RoadSegment::deserialize(Deserialize& in)
{
	deserializeEntity(in);
	LOADVAR(i, magnet);
	orientation = static_cast<Orientation>(in.u8("orientation"));
	const int width = in.i("width");
	const int height = in.i("height");
	setCollisionMask(CollisionMask(0, 0, width, height));
	road = nullptr;
	crossRoad = nullptr;

	// TODO serialize suspiciousScenario somehow?
}

void RoadSegment::serialize(Serialize& out) const
{
	serializeEntity(out);
	SAVEVAR(i, magnet);
	out.u8("orientation", static_cast<uint8_t>(orientation));
	// mask not (yet) saved via serializeEntity, so manually do it
	out.i("width", getMask().getBoundingBox().width);
	out.i("height", getMask().getBoundingBox().height);
	// TODO: don't ignore road / crossRoad
}

} // sl
