#ifndef SL_ROADSEGMENT_HPP
#define SL_ROADSEGMENT_HPP

#include "Engine/Entity.hpp"
#include "Engine/Serialization/DeserializeConstructor.hpp"

namespace sl
{

class Road;

class RoadSegment : public Entity
{
public:
	DECLARE_SERIALIZABLE_METHODS(RoadSegment)
	enum class Orientation
	{
		North = 0,
		East,
		West,
		South
	};

	RoadSegment(DeserializeConstructor);

	RoadSegment(const Rect<int>& bounds, int magnet, Orientation orientation, const Road& road, const Road *crossRoad = nullptr);

	std::string getName() const;

	/**
	 * @return Where to move to (X in West/East, Y in North/South) to be in the center of the road
	 */
	int getMagnet() const;

	const Vector2f& getDirection() const;

	Orientation getOrientation() const;

private:
	const Types::Type getType() const override;

	void onUpdate() override;

	int magnet;
	Orientation orientation;
	const Road *road;
	const Road *crossRoad;
};

} // sl

#endif // SL_ROADSEGMENT_HPP
