#ifndef SL_STATICGEOMETRY_HPP
#define SL_STATICGEOMETRY_HPP

#include <memory>

#include "Engine/Entity.hpp"

namespace sl
{

class StaticGeometry : public Entity
{
public:
	StaticGeometry(int x, int y, int w, int h, std::unique_ptr<Drawable> drawable);

	StaticGeometry(int x, int y, int w, int h, const Rect<int>& behindCheckBox, std::initializer_list<Drawable*> drawables, float transparencyWhenBehind);
	
	StaticGeometry(int x, int y, int w, int h, const Rect<int>& behindCheckBox, std::initializer_list<Drawable*> drawables, std::initializer_list<Drawable*> drawablesWhenBehind);


	const Types::Type getType() const override;

	bool isStatic() const override;

	void doNotSerialize();

	void serialize(Serialize& out) const override;

	void deserialize(Deserialize& in) override;

	std::string serializationType() const override;

	bool shouldSerialize() const override;

private:
	enum class PlayerBehindPolicy
	{
		DoNothing,
		BecomeTransparent,
		AlternativeDrawables
	};

	void onUpdate() override;

	PlayerBehindPolicy policy;
	Rect<int> behindCheckBox;
	float alpha;
	bool serializeThis;
};

} // sl

#endif // SL_STATICGEOMETRY_HPP
