#include "Town/ThreeWayTrafficController.hpp"

#include "Town/TrafficLight.hpp"

namespace sl
{

ThreeWayTrafficController::ThreeWayTrafficController(TrafficLight& north, TrafficLight& east, TrafficLight& west)
	:Entity()
	,north(north)
	,east(east)
	,west(west)
	,greenTime(90)
	,yellowTime(30)
	,state(0)
	,alarm(greenTime + yellowTime)
{
	// straight (repeat twice)
	states.push_back(State(false, true, false, true));
	states.push_back(State(false, true, false, true));
	// left
	states.push_back(State(true, false, true, false));
}

const Types::Type ThreeWayTrafficController::getType() const
{
	return "ThreeWayTrafficController";
}

void ThreeWayTrafficController::onUpdate()
{
	--alarm;
	if (alarm == yellowTime)
	{
		const State& currentState = states[state];
		const State& nextState = states[(state + 1) % states.size()];
		// if the next one is green, don't bother going to yellow to avoid
		// a green->yellow->green change because that is fucking stupid
		if (!nextState.northLeft)
		{
			north.setColour(currentState.northLeft ? TrafficLight::Yellow : TrafficLight::Red);
		}
		if (!nextState.eastStraight)
		{
			east.setColour(currentState.eastStraight ? TrafficLight::Yellow : TrafficLight::Red);
		}
		if (!nextState.eastLeft)
		{
			east.setLeftColour(currentState.eastLeft ? TrafficLight::Yellow : TrafficLight::Red);
		}
		if (!nextState.westLeft)
		{
			west.setLeftColour(currentState.westLeft ? TrafficLight::Yellow : TrafficLight::Red);
		}
	}
	else if (alarm == 0)
	{
		state = (state + 1) % states.size();

		const State& currentState = states[state];

		north.setLeftColour(currentState.northLeft ? TrafficLight::Green : TrafficLight::Red);
		east.setColour(currentState.eastStraight ? TrafficLight::Green : TrafficLight::Red);
		east.setLeftColour(currentState.eastLeft ? TrafficLight::Green : TrafficLight::Red);
		west.setLeftColour(currentState.westLeft ? TrafficLight::Green : TrafficLight::Red);

		alarm = greenTime + yellowTime;
	}
}

}