#ifndef SL_TOWNTILES_HPP
#define SL_TOWNTILES_HPP

namespace sl
{

enum TileType
{
	// Basic fill types
	Dirt = 0,
	Grass,
	Pavement,
	// Roads
	RoadH,
	RoadV,
	RoadRest,
	// Sidewalks
	SidewalkN,
	SidewalkS,
	SidewalkW,
	SidewalkE,
	SidewalkNW,
	SidewalkNE,
	SidewalkSW,
	SidewalkSE,
	SidewalkNWI,
	SidewalkNEI,
	SidewalkSWI,
	SidewalkSEI,
	SidewalkInner,
	DrivewayJoinL,
	DrivewayJoinR,
	// Misc Road-Related
	Intersection,
	CrosswalkH,
	CrosswalkV,
	DrivewayL,
	DrivewayR,
	DrivewayC,
	Walkway,
	// Park paths
	ParkPathN,
	ParkPathE,
	ParkPathW,
	ParkPathS,
	ParkPathNS,
	ParkPathEW,
	ParkPathNE,
	ParkPathNW,
	ParkPathES,
	ParkPathWS,
	ParkPathNEW,
	ParkPathNWS,
	ParkPathNES,
	ParkPathEWS,
	ParkPathNEWS,
	// Parking lot marking
	ParkingLotL,
	ParkingLotR,
	ParkingLotM,

	HACK,


	NumOfTiles  //  Keep NumOfTiles at the end and don't use it anywhere
};

#define CASE_ROAD \
	     RoadH: \
	case RoadV: \
	case RoadRest \

inline bool isRoad(TileType t)
{
	switch (t)
	{
		case CASE_ROAD:
			return true;
		default:
			return false;
	}
	return false;
}

#define CASE_SIDEWALK \
	     SidewalkN: \
	case SidewalkS: \
	case SidewalkW: \
	case SidewalkE: \
	case SidewalkNW: \
	case SidewalkNE: \
	case SidewalkSW: \
	case SidewalkSE: \
	case SidewalkNWI: \
	case SidewalkNEI: \
	case SidewalkSWI: \
	case SidewalkSEI: \
	case SidewalkInner \

inline bool isSidewalk(TileType t)
{
	switch (t)
	{
	case CASE_SIDEWALK:
			return true;
		default:
			return false;
	}
	return false;
}

#define CASE_PARKPATH \
	     ParkPathN: \
	case ParkPathE: \
	case ParkPathW: \
	case ParkPathS: \
	case ParkPathNS: \
	case ParkPathEW: \
	case ParkPathNE: \
	case ParkPathNW: \
	case ParkPathES: \
	case ParkPathWS: \
	case ParkPathNEW: \
	case ParkPathNWS: \
	case ParkPathNES: \
	case ParkPathEWS: \
	case ParkPathNEWS \

inline bool isParkPath(TileType t)
{
	switch (t)
	{
		case CASE_PARKPATH:
			return true;
		default:
			return false;
	}
	return false;
}

#define CASE_PARKINGLOT \
	     ParkingLotR: \
	case ParkingLotL \

inline bool isParkingLot(TileType t)
{
	switch (t)
	{
		case CASE_PARKINGLOT:
			return true;
		default:
			return false;
	}
	return false;
}

} // sl

#endif
