#ifndef SL_TRAFFICLIGHT_HPP
#define SL_TRAFFICLIGHT_HPP

#include "Engine/Entity.hpp"
#include "Pathfinding/NodeAvailabilityChecker.hpp"

namespace sl
{

class TrafficLight : public Entity
{
public:
	DECLARE_SERIALIZABLE_METHODS(TrafficLight)
	enum Direction
	{
		North = 0,
		East,
		West,
		South
	};
	enum Colour
	{
		Green,
		Yellow,
		Red
	};

	TrafficLight(int x, int y, Direction direction, bool canGoLeft);


	const Types::Type getType() const override;

	bool isStatic() const override;

	const pf::NodeAvailabilityChecker& getStraightChecker() const;

	const pf::NodeAvailabilityChecker& getLeftChecker() const;

	void setColour(Colour colour);

	void setLeftColour(Colour color);
	
	int getLeftLoad() const;
	
	int getStraightLoad() const;
	
	int getRightLoad() const;

private:
	void onUpdate() override;

	void onEnter() override;

	void onExit() override;
	
	void computeLoads() const;

	void createDrawables();


	
	pf::NodeAvailabilityChecker straightAvailabilityChecker;

	pf::NodeAvailabilityChecker leftAvailabilityChecker;

	Direction direction;

	bool canGoLeft;
	
	mutable int cachedLeftLoad;
	mutable int cachedStraightLoad;
	mutable int cachedRightLoad;

	sf::Vector2i polePos;


	class Pole : public Entity
	{
	public:
		Pole(int x, int y, Direction direction);

		const Types::Type getType() const override;

		bool isStatic() const override;

	private:
		void onUpdate() override;
	};
};

}

#endif