#ifndef SL_ASSERT_HPP
#define SL_ASSERT_HPP

//	so it only prints starting at the project directory
#ifndef PROJPATHLEN
#define PROJPATHLEN (sizeof(__FILE__) - sizeof("src/Utility/Assert.hpp") - 1)
#endif

#include <sstream>
#include <cstdlib>

namespace sl
{
namespace ass
{

enum AssertionLevel
{
	DebugOnly,
	Fatal
};

extern void onAssertionFail(AssertionLevel level, const std::string& message);

} // ass
} // sl

#ifdef SL_BREAK_ON_ASSERT
	#ifdef _MSC_VER
		#define SL_DEBUGBREAK() __debugbreak()
	#elif defined(__builtin_trap)
		#define SL_DEBUGBREAK() __builtin_trap()
	#else
		#define SL_DEBUGBREAK() \
		do \
		{ \
			const int zero = &PROJPATHLEN  == NULL? 1 : 0; \
			const int divideByZero = 1 / zero; \
		} \
		while (0)
	#endif
#else
	#define SL_DEBUGBREAK() ((void)0)
#endif

#define ASSERT_WITH_LEVEL(x, l) \
	do \
	{ \
		const bool failed = !(x); \
		if (failed) \
		{ \
			SL_DEBUGBREAK(); \
			std::stringstream ss; \
			ss << "Failed assertion (" << #x << ") on line number " << __LINE__ << " in file " << (__FILE__ + PROJPATHLEN) << std::endl; \
			onAssertionFail(l, ss.str()); \
		} \
	} \
	while (0)

#define ASSERT(x) ASSERT_WITH_LEVEL(x, sl::ass::Fatal)

#define ERROR(x) \
	do \
	{ \
		SL_DEBUGBREAK(); \
		std::stringstream ss; \
		ss << "Error (" << x << ") on line number " << __LINE__ << " in file " << (__FILE__ + PROJPATHLEN) << std::endl; \
		onAssertionFail(sl::ass::Fatal, ss.str()); \
	} \
	while (0)

#endif
