#include "Utility/DataToString.hpp"

#include <iomanip>
#include <sstream>

#include "Engine/Settings.hpp"
#include "Utility/Time/GameDate.hpp"
#include "Utility/Time/TimeOfDay.hpp"

namespace sl
{

const char * const dayNames[7] = {
	"Monday",
	"Tuesday",
	"Wednesday",
	"Thursday",
	"Friday",
	"Saturday",
	"Sunday"
};

const char * const monthNames[12] = {
	"January",
	"February",
	"March",
	"April",
	"May",
	"June",
	"July",
	"August",
	"September",
	"October",
	"November",
	"December"
};

std::string formatAge(int ageInDays, bool showMonths, bool showDays)
{
	const int daysInYear = 365;
	const int years = ageInDays / daysInYear;
	ageInDays -= years * daysInYear;

	const int daysInMonth = 30;// @todo handle specific months?
	const int months = ageInDays / daysInMonth;

	const int days = ageInDays - months * daysInMonth;

	std::stringstream ss;

	ss << years << " years";

	if (showMonths)
	{
		ss << " " << months << " months";
	}

	if (showDays)
	{
		ss << " " << days << " days";
	}

	return ss.str();
}

std::string formatWeight(float weightInKg)
{
	std::stringstream ss;
	if (Settings::fuckYouSingletonHack()->get("metric"))
	{
		ss << (int) weightInKg << "kg";
	}
	else
	{
		ss << (int) (weightInKg * 2.2) << "lbs";
	}
	return ss.str();
}

std::string formatHeight(float heightInCm)
{
	std::stringstream ss;
	if (Settings::fuckYouSingletonHack()->get("metric"))
	{
		ss << (int) heightInCm << "cm";
	}
	else
	{
		const int inches = heightInCm / 2.54f;
		ss << (inches / 12) << "'" << (inches % 12) << "\"";
	}
	return ss.str();
}

std::string formatLength(float lengthInCm)
{
	std::stringstream ss;
	if (Settings::fuckYouSingletonHack()->get("metric"))
	{
		ss << std::setprecision(3) << lengthInCm << "cm";
	}
	else
	{
		const float inches = lengthInCm / 2.54f;
		ss << std::setprecision(3) << inches << "\"";
	}
	return ss.str();
}

std::string formatPercent(float number)
{
	std::stringstream ss;
	ss << std::setprecision(3) << (100.f * number) << "%";
	return ss.str();
}

std::string formatDateShort(const GameDate& date)
{
	std::stringstream ss;
	ss << (date.getMonthDay() + 1) << "/" << (date.getMonth() + 1) << "/" << date.getYear();
	return ss.str();
}

std::string formatDateLong(const GameDate& date)
{
	std::stringstream ss;
	ss << dayNames[date.getDayOfWeek()] << ", " << monthNames[date.getMonth()] << " " << (date.getMonthDay() + 1) << ", " << date.getYear();
	return ss.str();
}

std::string formatTime(const TimeOfDay& time)
{
	std::stringstream ss;
	const int hour = time.getHour() % 12;
	const bool am = time.getHour() < 12;
	ss << (hour == 0 ? 12 : hour) << ":";
	if (time.getMinutes() < 10)
	{
		ss << "0";
	}
	ss << time.getMinutes() << (am ? "AM" : "PM");
	return ss.str();
}

} // sl
