/**
 * @section DESCRIPTION
 * These are all infinite series (some just return 0 after a certain point to achieve this though) that all converge to 1.
 * This can be pretty useful to model diminishing returns and such, subject to the constraint that it will never go above 1.
 * If you need a different number, you can just add in a scalar.
 * NOTE: All inputs start at n=1.
 */
#ifndef SL_CONVERGENTSERIES_HPP
#define SL_CONVERGENTSERIES_HPP

#include <functional>

namespace sl
{

// TODO: make some slower-converging sums

/**
 * 1/2^n. 0.5 + 0.25 + 0.125 + ...
 */
extern float geometricSeries(int n);

/**
 * p-series with p=2. ie 1/(n^2). approx 0.6 + 0.15 + 0.06 + 0.037 + ...
 */
extern float baselSeries(int n);

class FiniteSeries
{
public:
	/**
	 * Samples f(n) until it is approximately 0 (or less), then normalizes the summation up to that point
	 */
	FiniteSeries(std::function<float(int)> f);

	/**
	 * Normalizes f(n) over 1 to nonZeroTerms, then returns 0 past that point
	 */
	FiniteSeries(std::function<float(int)> f, int nonZeroTerms);

	float operator()(int n) const;

private:
	float computeNormalizationScalar() const;

	static int computeNonZeroTermsIndex(const std::function<float(int)>& f);

	std::function<float(int)> f;
	const int nonZeroTerms;
	const float normalizationScalar;
};

} // s;

#endif // SL_CONVERGENTSERIES_HPP
