#ifndef SL_ITER_ITERATORUTIL_HPP
#define SL_ITER_ITERATORUTIL_HPP

#include <iterator>

namespace sl
{

namespace iter
{
namespace impl
{

template <typename RAIter>
void tryAdvance(RAIter& iter, const RAIter& end, typename std::iterator_traits<RAIter>::difference_type dist, std::random_access_iterator_tag)
{
	std::advance(iter, std::min(dist, std::distance(iter, end)));
}

template <typename Iter>
void tryAdvance(Iter& iter, const Iter& end, typename std::iterator_traits<Iter>::difference_type dist, std::forward_iterator_tag)
{
	while (iter != end && dist > 0)
	{
		++iter;
		--dist;
	}
}
} // impl

template <typename Iter>
void tryAdvance(Iter& iter, const Iter& end, typename std::iterator_traits<Iter>::difference_type dist)
{
	return impl::tryAdvance(iter, end, dist, std::iterator_traits<Iter>::iterator_category());
}

template <typename Iter>
Iter tryNext(const Iter& iter, const Iter& end, typename std::iterator_traits<Iter>::difference_type dist)
{
	Iter ret(iter);
	tryAdvance(ret, end, dist);
	return ret;
}

} // iter
} // sl

#endif // SL_ITER_ITERATORUTIL_HPP