#include "Utility/JoinableSet.hpp"

#include "Utility/Assert.hpp"

namespace sl
{

JoinableSet::JoinableSet(int n)
	:nodes(n)
{
	for (int i = 0; i < n; ++i)
	{
		nodes[i].parent = i;
		nodes[i].rank = 0;
		nodes[i].order = 1;
	}
}

int JoinableSet::getComponent(int u)
{
	ASSERT(0 <= u && u < nodes.size());
	if (nodes[u].parent != u)
	{
		nodes[u].parent = getComponent(nodes[u].parent);
	}
	return nodes[u].parent;
}

void JoinableSet::join(int u, int v)
{
	ASSERT(0 <= u && u < nodes.size());
	ASSERT(0 <= v && v < nodes.size());
	const int uRoot = getComponent(u);
	const int vRoot = getComponent(v);
	if (uRoot != vRoot)
	{
		if (nodes[uRoot].rank < nodes[vRoot].rank)
		{
			nodes[vRoot].order += nodes[uRoot].order;
			nodes[uRoot].order = -1;
			nodes[uRoot].parent = vRoot;
		}
		else if (nodes[uRoot].rank > nodes[vRoot].rank)
		{
			nodes[uRoot].order += nodes[vRoot].order;
			nodes[vRoot].order = -1;
			nodes[vRoot].parent = uRoot;
		}
		else
		{
			nodes[uRoot].order += nodes[vRoot].order;
			nodes[vRoot].order = -1;
			nodes[vRoot].parent = uRoot;
			++nodes[uRoot].rank;
		}
	}
}

int JoinableSet::componentSize(int component) const
{
	ASSERT(0 <= component && component < nodes.size());
	const int order = nodes[component].order;
	ASSERT(order != -1); // ensure that it is actually a component (tree root)
	return order;
}

} // sl
