#include "Utility/JsonUtil.hpp"

#include <fstream>

namespace sl
{

json11::Json readJsonFile(const std::string& fname, std::string& err)
{
	std::ifstream file(fname, std::ios::in | std::ios::binary);
	json11::Json json;
	if (file)
	{
		std::string rawstring;
		file.seekg(0, std::ios::end);
		rawstring.resize(file.tellg());
		file.seekg(0, std::ios::beg);
		file.read(&rawstring.front(), rawstring.size());
		file.close();

		json = json11::Json::parse(rawstring, err);
	}
	else
	{
		err = "Could not open file: " + fname;
	}
	return std::move(json);
}

json11::Json tryReadJsonFile(const std::string& fname)
{
	std::string ignored;
	return readJsonFile(fname, ignored);
}

std::string saveJsonFile(const std::string& fname, json11::Json json)
{
	std::string buffer;
	json.dump(buffer);

	std::ofstream file(fname, std::ios::out | std::ios::binary);
	std::string err;
	if (file)
	{
		file << buffer;
	}
	else
	{
		err = "Could not save to file: " + fname;
	}
	return err;
}

Rect<int> parseRect(const json11::Json::object& obj)
{
	auto x = obj.find("x");
	auto y = obj.find("y");
	return Rect<int>(x == obj.end() ? 0 : x->second.int_value(), y == obj.end() ? 0 : y->second.int_value(), obj.at("w").int_value(), obj.at("h").int_value());
}

json11::Json::object writeRect(const Rect<int>& rect)
{
	json11::Json::object obj;
	obj["x"] = rect.left;
	obj["y"] = rect.top;
	obj["w"] = rect.width;
	obj["h"] = rect.height;
	return obj;
}

} // sl
