#ifndef SL_JSONUTIL_HPP
#define SL_JSONUTIL_HPP

#include "json11/json11.hpp"

#include <string>

#include "Utility/Rect.hpp"

namespace sl
{

/**
 * Opens and reads a json file into a json11 object
 * @param fname The filename to open
 * @param err (output) is set to an error string if an error occurs opening or reading the file
 * @return the json object read from the file
 */
extern json11::Json readJsonFile(const std::string& fname, std::string& err);

/**
 * Opens and reads a json file to a json11 object if the file exists. No error checking is done.
 * @param fname The filename to attempt to open
 * @return The json object read (json11::NUL if couldn't read)
 */
extern json11::Json tryReadJsonFile(const std::string& fname);

extern std::string saveJsonFile(const std::string& fname, json11::Json json);

/**
 * Reads a JSON object with fields x (optional - defaults to 0), y (optional - defaults to 0), w, h
 * @param obj The JSON object to parse
 * @return a rect of the form Rect<int>(x, y, w, h) with x/y being 0 if not in obj
 */
extern Rect<int> parseRect(const json11::Json::object& obj);

/**
 * Converts a Rect<int> to a JSON object
 * @param rect The rect to convert
 * @return JSON object { "x" : rect.left, "y" : rect.top, "w" : rect.width, "h" : rect.height }
 */
extern json11::Json::object writeRect(const Rect<int>& rect);

} // sl

#endif // SL_JSONUTIL_HPP
