#include "Utility/Logger.hpp"

#include <iostream>

#include "Utility/Assert.hpp"

namespace sl
{

/*****************************************************************************
 *                        Logger implementation                              *
 *****************************************************************************/

Logger *Logger::instance = nullptr;

Logger::Logger()
	:enabled(true)
{
	for (int i = 0; i < AREAS; ++i)
	{
		priorities[i] = Priority::Useless;
		for (int j = 0; j < PRIORITIES; ++j)
		{
			outputs[j][i] = OutputLocation::Console;
		}
	}
}


Logger::Stream Logger::log(Area area, Priority priority)
{
	ASSERT(instance);
	if (instance->enabled && priority >= instance->priorities[area])
	{
		return Stream(instance->outputs[priority][area]);
	}
	return Stream(Ignored);
}

void Logger::setDefaultLogger(Logger *logger)
{
	instance = logger;
}

Logger::Stream Logger::logTo(Area area, Priority priority) const
{
	if (enabled && priority >= priorities[area])
	{
		return Stream(outputs[priority][area]);
	}
	return Stream(Ignored);
}

void Logger::setOutputLocation(OutputLocation location)
{
	for (int i = 0; i < AREAS; ++i)
	{
		for (int j = 0; j < PRIORITIES; ++j)
		{
			outputs[j][i] = location;
		}
	}
}

void Logger::setOutputLocation(Area area, OutputLocation location)
{
	for (int i = 0; i < PRIORITIES; ++i)
	{
		outputs[i][area] = location;
	}
}

void Logger::setOutputLocation(Priority priority, OutputLocation location)
{
	for (OutputLocation& loc : outputs[priority])
	{
		loc = location;
	}
}

void Logger::setOutputLocation(Area area, Priority priority, OutputLocation location)
{
	outputs[priority][area] = location;
}

void Logger::setMinimumPriority(Priority priority)
{
	for (Priority& p : priorities)
	{
		p = priority;
	}
}

void Logger::setMinimumPriority(Area area, Priority priority)
{
	priorities[area] = priority;
}


void Logger::enable()
{
	enabled = true;
}

void Logger::disable()
{
	enabled = false;
}



/*****************************************************************************
 *                        Stream implementation                              *
 *****************************************************************************/

Logger::Stream::Stream(Logger::OutputLocation outputLocation)
	:outputLocation(outputLocation)
{
	if (outputLocation == Logger::OutputLocation::File)
	{
		fileStream.open("logs.txt");
	}
}

Logger::Stream::Stream(Stream&& rhs)
	:outputLocation(rhs.outputLocation)
//	,fileStream(std::move(rhs.fileStream))
//	,popupStream(std::move(rhs.popupStream))
{
}

Logger::Stream::~Stream()
{
	if (outputLocation == Logger::OutputLocation::File)
	{
		 fileStream.close();
	}
	else if (outputLocation	== Logger::OutputLocation::Popup)
	{
		// @todo Make this an actual popup
		ERROR(popupStream.str());
	}
}

std::ostream& Logger::Stream::get()
{
	switch (outputLocation)
	{
	case Ignored:
		return ignoreStream;
	case File:
		return fileStream;
	case Console:
		return std::cout;
	case Screen:
		ERROR("not implemented yet");
		return std::cout;
	case Popup:
		return popupStream;
	}
	return ignoreStream;
}

} // namespace sl
