#ifndef SL_MAPPING_HPP
#define SL_MAPPING_HPP

#include <map>
#include <set>
#include <type_traits>
#include <vector>

namespace sl
{

// TODO: replace all with iterator-based then have a templated generic one to convert to the iterator ones via std::begin()?
template <typename It, typename F>
auto mapVec(It begin, It end, const F& f) -> std::vector<typename std::result_of<F(typename const decltype(*begin)&)>::type>
{
	typename std::vector<typename std::result_of<F(typename const decltype(*begin)&)>::type> ret;
	ret.reserve(std::distance(begin, end));
	for (auto it = begin; it != end; ++it)
	{
		ret.push_back(f(*it));
	}
	return ret;
}

template <typename C, typename F>
auto mapVec(const C& src, const F& f) -> std::vector<typename std::result_of<F(typename const C::value_type&)>::type>
{
	typename std::vector<typename std::result_of<F(typename const C::value_type&)>::type> ret;
	ret.reserve(src.size());
	for (const auto& s : src)
	{
		ret.push_back(f(s));
	}
	return ret;
}

template <typename C, typename F>
auto mapVecIndexed(const C& src, const F& f) -> std::vector<typename std::result_of<F(int, typename const C::value_type&)>::type>
{
	typename std::vector<typename std::result_of<F(int, typename const C::value_type&)>::type> ret;
	ret.reserve(src.size());
	for (int i = 0; i < src.size(); ++i)
	{
		ret.push_back(f(i, src[i]));
	}
	return ret;
}

// TOOD: why do I need a diff name? it should match fine.
template <typename C, typename F>
auto mapVecMove(C&& src, const F& f) -> std::vector<typename std::result_of<F(typename C::value_type&&)>::type>
{
	typename std::vector<typename std::result_of<F(typename C::value_type&&)>::type> ret;
	ret.reserve(src.size());
	for (auto& s : src)
	{
		ret.push_back(f(std::move(s)));
	}
	return ret;
}

// TODO: write a generic one with some kind of generic inserter? might only make sense for order-preserving inserts (ie not map/set)
template <typename C, typename F>
auto mapSet(const C& src, const F& f) -> std::set<typename std::result_of<F(typename const C::value_type&)>::type>
{
	typename std::set<typename std::result_of<F(typename const C::value_type&)>::type> ret;
	for (const auto& s : src)
	{
		ret.insert(f(s));
	}
	return ret;
}

template <typename C, typename F>
auto mapMap(const C& src, const F& f) -> std::map<
	typename std::result_of<F(typename const C::value_type&)>::type::first_type,
	typename std::result_of<F(typename const C::value_type&)>::type::second_type>
{
	typename std::map<
		typename std::result_of<F(typename const C::value_type&)>::type::first_type,
		typename std::result_of<F(typename const C::value_type&)>::type::second_type
	> ret;
	for (const auto& s : src)
	{
		ret.insert(f(s));
	}
	return ret;
}

} // sl

#endif // SL_MAPPING_HPP