#ifndef SL_MEMORY_HPP
#define SL_MEMORY_HPP

#include <memory>
#include <type_traits>
#include <utility>

namespace sl
{

	template <typename T, typename... Args>
	std::unique_ptr<T> unique(Args&&... args)
	{
		return std::unique_ptr<T>(new T(std::forward<Args>(args)...));
	}

	template <typename D, typename B, typename... Args>
	std::unique_ptr<B> uniqueAs(Args&&... args)
	{
		return std::unique_ptr<B>(new D(std::forward<Args>(args)...));
	}

	template <typename T, typename... Args>
	std::shared_ptr<T> shared(Args&&... args)
	{
		return std::shared_ptr<T>(new T(std::forward<Args>(args)...));
	}

	namespace memory	//	hide implementation stuff
	{

		/*		taken from channel9.msdn.com/Series/C9-Lectures-Stephan-T-Lavavej-Core-C-/STLCCSeries6		*/
		template <typename T, typename... Args>
		std::unique_ptr<T> unique_ptr_helper(std::false_type, Args&&... args)
		{
			return std::unique_ptr<T>(new T(std::forward<Args>(args)...));
		}
		/*		taken from channel9.msdn.com/Series/C9-Lectures-Stephan-T-Lavavej-Core-C-/STLCCSeries6		*/
		template <typename T, typename... Args>
		std::unique_ptr<T> unique_ptr_helper(std::true_type, Args&&... args)
		{
			static_assert(std::extent<T>::value == 0, "unique_ptr<T[N]>() is forbidden, please use unique_ptr<T[]>().");

			typedef typename std::remove_extent<T>::type U;
			return std::unique_ptr<T>(new U[sizeof...(Args)]{ std::forward<Args>(args)... });
		}

	}	//	end memory namespace

} // end of sl namespace

#endif