#ifndef SL_MULTIVARIATELINEARINTERPOLATION_HPP
#define SL_MULTIVARIATELINEARINTERPOLATION_HPP

#include "Utility/Mapping.hpp"
#include "Utility/MultivariateVoronoi.hpp"
#include "Utility/Assert.hpp"

namespace sl
{

template <typename Vec, typename F>
F voronoiLerp(const std::vector<Vec>& points, const std::vector<F> values, const Vec& query, const typename Vec::Field dotContrib, const typename Vec::Field distContrib)
{
	using T = Vec::Field;
	static constexpr T zero(0);
	static constexpr T one(1);

	ASSERT(distContrib + dotContrib <= one);
	const T restContrib = one - distContrib - dotContrib;
	ASSERT(points.size() == values.size());
	const std::vector<int> neighbors = dotContrib < one
		? computeVoronoiNeighbors<Vec, false>(points, query)
		: computeVoronoiNeighbors<Vec, true>(points, query);
	const Vec& u = points[neighbors.front()];
	const F& uf = values[neighbors.front()];
	// accumulation (based on values) for dot product
	F rdot = F();
	// total contribution for dot product (values not involved)
	T ndot = zero;
	// accumation for distance method
	F rdist = F();
	// total contribution for distance method
	T ndist = zero;
	// accumulation for the rest
	F rrest = F();
	// total contribution for the rest
	const T nrest = T(neighbors.size() - 1);
	const Vec uq = query - u;
	for (int i = 1; i < neighbors.size(); ++i)
	{
		const Vec& v = points[neighbors[i]];
		const F& vf = values[neighbors[i]];
		const Vec uv = v - u;
		const T uvlen = uv.length();
		const Vec& uqprojuv = proj(uq, uv);
		const T hi = uqprojuv.length();
		const T lo = uvlen - hi;
		const F vraw = (uf * lo + vf * hi) / uvlen;
		const T curdot = dot(uq, uv);
		rdot += vraw * curdot;
		ndot += curdot;
		rdist += vraw * uvlen;
		ndist += uvlen;
		rrest += vraw;
	}
	return (rrest / nrest) * restContrib
	     + (rdist / ndist) * distContrib
		 + (ndot > zero ? (rdot / ndot) * dotContrib : zero);
}

} // sl

#endif // SL_MULTIVARIATELINEARINTERPOLATION_HPP