#include "Utility/Sequencer/SequenceJoin.hpp"

namespace sl
{

SequenceJoin::SequenceJoin()
	:joinCounter(new int(0))
	,reached(false)
{
}

SequenceJoin::SequenceJoin(const SequenceJoin& copy)
	:joinCounter(copy.joinCounter)
	,reached(false)
{
}

SequenceJoin::SequenceJoin(SequenceJoin&& move)
	:joinCounter(move.joinCounter)
	,reached(move.reached)
{
}

SequenceJoin::~SequenceJoin()
{
	signalNotReached();
}

SequenceJoin& SequenceJoin::operator=(const SequenceJoin& rhs)
{
	joinCounter = rhs.joinCounter;
	reached = false;
	return *this;
}

SequenceJoin& SequenceJoin::operator=(SequenceJoin&& rhs)
{
	joinCounter = rhs.joinCounter;
	reached = rhs.reached;
	return *this;
}

bool SequenceJoin::isInstantaneouslySequenced() const
{
	return true;
}

bool SequenceJoin::sequence()
{
	signalReached();
	return reached;
}

void SequenceJoin::resetSequence()
{
	signalNotReached();
}

/*				private					*/

void SequenceJoin::signalReached()
{
	if (!reached)
	{
		reached = true;
		++*joinCounter;
	}
}

void SequenceJoin::signalNotReached()
{
	if (reached)
	{
		reached = false;
		--*joinCounter;
	}
}

} // sl
