#include "Utility/Sequencer/SequencedFunction.hpp"

#include "Utility/Assert.hpp"

namespace sl
{

/*						SequencedFunction						*/

SequencedFunction::SequencedFunction(std::function<void()> function)
	:function(function)
	,isInstantaneous(true)
	,countdown(-1)
{
}

SequencedFunction::SequencedFunction(std::function<void()> function, int preTime, int postTime)
	:function(function)
	,isInstantaneous(false)
	,frameToFire(postTime + 1)
	,countdown(preTime + postTime + 1)
{
	ASSERT(preTime >= 0 && postTime >= 0);
}

bool SequencedFunction::sequence()
{
	if (isInstantaneous || --countdown == frameToFire)
	{
		function();
	}
	return countdown <= 0;
}

bool SequencedFunction::isInstantaneouslySequenced() const
{
	return isInstantaneous;
}


/*						SequencedFunction						*/

SequencedBoolFunction::SequencedBoolFunction(std::function<bool()> function, bool isInstantaneous)
	:function(function)
	,isInstantaneous(isInstantaneous)
{
}

bool SequencedBoolFunction::sequence()
{
	return function();
}

bool SequencedBoolFunction::isInstantaneouslySequenced() const
{
	return isInstantaneous;
}


} // sl