#include "Utility/String.hpp"

#include <cstring>

#include "Utility/Assert.hpp"

namespace sl
{

String::String(const std::string& str)
	:ref(new Ref{ new char[str.size()], (int)str.size(), 1 })
{
	memcpy(this->ref->str, str.c_str(), str.size());
}

String::String(const String& other)
	:ref(other.ref)
{
	addref();
}

String::String(String&& other)
	:ref(other.ref)
{
	addref();
}

String::~String()
{
	unref();
}

String& String::operator=(const String& rhs)
{
	rhs.addref();
	unref();
	ref = rhs.ref;
	return *this;
}

String& String::operator=(String&& rhs)
{
	rhs.addref();
	unref();
	ref = rhs.ref;
	return *this;
}

std::string String::toStd() const
{
	return std::string(ref->str);
}

const char* String::toC() const
{
	return ref->str;
}

String::operator const char*() const
{
	return toC();
}

// private
void String::addref() const
{
	++ref->count;
}

void String::unref()
{
	if (--ref->count == 0)
	{
		delete ref->str;
		delete ref;
	}
	ASSERT(ref->count >= 0);
}

void String::breakOff()
{
	if (ref->count > 1)
	{
		unref();
		Ref *newRef = new Ref{new char[ref->len], ref->len, 1};
		memcpy(newRef->str, ref->str, ref->len);
		ref = newRef;
	}
	ASSERT(ref->count == 1);
}

} // sl
