#include "Utility/Time/GameDate.hpp"

#include "Utility/Time/DateUtil.hpp"

// TODO: sort out leap year stuff instead of doing *365 everywhere. there's bound to be bugs.

namespace sl
{

GameDate::GameDate(int year, int yearDay)
	:year(year)
	,day(yearDay)
{
}

GameDate::GameDate(int year, Month month, int monthDay)
	:year(year)
	,day(monthDay)
{
	for (int i = 0; i < static_cast<int>(month); ++i)
	{
		day += daysInMonth(i, year);
	}
}

int GameDate::getYear() const
{
	return year;
}

int GameDate::getMonthDay() const
{
	int days = day;
	int month = 0;
	while (days >= daysInMonth(month, year))
	{
		days -= daysInMonth(month, year);
		++month;
	}
	return days;
}

int GameDate::getDay() const
{
	return day;
}

int GameDate::getMonth() const
{
	int days = day;
	int month = 0;
	while (days >= daysInMonth(month, year))
	{
		days -= daysInMonth(month, year);
		++month;
	}
	return month;
}

int GameDate::getDayOfWeek() const
{
	const int yearMinusOne = year - 1;
	// from https://en.wikipedia.org/wiki/Determination_of_the_day_of_the_week#Gauss.27_algorithm
	return (((5 * (yearMinusOne % 4) + 4 * (yearMinusOne % 100) + 6 * (yearMinusOne % 400)) % 7) + 7) % 7;
}

void GameDate::advance(int days)
{
	day += days;
	if (day >= 0)
	{
		const int yearsToAdvance = day / 365;
		if (yearsToAdvance)
		{
			year += yearsToAdvance;
			day %= 365;
		}
	}
	else
	{
		while (day < 0)
		{
			--year;
			day += 365;
		}
	}
}


bool GameDate::operator==(const GameDate& rhs) const
{
	return year == rhs.year && day == rhs.day;
}

bool GameDate::operator<(const GameDate& rhs) const
{
	return year < rhs.year || (year == rhs.year && day < rhs.day);
}

uint64_t GameDate::u64() const
{
	return 365 * year + day;
}

} // sl