#include "Utility/Time/GameTime.hpp"

#include "Utility/Assert.hpp"

namespace sl
{

GameTime::GameTime(const GameDate& date, const TimeOfDay& time)
	:date(date)
	,time(time)
{
}

GameTime::GameTime(int year, int yearDay, const TimeOfDay& time)
	:date(year, yearDay)
	,time(time)
{
}

GameTime::GameTime(int year, Month month, int monthDay, const TimeOfDay& time)
	:date(year, month, monthDay)
	,time(time)
{
}

bool GameTime::operator==(const GameTime& rhs) const
{
	return date == rhs.date && time.getTotalSeconds() == rhs.time.getTotalSeconds();
}

bool GameTime::operator<(const GameTime& rhs) const
{
	return date < rhs.date
	    || (date == rhs.date && time.getTotalSeconds() < rhs.time.getTotalSeconds());
}

uint64_t GameTime::u64() const
{
	return dayToSeconds * date.u64() + time.getTotalSeconds();
}


// --------------------------- TimeInterval --------------------------------

TimeInterval::TimeInterval(const GameTime& start, const GameTime& end)
	:start(start)
	,end(end)
{
	ASSERT(start < end);
}

bool TimeInterval::isWithin(const GameTime& time) const
{
	return start <= time && time <= end;
}

} // sl
