#include "Utility/Time/RecurringTimeInterval.hpp"

#include "Utility/Assert.hpp"
#include "Utility/Time/GameTime.hpp"

namespace sl
{

RecurringTimeInterval::RecurringTimeInterval(const TimeOfDay& timeStart, int intervalSize, uint8_t requiredWeekdays, uint16_t requiredMonths)
	:requiredWeekdays(requiredWeekdays)
	,requiredMonths(requiredMonths)
	,timeStart(timeStart)
	,intervalSize(intervalSize)
{
	if (intervalSize + timeStart.getSeconds() >= 24 * 60 * 60)
	{
		ERROR("intervals that wrap around to the next day are not currently supported - please implement this later you lazy fuck");
	}
}


bool RecurringTimeInterval::isWithin(const GameTime& time) const
{
	if (isDateWithin(time.date))
	{
		const int diff = time.time.getTotalSeconds() - timeStart.getTotalSeconds();
		if (diff >= 0 && diff < intervalSize)
		{
			return true;
		}
	}
	return false;
}

bool RecurringTimeInterval::isDateWithin(const GameDate& date) const
{
	return requiredWeekdays & weekDayToIntervalWeekday(date.getDayOfWeek()) &&
	       requiredMonths & monthToIntervalMonths(date.getMonth());
}

const TimeOfDay& RecurringTimeInterval::getTimeOfDayStart() const
{
	return timeStart;
}

int RecurringTimeInterval::getDurationSeconds() const
{
	return intervalSize;
}


/*static*/RecurringTimeInterval::WeekDays RecurringTimeInterval::weekDayToIntervalWeekday(int weekday)
{
	static const WeekDays table[7] = {
		WeekDays::Monday,
		WeekDays::Tuesday,
		WeekDays::Wednesday,
		WeekDays::Thursday,
		WeekDays::Friday,
		WeekDays::Saturday,
		WeekDays::Sunday
	};
	ASSERT(weekday >= 0 && weekday < 7);
	return table[weekday];
}

/*static*/RecurringTimeInterval::Months RecurringTimeInterval::monthToIntervalMonths(int month)
{
	static const Months table[12] = {
		Months::January,
		Months::February,
		Months::March,
		Months::April,
		Months::May,
		Months::June,
		Months::July,
		Months::August,
		Months::September,
		Months::October,
		Months::November,
		Months::December
	};
	ASSERT(month >= 0 && month < 12);
	return table[month];
}

/*static*/uint32_t RecurringTimeInterval::monthDayToFlag(int dayZeroIndexed)
{
	ASSERT(dayZeroIndexed >= 0 && dayZeroIndexed <= 30);
	return 1 << dayZeroIndexed;
}

} // sl