#ifndef SL_RECURRINGTIMEINTERVAL_HPP
#define SL_RECURRINGTIMEINTERVAL_HPP

#include <cstdint>

#include "Utility/Time/TimeOfDay.hpp"

namespace sl
{

class GameDate;
class GameTime;

class RecurringTimeInterval
{
public:
	enum WeekDays
	{
		Monday = 1,
		Tuesday = 2,
		Wednesday = 4,
		Thursday = 8,
		Friday = 16,
		Saturday = 32,
		Sunday = 64,
		Weekdays = Monday | Tuesday | Wednesday | Thursday | Friday,
		Weekend = Saturday | Sunday,
		AnyDayOfWeek = Weekdays | Weekend
	};
	enum Months
	{
		January = 1,
		February = 2,
		March = 4,
		April = 8,
		May = 16,
		June = 32,
		July = 64,
		August = 128,
		September = 256,
		October = 512,
		November = 1024,
		December = 2048,
		AnyMonth = 4095
	};
	/**
	 * @param timeStart The start of the time period (per-day)
	 * @param intervalSize The duration (in seconds!) for this interval to last for.
	 *
	 */
	RecurringTimeInterval(const TimeOfDay& timeStart, int intervalSize = 24 * 60 * 60, uint8_t requiredWeekdays = WeekDays::AnyDayOfWeek, uint16_t requiredMonths = Months::AnyMonth);

	bool isWithin(const GameTime& time) const;

	bool isDateWithin(const GameDate& date) const;

	const TimeOfDay& getTimeOfDayStart() const;

	int getDurationSeconds() const;

	static WeekDays weekDayToIntervalWeekday(int weekday);

	static Months monthToIntervalMonths(int month);

	template <typename... Args>
	static uint32_t monthDaysToFlag(int dayZeroIndexed, Args... tail)
	{
		return monthDayToFlag(dayZeroIndexed) | monthDaysToFlag(tail...);
	}

	static uint32_t monthDayToFlag(int dayZeroIndexed);

private:

	uint8_t requiredWeekdays;
	uint32_t requiredDaysOfMonth;
	uint16_t requiredMonths;
	TimeOfDay timeStart;
	int intervalSize;
};

} // sl

#endif // SL_RECURRINGTIMEINTERVAL_HPP