#ifndef SL_TIMEINTERVALGENERATOR_HPP
#define SL_TIMEINTERVALGENERATOR_HPP

#include <vector>

#include "Utility/Time/GameTime.hpp"

namespace sl
{

class TimeIntervalGenerator
{
public:
	virtual ~TimeIntervalGenerator() = default;

	virtual std::vector<TimeInterval> generate(const GameTime& start, const GameTime& end) const = 0;

	virtual bool isWithin(const GameTime& time) const = 0;
};

} // sl

namespace sl
{

class FixedGameTimeIntervalGenerator : public TimeIntervalGenerator
{
public:
	FixedGameTimeIntervalGenerator(const TimeInterval& interval)
		:interval(interval)
	{
	}

	std::vector<TimeInterval> generate(const GameTime& start, const GameTime& end) const override
	{
		return { interval };
	}

	bool isWithin(const GameTime& time) const override
	{
		return interval.isWithin(time);
	}

private:
	TimeInterval interval;
};

} // sl

#endif // SL_TIMEINTERVALGENERATOR_HPP