#include "Utility/Trig.hpp"

namespace sl
{

float lengthdirX(float length, float dir)
{
	return length * cos(dir / 180 * pi);
}

float lengthdirY(float length, float dir)
{
	return -length * sin(dir / 180 * pi);
}

Vector2<float> lengthdir(float length, float dir)
{
	return Vector2<float>(lengthdirX(length, dir), lengthdirY(length, dir));
}


float pointDistance(const Vector2<float>& a, const Vector2<float>& b)
{
	return sqrt((a.x - b.x) * (a.x - b.x) + (a.y - b.y) * (a.y - b.y));
}

float pointDistance(float x1, float y1, float x2, float y2)
{
	return sqrt((x1 - x2) * (x1 - x2) + (y1 - y2) * (y1 - y2));
}


float pointDirection(const Vector2<float>& a, const Vector2<float>& b)
{
	float angle;
	if (b.y != a.y)
	{
		if (b.x - a.x == 0.f)
		{
			if (b.y - a.y < 0.f)
				angle = 90.f;
			else
				angle = 270.f;
		}
		else
			angle = (180.f / pi) * atan((a.y - b.y) / (b.x - a.x));

		if (b.x - a.x < 0.f)
			angle += 180.f;
		else if (b.x - a.x > 0 && b.y - a.y > 0.f)
			angle += 360.f;
	}
	else
	{
		// to avoid returning an angle of 360.f
		angle = a.x > b.x ? 180.f : 0.f;
	}
	return angle;
}

float direction(const Vector2f& a)
{
	return pointDirection(Vector2f(0.f, 0.f), a);
}

float pointDirection(float x1, float y1, float x2, float y2)
{
	float angle;
	if (x2 - x1 == 0)
	{
		if (y2 - y1 < 0)
			angle = 90;
		else
			angle = 270;
	}
	else
		angle = 180 / pi * atan((y1 - y2) / (x2 - x1));

	if (x2 - x1 < 0)
		angle += 180;
	else if (x2 - x1 > 0 && y2 - y1 > 0)
		angle += 360;
	return angle;
}

}

