#ifndef SL_TYPETRAITS_HPP
#define SL_TYPETRAITS_HPP

#include <type_traits>

#define IsMemcopyable_Error "Can only store POD types - if your class is memcpyable, add an exception in Utility/TypeTraits.hpp"

namespace sl
{

//! Determines whether or not a type is copyable by simply copying its bytes
template <typename T>
struct IsMemcopyable
{
	static constexpr bool value = std::is_trivially_copyable<T>::value;//std::is_pod<T>::value;
};

// Exceptions are added here:
// This is for classes that, while they have non-trivial constructors, they can still be
// safely memcpy'd as a trivial copy is all that is necessary.
#define ISMEMCOPYABLE_WHITELIST(T) class T; \
	template <> \
	struct IsMemcopyable<Target> { \
		static const bool value = true; \
	}

//ISMEMCOPYABLE_WHITELIST(Target);

} // sl

#endif // SL_TYPETRAITS_HPP