#include "NPC/Stats/DNA.hpp"

#include "Utility/Assert.hpp"

namespace sl
{

DNA::DNA()
	:genes()
{
}

DNA::DNA(const DNA& mother, const DNA& father)
	:genes(mother.genes | father.genes)
{
}

bool DNA::isMale() const
{
	return genes[Gender];
}

DNA::HairColor DNA::getHairColor() const
{
	int darkHairGenes = 0;
	if (genes[Hair1])
	{
		++darkHairGenes;
	}
	if (genes[Hair2])
	{
		++darkHairGenes;
	}
	if (genes[Hair3])
	{
		++darkHairGenes;
	}
	if (genes[Hair4])
	{
		++darkHairGenes;
	}
	if (!genes[Redhead])
	{
		if (darkHairGenes == 0)
		{
			return HairColor::LightRed;
		}
		else if (darkHairGenes < 3)
		{
			return HairColor::Red;
		}
	}
	if (darkHairGenes == 0)
	{
		return HairColor::PlatinumBlonde;
	}
	else if (darkHairGenes == 1)
	{
		if (genes[Hair1] || genes[Hair3])
		{
			return HairColor::Blonde;
		}
		else
		{
			return HairColor::GoldenBlonde;
		}
	}
	else if (darkHairGenes == 2)
	{
		if (genes[Hair4])
		{
			return HairColor::LightBrown;
		}
		else
		{
			return HairColor::LightBrownGreyish;
		}
	}
	else if (darkHairGenes == 3)
	{
		return HairColor::DarkBrown;
	}
	ASSERT(darkHairGenes == 4);
	return HairColor::Black;
}

/*static*/ const std::string& DNA::getHairColorString(HairColor hairColor)
{
	static const std::string strings[] = {
		"light blonde",
		"blonde",
		"golden blonde",
		"greyish brown",
		"light brown",
		"dark brown",
		"black",
		"red",
		"light red"
	};
	return strings[static_cast<int>(hairColor)];
}

DNA::EyeColor DNA::getEyeColor() const
{
	/*
	 * See http://genetics.thetech.org/ask/ask316 for explanation of Herc2/Gey
	 */
	if (genes[Herc2])
	{
		if (!genes[EyeCollagen])
		{
			return EyeColor::DarkBrown;
		}
		else
		{
			return EyeColor::Brown;
		}
	}
	else if (genes[Gey])
	{
		return !genes[EyeCollagen] ? EyeColor::Hazel : EyeColor::Green;
	}
	else if (!genes[LightEyes])
	{
		return !genes[EyeCollagen] ? EyeColor::Grey : EyeColor::LightBlue;
	}
	return !genes[EyeCollagen] ? EyeColor::Grey : EyeColor::Blue;
}

/*static*/ const std::string& DNA::getEyeColorString(EyeColor eyeColor)
{
	static const std::string strings[] = {
		"brown",
		"blue",
		"green",
		"dark brown",
		"grey",
		"hazel",
		"light blue",
		"cyan",
		"red",
		"black"
	};
	return strings[static_cast<int>(eyeColor)];
}

// indices go [Dark1][Dark2][Hue1][Tint1][Tint2]
static const DNA::SkinColor skinColours[2][2][2][2][2] = {
	{ // Dark1 = 0
		{ // Dark2 = 0
			{ // Hue1 = 0
				{DNA::SkinColor::Nordic,    DNA::SkinColor::Caucasian}, // Tint1 = 0
				{DNA::SkinColor::Caucasian, DNA::SkinColor::Mediterranean}  // Tint1 = 1
			},
			{// Hue1 = 1
				{DNA::SkinColor::Asian, DNA::SkinColor::DarkAsian}, // Tint1 = 0
				{DNA::SkinColor::SEAsian, DNA::SkinColor::DSEA}  // Tint1 = 1
			}
		},
		{ // Dark2 = 1
			{ // Hue1 = 0
				{DNA::SkinColor::Caucasian, DNA::SkinColor::Mediterranean}, // Tint1 = 0
				{DNA::SkinColor::Mediterranean, DNA::SkinColor::Hispanic} // Tint1 = 1
			},
			{ // Hue1 = 1
				{DNA::SkinColor::Arabic, DNA::SkinColor::DarkArabic}, // Tint1 = 0
				{DNA::SkinColor::Indian, DNA::SkinColor::DarkIndian}  // Tint1 = 1
			}
		}
	},
	{ // Dark1 = 1
		{ //Dark2 = 0
			{ // Hue1 = 0
				{DNA::SkinColor::Hispanic, DNA::SkinColor::Moreno}, // Tint1 = 0
				{DNA::SkinColor::Moreno2, DNA::SkinColor::DarkMoreno2}, // Tint1 = 1
			},
			{ // Hue1 = 1
				{DNA::SkinColor::Tropical,DNA::SkinColor::Nigger}, // Tint1 = 0
				{DNA::SkinColor::Tropical, DNA::SkinColor::ExtraNigger}  // Tint1 = 1
			}
		},
		{ //Dark2 = 1
			{ // Hue1 = 0
				{DNA::SkinColor::Moreno, DNA::SkinColor::DarkMoreno}, // Tint1 = 0
				{DNA::SkinColor::Moreno2, DNA::SkinColor::Nigger}  // Tint1 = 1
			},
			{ // Hue1 = 1
				{DNA::SkinColor::Nigger, DNA::SkinColor::ExtraNigger}, // Tint1 = 0
				{DNA::SkinColor::ExtraNigger, DNA::SkinColor::GiggaNigga}  // Tint1 = 1
			}
		}
	}
};

DNA::SkinColor DNA::getSkinColor() const
{
	return skinColours[genes[SkinDark1]][genes[SkinDark2]][genes[SkinHue1]][genes[SkinTint1]][genes[SkinTint2]];
}

/*static*/ const std::string& DNA::getSkinColorString(SkinColor skinColor)
{
	static const std::string strings[] = {
		"Caucasian",
		"Nordic",
		"Asian",
		"Hispanic",
		"Nigger",
		"Tropical",
		"Moreno",
		"Indian",
		"Arabic",
		"Mediterranean",
		"SEAsian",
		"ExtraNigger",
		"Moreno2",
		"DarkMoreno",
		"DarkMoreno2",
		"DarkIndian",
		"DarkArabic",
		"DSEA",
		"GiggaNigga",
		"DarkAsian"
	};
	return strings[static_cast<int>(skinColor)];
}

} // sl
