/**
 * @section DESCRIPTION
 * Represents 1 chromosome of an NPC
 */
#ifndef SL_DNA_HPP
#define SL_DNA_HPP

#include <bitset>

const int DNA_SIZE = 32;

namespace sl
{

class DNA
{
public:
	enum Index
	{
		// HAIR
		Redhead = 0,
		Hair1,
		Hair2,
		Hair3,
		Hair4,
		// SKIN
		SkinDark1,
		SkinDark2,
		SkinHue1,
		SkinTint1,
		SkinTint2,
		// EYE
		EyeCollagen,
		LightEyes,
		Herc2,
		Gey,
		// BLOOD
		BloodA,
		BloodB,
		// SEX-RELATED
		Gender,
		Whore,
		AversionToIncest,
		// PERSONALITY
		Bimbo,
		Defect1,
		Defect2,
		Defect3,
		Defect4,
		Defect5,
		Defect6,
		Defect7,
		Defect8,
		Defect9
	};

	DNA();
	/**
	 * @param mother The chromosome from the mother's side
	 * @param father The chromosome from the father's side
	 */
	DNA(const DNA& mother, const DNA& father);

	/**
	 * @return Whether the chromosome is Y (male)
	 */
	bool isMale() const;

	enum class HairColor
	{
		PlatinumBlonde,
		Blonde,
		GoldenBlonde,
		LightBrownGreyish,
		LightBrown,
		DarkBrown,
		Black,
		Red,
		LightRed
	};
	/**
	 * @return The expressed hair colour
	 */
	HairColor getHairColor() const;

	static const std::string& getHairColorString(HairColor hairColor);

	enum class EyeColor
	{
		Brown,
		Blue,
		Green,
		DarkBrown,
		Grey,
		Hazel,
		LightBlue,
		Cyan,
		Red,
		Black
	};
	/**
	 * @return The expressed eye colour
	 */
	EyeColor getEyeColor() const;

	static const std::string& getEyeColorString(EyeColor eyeColor);

	enum class SkinColor
	{
		Caucasian,
		Nordic,
		Asian,
		Hispanic,
		Nigger,
		Tropical,
		Moreno,
		Indian,
		Arabic,
		Mediterranean,
		SEAsian,
		ExtraNigger,
		Moreno2,
		DarkMoreno,
		DarkMoreno2,
		DarkIndian,
		DarkArabic,
		DSEA,
		GiggaNigga,
		DarkAsian
	};
	/**
	 * @return The expressed skin colour
	 */
	SkinColor getSkinColor() const;

	static const std::string& getSkinColorString(SkinColor skinColor);

	//! a bitset with a 0 for the recessive allele and 1 for a dominant allele for each gene
	std::bitset<DNA_SIZE> genes;
};

}

#endif
